@@ -17,14 +17,14 @@ namespace at {
 /// Return the Device of a Tensor, if the Tensor is defined.
 inline std::optional<Device> device_of(const Tensor& t) {
   if (t.defined()) {
-    return c10::make_optional(t.device());
+    return std::make_optional(t.device());
   } else {
-    return c10::nullopt;
+    return std::nullopt;
   }
 }
 
 inline std::optional<Device> device_of(const std::optional<Tensor>& t) {
-  return t.has_value() ? device_of(t.value()) : c10::nullopt;
+  return t.has_value() ? device_of(t.value()) : std::nullopt;
 }
 
 /// Return the Device of a TensorList, if the list is non-empty and
@@ -34,7 +34,7 @@ inline std::optional<Device> device_of(ITensorListRef t) {
   if (!t.empty()) {
     return device_of(t.front());
   } else {
-    return c10::nullopt;
+    return std::nullopt;
   }
 }
 
