@@ -62,7 +62,7 @@ static bool is_allowed_dim_on_scalar_tensor(int64_t dim) {
   return dim == 0 || dim == -1;
 }
 
-Tensor sum_batching_rule(const Tensor& self, OptionalIntArrayRef opt_dims, bool keepdim, optional<ScalarType> dtype) {
+Tensor sum_batching_rule(const Tensor& self, OptionalIntArrayRef opt_dims, bool keepdim, std::optional<ScalarType> dtype) {
   if (opt_dims.has_value()) {
     auto dims = opt_dims.value();
     // PyTorch has a special case where sum(scalar_tensor, dim=0) does not fail
@@ -198,7 +198,7 @@ std::vector<Tensor> chunk_batching_rule(const Tensor& self, int64_t chunks, int6
   return result;
 }
 
-Tensor clamp_batching_rule(const Tensor& self, const optional<Scalar>& min, const optional<Scalar>& max) {
+Tensor clamp_batching_rule(const Tensor& self, const std::optional<Scalar>& min, const std::optional<Scalar>& max) {
   auto self_physical = MultiBatchVmapTransform::logicalToPhysical(self);
   auto result = at::clamp(self_physical.tensor(), min, max);
   return self_physical.getPhysicalToLogicalMap().apply(result);
@@ -508,11 +508,11 @@ static void checkBatchDimsAtFrontInLayout(IntArrayRef physical_strides, int64_t
 // given (sizes, strides, storage_offset) returns the maximum location that
 // can be indexed (or nullopt if such a location doesn't exist, e.g., tensors
 // with zero-size dims).
-static optional<int64_t> maximum_indexable_location(
+static std::optional<int64_t> maximum_indexable_location(
     IntArrayRef sizes, IntArrayRef strides, int64_t storage_offset) {
   auto result = native::storage_size_for(sizes, strides);
   if (result == 0) {
-    return nullopt;
+    return std::nullopt;
   }
   return result + storage_offset;
 }
@@ -526,7 +526,7 @@ static void checkBasicAsStridedValidForSlice(
     int64_t num_batch_dims,
     IntArrayRef sizes,
     IntArrayRef strides,
-    optional<int64_t> maybe_storage_offset) {
+    std::optional<int64_t> maybe_storage_offset) {
   auto slice_sizes = physical_tensor.sizes().slice(num_batch_dims);
   auto slice_strides = physical_tensor.strides().slice(num_batch_dims);
   auto base_offset = physical_tensor.storage_offset();
@@ -558,7 +558,7 @@ static void checkBasicAsStridedValidForSlice(
       "rewrite the `as_strided` call as a sequence of PyTorch view operations");
 }
 
-Tensor _reshape_alias_batching_rule(const Tensor& self, IntArrayRef sizes, IntArrayRef strides) {
+Tensor _reshape_alias_batching_rule(const Tensor& self, IntArrayRef sizes, IntArrayRef strides [[maybe_unused]]) {
   return reshape_batching_rule(self, sizes);
 }
 
@@ -614,7 +614,7 @@ Tensor as_strided_batching_rule(
     const Tensor& tensor,
     IntArrayRef sizes,
     IntArrayRef strides,
-    optional<int64_t> storage_offset) {
+    std::optional<int64_t> storage_offset) {
   auto physical_view = at::MultiBatchVmapTransform::logicalToPhysical(tensor);
   auto num_batch_dims = physical_view.numBatchDims();
   auto physical_sizes = physical_view.getPhysicalShape(sizes);
@@ -763,7 +763,7 @@ Tensor pow_scalar_Tensor_batching_rule(const Scalar& other, const Tensor& self)
   return makeBatched(output_physical, BatchDims(old_bdims.begin(), old_bdims.end()));
 }
 
-Tensor clone_batching_rule(const Tensor& self, optional<MemoryFormat> memory_format) {
+Tensor clone_batching_rule(const Tensor& self, std::optional<MemoryFormat> memory_format) {
   // Memory format support is a little tricky because vmap is allowed to move
   // around batch dimensions and some memory formats are rank-dependent.
   // Another weird case is:
@@ -958,12 +958,12 @@ Tensor stack_batching_rule(TensorList tensors, int64_t dim) {
 // unwrap_and_call<..., at::to> because at::to takes TensorOptions& (!!)
 Tensor to_dtype_layout_batching_rule(
     const Tensor& self,
-    optional<ScalarType> dtype,
-    optional<Layout> layout,
-    optional<Device> device,
-    optional<bool> pin_memory,
+    std::optional<ScalarType> dtype,
+    std::optional<Layout> layout,
+    std::optional<Device> device,
+    std::optional<bool> pin_memory,
     bool non_blocking, bool copy,
-    optional<MemoryFormat> memory_format) {
+    std::optional<MemoryFormat> memory_format) {
   auto options = TensorOptions()
     .dtype(dtype)
     .layout(layout)
@@ -978,10 +978,10 @@ Tensor to_dtype_layout_batching_rule(
 Tensor new_zeros_batching_rule(
     const Tensor& self,
     IntArrayRef size,
-    optional<ScalarType> dtype,
-    optional<Layout> layout,
-    optional<Device> device,
-    optional<bool> pin_memory) {
+    std::optional<ScalarType> dtype,
+    std::optional<Layout> layout,
+    std::optional<Device> device,
+    std::optional<bool> pin_memory) {
   auto physical_view = MultiBatchVmapTransform::logicalToPhysical(self);
   auto physical_size = physical_view.getPhysicalShape(size);
   auto options = TensorOptions()
@@ -1010,10 +1010,10 @@ Tensor new_empty_strided_batching_rule(
     const Tensor& self,
     IntArrayRef size,
     IntArrayRef stride,
-    optional<ScalarType> dtype,
-    optional<Layout> layout,
-    optional<Device> device,
-    optional<bool> pin_memory) {
+    std::optional<ScalarType> dtype,
+    std::optional<Layout> layout,
+    std::optional<Device> device,
+    std::optional<bool> pin_memory) {
   auto physical_view = MultiBatchVmapTransform::logicalToPhysical(self);
   auto physical_size = physical_view.getPhysicalShape(size);
 
@@ -1181,9 +1181,9 @@ TORCH_LIBRARY_IMPL(aten, Batched, m) {
     m.impl(name, unwrap_and_call_method< \
         to_type, &Tensor::to, __VA_ARGS__>);\
   }
-  TO_BATCHING_RULE("to.device", Device, ScalarType, bool, bool, optional<MemoryFormat>)
-  TO_BATCHING_RULE("to.dtype", ScalarType, bool, bool, optional<MemoryFormat>)
-  TO_BATCHING_RULE("to.other", const Tensor&, bool, bool, optional<MemoryFormat>)
+  TO_BATCHING_RULE("to.device", Device, ScalarType, bool, bool, std::optional<MemoryFormat>)
+  TO_BATCHING_RULE("to.dtype", ScalarType, bool, bool, std::optional<MemoryFormat>)
+  TO_BATCHING_RULE("to.other", const Tensor&, bool, bool, std::optional<MemoryFormat>)
   m.impl("to.dtype_layout", to_dtype_layout_batching_rule);
 #undef TO_BATCHING_RULE
   m.impl("clone", clone_batching_rule);
