@@ -180,7 +180,7 @@ void boxed_func_for_outofplace_multi_op(const OperatorHandle& /*opHandle*/, Stac
 // functional
 
 void expectBoxedCallingWithReturnWorks(const KernelFunction& func) {
-  called_with_args = c10::nullopt;
+  called_with_args = std::nullopt;
   vector<IValue> stack {3, 4};
   OperatorHandle dummy = makeDummyOperatorHandle();
 
@@ -194,7 +194,7 @@ void expectBoxedCallingWithReturnWorks(const KernelFunction& func) {
 }
 
 void expectBoxedCallingWithoutReturnWorks(const KernelFunction& func) {
-  called_with_args = c10::nullopt;
+  called_with_args = std::nullopt;
   vector<IValue> stack {3, 4};
   OperatorHandle dummy = makeDummyOperatorHandle();
 
@@ -206,7 +206,7 @@ void expectBoxedCallingWithoutReturnWorks(const KernelFunction& func) {
 }
 
 void expectBoxedCallingWithMultiReturnWorks(const KernelFunction& func) {
-  called_with_args = c10::nullopt;
+  called_with_args = std::nullopt;
   vector<IValue> stack {3, 4};
   OperatorHandle dummy = makeDummyOperatorHandle();
 
@@ -284,7 +284,7 @@ void expectOutOfPlaceMultiBoxedCallingWorks(const KernelFunction& func) {
 // make an unboxed call to a kernel that returns a single value.
 //
 void expectUnboxedCallingWithReturnWorks(const KernelFunction& func) {
-  called_with_args = c10::nullopt;
+  called_with_args = std::nullopt;
   OperatorHandle dummy = makeDummyOperatorHandle();
 
   int64_t result = func.call<int64_t, int64_t, int64_t>(dummy, CPU_TEST_SET, 3, 4);
@@ -297,7 +297,7 @@ void expectUnboxedCallingWithReturnWorks(const KernelFunction& func) {
 // make an unboxed call to a kernel that returns nothing.
 //
 void expectUnboxedCallingWithoutReturnWorks(const KernelFunction& func) {
-  called_with_args = c10::nullopt;
+  called_with_args = std::nullopt;
   OperatorHandle dummy = makeDummyOperatorHandle();
 
   func.call<void, int64_t, int64_t>(dummy, CPU_TEST_SET, 3, 4);
@@ -310,7 +310,7 @@ void expectUnboxedCallingWithoutReturnWorks(const KernelFunction& func) {
 // When calling unboxed, multiple values are returned as a tuple.
 //
 void expectUnboxedCallingWithMultiReturnWorks(const KernelFunction& func) {
-  called_with_args = c10::nullopt;
+  called_with_args = std::nullopt;
   OperatorHandle dummy = makeDummyOperatorHandle();
 
   auto result = func.call<std::tuple<int64_t, int64_t>, int64_t, int64_t>(dummy, CPU_TEST_SET, 3, 4);
