@@ -2,7 +2,7 @@
 
 #include <ATen/ATen.h>
 
-namespace at { namespace native {
+namespace at::native {
 
 cudnnDataType_t getCudnnDataTypeFromScalarType(const at::ScalarType dtype) {
   if (dtype == c10::kQInt8) {
@@ -13,9 +13,7 @@ cudnnDataType_t getCudnnDataTypeFromScalarType(const at::ScalarType dtype) {
     return CUDNN_DATA_DOUBLE;
   } else if (dtype == at::kHalf) {
     return CUDNN_DATA_HALF;
-  }
-#if defined(CUDNN_VERSION) && CUDNN_VERSION >= 8200
-  else if (dtype == at::kBFloat16) {
+  } else if (dtype == at::kBFloat16) {
     return CUDNN_DATA_BFLOAT16;
   } else if (dtype == at::kInt) {
     return CUDNN_DATA_INT32;
@@ -24,7 +22,6 @@ cudnnDataType_t getCudnnDataTypeFromScalarType(const at::ScalarType dtype) {
   } else if (dtype == at::kChar) {
     return CUDNN_DATA_INT8;
   }
-#endif
   std::string msg("getCudnnDataTypeFromScalarType() not supported for ");
   msg += toString(dtype);
   throw std::runtime_error(msg);
@@ -38,4 +35,4 @@ int64_t cudnn_version() {
   return CUDNN_VERSION;
 }
 
-}}  // namespace at::cudnn
+} // namespace at::native
