@@ -14,9 +14,9 @@
 namespace at::functorch {
 
 template <typename F, F Func, typename... ExtraArgs>
-std::tuple<Tensor,optional<int64_t>> _binary_pointwise_batch_rule(
-    const Tensor& tensor, optional<int64_t> tensor_batch_dim,
-    const Tensor& other, optional<int64_t> other_batch_dim,
+std::tuple<Tensor, std::optional<int64_t>> _binary_pointwise_batch_rule(
+    const Tensor& tensor, std::optional<int64_t> tensor_batch_dim,
+    const Tensor& other, std::optional<int64_t> other_batch_dim,
     ExtraArgs... extra_args) {
 
   auto tensor_other = _binary_pointwise_helper(
@@ -33,9 +33,9 @@ struct BinaryPointwiseBatchRuleHelper;
 
 template <typename F, F Func, typename T1, typename T2, typename... T>
 struct BinaryPointwiseBatchRuleHelper<F, Func, typelist<T1, T2, T...>> {
-  static std::tuple<Tensor,optional<int64_t>> apply(
-      const Tensor& tensor, optional<int64_t> tensor_batch_dim,
-      const Tensor& other, optional<int64_t> other_batch_dim,
+  static std::tuple<Tensor, std::optional<int64_t>> apply(
+      const Tensor& tensor, std::optional<int64_t> tensor_batch_dim,
+      const Tensor& other, std::optional<int64_t> other_batch_dim,
       T... extra_args) {
     return _binary_pointwise_batch_rule<F, Func, T...>(
         tensor, tensor_batch_dim, other, other_batch_dim,
@@ -94,8 +94,8 @@ struct BinaryRandomPointwiseBatchRuleHelper<F, Func, typelist<T1, T2, T...>> {
 
 template <typename M, M Meth, typename... ExtraArgs>
 void binary_pointwise_inplace_batch_rule(
-    Tensor& tensor, optional<int64_t> tensor_batch_dim,
-    const Tensor& other, optional<int64_t> other_batch_dim,
+    Tensor& tensor, std::optional<int64_t> tensor_batch_dim,
+    const Tensor& other, std::optional<int64_t> other_batch_dim,
     ExtraArgs... extra_args) {
   if (!tensor_batch_dim && other_batch_dim) {
     vmapIncompatibleInplaceError("inplace arithmetic");
@@ -120,9 +120,9 @@ void binary_pointwise_inplace_batch_rule(
 }
 
 template <typename F, F Func>
-std::tuple<Tensor,optional<int64_t>> comparison_pointwise_batch_rule(
-    const Tensor& tensor, optional<int64_t> tensor_batch_dim,
-    const Tensor& other, optional<int64_t> other_batch_dim) {
+std::tuple<Tensor, std::optional<int64_t>> comparison_pointwise_batch_rule(
+    const Tensor& tensor, std::optional<int64_t> tensor_batch_dim,
+    const Tensor& other, std::optional<int64_t> other_batch_dim) {
   // compute max logical rank
   auto tensor_logical_rank = rankWithoutBatchDim(tensor, tensor_batch_dim);
   auto other_logical_rank = rankWithoutBatchDim(other, other_batch_dim);
@@ -142,9 +142,9 @@ std::tuple<Tensor,optional<int64_t>> comparison_pointwise_batch_rule(
   return std::make_tuple( std::move(result), 0 );
 }
 
-static std::tuple<Tensor,optional<int64_t>> where_self_batch_rule(
-    const Tensor& condition, optional<int64_t> condition_bdim,
-    const Tensor& self, optional<int64_t> self_bdim, const Tensor& other, optional<int64_t> other_bdim) {
+static std::tuple<Tensor, std::optional<int64_t>> where_self_batch_rule(
+    const Tensor& condition, std::optional<int64_t> condition_bdim,
+    const Tensor& self, std::optional<int64_t> self_bdim, const Tensor& other, std::optional<int64_t> other_bdim) {
   auto condition_logical_rank = rankWithoutBatchDim(condition, condition_bdim);
   auto tensor_logical_rank = rankWithoutBatchDim(self, self_bdim);
   auto other_logical_rank = rankWithoutBatchDim(other, other_bdim);
@@ -160,8 +160,8 @@ static std::tuple<Tensor,optional<int64_t>> where_self_batch_rule(
   return std::make_tuple(at::where(condition_, self_, other_), 0);
 }
 
-static std::tuple<Tensor, optional<int64_t>> gelu_backward_batch_rule(
-    const Tensor& grad_out, optional<int64_t> grad_out_bdim, const Tensor& input, optional<int64_t> input_bdim,
+static std::tuple<Tensor, std::optional<int64_t>> gelu_backward_batch_rule(
+    const Tensor& grad_out, std::optional<int64_t> grad_out_bdim, const Tensor& input, std::optional<int64_t> input_bdim,
     c10::string_view approximate) {
 
   // repeat the preprocessing from _binary_pointwise_batch_rule
@@ -177,9 +177,9 @@ static std::tuple<Tensor, optional<int64_t>> gelu_backward_batch_rule(
   return std::make_tuple(at::gelu_backward(grad_out_, input_, approximate), 0);
 }
 
-static std::tuple<Tensor,optional<int64_t>> masked_select_batch_rule(
-    const Tensor& self, optional<int64_t> self_bdim,
-    const Tensor& mask, optional<int64_t> mask_bdim) {
+static std::tuple<Tensor, std::optional<int64_t>> masked_select_batch_rule(
+    const Tensor& self, std::optional<int64_t> self_bdim,
+    const Tensor& mask, std::optional<int64_t> mask_bdim) {
   TORCH_CHECK(!mask_bdim.has_value(),
       "vmap: Attempted to vmap over `mask` in torch.masked_select(self, mask) ",
       "We cannot support this because for each batch this would return a ",
@@ -196,10 +196,10 @@ static std::tuple<Tensor,optional<int64_t>> masked_select_batch_rule(
   return std::make_tuple(result, 0);
 }
 
-static std::tuple<Tensor,optional<int64_t>> masked_select_backward_batch_rule(
-    const Tensor& grad, optional<int64_t> grad_bdim,
-    const Tensor& self, optional<int64_t> self_bdim,
-    const Tensor& mask, optional<int64_t> mask_bdim) {
+static std::tuple<Tensor, std::optional<int64_t>> masked_select_backward_batch_rule(
+    const Tensor& grad, std::optional<int64_t> grad_bdim,
+    const Tensor& self, std::optional<int64_t> self_bdim,
+    const Tensor& mask, std::optional<int64_t> mask_bdim) {
   TORCH_CHECK(!mask_bdim.has_value(),
       "vmap: Attempted to vmap over `mask` in torch.masked_select_backward(grad, self, mask) ",
       "We cannot support this because for each batch this would return a ",
@@ -221,12 +221,12 @@ static std::tuple<Tensor,optional<int64_t>> masked_select_backward_batch_rule(
   return std::make_tuple(result, 0);
 }
 
-static std::tuple<Tensor,optional<int64_t>> cdist_backward_batch_rule(
-    const Tensor& grad, optional<int64_t> grad_bdim,
-    const Tensor& x1, optional<int64_t> x1_bdim,
-    const Tensor& x2, optional<int64_t> x2_bdim,
+static std::tuple<Tensor, std::optional<int64_t>> cdist_backward_batch_rule(
+    const Tensor& grad, std::optional<int64_t> grad_bdim,
+    const Tensor& x1, std::optional<int64_t> x1_bdim,
+    const Tensor& x2, std::optional<int64_t> x2_bdim,
     const double p,
-    const Tensor& cdist, optional<int64_t> cdist_bdim) {
+    const Tensor& cdist, std::optional<int64_t> cdist_bdim) {
 
   auto x1_ = x1;
   if (cdist_bdim && !x1_bdim) {
@@ -258,7 +258,7 @@ static std::tuple<Tensor,optional<int64_t>> cdist_backward_batch_rule(
 
   auto out = at::_cdist_backward(grad_, x1_, x2_, p, cdist);
 
-  optional<int64_t> out_bdim = nullopt;
+  std::optional<int64_t> out_bdim = std::nullopt;
   if (x1_bdim || x2_bdim) {
     out_bdim = 0;
   }
@@ -268,16 +268,16 @@ static std::tuple<Tensor,optional<int64_t>> cdist_backward_batch_rule(
 
 static void fill__Tensor_batch_rule(
     Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     const Tensor& other,
-    optional<int64_t> other_bdim) {
+    std::optional<int64_t> other_bdim) {
   if (!other_bdim.has_value()) {
     // Optimization: fill_ is faster than the other path which does
     // reshaping + copy_
     self.fill_(other);
     return;
   }
-  if (!self_bdim && other_bdim) {
+  if (!self_bdim) {
     vmapIncompatibleInplaceError("fill_");
   }
   auto self_and_other = _binary_pointwise_helper(
@@ -285,10 +285,10 @@ static void fill__Tensor_batch_rule(
   std::get<0>(self_and_other).copy_(std::get<1>(self_and_other));
 }
 
-static std::tuple<Tensor, optional<int64_t>> log_sigmoid_backward_batch_rule(
-  Tensor& grad, optional<int64_t> grad_bdim,
-  Tensor& self, optional<int64_t> self_bdim,
-  Tensor& buffer, optional<int64_t> buffer_bdim) {
+static std::tuple<Tensor, std::optional<int64_t>> log_sigmoid_backward_batch_rule(
+  Tensor& grad, std::optional<int64_t> grad_bdim,
+  Tensor& self, std::optional<int64_t> self_bdim,
+  Tensor& buffer, std::optional<int64_t> buffer_bdim) {
   // NB: This emulates handle_pointwise_ops except we ignore the last argument, buffer
   // when any of the inputs are on cuda.
   // We do this because on cuda, buffer is a dummy tensor always of logical rank 1 and
