@@ -16,8 +16,8 @@ namespace at::functorch {
 // PyTorch's convolution is different from JAX's conv_general_dilated:
 // we do not support batch_group_count (which is needed for convolution backwards).
 // Instead, there's a convolution_backward op that needs a batching rule.
-static std::tuple<Tensor,optional<int64_t>>
-convolution_batch_rule(const Tensor& lhs, optional<int64_t> lhs_bdim, const Tensor& rhs, optional<int64_t> rhs_bdim, const optional<Tensor>& bias, optional<int64_t> bias_bdim, c10::SymIntArrayRef stride, c10::SymIntArrayRef padding, c10::SymIntArrayRef dilation, bool transposed, c10::SymIntArrayRef output_padding, c10::SymInt groups) {
+static std::tuple<Tensor, std::optional<int64_t>>
+convolution_batch_rule(const Tensor& lhs, std::optional<int64_t> lhs_bdim, const Tensor& rhs, std::optional<int64_t> rhs_bdim, const std::optional<Tensor>& bias, std::optional<int64_t> bias_bdim, c10::SymIntArrayRef stride, c10::SymIntArrayRef padding, c10::SymIntArrayRef dilation, bool transposed, c10::SymIntArrayRef output_padding, c10::SymInt groups) {
   DimVector lhs_spec(stride.size() + 2);
   std::iota(lhs_spec.begin(), lhs_spec.end(), 0);
   DimVector rhs_spec = lhs_spec;
@@ -28,18 +28,18 @@ convolution_batch_rule(const Tensor& lhs, optional<int64_t> lhs_bdim, const Tens
   }
 
   // If we have a batched bias or weight, we need to perform the computation separately.
-  optional<Tensor> unbatched_bias;
+  std::optional<Tensor> unbatched_bias;
   bool separate_bias = false;
   if ((rhs_bdim && bias && bias->defined()) || bias_bdim) {
     TORCH_INTERNAL_ASSERT(bias.has_value());
     TORCH_INTERNAL_ASSERT(bias->defined());
-    unbatched_bias = nullopt;
+    unbatched_bias = std::nullopt;
     separate_bias = true;
   } else {
     unbatched_bias = bias;
     separate_bias = false;
   }
-  std::tuple<Tensor, optional<int64_t>> result;
+  std::tuple<Tensor, std::optional<int64_t>> result;
   if (lhs_bdim && !rhs_bdim) {
     auto new_x = reshape_dim_into(*lhs_bdim, lhs_spec[0], lhs);
     auto out = at::convolution_symint(new_x, rhs, unbatched_bias, stride, padding, dilation, transposed, output_padding, groups);
@@ -103,7 +103,7 @@ convolution_batch_rule(const Tensor& lhs, optional<int64_t> lhs_bdim, const Tens
     out = reshape_dim_outof_symint(out_spec[1], lhs.sizes()[*lhs_bdim], out);
     result = std::make_tuple(out, out_spec[1]);
   } else {
-    result = std::make_tuple(at::convolution_symint(lhs, rhs, unbatched_bias, stride, padding, dilation, transposed, output_padding, groups), nullopt);
+    result = std::make_tuple(at::convolution_symint(lhs, rhs, unbatched_bias, stride, padding, dilation, transposed, output_padding, groups), std::nullopt);
   }
   if (separate_bias) {
     auto A = std::get<0>(result);
@@ -138,81 +138,6 @@ static Tensor _convolution_decomp(
       input_r, weight_r, bias_r_opt, stride_, padding_, dilation_, transposed_, output_padding_, groups_);
 }
 
-// TODO: delete the following after confirming performance
-// bool first_dim_has_size_1(const Tensor& value, int64_t bdim) {
-//   if (bdim == 0) {
-//     return value.size(1) == 1;
-//   }
-//   return value.size(0) == 1;
-// }
-//
-// std::tuple<Tensor,int64_t,Tensor,int64_t> cudnn_conv_per_sample_grad_rule(
-//     const Tensor& self, optional<int64_t> self_bdim,
-//     const Tensor& grad_output, optional<int64_t> grad_output_bdim,
-//     const Tensor& weight, optional<int64_t> weight_bdim,
-//     IntArrayRef padding, IntArrayRef stride, IntArrayRef dilation, int64_t groups, bool benchmark,
-//     bool deterministic, bool allow_tf32, std::array<bool, 2> output_mask) {
-//   TORCH_INTERNAL_ASSERT(self_bdim && grad_output_bdim && !weight_bdim);
-//   // TODO: No clue if this works if the first non-batch dim isn't size 1
-//   TORCH_INTERNAL_ASSERT(first_dim_has_size_1(self, *self_bdim));
-//   TORCH_INTERNAL_ASSERT(self.dim() == 5);
-//
-//   auto bdim_size = self.size(*self_bdim);
-//   auto self_ = reshape_dim_into(*self_bdim, 0, self);
-//   auto in_channels = self_.size(1);
-//   auto grad_output_ = reshape_dim_into(*grad_output_bdim, 0, grad_output);
-//
-//   auto grad_self = at::cudnn_convolution_backward_input(
-//       self_.sizes(), grad_output_, weight,
-//       padding, stride, dilation, groups, benchmark, deterministic, allow_tf32);
-//   grad_self = reshape_dim_outof(0, bdim_size, grad_self);
-//
-//   // Copied from https://github.com/pytorch/opacus/blob/master/opacus/grad_sample/conv.py
-//   auto A = at::im2col(self_, {weight.size(2), weight.size(3)}, dilation, padding, stride);
-//   auto B = grad_output_.reshape({bdim_size, -1, A.size(-1)});
-//   auto grad_sample = at::einsum("noq,npq->nop", {B, A});
-//   grad_sample = grad_sample.view({
-//       bdim_size, groups, -1, groups, in_channels / groups,
-//       weight.size(2) * weight.size(3) });
-//   grad_sample = at::einsum("ngrg...->ngr...", {grad_sample});
-//   grad_sample = grad_sample.reshape(
-//       {bdim_size, weight.size(0), weight.size(1), weight.size(2), weight.size(3)});
-//
-//   return std::make_tuple(grad_self, 0, grad_sample, 0);
-// }
-//
-// std::tuple<Tensor,Tensor> cudnn_convolution_backward_plumbing(const Tensor & self, const Tensor & grad_output, const Tensor & weight, IntArrayRef padding, IntArrayRef stride, IntArrayRef dilation, int64_t groups, bool benchmark, bool deterministic, bool allow_tf32, std::array<bool, 2> output_mask) {
-//   auto maybe_layer = maybeCurrentDynamicLayer();
-//   TORCH_INTERNAL_ASSERT(maybe_layer.has_value());
-//   int64_t cur_level = maybe_layer->layerId();
-//
-//   Tensor self_value;
-//   optional<int64_t> self_bdim;
-//   std::tie(self_value, self_bdim) = unwrapTensorAtLevel(self, cur_level);
-//   Tensor grad_output_value;
-//   optional<int64_t> grad_output_bdim;
-//   std::tie(grad_output_value, grad_output_bdim) = unwrapTensorAtLevel(grad_output, cur_level);
-//   Tensor weight_value;
-//   optional<int64_t> weight_bdim;
-//   std::tie(weight_value, weight_bdim) = unwrapTensorAtLevel(weight, cur_level);
-//
-//   if (self_bdim.has_value() && self_value.dim() == 5 && first_dim_has_size_1(self_value, *self_bdim) && grad_output_bdim.has_value() && !weight_bdim.has_value()) {
-//     c10::impl::ExcludeDispatchKeyGuard guard(DispatchKey::FuncTorchBatched);
-//     auto result = cudnn_conv_per_sample_grad_rule(
-//         self_value, self_bdim,
-//         grad_output_value, grad_output_bdim,
-//         weight_value, weight_bdim,
-//         padding, stride, dilation, groups,
-//         benchmark, deterministic, allow_tf32, output_mask);
-//     return std::make_tuple(
-//         makeBatched(std::get<0>(result), std::get<1>(result), cur_level),
-//         makeBatched(std::get<2>(result), std::get<3>(result), cur_level));
-//   }
-//
-//   static auto op = c10::Dispatcher::singleton()
-//     .findSchemaOrThrow("aten::cudnn_convolution_backward", "");
-//   return slow_fallback<Tensor,Tensor>(op, { self, grad_output, weight, padding, stride, dilation, groups, benchmark, deterministic, allow_tf32, output_mask });
-
 static Tensor compute_grad_bias(
     const Tensor& grad_output_, std::array<bool, 3> output_mask) {
   if (!output_mask[2]) {
@@ -227,7 +152,7 @@ static Tensor compute_grad_bias(
 
 // reshapes the batch_size into dim
 static Tensor make_dummy(
-    const Tensor& tensor, optional<int64_t> tensor_bdim,
+    const Tensor& tensor, std::optional<int64_t> tensor_bdim,
     int64_t dim, int64_t batch_size) {
   auto tensor_ = tensor_bdim ? tensor.select(*tensor_bdim, 0) : tensor;
   auto orig_size = tensor_.size(dim);
@@ -239,11 +164,11 @@ static Tensor make_dummy(
   return tensor_.new_empty({}).expand(expand_shape);
 }
 
-static std::tuple<Tensor,optional<int64_t>>
+static std::tuple<Tensor, std::optional<int64_t>>
 convolution_backward_input_batch_rule(
-    const Tensor& grad_output, optional<int64_t> grad_output_bdim,
-    const Tensor& input, optional<int64_t> input_bdim,
-    const Tensor& weight, optional<int64_t> weight_bdim,
+    const Tensor& grad_output, std::optional<int64_t> grad_output_bdim,
+    const Tensor& input, std::optional<int64_t> input_bdim,
+    const Tensor& weight, std::optional<int64_t> weight_bdim,
     c10::SymIntArrayRef stride, c10::SymIntArrayRef padding, c10::SymIntArrayRef dilation, bool transposed,
     c10::SymIntArrayRef output_padding, const c10::SymInt& groups) {
   const std::array<bool, 3> mask = {true, false, false};
@@ -255,7 +180,7 @@ convolution_backward_input_batch_rule(
     const auto weight_ = reshape_dim_into(*weight_bdim, 0, weight);
     auto dummy_input = make_dummy(input, input_bdim, 1, batch_size);
     const auto result = at::convolution_backward_symint(
-        grad_output_, dummy_input, weight_, nullopt, stride, padding,
+        grad_output_, dummy_input, weight_, std::nullopt, stride, padding,
         dilation, transposed, output_padding, groups * batch_size, mask);
     const auto grad_input = reshape_dim_outof(1, batch_size, std::get<0>(result));
     return std::make_tuple(grad_input, 1);
@@ -266,7 +191,7 @@ convolution_backward_input_batch_rule(
     const auto grad_output_ = reshape_dim_into(*grad_output_bdim, 0, grad_output);
     auto dummy_input = make_dummy(input, input_bdim, 0, batch_size);
     const auto result = at::convolution_backward_symint(
-        grad_output_, dummy_input, weight, nullopt, stride, padding,
+        grad_output_, dummy_input, weight, std::nullopt, stride, padding,
         dilation, transposed, output_padding, groups, mask);
     const auto grad_input = reshape_dim_outof(0, batch_size, std::get<0>(result));
     return std::make_tuple(grad_input, 0);
@@ -279,7 +204,7 @@ convolution_backward_input_batch_rule(
       const auto weight_ = reshape_dim_into(*weight_bdim, in_ch_dim, weight);
       auto dummy_input = make_dummy(input, input_bdim, 1, batch_size);
       const auto result = at::convolution_backward_symint(
-          grad_output, dummy_input, weight_, nullopt, stride, padding,
+          grad_output, dummy_input, weight_, std::nullopt, stride, padding,
           dilation, transposed, output_padding, groups, mask);
       const auto grad_input = reshape_dim_outof(1, batch_size, std::get<0>(result));
       return std::make_tuple(grad_input, 1);
@@ -290,7 +215,7 @@ convolution_backward_input_batch_rule(
       const auto weight_ = reshape_dim_into(*weight_bdim, 1, weight);
       auto dummy_input = make_dummy(input, input_bdim, 1, batch_size);
       const auto result = at::convolution_backward_symint(
-          grad_output, dummy_input, weight_, nullopt, stride, padding,
+          grad_output, dummy_input, weight_, std::nullopt, stride, padding,
           dilation, transposed, output_padding, groups, mask);
       grad_input = std::get<0>(result); // N(GBI)
     } else {
@@ -301,7 +226,7 @@ convolution_backward_input_batch_rule(
       weight_ = weight_.flatten(0, 2);                         // (GBI)O
       const auto dummy_input = make_dummy(input, input_bdim, 1, batch_size);
       const auto result = at::convolution_backward_symint(
-          grad_output, dummy_input, weight_, nullopt, stride, padding,
+          grad_output, dummy_input, weight_, std::nullopt, stride, padding,
           dilation, transposed, output_padding, groups, mask);
       grad_input = std::get<0>(result); // N(GBI)
     }
@@ -315,16 +240,16 @@ convolution_backward_input_batch_rule(
     TORCH_INTERNAL_ASSERT(input_bdim);
     const auto dummy_input = make_dummy(input, input_bdim, 0, 1);
     const auto result = at::convolution_backward_symint(
-        grad_output, dummy_input, weight, nullopt, stride, padding,
+        grad_output, dummy_input, weight, std::nullopt, stride, padding,
         dilation, transposed, output_padding, groups, mask);
-    return std::make_tuple(std::get<0>(result), nullopt);
+    return std::make_tuple(std::get<0>(result), std::nullopt);
   }
 }
-static std::tuple<Tensor,optional<int64_t>>
+static std::tuple<Tensor, std::optional<int64_t>>
 convolution_backward_weight_batch_rule(
-    const Tensor& grad_output, optional<int64_t> grad_output_bdim,
-    const Tensor& input, optional<int64_t> input_bdim,
-    const Tensor& weight, optional<int64_t> weight_bdim,
+    const Tensor& grad_output, std::optional<int64_t> grad_output_bdim,
+    const Tensor& input, std::optional<int64_t> input_bdim,
+    const Tensor& weight, std::optional<int64_t> weight_bdim,
     c10::SymIntArrayRef stride, c10::SymIntArrayRef padding, c10::SymIntArrayRef dilation, bool transposed,
     c10::SymIntArrayRef output_padding, const c10::SymInt& groups) {
   const std::array<bool, 3> mask = {false, true, false};
@@ -335,7 +260,7 @@ convolution_backward_weight_batch_rule(
     const auto input_ = reshape_dim_into(*input_bdim, 1, input);
     const auto dummy_weight = make_dummy(weight, weight_bdim, 0, batch_size);
     const auto result = at::convolution_backward_symint(
-        grad_output_, input_, dummy_weight, nullopt, stride, padding,
+        grad_output_, input_, dummy_weight, std::nullopt, stride, padding,
         dilation, transposed, output_padding, groups * batch_size, mask);
     auto grad_weight = std::get<1>(result);
     grad_weight = reshape_dim_outof_symint(0, batch_size, grad_weight);
@@ -349,7 +274,7 @@ convolution_backward_weight_batch_rule(
       const auto out_ch_dim = transposed ? 1 : 0;
       const auto dummy_weight = make_dummy(weight, weight_bdim, out_ch_dim, batch_size);
       const auto result = at::convolution_backward_symint(
-          grad_output_, input, dummy_weight, nullopt, stride, padding,
+          grad_output_, input, dummy_weight, std::nullopt, stride, padding,
           dilation, transposed, output_padding, groups, mask);
       auto grad_weight = std::get<1>(result);
       grad_weight = reshape_dim_outof_symint(out_ch_dim, batch_size, grad_weight);
@@ -363,7 +288,7 @@ convolution_backward_weight_batch_rule(
         // BN(GO), N(GI) -> N(GBO), N(GI) -> (GBO)I
         const auto dummy_weight = make_dummy(weight, weight_bdim, 0, batch_size);
         const auto result = at::convolution_backward_symint(
-            grad_output_, input, dummy_weight, nullopt, stride, padding,
+            grad_output_, input, dummy_weight, std::nullopt, stride, padding,
             dilation, transposed, output_padding, groups, mask);
         auto grad_weight = std::get<1>(result);
         grad_weight = grad_weight.unflatten_symint(0, { groups, batch_size, -1 }); // GBOI
@@ -374,7 +299,7 @@ convolution_backward_weight_batch_rule(
         // BN(GO), N(GI) -> N(GBO), N(GI) -> (GI)(BO)
         const auto dummy_weight = make_dummy(weight, weight_bdim, 1, batch_size);
         const auto result = at::convolution_backward_symint(
-            grad_output_, input, dummy_weight, nullopt, stride, padding,
+            grad_output_, input, dummy_weight, std::nullopt, stride, padding,
             dilation, transposed, output_padding, groups, mask);
         auto grad_weight = std::get<1>(result);
         grad_weight = reshape_dim_outof_symint(1, batch_size, grad_weight);
@@ -390,7 +315,7 @@ convolution_backward_weight_batch_rule(
       const auto in_ch_dim = transposed ? 0 : 1;
       const auto dummy_weight = make_dummy(weight, weight_bdim, in_ch_dim, batch_size);
       const auto result = at::convolution_backward_symint(
-          grad_output, input_, dummy_weight, nullopt, stride, padding,
+          grad_output, input_, dummy_weight, std::nullopt, stride, padding,
           dilation, transposed, output_padding, groups, mask);
       auto grad_weight = std::get<1>(result);
       grad_weight = reshape_dim_outof_symint(in_ch_dim, batch_size, grad_weight);
@@ -404,7 +329,7 @@ convolution_backward_weight_batch_rule(
         // regular: N(GO), BN(GI) -> N(GO), N(GBI) -> (GO)(BI)
         const auto dummy_weight = make_dummy(weight, weight_bdim, 1, batch_size);
         const auto result = at::convolution_backward_symint(
-            grad_output, input_, dummy_weight, nullopt, stride, padding,
+            grad_output, input_, dummy_weight, std::nullopt, stride, padding,
             dilation, transposed, output_padding, groups, mask);
         auto grad_weight = std::get<1>(result);
         grad_weight = reshape_dim_outof_symint(1, batch_size, grad_weight);
@@ -413,7 +338,7 @@ convolution_backward_weight_batch_rule(
         // transposed: N(GO), BN(GI) -> N(GO), N(GBI) -> (GBI)O
         const auto dummy_weight = make_dummy(weight, weight_bdim, 0, batch_size);
         const auto result = at::convolution_backward_symint(
-            grad_output, input_, dummy_weight, nullopt, stride, padding,
+            grad_output, input_, dummy_weight, std::nullopt, stride, padding,
             dilation, transposed, output_padding, groups, mask);
         auto grad_weight = std::get<1>(result);
         grad_weight = grad_weight.unflatten_symint(0, { groups, batch_size, -1 }); // GBIO
@@ -426,9 +351,9 @@ convolution_backward_weight_batch_rule(
     TORCH_INTERNAL_ASSERT(weight_bdim);
     const auto dummy_weight = make_dummy(weight, weight_bdim, 0, 1);
     const auto result = at::convolution_backward_symint(
-        grad_output, input, dummy_weight, nullopt, stride, padding,
+        grad_output, input, dummy_weight, std::nullopt, stride, padding,
         dilation, transposed, output_padding, groups, mask);
-    return std::make_tuple(std::get<1>(result), nullopt);
+    return std::make_tuple(std::get<1>(result), std::nullopt);
 
   }
 }
@@ -482,7 +407,7 @@ static std::tuple<Tensor,Tensor,Tensor> convolution_backward_plumbing(
     input = reshape_dim_into(*input_bdim, 1, input);
     weight = reshape_dim_into(*weight_bdim, 0, weight);
     const auto result = at::convolution_backward_symint(
-        grad_output, input, weight, nullopt, stride, padding, dilation,
+        grad_output, input, weight, std::nullopt, stride, padding, dilation,
         transposed, output_padding, batch_size * groups, output_mask);
     // N(BI), (BO)I -> NBI, BOI
     const auto grad_input = output_mask[0] ?
