@@ -6,12 +6,14 @@
 
 #include <ATen/functorch/BatchRulesHelper.h>
 
-namespace at { namespace functorch {
+namespace at::functorch {
 
-typedef std::tuple<Tensor, optional<int64_t>> oneOutput;
-typedef std::tuple<Tensor, optional<int64_t>, Tensor, optional<int64_t>> twoOutputs;
-typedef std::tuple<Tensor, optional<int64_t>, Tensor, optional<int64_t>, Tensor, optional<int64_t>> threeOutputs;
-typedef std::tuple<Tensor, optional<int64_t>, Tensor, optional<int64_t>, Tensor, optional<int64_t>, Tensor, optional<int64_t>> fourOutputs;
+typedef std::tuple<Tensor, std::optional<int64_t>> oneOutput;
+typedef std::tuple<Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>> twoOutputs;
+typedef std::tuple<Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>> threeOutputs;
+typedef std::tuple<Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>> fourOutputs;
+
+namespace {
 
 // Note [Batching rules for matmul-like operators]
 // at::matmul doesn't "de-expand" arguments to get better performance (maybe
@@ -20,7 +22,7 @@ typedef std::tuple<Tensor, optional<int64_t>, Tensor, optional<int64_t>, Tensor,
 // only one of the two arguments is a BatchedTensor, then we should try
 // not to expand batch dimensions onto the other arg.
 
-std::tuple<Tensor, optional<int64_t>> dot_batch_rule(const Tensor& A, optional<int64_t> A_bdim, const Tensor& B, optional<int64_t> B_bdim) {
+std::tuple<Tensor, std::optional<int64_t>> dot_batch_rule(const Tensor& A, std::optional<int64_t> A_bdim, const Tensor& B, std::optional<int64_t> B_bdim) {
   TORCH_CHECK(A.dim() - A_bdim.has_value() == 1 && B.dim() - B_bdim.has_value() == 1, "Got wrong shapes for dot");
   auto A_ = moveBatchDimToFront(A, A_bdim);
   auto B_ = moveBatchDimToFront(B, B_bdim);
@@ -37,9 +39,9 @@ Tensor vdot_decomp(const Tensor& A, const Tensor& B) {
 // NB: I wrote this like this because we *might* want its for a future matmul
 // batch rule that isn't decomposed...
 // "tv" = tensor @ vector
-static std::tuple<Tensor, optional<int64_t>> tv_batch_rule(
-    const Tensor& self, optional<int64_t> self_bdim,
-    const Tensor& other, optional<int64_t> other_bdim) {
+static std::tuple<Tensor, std::optional<int64_t>> tv_batch_rule(
+    const Tensor& self, std::optional<int64_t> self_bdim,
+    const Tensor& other, std::optional<int64_t> other_bdim) {
   if (self_bdim && other_bdim) {
     // See Note [Batching rules for matmul-like operators]
     // B...OI, BI -> ...BOI, BI1 -> ...BO1 -> ...BO
@@ -64,9 +66,9 @@ static std::tuple<Tensor, optional<int64_t>> tv_batch_rule(
   TORCH_INTERNAL_ASSERT(false, "can't get here");
 }
 
-static std::tuple<Tensor, optional<int64_t>> mv_batch_rule(
-    const Tensor& self, optional<int64_t> self_bdim,
-    const Tensor& other, optional<int64_t> other_bdim) {
+static std::tuple<Tensor, std::optional<int64_t>> mv_batch_rule(
+    const Tensor& self, std::optional<int64_t> self_bdim,
+    const Tensor& other, std::optional<int64_t> other_bdim) {
   auto self_logical_rank = rankWithoutBatchDim(self, self_bdim);
   auto other_logical_rank = rankWithoutBatchDim(other, other_bdim);
   TORCH_CHECK(self_logical_rank == 2 && other_logical_rank == 1,
@@ -77,9 +79,9 @@ static std::tuple<Tensor, optional<int64_t>> mv_batch_rule(
   return tv_batch_rule(self, self_bdim, other, other_bdim);
 }
 
-static std::tuple<Tensor, optional<int64_t>> mm_batch_rule(
-    const Tensor& self, optional<int64_t> self_bdim,
-    const Tensor& other, optional<int64_t> other_bdim) {
+static std::tuple<Tensor, std::optional<int64_t>> mm_batch_rule(
+    const Tensor& self, std::optional<int64_t> self_bdim,
+    const Tensor& other, std::optional<int64_t> other_bdim) {
   auto self_logical_rank = rankWithoutBatchDim(self, self_bdim);
   auto other_logical_rank = rankWithoutBatchDim(other, other_bdim);
   TORCH_CHECK(self_logical_rank == 2 && other_logical_rank == 2,
@@ -92,9 +94,9 @@ static std::tuple<Tensor, optional<int64_t>> mm_batch_rule(
   return std::make_tuple( at::matmul(self_, other_), 0 );
 }
 
-static std::tuple<Tensor, optional<int64_t>> bmm_batch_rule(
-    const Tensor& self, optional<int64_t> self_bdim,
-    const Tensor& other, optional<int64_t> other_bdim) {
+static std::tuple<Tensor, std::optional<int64_t>> bmm_batch_rule(
+    const Tensor& self, std::optional<int64_t> self_bdim,
+    const Tensor& other, std::optional<int64_t> other_bdim) {
   auto self_logical_rank = rankWithoutBatchDim(self, self_bdim);
   auto other_logical_rank = rankWithoutBatchDim(other, other_bdim);
   TORCH_CHECK(self_logical_rank == 3 && other_logical_rank == 3,
@@ -144,35 +146,20 @@ Tensor baddbmm_decomp(
   return out;
 }
 
-Tensor linear_decomp(
-    const Tensor& input, const Tensor& weight,
-    const c10::optional<Tensor>& bias_opt) {
-  auto result = input.matmul(weight.t());
-  if (bias_opt) {
-    // NB: It's too much work to figure out how to actually fuse the bias so
-    // we're not going to.
-    // TODO: if the result isn't batched but bias is, then we need to do the following.
-    // Otherwise, it can just be in-place. We should write a more nuanced
-    // decomposition rule
-    return result.add(*bias_opt);
-  }
-  return result;
-}
-
 Tensor addmm_decomp(const Tensor& self, const Tensor& mat1, const Tensor& mat2, const Scalar& beta, const Scalar& alpha) {
   // Decomposition that is probably not very fast...
   return at::add(self * beta, at::mm(mat1, mat2), alpha);
 }
 
-void _linalg_check_errors_batch_rule(const Tensor& info, optional<int64_t> info_bdim, c10::string_view api_name, bool is_matrix) {
+void _linalg_check_errors_batch_rule(const Tensor& info, std::optional<int64_t> info_bdim, c10::string_view api_name, bool is_matrix) {
   auto info_ = moveBatchDimToFront(info, info_bdim);
   // Not a matrix means this is a batch of matrices
   at::_linalg_check_errors(info_, api_name, false);
 }
 
-std::tuple<Tensor, c10::optional<int64_t>>
-householder_product_batch_rule(const Tensor &input, c10::optional<int64_t> input_bdim,
-                               const Tensor &tau, c10::optional<int64_t> tau_bdim)
+std::tuple<Tensor, std::optional<int64_t>>
+householder_product_batch_rule(const Tensor &input, std::optional<int64_t> input_bdim,
+                               const Tensor &tau, std::optional<int64_t> tau_bdim)
 {
   auto input_ = moveBatchDimToFront(input, input_bdim);
   auto tau_ = moveBatchDimToFront(tau, tau_bdim);
@@ -189,14 +176,14 @@ struct LinalgCheckMatrixUnaryRuleHelper;
 
 template <char const *op_name, typename F, F Func, typename A, typename... T>
 struct LinalgCheckMatrixUnaryRuleHelper<op_name, F, Func, typelist<A, T...>> {
-  static inline Tensor check_and_reshape_input(const Tensor& tensor, optional<int64_t> batch_dim) {
+  static inline Tensor check_and_reshape_input(const Tensor& tensor, std::optional<int64_t> batch_dim) {
     TORCH_CHECK(rankWithoutBatchDim(tensor, batch_dim) >= 2, op_name, ": The input tensor A must have at least 2 dimensions.");
     return moveBatchDimToFront(tensor, batch_dim);
   }
 
   static oneOutput apply_one(
       const Tensor& tensor,
-      optional<int64_t> batch_dim,
+      std::optional<int64_t> batch_dim,
       T... extra_args) {
     const auto tensor_ = check_and_reshape_input(tensor, batch_dim);
     return std::make_tuple(Func(tensor_, std::forward<T>(extra_args)...), 0);
@@ -204,7 +191,7 @@ struct LinalgCheckMatrixUnaryRuleHelper<op_name, F, Func, typelist<A, T...>> {
 
   static twoOutputs apply_two(
       const Tensor& tensor,
-      optional<int64_t> batch_dim,
+      std::optional<int64_t> batch_dim,
       T... extra_args) {
     const auto tensor_ = check_and_reshape_input(tensor, batch_dim);
     const auto res = Func(tensor_, std::forward<T>(extra_args)...);
@@ -213,7 +200,7 @@ struct LinalgCheckMatrixUnaryRuleHelper<op_name, F, Func, typelist<A, T...>> {
 
   static threeOutputs apply_three(
       const Tensor& tensor,
-      optional<int64_t> batch_dim,
+      std::optional<int64_t> batch_dim,
       T... extra_args) {
     const auto tensor_ = check_and_reshape_input(tensor, batch_dim);
     const auto res = Func(tensor_, std::forward<T>(extra_args)...);
@@ -222,7 +209,7 @@ struct LinalgCheckMatrixUnaryRuleHelper<op_name, F, Func, typelist<A, T...>> {
 
   static fourOutputs apply_four(
       const Tensor& tensor,
-      optional<int64_t> batch_dim,
+      std::optional<int64_t> batch_dim,
       T... extra_args) {
     const auto tensor_ = check_and_reshape_input(tensor, batch_dim);
     const auto res = Func(tensor_, std::forward<T>(extra_args)...);
@@ -236,8 +223,8 @@ struct LinalgCheckMatrixBinaryRuleHelper;
 template <char const *op_name, typename F, F Func, typename A, typename B, typename... T>
 struct LinalgCheckMatrixBinaryRuleHelper<op_name, F, Func, typelist<A, B, T...>> {
   static inline std::tuple<Tensor, Tensor> check_inputs_and_reshape_inputs(
-      const Tensor& first, optional<int64_t> first_bdim,
-      const Tensor& second, optional<int64_t> second_bdim) {
+      const Tensor& first, std::optional<int64_t> first_bdim,
+      const Tensor& second, std::optional<int64_t> second_bdim) {
     TORCH_CHECK(rankWithoutBatchDim(first, first_bdim) >= 2,
                 op_name, ": The input tensor A must have at least 2 dimensions.");
     TORCH_CHECK(rankWithoutBatchDim(second, second_bdim) >= 2,
@@ -246,8 +233,8 @@ struct LinalgCheckMatrixBinaryRuleHelper<op_name, F, Func, typelist<A, B, T...>>
   }
 
   static oneOutput apply_one(
-      const Tensor& first, optional<int64_t> first_bdim,
-      const Tensor& second, optional<int64_t> second_bdim,
+      const Tensor& first, std::optional<int64_t> first_bdim,
+      const Tensor& second, std::optional<int64_t> second_bdim,
       T... extra_args) {
     const auto tensor_other = check_inputs_and_reshape_inputs(first, first_bdim, second, second_bdim);
     const auto tensor_ = std::get<0>(tensor_other);
@@ -256,8 +243,8 @@ struct LinalgCheckMatrixBinaryRuleHelper<op_name, F, Func, typelist<A, B, T...>>
   }
 
   static twoOutputs apply_two(
-      const Tensor& first, optional<int64_t> first_bdim,
-      const Tensor& second, optional<int64_t> second_bdim,
+      const Tensor& first, std::optional<int64_t> first_bdim,
+      const Tensor& second, std::optional<int64_t> second_bdim,
       T... extra_args) {
     const auto tensor_other = check_inputs_and_reshape_inputs(first, first_bdim, second, second_bdim);
     const auto tensor_ = std::get<0>(tensor_other);
@@ -269,7 +256,7 @@ struct LinalgCheckMatrixBinaryRuleHelper<op_name, F, Func, typelist<A, B, T...>>
 
 static void expect_at_least_rank(
     const Tensor& tensor,
-    optional<int64_t> tensor_bdim,
+    std::optional<int64_t> tensor_bdim,
     int64_t expected_rank,
     const char* name) {
   auto rank = rankWithoutBatchDim(tensor, tensor_bdim);
@@ -278,10 +265,32 @@ static void expect_at_least_rank(
       rank, " dimensions instead.");
 }
 
+threeOutputs linalg_lu_unpack_batch_rule(
+    const Tensor& LU, std::optional<int64_t> LU_bdim,
+    const Tensor& pivots, std::optional<int64_t> pivots_bdim,
+    bool unpack_data, bool unpack_pivots) {
+  auto LU_ = moveBatchDimToFront(LU, LU_bdim);
+  auto pivots_ = moveBatchDimToFront(pivots, pivots_bdim);
+
+  // LU and pivots's first {N-2} (for LU), {N-1} (for pivots) dimensions must
+  // match So if only one of them is being vmapped over, we must expand out that
+  // dimension.
+  if (LU_bdim.has_value() != pivots_bdim.has_value()) {
+    auto bdim_size = get_bdim_size2(LU, LU_bdim, pivots, pivots_bdim);
+    LU_ = ensure_has_bdim(LU_, LU_bdim.has_value(), bdim_size);
+    pivots_ = ensure_has_bdim(pivots_, pivots_bdim.has_value(), bdim_size);
+    pivots_bdim = 0;
+    LU_bdim = 0;
+  }
+
+  const auto res = at::lu_unpack(LU_, pivots_, unpack_data, unpack_pivots);
+  return std::make_tuple(std::get<0>(res), 0, std::get<1>(res), 0, std::get<2>(res), 0);
+}
+
 oneOutput linalg_lu_solve_batch_rule(
-    const Tensor& LU, optional<int64_t> LU_bdim,
-    const Tensor& pivots, optional<int64_t> pivots_bdim,
-    const Tensor& B, optional<int64_t> B_bdim,
+    const Tensor& LU, std::optional<int64_t> LU_bdim,
+    const Tensor& pivots, std::optional<int64_t> pivots_bdim,
+    const Tensor& B, std::optional<int64_t> B_bdim,
     bool left, bool adjoint) {
   const auto LU_min_rank = 2;
   const auto pivots_min_rank = 1;
@@ -321,8 +330,8 @@ oneOutput linalg_lu_solve_batch_rule(
 }
 
 oneOutput cholesky_solve_batch_rule(
-    const Tensor& self, c10::optional<int64_t> self_bdim,
-    const Tensor& A, c10::optional<int64_t> A_bdim,
+    const Tensor& self, std::optional<int64_t> self_bdim,
+    const Tensor& A, std::optional<int64_t> A_bdim,
     bool upper) {
   TORCH_CHECK(rankWithoutBatchDim(self, self_bdim) >= 2,
            "b should have at least 2 dimensions, but has ", self.dim(), " dimensions instead");
@@ -336,22 +345,22 @@ oneOutput cholesky_solve_batch_rule(
 }
 
 threeOutputs linalg_lu_factor_ex_batch_rule(
-    const Tensor& A, c10::optional<int64_t> A_bdim, bool pivot, bool check_errors) {
-  TORCH_CHECK(rankWithoutBatchDim(A, A_bdim) >= 2, "torch.lu_factor: Expected tensor with 2 or more dimensions. Got size: ", A.sizes(), " instead");
+    const Tensor& A, std::optional<int64_t> A_bdim, bool pivot, bool check_errors) {
+  TORCH_CHECK(rankWithoutBatchDim(A, A_bdim) >= 2, "torch.lu_factor_ex: Expected tensor with 2 or more dimensions. Got size: ", A.sizes(), " instead");
   const auto A_ = moveBatchDimToFront(A, A_bdim);
   const auto res = at::linalg_lu_factor_ex(A_, pivot, check_errors);
   return std::make_tuple(std::get<0>(res), 0, std::get<1>(res), 0, std::get<2>(res), 0);
 }
 
-oneOutput matrix_exp_batch_rule(const Tensor& self, c10::optional<int64_t> self_bdim) {
+oneOutput matrix_exp_batch_rule(const Tensor& self, std::optional<int64_t> self_bdim) {
   TORCH_CHECK(rankWithoutBatchDim(self, self_bdim) >= 2, "torch.matrix_exp: The input tensor A must have at least 2 dimensions.");
   const auto self_ = moveBatchDimToFront(self, self_bdim).contiguous();  // seems to be a bug
   return std::make_tuple(at::matrix_exp(self_), 0);
 }
 
 fourOutputs solve_ex_batch_rule(
-    const Tensor& A, optional<int64_t> A_bdim,
-    const Tensor& B, optional<int64_t> B_bdim,
+    const Tensor& A, std::optional<int64_t> A_bdim,
+    const Tensor& B, std::optional<int64_t> B_bdim,
     bool left, bool check_errors) {
   auto batch_size = get_bdim_size2(A, A_bdim, B, B_bdim);
   const auto A_logical_rank = rankWithoutBatchDim(A, A_bdim);
@@ -361,7 +370,7 @@ fourOutputs solve_ex_batch_rule(
   TORCH_CHECK(A_logical_rank >= 2,
             "linalg.solve: The input tensor A must have at least 2 dimensions.");
 
-  int b_logical_rank = max_logical_rank;
+  auto b_logical_rank = max_logical_rank;
   if (A_logical_rank > B_logical_rank) {  // vector case: B was a vector or batched vector
     // not accurate but matches linalg error message
     TORCH_CHECK(B_logical_rank >= 1, "linalg.solve: The input tensor B must have at least 2 dimensions.");
@@ -391,8 +400,8 @@ fourOutputs solve_ex_batch_rule(
   return std::make_tuple(std::get<0>(res), 0, std::get<1>(res), 0, std::get<2>(res), 0, std::get<3>(res), 0);
 }
 
-oneOutput cross_batch_rule(const Tensor& self, c10::optional<int64_t> self_bdim,
-                           const Tensor& other, c10::optional<int64_t> other_bdim, const int64_t dim) {
+oneOutput cross_batch_rule(const Tensor& self, std::optional<int64_t> self_bdim,
+                           const Tensor& other, std::optional<int64_t> other_bdim, const int64_t dim) {
   // match cross dimension checks
   TORCH_CHECK(rankWithoutBatchDim(self, self_bdim) == rankWithoutBatchDim(other, other_bdim),
     "linalg.cross: inputs must have the same number of dimensions."
@@ -409,16 +418,16 @@ oneOutput cross_batch_rule(const Tensor& self, c10::optional<int64_t> self_bdim,
   return std::make_tuple(linalg_cross(self_, other_, dim_), 0);
 }
 
-c10::optional<int64_t> batch_dim_if_not_empty(const Tensor& t) {
+std::optional<int64_t> batch_dim_if_not_empty(const Tensor& t) {
   if (t.dim() == 1 && t.size(0) == 0) {
-    return c10::optional<int64_t>();
+    return std::optional<int64_t>();
   }
-  return c10::optional<int64_t>(0);
+  return std::optional<int64_t>(0);
 }
 
 fourOutputs linalg_lstsq_batch_rule(
-    const Tensor& self, c10::optional<int64_t> self_bdim, const Tensor& b, c10::optional<int64_t> b_bdim,
-    c10::optional<double> rcond, c10::optional<c10::string_view> driver) {
+    const Tensor& self, std::optional<int64_t> self_bdim, const Tensor& b, std::optional<int64_t> b_bdim,
+    std::optional<double> rcond, std::optional<c10::string_view> driver) {
   TORCH_CHECK(rankWithoutBatchDim(self, self_bdim) >= 2, "torch.linalg.lstsq: input must have at least 2 dimensions.");
   TORCH_CHECK(rankWithoutBatchDim(b, b_bdim) >= 1, "torch.linalg.lstsq: other must have at least 1 dimension.");
 
@@ -430,8 +439,7 @@ fourOutputs linalg_lstsq_batch_rule(
   const auto self_ = ensure_has_bdim(std::get<0>(tensor_other), self_bdim.has_value(), batch_size);
   const auto b_ = ensure_has_bdim(std::get<1>(tensor_other), b_bdim.has_value(), batch_size);
 
-  Tensor res, res_1, res_2, res_3;
-  std::tie(res, res_1, res_2, res_3) = at::linalg_lstsq(self_, b_, rcond, driver);
+  auto [res, res_1, res_2, res_3] = at::linalg_lstsq(self_, b_, rcond, driver);
 
   // everything but the 0th output are only sometimes computed. When they aren't, they're empty tensors without a bdim
   const auto res_1_bdim = batch_dim_if_not_empty(res_1);
@@ -441,11 +449,11 @@ fourOutputs linalg_lstsq_batch_rule(
 }
 
 template<typename F>
-std::tuple<Tensor, c10::optional<int64_t>>
+std::tuple<Tensor, std::optional<int64_t>>
 atol_rtol_tensor_batch_rule(
-    F Func, const Tensor& input, optional<int64_t> input_bdim,
-    const optional<Tensor>& atol, const optional<int64_t> atol_bdim,
-    const optional<Tensor>& rtol, const optional<int64_t> rtol_bdim, bool hermitian, char const *op_name) {
+    F Func, const Tensor& input, std::optional<int64_t> input_bdim,
+    const std::optional<Tensor>& atol, const std::optional<int64_t> atol_bdim,
+    const std::optional<Tensor>& rtol, const std::optional<int64_t> rtol_bdim, bool hermitian, char const *op_name) {
   auto input_logical_rank = rankWithoutBatchDim(input, input_bdim);
 
   TORCH_CHECK(input_logical_rank >= 2,
@@ -470,14 +478,169 @@ atol_rtol_tensor_batch_rule(
   return std::make_tuple(Func(input_, atol_, rtol_, hermitian), 0);
 }
 
-std::tuple<Tensor, c10::optional<int64_t>>
+static std::tuple<Tensor, std::optional<int64_t>>
 pinv_batch_rule(
-    const Tensor& input, c10::optional<int64_t> input_bdim, const optional<Tensor>& atol,
-    const c10::optional<int64_t> atol_bdim, const optional<Tensor>& rtol,
-    const c10::optional<int64_t> rtol_bdim, bool hermitian) {
+    const Tensor& input, std::optional<int64_t> input_bdim, const std::optional<Tensor>& atol,
+    const std::optional<int64_t> atol_bdim, const std::optional<Tensor>& rtol,
+    const std::optional<int64_t> rtol_bdim, bool hermitian) {
   return atol_rtol_tensor_batch_rule(ATEN_FN2(linalg_pinv, atol_rtol_tensor), input, input_bdim, atol, atol_bdim, rtol, rtol_bdim, hermitian, "linalg.pinv");
 }
 
+std::tuple<Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, SymInt, SymInt, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>>
+_scaled_dot_product_flash_attention_batch_rule(
+  const Tensor& query, std::optional<int64_t> query_bdim,
+  const Tensor& key, std::optional<int64_t> key_bdim,
+  const Tensor& value, std::optional<int64_t> value_bdim,
+  double dropout_p,
+  bool is_causal,
+  bool return_debug_mask,
+  c10::optional<double> scale
+) {
+  if (dropout_p > 0) {
+    auto maybe_layer = maybeCurrentDynamicLayer();
+    RandomnessType randomness = maybe_layer->randomness();
+    check_randomness(randomness, query_bdim.has_value() || key_bdim.has_value() || value_bdim.has_value());
+  }
+  auto batch_size = get_bdim_size3(query, query_bdim, key, key_bdim, value, value_bdim);
+  auto query_ = moveBatchDimToFront(query, query_bdim);
+  auto key_ = moveBatchDimToFront(key, key_bdim);
+  auto value_ = moveBatchDimToFront(value, value_bdim);
+  query_ = ensure_has_bdim(query_, query_bdim.has_value(), batch_size);
+  key_ = ensure_has_bdim(key_, key_bdim.has_value(), batch_size);
+  value_ = ensure_has_bdim(value_, value_bdim.has_value(), batch_size);
+  query_ = query_.flatten(0, 1);
+  key_ = key_.flatten(0, 1);
+  value_ = value_.flatten(0, 1);
+
+  const auto [res0, res1, res2, res3, res4, res5, res6, res7, res8] = at::_scaled_dot_product_flash_attention(
+      query_, key_, value_, dropout_p, is_causal, return_debug_mask, scale);
+
+  const auto res0_ = reshape_dim_outof(0, batch_size, res0);
+  const auto res1_ = reshape_dim_outof(0, batch_size, res1);
+  // res2 and res3 (cum_seq_q and cum_seq_k) are always [0] for dense tensors
+  // res4 and res5 (max_q and max_k) are SymInts, so they don't need reshaping
+  // res6 and res7 (philox seed and offset) are always non-batched
+  const auto res8_ = return_debug_mask ? reshape_dim_outof(0, batch_size, res8) : res8;
+
+  return std::make_tuple(
+    res0_, 0,
+    res1_, 0,
+    res2, std::nullopt,
+    res3, std::nullopt,
+    res4,
+    res5,
+    res6, std::nullopt,
+    res7, std::nullopt,
+    res8_, return_debug_mask ? std::optional<int64_t>(0) : std::nullopt
+  );
+}
+
+fourOutputs _scaled_dot_product_efficient_attention_batch_rule(
+  const Tensor& query, optional<int64_t> query_bdim,
+  const Tensor& key, optional<int64_t> key_bdim,
+  const Tensor& value, optional<int64_t> value_bdim,
+  const std::optional<Tensor>& attn_bias, optional<int64_t> attn_bias_bdim,
+  bool compute_log_sumexp,
+  double dropout_p,
+  bool is_causal,
+  c10::optional<double> scale
+) {
+  if (dropout_p > 0) {
+    auto maybe_layer = maybeCurrentDynamicLayer();
+    RandomnessType randomness = maybe_layer->randomness();
+    check_randomness(randomness, query_bdim.has_value() || key_bdim.has_value() || value_bdim.has_value());
+  }
+  auto batch_size = get_bdim_size3(query, query_bdim, key, key_bdim, value, value_bdim);
+  auto query_ = moveBatchDimToFront(query, query_bdim);
+  auto key_ = moveBatchDimToFront(key, key_bdim);
+  auto value_ = moveBatchDimToFront(value, value_bdim);
+  query_ = ensure_has_bdim(query_, query_bdim.has_value(), batch_size);
+  key_ = ensure_has_bdim(key_, key_bdim.has_value(), batch_size);
+  value_ = ensure_has_bdim(value_, value_bdim.has_value(), batch_size);
+
+  query_ = query_.flatten(0, 1);
+  key_ = key_.flatten(0, 1);
+  value_ = value_.flatten(0, 1);
+
+  std::optional<Tensor> attn_bias_;
+  if (attn_bias.has_value() && attn_bias->defined()) {
+    attn_bias_ = attn_bias_bdim.has_value() ? reshape_dim_into(*attn_bias_bdim, 0, attn_bias.value()) : attn_bias.value();
+  }
+  const auto [res0, res1, res2, res3] = at::_scaled_dot_product_efficient_attention(
+      query_, key_, value_, attn_bias_, compute_log_sumexp, dropout_p, is_causal, scale);
+  const auto res0_ = reshape_dim_outof(0, batch_size, res0);
+  const auto res1_ = reshape_dim_outof(0, batch_size, res1);
+  // philox seed is always non-batched
+  return std::make_tuple(res0_, 0, res1_, 0, res2, std::nullopt, res3, std::nullopt);
+}
+
+// Please unify SDPA APIs!!!
+std::tuple<Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, SymInt, SymInt, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>, Tensor, std::optional<int64_t>>
+_scaled_dot_product_cudnn_attention_batch_rule(
+  const Tensor& query, std::optional<int64_t> query_bdim,
+  const Tensor& key, std::optional<int64_t> key_bdim,
+  const Tensor& value, std::optional<int64_t> value_bdim,
+  const std::optional<Tensor>& attn_bias, std::optional<int64_t> attn_bias_bdim,
+  bool compute_log_sumexp,
+  double dropout_p,
+  bool is_causal,
+  bool return_debug_mask,
+  c10::optional<double> scale
+) {
+  if (dropout_p > 0) {
+    auto maybe_layer = maybeCurrentDynamicLayer();
+    RandomnessType randomness = maybe_layer->randomness();
+    check_randomness(randomness, query_bdim.has_value() || key_bdim.has_value() || value_bdim.has_value());
+  }
+  auto batch_size = get_bdim_size3(query, query_bdim, key, key_bdim, value, value_bdim);
+  auto query_ = moveBatchDimToFront(query, query_bdim);
+  auto key_ = moveBatchDimToFront(key, key_bdim);
+  auto value_ = moveBatchDimToFront(value, value_bdim);
+  query_ = ensure_has_bdim(query_, query_bdim.has_value(), batch_size);
+  key_ = ensure_has_bdim(key_, key_bdim.has_value(), batch_size);
+  value_ = ensure_has_bdim(value_, value_bdim.has_value(), batch_size);
+  query_ = query_.flatten(0, 1);
+  key_ = key_.flatten(0, 1);
+  value_ = value_.flatten(0, 1);
+
+  std::optional<Tensor> attn_bias_;
+  if (attn_bias.has_value() && attn_bias->defined()) {
+    attn_bias_ = attn_bias_bdim.has_value() ? reshape_dim_into(*attn_bias_bdim, 0, attn_bias.value()) : attn_bias.value();
+  }
+
+  const auto [res0, res1, res2, res3, res4, res5, res6, res7, res8] = at::_scaled_dot_product_cudnn_attention(
+      query_, key_, value_, attn_bias_, compute_log_sumexp, dropout_p, is_causal, return_debug_mask, scale);
+
+  const auto res0_ = reshape_dim_outof(0, batch_size, res0);
+  Tensor res1_;
+  std::optional<int64_t> res1_bdim;
+  if (compute_log_sumexp) {
+    res1_ = reshape_dim_outof(0, batch_size, res1);
+    res1_bdim = 0;
+  } else {
+    res1_ = res1;
+    res1_bdim = std::nullopt;
+  }
+  // res2 and res3 (cum_seq_q and cum_seq_k) are always [0] for dense tensors
+  // res4 and res5 (max_q and max_k) are SymInts, so they don't need reshaping
+  // res6 and res7 (philox seed and offset) are always non-batched
+  const auto res8_ = return_debug_mask ? reshape_dim_outof(0, batch_size, res8) : res8;
+
+  return std::make_tuple(
+    res0_, 0,
+    res1_, res1_bdim,
+    res2, std::nullopt,
+    res3, std::nullopt,
+    res4,
+    res5,
+    res6, std::nullopt,
+    res7, std::nullopt,
+    res8_, return_debug_mask ? std::optional<int64_t>(0) : std::nullopt
+  );
+}
+
+}
+
 #define LINALG_CHECK_MATRIX_UNARY_BATCH_RULE(fn, num_out) SINGLE_ARG(\
   LinalgCheckMatrixUnaryRuleHelper<\
     func_string_##fn,\
@@ -565,25 +728,24 @@ pinv_batch_rule(
   }
 
 // These need to be outside. String constant must be declared outside of a macro to be used as template param
+// NOLINTBEGIN(*array*)
 LINALG_CHECK_MATRIX_UNARY_ONE_OUT(cholesky, cholesky);
 LINALG_CHECK_MATRIX_UNARY_ONE_OUT(cholesky_inverse, cholesky_inverse);
 LINALG_CHECK_MATRIX_UNARY_TWO_OUT(linalg_cholesky_ex, linalg.cholesky);
 LINALG_CHECK_MATRIX_UNARY_TWO_OUT(linalg_eig, linalg.eig);
 LINALG_CHECK_MATRIX_UNARY_TWO_OUT(linalg_inv_ex, linalg.inv_ex);
 LINALG_CHECK_MATRIX_UNARY_THREE_OUT(linalg_ldl_factor_ex, torch.linalg.ldl_factor_ex);
-LINALG_CHECK_MATRIX_UNARY_ONE_OUT(linalg_pinv, linalg.pinv);
-LINALG_CHECK_MATRIX_UNARY_ONE_OUT2(linalg_pinv, atol_rtol_float, linalg.pinv);
 LINALG_CHECK_MATRIX_UNARY_TWO_OUT(linalg_qr, linalg.qr);
 LINALG_CHECK_MATRIX_UNARY_TWO_OUT(linalg_slogdet, linalg.slogdet);
 LINALG_CHECK_MATRIX_BINARY_ONE_OUT(linalg_solve_triangular, linalg.solve_triangular);
 
 LINALG_CHECK_MATRIX_UNARY_TWO_OUT(geqrf, geqrf);
-LINALG_CHECK_MATRIX_UNARY_ONE_OUT(logdet, logdet);
 LINALG_CHECK_MATRIX_BINARY_TWO_OUT(triangular_solve, triangular_solve);
 LINALG_CHECK_MATRIX_UNARY_THREE_OUT(_linalg_det, linalg.det);
 LINALG_CHECK_MATRIX_UNARY_TWO_OUT(_linalg_eigh, linalg.eigh);
 LINALG_CHECK_MATRIX_UNARY_FOUR_OUT(_linalg_slogdet, linalg.slogdet);
 LINALG_CHECK_MATRIX_UNARY_THREE_OUT(_linalg_svd, linalg.svd);
+// NOLINTEND(*array*)
 
 TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
   VMAP_SUPPORT(bmm, bmm_batch_rule);
@@ -594,7 +756,7 @@ TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
   VMAP_SUPPORT(dot, dot_batch_rule);
   VMAP_SUPPORT(mv, mv_batch_rule);
   VMAP_SUPPORT(mm, mm_batch_rule);
-  m.impl("linear", linear_decomp);
+  VMAP_SUPPORT(lu_unpack, linalg_lu_unpack_batch_rule);
   VMAP_SUPPORT(linalg_lu_solve, linalg_lu_solve_batch_rule);
   VMAP_SUPPORT(linalg_householder_product, householder_product_batch_rule);
   VMAP_SUPPORT(cholesky_solve, cholesky_solve_batch_rule);  // custom dim error
@@ -604,9 +766,13 @@ TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
   VMAP_SUPPORT(_linalg_solve_ex, solve_ex_batch_rule);
   VMAP_SUPPORT(linalg_cross, cross_batch_rule);
   VMAP_SUPPORT2(linalg_pinv, atol_rtol_tensor, pinv_batch_rule);
+  VMAP_SUPPORT(_scaled_dot_product_efficient_attention, _scaled_dot_product_efficient_attention_batch_rule);
+
+  VMAP_SUPPORT(_scaled_dot_product_flash_attention, _scaled_dot_product_flash_attention_batch_rule);
+  VMAP_SUPPORT(_scaled_dot_product_cudnn_attention, _scaled_dot_product_cudnn_attention_batch_rule);
 
   VMAP_SUPPORT(_linalg_check_errors, _linalg_check_errors_batch_rule);
 
   m.impl("vdot", vdot_decomp);
 }
-}}
+} // namespace at::functorch
