@@ -9,15 +9,15 @@
 #include <ATen/functorch/BatchedFallback.h>
 #include <ATen/core/dispatch/Dispatcher.h>
 
-namespace at { namespace functorch {
+namespace at::functorch {
 
 static bool is_empty_tensor(const Tensor& tensor) {
   const auto shape = tensor.sizes();
   return shape.size() == 1 && shape[0] == 0;
 }
 
-static optional<int64_t> compute_stat_bdim(
-    optional<int64_t> input_bdim,
+static std::optional<int64_t> compute_stat_bdim(
+    std::optional<int64_t> input_bdim,
     const Tensor& stat) {
   // There's a weird case where mean, rstd can both have shape (0,).
   // It's possible that this is a bug on the PyTorch side.
@@ -25,10 +25,10 @@ static optional<int64_t> compute_stat_bdim(
   if (input_bdim.has_value() && !is_empty_tensor(stat)) {
     return 0;
   }
-  return nullopt;
+  return std::nullopt;
 }
 
-static Tensor padRight(const Tensor& tensor, optional<int64_t> has_bdim, int64_t logical_rank) {
+static Tensor padRight(const Tensor& tensor, std::optional<int64_t> has_bdim, int64_t logical_rank) {
   // NB: Batch dim, if it exists, is assumed to be the first dim
   auto tensor_logical_rank = rankWithoutBatchDim(tensor, has_bdim);
   if (tensor_logical_rank >= logical_rank) {
@@ -42,13 +42,13 @@ static Tensor padRight(const Tensor& tensor, optional<int64_t> has_bdim, int64_t
 }
 
 template<typename F, F Func>
-std::tuple<Tensor,optional<int64_t>,Tensor,optional<int64_t>,Tensor,optional<int64_t>>
+std::tuple<Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>>
 batch_norm_batch_rule(
-    const Tensor& input, optional<int64_t> input_bdim,
-    const c10::optional<Tensor>& weight_opt, optional<int64_t> weight_bdim,
-    const c10::optional<Tensor>& bias_opt, optional<int64_t> bias_bdim,
-    const c10::optional<Tensor>& running_mean_opt, optional<int64_t> running_mean_bdim,
-    const c10::optional<Tensor>& running_var_opt, optional<int64_t> running_var_bdim,
+    const Tensor& input, std::optional<int64_t> input_bdim,
+    const std::optional<Tensor>& weight_opt, std::optional<int64_t> weight_bdim,
+    const std::optional<Tensor>& bias_opt, std::optional<int64_t> bias_bdim,
+    const std::optional<Tensor>& running_mean_opt, std::optional<int64_t> running_mean_bdim,
+    const std::optional<Tensor>& running_var_opt, std::optional<int64_t> running_var_bdim,
     bool training, double momentum, double eps) {
   c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight_opt);
   const Tensor& weight = *weight_maybe_owned;
@@ -63,7 +63,7 @@ batch_norm_batch_rule(
       "were not batched.\nIf you are using a module and do not need eval mode, please set `track_running_stats` to be False.",
       "If you are using a prebuilt module and do not need eval mode, please see the functorch website for resources on ",
       "how to patch your module to work with vmap");
-  c10::optional<int64_t> bdim_size;
+  std::optional<int64_t> bdim_size;
   Tensor result0;
   Tensor mean;
   Tensor rstd;
@@ -80,8 +80,8 @@ batch_norm_batch_rule(
     input_ = ensure_has_bdim(input_, input_bdim.has_value(), bdim_size.value());
     input_ = reshape_dim_into(0, /*channels dim*/1, input_);
 
-    c10::optional<Tensor> running_mean_;
-    c10::optional<Tensor> running_var_;
+    std::optional<Tensor> running_mean_;
+    std::optional<Tensor> running_var_;
     if (running_mean.defined()) {
       running_mean_ = moveBatchDimToFront(running_mean, running_mean_bdim);
       running_mean_ = ensure_has_bdim(*running_mean_, running_mean_bdim.has_value(), bdim_size.value());
@@ -114,7 +114,7 @@ batch_norm_batch_rule(
   if (bias.defined()) {
     const auto result_logical_rank = rankWithoutBatchDim(
         result0,
-        bdim_size.has_value() || weight_bdim.has_value() ? optional<int64_t>(0) : optional<int64_t>(nullopt));
+        bdim_size.has_value() || weight_bdim.has_value() ? std::optional<int64_t>(0) : std::optional<int64_t>(std::nullopt));
     auto bias_ = moveBatchDimToFront(bias, bias_bdim);
     bias_ = padRight(bias_, bias_bdim, result_logical_rank);
     result0 = result0 + bias_;
@@ -124,13 +124,13 @@ batch_norm_batch_rule(
 }
 
 template<typename F, F Func>
-std::tuple<at::Tensor,optional<int64_t>> batch_norm_backward_no_weight_bias_batch_rule(
-    const at::Tensor & grad_out, optional<int64_t> grad_out_bdim,
-    const at::Tensor & input, optional<int64_t> input_bdim,
-    const c10::optional<at::Tensor> & running_mean_opt, optional<int64_t> running_mean_bdim,
-    const c10::optional<at::Tensor> & running_var_opt, optional<int64_t> running_var_bdim,
-    const at::Tensor & mean, optional<int64_t> mean_bdim,
-    const at::Tensor & rstd, optional<int64_t> rstd_bdim,
+std::tuple<at::Tensor, std::optional<int64_t>> batch_norm_backward_no_weight_bias_batch_rule(
+    const at::Tensor & grad_out, std::optional<int64_t> grad_out_bdim,
+    const at::Tensor & input, std::optional<int64_t> input_bdim,
+    const std::optional<at::Tensor> & running_mean_opt, std::optional<int64_t> running_mean_bdim,
+    const std::optional<at::Tensor> & running_var_opt, std::optional<int64_t> running_var_bdim,
+    const at::Tensor & mean, std::optional<int64_t> mean_bdim,
+    const at::Tensor & rstd, std::optional<int64_t> rstd_bdim,
     bool training, double eps) {
   c10::MaybeOwned<Tensor> running_mean_maybe_owned = at::borrow_from_optional_tensor(running_mean_opt);
   const Tensor& running_mean = *running_mean_maybe_owned;
@@ -144,7 +144,7 @@ std::tuple<at::Tensor,optional<int64_t>> batch_norm_backward_no_weight_bias_batc
     const auto dummy_weight = at::ones(input.size(1), input.options());
     const auto result = Func(
         grad_out, input, dummy_weight, running_mean_opt, running_var_opt, mean, rstd, training, eps, {true, false, false});
-    return std::make_tuple(std::get<0>(result), nullopt);
+    return std::make_tuple(std::get<0>(result), std::nullopt);
   }
 
   auto grad_out_ = moveBatchDimToFront(grad_out, grad_out_bdim);
@@ -159,8 +159,8 @@ std::tuple<at::Tensor,optional<int64_t>> batch_norm_backward_no_weight_bias_batc
   mean_ = ensure_has_bdim(mean_, mean_bdim.has_value(), bdim_size);
   rstd_ = ensure_has_bdim(rstd_, rstd_bdim.has_value(), bdim_size);
 
-  optional<Tensor> running_mean_;
-  optional<Tensor> running_var_;
+  std::optional<Tensor> running_mean_;
+  std::optional<Tensor> running_var_;
   if (running_mean.defined()) {
     running_mean_ = moveBatchDimToFront(running_mean, running_mean_bdim);
     running_mean_ = ensure_has_bdim(*running_mean_, running_mean_bdim.has_value(), bdim_size);
@@ -199,11 +199,11 @@ template<typename F, F Func>
 std::tuple<at::Tensor,at::Tensor,at::Tensor> batch_norm_backward_plumbing(
     const at::Tensor & grad_out,
     const at::Tensor & input,
-    const c10::optional<at::Tensor> & weight_opt,
-    const c10::optional<at::Tensor> & running_mean_opt,
-    const c10::optional<at::Tensor> & running_var_opt,
-    const c10::optional<at::Tensor> & save_mean_opt,
-    const c10::optional<at::Tensor> & save_rstd_opt,
+    const std::optional<at::Tensor> & weight_opt,
+    const std::optional<at::Tensor> & running_mean_opt,
+    const std::optional<at::Tensor> & running_var_opt,
+    const std::optional<at::Tensor> & save_mean_opt,
+    const std::optional<at::Tensor> & save_rstd_opt,
     bool training,
     double eps,
     std::array<bool,3> output_mask) {
@@ -225,34 +225,26 @@ std::tuple<at::Tensor,at::Tensor,at::Tensor> batch_norm_backward_plumbing(
   vmap_check_escaped(maybe_layer, "batch_norm_backward_plumbing");
   int64_t cur_level = maybe_layer->layerId();
 
-  Tensor grad_out_value;
-  optional<int64_t> grad_out_bdim;
-  std::tie(grad_out_value, grad_out_bdim) = unwrapTensorAtLevel(grad_out, cur_level);
-  Tensor input_value;
-  optional<int64_t> input_bdim;
-  std::tie(input_value, input_bdim) = unwrapTensorAtLevel(input, cur_level);
+  auto [grad_out_value, grad_out_bdim] = unwrapTensorAtLevel(grad_out, cur_level);
+  auto [input_value, input_bdim] = unwrapTensorAtLevel(input, cur_level);
   Tensor mean_value;
-  optional<Tensor> weight_value;
-  optional<int64_t> weight_bdim;
+  std::optional<Tensor> weight_value;
+  std::optional<int64_t> weight_bdim;
   if (weight.defined()) {
     std::tie(weight_value, weight_bdim) = unwrapTensorAtLevel(weight, cur_level);
   }
-  optional<Tensor> running_mean_value;
-  optional<int64_t> running_mean_bdim;
+  std::optional<Tensor> running_mean_value;
+  std::optional<int64_t> running_mean_bdim;
   if (running_mean.defined()) {
     std::tie(running_mean_value, running_mean_bdim) = unwrapTensorAtLevel(running_mean, cur_level);
   }
-  optional<Tensor> running_var_value;
-  optional<int64_t> running_var_bdim;
+  std::optional<Tensor> running_var_value;
+  std::optional<int64_t> running_var_bdim;
   if (running_var.defined()) {
     std::tie(running_var_value, running_var_bdim) = unwrapTensorAtLevel(running_var, cur_level);
   }
-  Tensor save_mean_value;
-  optional<int64_t> save_mean_bdim;
-  std::tie(save_mean_value, save_mean_bdim) = unwrapTensorAtLevel(save_mean, cur_level);
-  Tensor save_rstd_value;
-  optional<int64_t> save_rstd_bdim;
-  std::tie(save_rstd_value, save_rstd_bdim) = unwrapTensorAtLevel(save_rstd, cur_level);
+  auto [save_mean_value, save_mean_bdim] = unwrapTensorAtLevel(save_mean, cur_level);
+  auto [save_rstd_value, save_rstd_bdim] = unwrapTensorAtLevel(save_rstd, cur_level);
 
   // results
   Tensor grad_bias;
@@ -267,16 +259,14 @@ std::tuple<at::Tensor,at::Tensor,at::Tensor> batch_norm_backward_plumbing(
     // NB: output isn't saved...
     auto mean = training ? save_mean : running_mean;
     auto var = training ? save_rstd : (1 / at::sqrt(running_var + eps));
-    const auto normalized_input = (input.transpose(0, 1) - padRight(mean, nullopt, input.dim())) * padRight(var, nullopt, input.dim());
+    const auto normalized_input = (input.transpose(0, 1) - padRight(mean, std::nullopt, input.dim())) * padRight(var, std::nullopt, input.dim());
     const auto expanded_grad_weight = normalized_input * grad_out.transpose(0, 1);
     grad_weight = expanded_grad_weight.sum(range(1, grad_out.dim()));
   }
   if (output_mask[0]) {
     const auto grad_normalized_input = weight.defined() ?
-      grad_out.transpose(0, 1) * padRight(weight, nullopt, grad_out.dim()) : grad_out.transpose(0, 1);           // [B0, C, B, *]
-    Tensor grad_normalized_input_value;
-    optional<int64_t> grad_normalized_input_bdim;
-    std::tie(grad_normalized_input_value, grad_normalized_input_bdim) =
+      grad_out.transpose(0, 1) * padRight(weight, std::nullopt, grad_out.dim()) : grad_out.transpose(0, 1);           // [B0, C, B, *]
+    auto [grad_normalized_input_value, grad_normalized_input_bdim] =
         unwrapTensorAtLevel(grad_normalized_input.transpose(0, 1), cur_level);       // [B0, B, C, *]
 
     c10::impl::ExcludeDispatchKeyGuard guard(DispatchKey::FuncTorchBatched);
@@ -294,8 +284,8 @@ std::tuple<at::Tensor,at::Tensor,at::Tensor> batch_norm_backward_plumbing(
 }
 
 static std::tuple<Tensor,Tensor,Tensor> native_group_norm_plumbing(
-    const Tensor & input, const c10::optional<Tensor> & weight_opt,
-    const c10::optional<Tensor> & bias_opt, int64_t N, int64_t C,
+    const Tensor & input, const std::optional<Tensor> & weight_opt,
+    const std::optional<Tensor> & bias_opt, int64_t N, int64_t C,
     int64_t HxW, int64_t group, double eps) {
   // See [Note: hacky wrapper removal for optional tensor]
   c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight_opt);
@@ -312,9 +302,7 @@ static std::tuple<Tensor,Tensor,Tensor> native_group_norm_plumbing(
     return at::native_group_norm(input, weight_opt, bias_opt, N, C, HxW, group, eps);
   }
 
-  Tensor input_value;
-  optional<int64_t> input_bdim;
-  std::tie(input_value, input_bdim) = unwrapTensorAtLevel(input, cur_level);
+  auto [input_value, input_bdim] = unwrapTensorAtLevel(input, cur_level);
 
   Tensor result0;
   Tensor mean;
@@ -324,36 +312,36 @@ static std::tuple<Tensor,Tensor,Tensor> native_group_norm_plumbing(
     const auto bdim_size = input_value.size(*input_bdim);
 
     c10::impl::ExcludeDispatchKeyGuard guard(DispatchKey::FuncTorchBatched);
-    const auto result = at::native_group_norm(input_, nullopt, nullopt, N * bdim_size, C, HxW, group, eps);
+    const auto result = at::native_group_norm(input_, std::nullopt, std::nullopt, N * bdim_size, C, HxW, group, eps);
     result0 = makeBatched(reshape_dim_outof(0, bdim_size, std::get<0>(result)), 0, cur_level);
     mean = makeBatched(reshape_dim_outof(0, bdim_size, std::get<1>(result)), 0, cur_level);
     rstd = makeBatched(reshape_dim_outof(0, bdim_size, std::get<2>(result)), 0, cur_level);
   } else {
     c10::impl::ExcludeDispatchKeyGuard guard(DispatchKey::FuncTorchBatched);
-    const auto result = at::native_group_norm(input_value, nullopt, nullopt, N, C, HxW, group, eps);
+    const auto result = at::native_group_norm(input_value, std::nullopt, std::nullopt, N, C, HxW, group, eps);
     result0 = std::get<0>(result);
     mean = std::get<1>(result);
     rstd = std::get<2>(result);
   }
 
   if (weight.defined()) {
-    const auto padded_weight = padRight(weight, nullopt, result0.dim() - 1);
+    const auto padded_weight = padRight(weight, std::nullopt, result0.dim() - 1);
     result0 = result0 * padded_weight;
   }
 
   if (bias.defined()) {
-    const auto padded_bias = padRight(bias, nullopt, result0.dim() - 1);
+    const auto padded_bias = padRight(bias, std::nullopt, result0.dim() - 1);
     result0 = result0 + padded_bias;
   }
 
   return std::make_tuple(result0, mean, rstd);
 }
 
-static std::tuple<at::Tensor,optional<int64_t>> group_norm_backward_no_weight_bias_batch_rule(
-    const at::Tensor & grad_out, optional<int64_t> grad_out_bdim,
-    const at::Tensor & input, optional<int64_t> input_bdim,
-    const at::Tensor & mean, optional<int64_t> mean_bdim,
-    const at::Tensor & rstd, optional<int64_t> rstd_bdim,
+static std::tuple<at::Tensor, std::optional<int64_t>> group_norm_backward_no_weight_bias_batch_rule(
+    const at::Tensor & grad_out, std::optional<int64_t> grad_out_bdim,
+    const at::Tensor & input, std::optional<int64_t> input_bdim,
+    const at::Tensor & mean, std::optional<int64_t> mean_bdim,
+    const at::Tensor & rstd, std::optional<int64_t> rstd_bdim,
     int64_t N, int64_t C, int64_t HxW, int64_t group) {
   auto grad_out_ = moveBatchDimToFront(grad_out, grad_out_bdim);
   auto input_ = moveBatchDimToFront(input, input_bdim);
@@ -376,7 +364,7 @@ static std::tuple<at::Tensor,optional<int64_t>> group_norm_backward_no_weight_bi
       input_.contiguous(),
       mean_.contiguous(),
       rstd_.contiguous(),
-      nullopt, N * bdim_size, C, HxW, group, {true, false, false});
+      std::nullopt, N * bdim_size, C, HxW, group, {true, false, false});
   auto result0 = std::get<0>(result);
   result0 = reshape_dim_outof(0, bdim_size, result0);
   return std::make_tuple(result0, 0);
@@ -384,7 +372,7 @@ static std::tuple<at::Tensor,optional<int64_t>> group_norm_backward_no_weight_bi
 
 static std::tuple<Tensor,Tensor,Tensor> native_group_norm_backward_plumbing(
   const Tensor & grad_out, const Tensor & input, const Tensor & mean,
-  const Tensor & rstd, const c10::optional<Tensor> & weight_opt,
+  const Tensor & rstd, const std::optional<Tensor> & weight_opt,
   int64_t N, int64_t C, int64_t HxW, int64_t group, std::array<bool,3> output_mask
 ) {
   // See [Note: hacky wrapper removal for optional tensor]
@@ -401,20 +389,14 @@ static std::tuple<Tensor,Tensor,Tensor> native_group_norm_backward_plumbing(
     return at::native_group_norm_backward(grad_out, input, mean, rstd, weight_opt, N, C, HxW, group, output_mask);
   }
 
-  Tensor input_value;
-  optional<int64_t> input_bdim;
-  std::tie(input_value, input_bdim) = unwrapTensorAtLevel(input, cur_level);
+  auto [input_value, input_bdim] = unwrapTensorAtLevel(input, cur_level);
   Tensor weight_value;
-  optional<int64_t> weight_bdim;
+  std::optional<int64_t> weight_bdim;
   if (weight.defined()){
     std::tie(weight_value, weight_bdim) = unwrapTensorAtLevel(weight, cur_level);
   }
-  Tensor mean_value;
-  optional<int64_t> mean_bdim;
-  std::tie(mean_value, mean_bdim) = unwrapTensorAtLevel(mean, cur_level);
-    Tensor rstd_value;
-  optional<int64_t> rstd_bdim;
-  std::tie(rstd_value, rstd_bdim) = unwrapTensorAtLevel(rstd, cur_level);
+  auto [mean_value, mean_bdim] = unwrapTensorAtLevel(mean, cur_level);
+  auto [rstd_value, rstd_bdim] = unwrapTensorAtLevel(rstd, cur_level);
 
   // results
   Tensor grad_input;
@@ -428,17 +410,15 @@ static std::tuple<Tensor,Tensor,Tensor> native_group_norm_backward_plumbing(
 
   if (output_mask[1] && weight.defined()) {
     const auto reshaped_input = reshape_dim_outof(1, group, input);
-    const auto normalized_input = (reshaped_input - padRight(mean, nullopt, reshaped_input.dim())) * padRight(rstd, nullopt, reshaped_input.dim());
+    const auto normalized_input = (reshaped_input - padRight(mean, std::nullopt, reshaped_input.dim())) * padRight(rstd, std::nullopt, reshaped_input.dim());
     const auto expanded_grad_weight = reshape_dim_into(1, 1, normalized_input) * grad_out;
     grad_weight = expanded_grad_weight.transpose(0, 1).sum(range(1, expanded_grad_weight.dim()));
   }
 
   if (output_mask[0]) {
     const auto grad_normalized_input = weight.defined() ?
-      grad_out * padRight(weight, nullopt, grad_out.dim() - 1) : grad_out;
-    Tensor grad_normalized_input_value;
-    optional<int64_t> grad_normalized_input_bdim;
-    std::tie(grad_normalized_input_value, grad_normalized_input_bdim) =
+      grad_out * padRight(weight, std::nullopt, grad_out.dim() - 1) : grad_out;
+    auto [grad_normalized_input_value, grad_normalized_input_bdim] =
         unwrapTensorAtLevel(grad_normalized_input, cur_level);
 
     c10::impl::ExcludeDispatchKeyGuard guard(DispatchKey::FuncTorchBatched);
@@ -455,7 +435,7 @@ static std::tuple<Tensor,Tensor,Tensor> native_group_norm_backward_plumbing(
 }
 
 C10_ALWAYS_INLINE bool has_same_shape(
-    const Tensor& tensor, optional<int64_t> tensor_bdim,
+    const Tensor& tensor, std::optional<int64_t> tensor_bdim,
     c10::SymIntArrayRef normalized_shape) {
   if (!tensor.defined()) {
     return true;
@@ -478,7 +458,7 @@ C10_ALWAYS_INLINE bool has_same_shape(
 }
 
 C10_ALWAYS_INLINE void check_same_shape(
-    const Tensor& tensor, optional<int64_t> tensor_bdim,
+    const Tensor& tensor, std::optional<int64_t> tensor_bdim,
     c10::SymIntArrayRef normalized_shape, const std::string& name) {
   TORCH_CHECK(has_same_shape(tensor, tensor_bdim, normalized_shape),
       "Expected ", name, " to be of same shape as normalized_shape, but got ",
@@ -491,10 +471,10 @@ C10_ALWAYS_INLINE void check_same_shape(
 // Ugh, hard to deduplicate
 C10_ALWAYS_INLINE void _check_layer_norm_inputs(
     SymIntArrayRef normalized_shape,
-    const Tensor& weight, optional<int64_t> weight_bdim,
-    const Tensor& bias, optional<int64_t> bias_bdim) {
+    const Tensor& weight, std::optional<int64_t> weight_bdim,
+    const Tensor& bias, std::optional<int64_t> bias_bdim) {
 
-  const int normalized_ndim = normalized_shape.size();
+  const auto normalized_ndim = normalized_shape.size();
   TORCH_CHECK(
       normalized_ndim >= 1,
       "Expected normalized_shape to be at least 1-dimensional, i.e., ",
@@ -504,12 +484,12 @@ C10_ALWAYS_INLINE void _check_layer_norm_inputs(
   check_same_shape(bias, bias_bdim, normalized_shape, "weight");
 }
 
-static std::tuple<Tensor,optional<int64_t>,Tensor,optional<int64_t>,Tensor,optional<int64_t>>
+static std::tuple<Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>>
 native_layer_norm_batch_rule(
-    const Tensor& input, optional<int64_t> input_bdim,
+    const Tensor& input, std::optional<int64_t> input_bdim,
     c10::SymIntArrayRef normalized_shape,
-    const c10::optional<Tensor>& weight_opt, optional<int64_t> weight_bdim,
-    const c10::optional<Tensor>& bias_opt, optional<int64_t> bias_bdim,
+    const std::optional<Tensor>& weight_opt, std::optional<int64_t> weight_bdim,
+    const std::optional<Tensor>& bias_opt, std::optional<int64_t> bias_bdim,
     double eps) {
   auto input_ = moveBatchDimToFront(input, input_bdim);
   if (!weight_bdim && !bias_bdim) {
@@ -528,7 +508,7 @@ native_layer_norm_batch_rule(
   _check_layer_norm_inputs(normalized_shape, weight, weight_bdim, bias, bias_bdim);
 
   const auto input_logical_rank = rankWithoutBatchDim(input, input_bdim);
-  const auto result = at::native_layer_norm_symint(input_, normalized_shape, nullopt, nullopt, eps);
+  const auto result = at::native_layer_norm_symint(input_, normalized_shape, std::nullopt, std::nullopt, eps);
   auto result0 = std::get<0>(result);
   const auto mean = std::get<1>(result);
   const auto rstd = std::get<2>(result);
@@ -542,7 +522,7 @@ native_layer_norm_batch_rule(
   if (bias.defined()) {
     const auto result_logical_rank = rankWithoutBatchDim(
         result0,
-        input_bdim.has_value() || weight_bdim.has_value() ? optional<int64_t>(0) : optional<int64_t>(nullopt));
+        input_bdim.has_value() || weight_bdim.has_value() ? std::optional<int64_t>(0) : std::optional<int64_t>(std::nullopt));
     auto bias_ = moveBatchDimToFront(bias, bias_bdim);
     bias_ = maybePadToLogicalRank(bias_, /*has_bdim*/bias_bdim, result_logical_rank);
     result0 = result0 + bias_;
@@ -550,18 +530,18 @@ native_layer_norm_batch_rule(
   return std::make_tuple(result0, 0, mean, stats_bdim, rstd, stats_bdim);
 }
 
-static std::tuple<at::Tensor,optional<int64_t>> native_layer_norm_backward_no_weight_bias_batch_rule(
-    const at::Tensor & grad_out, optional<int64_t> grad_out_bdim,
-    const at::Tensor & input, optional<int64_t> input_bdim,
+static std::tuple<at::Tensor, std::optional<int64_t>> native_layer_norm_backward_no_weight_bias_batch_rule(
+    const at::Tensor & grad_out, std::optional<int64_t> grad_out_bdim,
+    const at::Tensor & input, std::optional<int64_t> input_bdim,
     at::IntArrayRef normalized_shape,
-    const at::Tensor & mean, optional<int64_t> mean_bdim,
-    const at::Tensor & rstd, optional<int64_t> rstd_bdim) {
+    const at::Tensor & mean, std::optional<int64_t> mean_bdim,
+    const at::Tensor & rstd, std::optional<int64_t> rstd_bdim) {
 
   if (!grad_out_bdim.has_value() && !input_bdim.has_value() &&
       !mean_bdim.has_value() && !rstd_bdim.has_value()) {
     const auto result = at::native_layer_norm_backward(
-        grad_out, input, normalized_shape, mean, rstd, nullopt, nullopt, {true, false, false});
-    return std::make_tuple(std::get<0>(result), nullopt);
+        grad_out, input, normalized_shape, mean, rstd, std::nullopt, std::nullopt, {true, false, false});
+    return std::make_tuple(std::get<0>(result), std::nullopt);
   }
 
   auto grad_out_ = moveBatchDimToFront(grad_out, grad_out_bdim);
@@ -582,7 +562,7 @@ static std::tuple<at::Tensor,optional<int64_t>> native_layer_norm_backward_no_we
       normalized_shape,
       mean_.contiguous(),
       rstd_.contiguous(),
-      nullopt, nullopt, {true, false, false});
+      std::nullopt, std::nullopt, {true, false, false});
 
   return std::make_tuple(std::get<0>(result), 0);
 }
@@ -593,8 +573,8 @@ static std::tuple<at::Tensor,at::Tensor,at::Tensor> native_layer_norm_backward_p
     at::IntArrayRef normalized_shape,
     const at::Tensor & mean,
     const at::Tensor & rstd,
-    const c10::optional<at::Tensor> & weight_opt,
-    const c10::optional<at::Tensor> & bias_opt,
+    const std::optional<at::Tensor> & weight_opt,
+    const std::optional<at::Tensor> & bias_opt,
     std::array<bool,3> output_mask) {
   // See [Note: hacky wrapper removal for optional tensor]
   c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight_opt);
@@ -611,25 +591,17 @@ static std::tuple<at::Tensor,at::Tensor,at::Tensor> native_layer_norm_backward_p
     return at::native_layer_norm_backward(grad_out, input, normalized_shape, mean, rstd,
         weight_opt, bias_opt, output_mask);
   }
-  Tensor grad_out_value;
-  optional<int64_t> grad_out_bdim;
-  std::tie(grad_out_value, grad_out_bdim) = unwrapTensorAtLevel(grad_out, cur_level);
-  Tensor input_value;
-  optional<int64_t> input_bdim;
-  std::tie(input_value, input_bdim) = unwrapTensorAtLevel(input, cur_level);
-  Tensor mean_value;
-  optional<int64_t> mean_bdim;
-  std::tie(mean_value, mean_bdim) = unwrapTensorAtLevel(mean, cur_level);
-  Tensor rstd_value;
-  optional<int64_t> rstd_bdim;
-  std::tie(rstd_value, rstd_bdim) = unwrapTensorAtLevel(rstd, cur_level);
-  optional<Tensor> weight_value;
-  optional<int64_t> weight_bdim;
+  auto [grad_out_value, grad_out_bdim] = unwrapTensorAtLevel(grad_out, cur_level);
+  auto [input_value, input_bdim] = unwrapTensorAtLevel(input, cur_level);
+  auto [mean_value, mean_bdim] = unwrapTensorAtLevel(mean, cur_level);
+  auto [rstd_value, rstd_bdim] = unwrapTensorAtLevel(rstd, cur_level);
+  std::optional<Tensor> weight_value;
+  std::optional<int64_t> weight_bdim;
   if (weight.defined()) {
     std::tie(weight_value, weight_bdim) = unwrapTensorAtLevel(weight, cur_level);
   }
-  optional<Tensor> bias_value;
-  optional<int64_t> bias_bdim;
+  std::optional<Tensor> bias_value;
+  std::optional<int64_t> bias_bdim;
   if (bias.defined()) {
     std::tie(bias_value, bias_bdim) = unwrapTensorAtLevel(bias, cur_level);
   }
@@ -644,7 +616,7 @@ static std::tuple<at::Tensor,at::Tensor,at::Tensor> native_layer_norm_backward_p
     if (num_front_dims_to_reduce == 0) {
       grad_bias = grad_out;
     } else {
-      grad_bias = grad_out.sum(range(0, num_front_dims_to_reduce));
+      grad_bias = grad_out.sum(range(0, static_cast<int64_t>(num_front_dims_to_reduce)));
     }
   }
   if (output_mask[1] && weight_value.has_value()) {
@@ -656,15 +628,13 @@ static std::tuple<at::Tensor,at::Tensor,at::Tensor> native_layer_norm_backward_p
     if (num_front_dims_to_reduce == 0) {
       grad_weight = expanded_grad_weight;
     } else {
-      grad_weight = expanded_grad_weight.sum(range(0, num_front_dims_to_reduce));
+      grad_weight = expanded_grad_weight.sum(range(0, static_cast<int64_t>(num_front_dims_to_reduce)));
     }
   }
   if (output_mask[0]) {
     const auto grad_normalized_input = weight.defined() ?
       grad_out * weight : grad_out;
-    Tensor grad_normalized_input_value;
-    optional<int64_t> grad_normalized_input_bdim;
-    std::tie(grad_normalized_input_value, grad_normalized_input_bdim) =
+    auto [grad_normalized_input_value, grad_normalized_input_bdim] =
         unwrapTensorAtLevel(grad_normalized_input, cur_level);
 
     c10::impl::ExcludeDispatchKeyGuard guard(DispatchKey::FuncTorchBatched);
@@ -681,12 +651,12 @@ static std::tuple<at::Tensor,at::Tensor,at::Tensor> native_layer_norm_backward_p
 
 template <typename F, F Func>
 struct NativeBatchNormBatchRuleHelper {
-  static std::tuple<Tensor,optional<int64_t>,Tensor,optional<int64_t>,Tensor,optional<int64_t>> apply(
-    const Tensor& input, optional<int64_t> input_bdim,
-    const c10::optional<Tensor>& weight_opt, optional<int64_t> weight_bdim,
-    const c10::optional<Tensor>& bias_opt, optional<int64_t> bias_bdim,
-    const c10::optional<Tensor>& running_mean_opt, optional<int64_t> running_mean_bdim,
-    const c10::optional<Tensor>& running_var_opt, optional<int64_t> running_var_bdim,
+  static std::tuple<Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>> apply(
+    const Tensor& input, std::optional<int64_t> input_bdim,
+    const std::optional<Tensor>& weight_opt, std::optional<int64_t> weight_bdim,
+    const std::optional<Tensor>& bias_opt, std::optional<int64_t> bias_bdim,
+    const std::optional<Tensor>& running_mean_opt, std::optional<int64_t> running_mean_bdim,
+    const std::optional<Tensor>& running_var_opt, std::optional<int64_t> running_var_bdim,
     bool training, double momentum, double eps) {
     return batch_norm_batch_rule<F, Func>(
         input, input_bdim, weight_opt, weight_bdim, bias_opt, bias_bdim,
@@ -696,29 +666,29 @@ struct NativeBatchNormBatchRuleHelper {
 
 template <typename F, F Func>
 struct CudnnBatchNormBatchRuleHelper {
-  static std::tuple<Tensor,optional<int64_t>,Tensor,optional<int64_t>,Tensor,optional<int64_t>,Tensor,optional<int64_t>> apply(
-    const Tensor& input, optional<int64_t> input_bdim,
-    const Tensor& weight_opt, optional<int64_t> weight_bdim,
-    const c10::optional<Tensor>& bias_opt, optional<int64_t> bias_bdim,
-    const c10::optional<Tensor>& running_mean_opt, optional<int64_t> running_mean_bdim,
-    const c10::optional<Tensor>& running_var_opt, optional<int64_t> running_var_bdim,
+  static std::tuple<Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>> apply(
+    const Tensor& input, std::optional<int64_t> input_bdim,
+    const Tensor& weight_opt, std::optional<int64_t> weight_bdim,
+    const std::optional<Tensor>& bias_opt, std::optional<int64_t> bias_bdim,
+    const std::optional<Tensor>& running_mean_opt, std::optional<int64_t> running_mean_bdim,
+    const std::optional<Tensor>& running_var_opt, std::optional<int64_t> running_var_bdim,
     bool training, double momentum, double eps) {
     auto reserve = at::empty({0}, input.options().dtype(kByte));  // in experiments, reserve was never set to anything other than empty by cuda
     auto res = batch_norm_batch_rule<F, Func>(
         input, input_bdim, weight_opt, weight_bdim, bias_opt, bias_bdim,
         running_mean_opt, running_mean_bdim, running_var_opt, running_var_bdim, training, momentum, eps);
-    return std::tuple_cat(res, std::make_tuple(reserve, nullopt));
+    return std::tuple_cat(res, std::make_tuple(reserve, std::nullopt));
   }
 };
 
 template <typename F, F Func>
 struct MiopenBatchNormBatchRuleHelper {
-  static std::tuple<Tensor,optional<int64_t>,Tensor,optional<int64_t>,Tensor,optional<int64_t>> apply(
-    const Tensor& input, optional<int64_t> input_bdim,
-    const Tensor& weight_opt, optional<int64_t> weight_bdim,
-    const c10::optional<Tensor>& bias_opt, optional<int64_t> bias_bdim,
-    const c10::optional<Tensor>& running_mean_opt, optional<int64_t> running_mean_bdim,
-    const c10::optional<Tensor>& running_var_opt, optional<int64_t> running_var_bdim,
+  static std::tuple<Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>,Tensor, std::optional<int64_t>> apply(
+    const Tensor& input, std::optional<int64_t> input_bdim,
+    const Tensor& weight_opt, std::optional<int64_t> weight_bdim,
+    const std::optional<Tensor>& bias_opt, std::optional<int64_t> bias_bdim,
+    const std::optional<Tensor>& running_mean_opt, std::optional<int64_t> running_mean_bdim,
+    const std::optional<Tensor>& running_var_opt, std::optional<int64_t> running_var_bdim,
     bool training, double momentum, double eps) {
     return batch_norm_batch_rule<F, Func>(
         input, input_bdim, weight_opt, weight_bdim, bias_opt, bias_bdim,
@@ -746,11 +716,11 @@ struct NativeBatchNormBackwardBatchRuleHelper {
   static std::tuple<Tensor,Tensor,Tensor> apply(
     const at::Tensor & grad_out,
     const at::Tensor & input,
-    const c10::optional<at::Tensor> & weight_opt,
-    const c10::optional<at::Tensor> & running_mean_opt,
-    const c10::optional<at::Tensor> & running_var_opt,
-    const c10::optional<at::Tensor> & save_mean_opt,
-    const c10::optional<at::Tensor> & save_rstd_opt,
+    const std::optional<at::Tensor> & weight_opt,
+    const std::optional<at::Tensor> & running_mean_opt,
+    const std::optional<at::Tensor> & running_var_opt,
+    const std::optional<at::Tensor> & save_mean_opt,
+    const std::optional<at::Tensor> & save_rstd_opt,
     bool training,
     double eps,
     std::array<bool,3> output_mask) {
@@ -778,10 +748,10 @@ struct CudnnBatchNormBackwardBatchRuleHelper {
     const at::Tensor & input,
     const at::Tensor & grad_out,
     const at::Tensor & weight,
-    const c10::optional<at::Tensor> & running_mean_opt,
-    const c10::optional<at::Tensor> & running_var_opt,
-    const c10::optional<at::Tensor> & save_mean_opt,
-    const c10::optional<at::Tensor> & save_rstd_opt,
+    const std::optional<at::Tensor> & running_mean_opt,
+    const std::optional<at::Tensor> & running_var_opt,
+    const std::optional<at::Tensor> & save_mean_opt,
+    const std::optional<at::Tensor> & save_rstd_opt,
     double eps,
     const at::Tensor & reserve) {
 
@@ -807,10 +777,10 @@ struct MiopenBatchNormBackwardBatchRuleHelper {
     const at::Tensor & input,
     const at::Tensor & grad_out,
     const at::Tensor & weight,
-    const c10::optional<at::Tensor> & running_mean_opt,
-    const c10::optional<at::Tensor> & running_var_opt,
-    const c10::optional<at::Tensor> & save_mean_opt,
-    const c10::optional<at::Tensor> & save_rstd_opt,
+    const std::optional<at::Tensor> & running_mean_opt,
+    const std::optional<at::Tensor> & running_var_opt,
+    const std::optional<at::Tensor> & save_mean_opt,
+    const std::optional<at::Tensor> & save_rstd_opt,
     double eps) {
 
     auto maybe_layer = maybeCurrentDynamicLayer();
@@ -848,10 +818,10 @@ static std::tuple<at::Tensor,at::Tensor,at::Tensor> cudnn_batch_norm_backward_wr
     const at::Tensor & grad_out,
     const at::Tensor & input,
     const at::Tensor& weight_opt,
-    const c10::optional<at::Tensor> & running_mean_opt,
-    const c10::optional<at::Tensor> & running_var_opt,
-    const c10::optional<at::Tensor> & save_mean_opt,
-    const c10::optional<at::Tensor> & save_rstd_opt,
+    const std::optional<at::Tensor> & running_mean_opt,
+    const std::optional<at::Tensor> & running_var_opt,
+    const std::optional<at::Tensor> & save_mean_opt,
+    const std::optional<at::Tensor> & save_rstd_opt,
     bool training,
     double eps,
     std::array<bool,3> output_mask) {
@@ -864,10 +834,10 @@ static std::tuple<at::Tensor,at::Tensor,at::Tensor> miopen_batch_norm_backward_w
     const at::Tensor & grad_out,
     const at::Tensor & input,
     const at::Tensor& weight_opt,
-    const c10::optional<at::Tensor> & running_mean_opt,
-    const c10::optional<at::Tensor> & running_var_opt,
-    const c10::optional<at::Tensor> & save_mean_opt,
-    const c10::optional<at::Tensor> & save_rstd_opt,
+    const std::optional<at::Tensor> & running_mean_opt,
+    const std::optional<at::Tensor> & running_var_opt,
+    const std::optional<at::Tensor> & save_mean_opt,
+    const std::optional<at::Tensor> & save_rstd_opt,
     bool training,
     double eps,
     std::array<bool,3> output_mask) {
@@ -880,13 +850,13 @@ static std::tuple<at::Tensor,at::Tensor,at::Tensor> miopen_batch_norm_backward_w
 // work with dynamo anyway so we gain some buffer room to do wrong things here. The (reasonable) hope is that we will
 // make native_batch_norm composite implicit within a few weeks and we can fix this before vmap works with dynamo.
 static std::tuple<at::Tensor,at::Tensor,at::Tensor> _native_batch_norm_legit_batch(
-  const Tensor& self, const c10::optional<Tensor>& weight_opt, const c10::optional<Tensor>& bias_opt,
+  const Tensor& self, const std::optional<Tensor>& weight_opt, const std::optional<Tensor>& bias_opt,
   Tensor& running_mean, Tensor& running_var, bool train, double momentum, double eps) {
     return at::native_batch_norm(self, weight_opt, bias_opt, running_mean, running_var, train, momentum, eps);
 }
 
 static std::tuple<at::Tensor,at::Tensor,at::Tensor> _native_batch_norm_legit_no_stats_batch(
-  const Tensor& self, const c10::optional<Tensor>& weight_opt, const c10::optional<Tensor>& bias_opt,
+  const Tensor& self, const std::optional<Tensor>& weight_opt, const std::optional<Tensor>& bias_opt,
   bool train, double momentum, double eps) {
     return at::native_batch_norm(self, weight_opt, bias_opt, Tensor(), Tensor(), train, momentum, eps);
 }
@@ -906,4 +876,4 @@ TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
   m.impl("native_layer_norm_backward", native_layer_norm_backward_plumbing);
 }
 
-}}
+} // namespace at::functorch
