@@ -11,7 +11,7 @@
 
 namespace at::functorch {
 
-void vmap_check_escaped(const optional<DynamicLayer> &layer, const char* what) {
+void vmap_check_escaped(const std::optional<DynamicLayer> &layer, const char* what) {
   TORCH_CHECK(
     layer.has_value(),
     "Either your tensor may have escaped from inside a function being vmapped and this is a user error ",
@@ -22,7 +22,7 @@ void vmap_check_escaped(const optional<DynamicLayer> &layer, const char* what) {
   )
 }
 
-Tensor makeBatched(const Tensor& tensor, optional<int64_t> bdim, int64_t level) {
+Tensor makeBatched(const Tensor& tensor, std::optional<int64_t> bdim, int64_t level) {
   if (bdim.has_value()) {
     TORCH_INTERNAL_ASSERT(*bdim >= 0);
     TORCH_INTERNAL_ASSERT(*bdim < tensor.dim());
@@ -31,7 +31,7 @@ Tensor makeBatched(const Tensor& tensor, optional<int64_t> bdim, int64_t level)
   return tensor;
 }
 
-std::vector<Tensor> makeBatchedVector(const std::vector<Tensor>& tensors, optional<int64_t> bdim, int64_t level) {
+std::vector<Tensor> makeBatchedVector(const std::vector<Tensor>& tensors, std::optional<int64_t> bdim, int64_t level) {
   std::vector<Tensor> res;
   res.reserve(tensors.size());
   for (const auto & tensor : tensors) {
@@ -43,12 +43,12 @@ std::vector<Tensor> makeBatchedVector(const std::vector<Tensor>& tensors, option
 std::tuple<Tensor, std::optional<int64_t>> unwrapTensorAtLevel(const Tensor& tensor, int64_t level) {
   auto* batched = maybeGetBatchedImpl(tensor);
   if (!batched) {
-    return std::make_tuple(tensor, nullopt);
+    return std::make_tuple(tensor, std::nullopt);
   }
   if (batched->level() == level) {
     return std::make_tuple(batched->value(), batched->bdim());
   }
-  return std::make_tuple(tensor, nullopt);
+  return std::make_tuple(tensor, std::nullopt);
 }
 
 bool isBatchedAtLevel(const Tensor& tensor, int64_t level) {
@@ -82,7 +82,7 @@ bool isBatchedAtLevel(const c10::List<std::optional<Tensor>>& maybe_tensors, int
   return false;
 }
 
-bool areAnyBatchedAtLevel(ArrayRef<optional<Tensor>> maybe_tensors, int64_t level) {
+bool areAnyBatchedAtLevel(ArrayRef<std::optional<Tensor>> maybe_tensors, int64_t level) {
   for (const auto& maybe_tensor : maybe_tensors) {
     if (isBatchedAtLevel(maybe_tensor, level)) {
       return true;
