@@ -1,7 +1,6 @@
 #pragma once
 
-namespace at {
-namespace mps {
+namespace at::mps {
 
 static const char * indexing_metal_shaders = R"INDEX_METAL(
 #include <metal_stdlib>
@@ -9,27 +8,26 @@ static const char * indexing_metal_shaders = R"INDEX_METAL(
 
 using namespace metal;
 
-constant uint32_t num_indices            [[function_constant(0)]];
-
 struct IndexAB {
-    // Allow up to 16 indices
-    metal::array<constant void *, 16>  indexArray [[ id(0) ]];
+    constant int64_t* indexArray;
 };
 
-template<typename T>
+template<typename T, typename OffsetsT>
 kernel void index_select(
-    constant IndexAB  & indexAB           [[buffer(0)]],
+    constant IndexAB  * indexAB           [[buffer(0)]],
     constant void     * indexSizes        [[buffer(1)]],
     constant void     * indexStrides      [[buffer(2)]],
-    constant uint3    * offsets           [[buffer(3)]],
+    constant OffsetsT * offsets           [[buffer(3)]],
     constant void     * inputData         [[buffer(4)]],
     device   void     * outputData        [[buffer(5)]],
+    constant uint32_t & num_indices       [[buffer(6)]],
     uint thread_index [[thread_position_in_grid]]) {
     constant int64_t * index_sizes   = (constant int64_t *)indexSizes;
     constant int64_t * index_strides = (constant int64_t *)indexStrides;
     int64_t offset = 0;
     for (uint32_t i = 0; i < num_indices; i++) {
-        int64_t index = ((constant int64_t*)(indexAB.indexArray[i]))[offsets[thread_index].z / sizeof(int64_t)];
+        constant int64_t* indexArray = indexAB[i].indexArray;
+        int64_t index = indexArray[offsets[thread_index].z / sizeof(int64_t)];
         if (index < 0) {
             index += index_sizes[i];
         }
@@ -40,81 +38,169 @@ kernel void index_select(
     *out = *in;
 }
 
-template<typename T>
-kernel void index_put(
-    constant IndexAB  & indexAB           [[buffer(0)]],
-    constant void     * indexSizes        [[buffer(1)]],
-    constant void     * indexStrides      [[buffer(2)]],
-    constant uint3    * offsets           [[buffer(3)]],
-    constant void     * inputData         [[buffer(4)]],
-    device   void     * outputData        [[buffer(5)]],
-    uint thread_index [[thread_position_in_grid]]) {
-
-    constant int64_t * index_sizes   = (constant int64_t *)indexSizes;
-    constant int64_t * index_strides = (constant int64_t *)indexStrides;
+template<typename T, typename OffsetsT>
+void index_put_impl(
+    constant IndexAB  * indexAB,
+    constant int64_t  * index_sizes,
+    constant int64_t  * index_strides,
+    constant OffsetsT * offsets,
+    constant void     * inputData,
+    device   void     * outputData,
+    constant uint32_t & num_indices,
+    uint thread_index) {
     int64_t offset = 0;
     for (uint32_t i = 0; i < num_indices; i++) {
-        int64_t index = ((constant int64_t*)(indexAB.indexArray[i]))[offsets[thread_index].z / sizeof(int64_t)];
+        constant int64_t* indexArray = indexAB[i].indexArray;
+        int64_t index = indexArray[offsets[thread_index].z / sizeof(int64_t)];
+
         if (index < 0) {
             index += index_sizes[i];
         }
         offset += index * index_strides[i];
-     }
+    }
     device T * out = (device T*)((device char*)outputData + offsets[thread_index].x + offset);
     constant T * in  = (constant T*)((constant char*)inputData  + offsets[thread_index].y);
     *out = *in;
 }
 
-#define REGISTER_INDEX_OP(DTYPE_SIZE, DTYPE, INDEX_OP_TYPE)     \
-template                                                        \
-[[host_name("index_" #INDEX_OP_TYPE "_" #DTYPE_SIZE)]]          \
-kernel void index_ ## INDEX_OP_TYPE<DTYPE>(                     \
-    constant IndexAB & indexAB           [[buffer(0)]],         \
-    constant void    * indexSizes        [[buffer(1)]],         \
-    constant void    * indexStrides      [[buffer(2)]],         \
-    constant uint3   * offsets           [[buffer(3)]],         \
-    constant void    * inputData         [[buffer(4)]],         \
-    device   void    * outputData        [[buffer(5)]],         \
+template<typename T, typename OffsetsT>
+kernel void index_put_serial(
+    constant IndexAB  * indexAB           [[buffer(0)]],
+    constant void     * indexSizes        [[buffer(1)]],
+    constant void     * indexStrides      [[buffer(2)]],
+    constant OffsetsT * offsets           [[buffer(3)]],
+    constant void     * inputData         [[buffer(4)]],
+    device   void     * outputData        [[buffer(5)]],
+    constant uint32_t & num_indices       [[buffer(6)]],
+    constant uint     * numIters          [[buffer(7)]],
+    uint thread_index [[thread_position_in_grid]]) {
+
+    constant int64_t * index_sizes   = (constant int64_t *)indexSizes;
+    constant int64_t * index_strides = (constant int64_t *)indexStrides;
+
+    for (uint iter_i = 0; iter_i < *numIters; iter_i++) {
+        index_put_impl<T>(indexAB, index_sizes, index_strides, offsets, inputData, outputData, num_indices, iter_i);
+    }
+}
+
+template<typename T, typename OffsetsT>
+kernel void index_put(
+    constant IndexAB  * indexAB           [[buffer(0)]],
+    constant void     * indexSizes        [[buffer(1)]],
+    constant void     * indexStrides      [[buffer(2)]],
+    constant OffsetsT * offsets           [[buffer(3)]],
+    constant void     * inputData         [[buffer(4)]],
+    device   void     * outputData        [[buffer(5)]],
+    constant uint32_t & num_indices       [[buffer(6)]],
+    uint thread_index [[thread_position_in_grid]]) {
+
+    constant int64_t * index_sizes   = (constant int64_t *)indexSizes;
+    constant int64_t * index_strides = (constant int64_t *)indexStrides;
+    index_put_impl<T>(indexAB, index_sizes, index_strides, offsets, inputData, outputData, num_indices, thread_index);
+}
+
+#define REGISTER_INDEX_OP(DTYPE_SIZE, IDX_SIZE, DTYPE, INDEX_OP_TYPE, IDX_DTYPE)   \
+template                                                                           \
+[[host_name("index_" #INDEX_OP_TYPE "_" #DTYPE_SIZE "_" #IDX_SIZE)]]               \
+kernel void index_ ## INDEX_OP_TYPE<DTYPE, IDX_DTYPE>(                             \
+    constant IndexAB * indexAB           [[buffer(0)]],                            \
+    constant void    * indexSizes        [[buffer(1)]],                            \
+    constant void    * indexStrides      [[buffer(2)]],                            \
+    constant IDX_DTYPE   * offsets           [[buffer(3)]],                        \
+    constant void    * inputData         [[buffer(4)]],                            \
+    device   void    * outputData        [[buffer(5)]],                            \
+    constant uint32_t & num_indices      [[buffer(6)]],                            \
     uint thread_index [[thread_position_in_grid]]);
 
 #define REGISTER_INDEX_OP_ALL_DTYPES(INDEX_OP_TYPE)     \
-    REGISTER_INDEX_OP(8bit,  char,  INDEX_OP_TYPE);     \
-    REGISTER_INDEX_OP(16bit, short, INDEX_OP_TYPE);     \
-    REGISTER_INDEX_OP(32bit, int,   INDEX_OP_TYPE);     \
-    REGISTER_INDEX_OP(64bit, long,  INDEX_OP_TYPE);
+    REGISTER_INDEX_OP(8bit,  idx32, char,  INDEX_OP_TYPE, uint3);     \
+    REGISTER_INDEX_OP(8bit,  idx64, char,  INDEX_OP_TYPE, ulong3);    \
+    REGISTER_INDEX_OP(16bit, idx32, short, INDEX_OP_TYPE, uint3);     \
+    REGISTER_INDEX_OP(16bit, idx64, short, INDEX_OP_TYPE, ulong3);    \
+    REGISTER_INDEX_OP(32bit, idx32, int,   INDEX_OP_TYPE, uint3);     \
+    REGISTER_INDEX_OP(32bit, idx64, int,   INDEX_OP_TYPE, ulong3);    \
+    REGISTER_INDEX_OP(64bit, idx32, long,  INDEX_OP_TYPE, uint3);     \
+    REGISTER_INDEX_OP(64bit, idx64, long,  INDEX_OP_TYPE, ulong3);
 
 REGISTER_INDEX_OP_ALL_DTYPES(select);
 REGISTER_INDEX_OP_ALL_DTYPES(put);
 
-kernel void kernel_index_offsets(constant packed_uint3 * strides         [[buffer(0)]],
-                                 device uint3          * data_offsets    [[buffer(1)]],
-                                 constant uint         * iter_shape      [[buffer(2)]],
-                                 constant uint         & num_dimensions  [[buffer(3)]],
-                                 constant uint         & num_offsets     [[buffer(4)]],
-                                 uint thread_index [[thread_position_in_grid]]) {
+#define REGISTER_SINGLE_THREADED_INDEX_OP(DTYPE_SIZE, IDX_SIZE, DTYPE, INDEX_OP_TYPE, IDX_DTYPE)   \
+template                                                                                           \
+[[host_name("index_" #INDEX_OP_TYPE "_" #DTYPE_SIZE "_" #IDX_SIZE)]]                               \
+kernel void index_ ## INDEX_OP_TYPE<DTYPE, IDX_DTYPE>(                                             \
+    constant IndexAB   * indexAB           [[buffer(0)]],                                          \
+    constant void      * indexSizes        [[buffer(1)]],                                          \
+    constant void      * indexStrides      [[buffer(2)]],                                          \
+    constant IDX_DTYPE * offsets           [[buffer(3)]],                                          \
+    constant void      * inputData         [[buffer(4)]],                                          \
+    device   void      * outputData        [[buffer(5)]],                                          \
+    constant uint32_t  & num_indices       [[buffer(6)]],                                          \
+    constant uint      * numIters          [[buffer(7)]],                                          \
+    uint thread_index [[thread_position_in_grid]]);
+
+#define REGISTER_SINGLE_THREADED_INDEX_OP_ALL_DTYPES(INDEX_OP_TYPE)                   \
+    REGISTER_SINGLE_THREADED_INDEX_OP(8bit,  idx32, char,  INDEX_OP_TYPE, uint3);     \
+    REGISTER_SINGLE_THREADED_INDEX_OP(8bit,  idx64, char,  INDEX_OP_TYPE, ulong3);    \
+    REGISTER_SINGLE_THREADED_INDEX_OP(16bit, idx32, short, INDEX_OP_TYPE, uint3);     \
+    REGISTER_SINGLE_THREADED_INDEX_OP(16bit, idx64, short, INDEX_OP_TYPE, ulong3);    \
+    REGISTER_SINGLE_THREADED_INDEX_OP(32bit, idx32, int,   INDEX_OP_TYPE, uint3);     \
+    REGISTER_SINGLE_THREADED_INDEX_OP(32bit, idx64, int,   INDEX_OP_TYPE, ulong3);    \
+    REGISTER_SINGLE_THREADED_INDEX_OP(64bit, idx32, long,  INDEX_OP_TYPE, uint3);     \
+    REGISTER_SINGLE_THREADED_INDEX_OP(64bit, idx64, long,  INDEX_OP_TYPE, ulong3);
+
+REGISTER_SINGLE_THREADED_INDEX_OP_ALL_DTYPES(put_serial);
+
+template<typename StridesT, typename DataT>
+kernel void kernel_index_offsets(constant StridesT * strides         [[buffer(0)]],
+                                device DataT      * data_offsets    [[buffer(1)]],
+                                constant uint     * iter_shape      [[buffer(2)]],
+                                constant uint     & num_dimensions  [[buffer(3)]],
+                                uint thread_index [[thread_position_in_grid]]) {
+    data_offsets[thread_index] = 0;
     uint32_t idx = thread_index;
     for (uint32_t dim = 0; dim < num_dimensions; dim++) {
         uint32_t remainder = idx % iter_shape[dim];
         idx /= iter_shape[dim];
 
-        for (uint32_t offset = 0; offset < num_offsets; offset++)
-            data_offsets[thread_index][offset] += remainder * strides[dim][offset];
+        data_offsets[thread_index] += remainder * DataT(strides[dim]);
     }
 }
 
-template<typename T, typename E>
-kernel void index_put_accumulate_native_dtypes(constant IndexAB & indexAB      [[buffer(0)]],
-                                               constant void    * indexSizes   [[buffer(1)]],
-                                               constant void    * indexStrides [[buffer(2)]],
-                                               constant uint3   * offsets      [[buffer(3)]],
-                                               constant void    * inputData    [[buffer(4)]],
-                                               device       void    * outputData   [[buffer(5)]],
-                                               uint thread_index [[thread_position_in_grid]]) {
+template
+[[host_name("kernel_index_offsets_32")]]
+kernel void kernel_index_offsets<packed_uint3, uint3>(
+                constant packed_uint3 * strides         [[buffer(0)]],
+                device uint3          * data_offsets    [[buffer(1)]],
+                constant uint         * iter_shape      [[buffer(2)]],
+                constant uint         & num_dimensions  [[buffer(3)]],
+                uint thread_index [[thread_position_in_grid]]);
+
+template
+[[host_name("kernel_index_offsets_64")]]
+kernel void kernel_index_offsets<packed_uint3, ulong3>(
+                constant packed_uint3 * strides         [[buffer(0)]],
+                device ulong3          * data_offsets    [[buffer(1)]],
+                constant uint         * iter_shape      [[buffer(2)]],
+                constant uint         & num_dimensions  [[buffer(3)]],
+                uint thread_index [[thread_position_in_grid]]);
+
+template<typename T, typename E, typename OffsetsT>
+kernel void index_put_accumulate_native_dtypes(
+    constant IndexAB  * indexAB     [[buffer(0)]],
+    constant void     * indexSizes   [[buffer(1)]],
+    constant void     * indexStrides [[buffer(2)]],
+    constant OffsetsT * offsets      [[buffer(3)]],
+    constant void     * inputData    [[buffer(4)]],
+    device void       * outputData   [[buffer(5)]],
+    constant uint32_t & num_indices  [[buffer(6)]],
+    uint thread_index [[thread_position_in_grid]]) {
     constant int64_t * index_sizes   = (constant int64_t *)indexSizes;
     constant int64_t * index_strides = (constant int64_t *)indexStrides;
     int64_t offset = 0;
     for (uint32_t i = 0; i < num_indices; i++) {
-        int64_t index = ((constant int64_t*)(indexAB.indexArray[i]))[offsets[thread_index].z / sizeof(int64_t)];
+        constant int64_t* indexArray = indexAB[i].indexArray;
+        int64_t index = indexArray[offsets[thread_index].z / sizeof(int64_t)];
         if (index < 0) {
             index += index_sizes[i];
         }
@@ -135,19 +221,22 @@ __attribute__((__always_inline__)) void atomic_fetch_add_relaxed(device void * a
     }
 }
 
-template<typename T>
-kernel void atomic_index_put_accumulate(constant IndexAB & indexAB           [[buffer(0)]],
-                                        constant void    * indexSizes        [[buffer(1)]],
-                                        constant void    * indexStrides      [[buffer(2)]],
-                                        constant uint3   * offsets           [[buffer(3)]],
-                                        constant void    * inputData         [[buffer(4)]],
-                                        device   void    * outputData        [[buffer(5)]],
-                                        uint thread_index [[thread_position_in_grid]]) {
+template<typename T, typename OffsetsT>
+kernel void atomic_index_put_accumulate(
+    constant IndexAB  * indexAB           [[buffer(0)]],
+    constant void     * indexSizes        [[buffer(1)]],
+    constant void     * indexStrides      [[buffer(2)]],
+    constant OffsetsT * offsets           [[buffer(3)]],
+    constant void     * inputData         [[buffer(4)]],
+    device   void     * outputData        [[buffer(5)]],
+    constant uint32_t & num_indices       [[buffer(6)]],
+    uint thread_index [[thread_position_in_grid]]) {
     constant int64_t * index_sizes   = (constant int64_t *)indexSizes;
     constant int64_t * index_strides = (constant int64_t *)indexStrides;
     int64_t offset = 0;
     for (uint32_t i = 0; i < num_indices; i++) {
-        int64_t index = ((constant int64_t*)(indexAB.indexArray[i]))[offsets[thread_index].z / sizeof(int64_t)];
+        constant int64_t* indexArray = indexAB[i].indexArray;
+        int64_t index = indexArray[offsets[thread_index].z / sizeof(int64_t)];
         if (index < 0) {
             index += index_sizes[i];
         }
@@ -159,23 +248,52 @@ kernel void atomic_index_put_accumulate(constant IndexAB & indexAB           [[b
 }
 
 template
-[[host_name("index_put_accumulate_32bit_float")]]
-kernel void atomic_index_put_accumulate<float>(constant IndexAB & indexAB      [[buffer(0)]],
-                                               constant void    * indexSizes   [[buffer(1)]],
-                                               constant void    * indexStrides [[buffer(2)]],
-                                               constant uint3   * offsets      [[buffer(3)]],
-                                               constant void    * inputData    [[buffer(4)]],
-                                               device   void    * outputData   [[buffer(5)]],
-                                               uint thread_index [[thread_position_in_grid]]);
+[[host_name("index_put_accumulate_32bit_float_idx32")]]
+kernel void atomic_index_put_accumulate<float, uint3>(
+    constant IndexAB  * indexAB     [[buffer(0)]],
+    constant void     * indexSizes   [[buffer(1)]],
+    constant void     * indexStrides [[buffer(2)]],
+    constant uint3    * offsets      [[buffer(3)]],
+    constant void     * inputData    [[buffer(4)]],
+    device   void     * outputData   [[buffer(5)]],
+    constant uint32_t & num_indices  [[buffer(6)]],
+    uint thread_index [[thread_position_in_grid]]);
+
 template
-[[host_name("index_put_accumulate_32bit_int")]]
-kernel void index_put_accumulate_native_dtypes<atomic_int, int>(constant IndexAB & indexAB      [[buffer(0)]],
-                                                                constant void    * indexSizes   [[buffer(1)]],
-                                                                constant void    * indexStrides [[buffer(2)]],
-                                                                constant uint3   * offsets      [[buffer(3)]],
-                                                                constant void    * inputData    [[buffer(4)]],
-                                                                device   void    * outputData   [[buffer(5)]],
-                                                                uint thread_index [[thread_position_in_grid]]);
+[[host_name("index_put_accumulate_32bit_float_idx64")]]
+kernel void atomic_index_put_accumulate<float, ulong3>(
+    constant IndexAB  * indexAB     [[buffer(0)]],
+    constant void     * indexSizes   [[buffer(1)]],
+    constant void     * indexStrides [[buffer(2)]],
+    constant ulong3   * offsets      [[buffer(3)]],
+    constant void     * inputData    [[buffer(4)]],
+    device   void     * outputData   [[buffer(5)]],
+    constant uint32_t & num_indices  [[buffer(6)]],
+    uint thread_index [[thread_position_in_grid]]);
+
+template
+[[host_name("index_put_accumulate_32bit_int_idx32")]]
+kernel void index_put_accumulate_native_dtypes<atomic_int, int, uint3>(
+    constant IndexAB  * indexAB     [[buffer(0)]],
+    constant void     * indexSizes   [[buffer(1)]],
+    constant void     * indexStrides [[buffer(2)]],
+    constant uint3    * offsets      [[buffer(3)]],
+    constant void     * inputData    [[buffer(4)]],
+    device   void     * outputData   [[buffer(5)]],
+    constant uint32_t & num_indices [[buffer(6)]],
+    uint thread_index [[thread_position_in_grid]]);
+
+template
+[[host_name("index_put_accumulate_32bit_int_idx64")]]
+kernel void index_put_accumulate_native_dtypes<atomic_int, int, ulong3>(
+    constant IndexAB  * indexAB     [[buffer(0)]],
+    constant void     * indexSizes   [[buffer(1)]],
+    constant void     * indexStrides [[buffer(2)]],
+    constant ulong3   * offsets      [[buffer(3)]],
+    constant void     * inputData    [[buffer(4)]],
+    device   void     * outputData   [[buffer(5)]],
+    constant uint32_t & num_indices [[buffer(6)]],
+    uint thread_index [[thread_position_in_grid]]);
 )INDEX_METAL";
 
 static const char *SCATTER_OPS_TEMPLATE = R"METAL_SCATTER(
@@ -183,6 +301,14 @@ struct __attribute__ ((packed)) packed_uint5{{
   uint32_t x; uint32_t y; uint32_t z; uint32_t w; uint32_t u;
 }};
 
+template<typename Y, typename X>
+Y cast(const X x);
+
+template<>
+{1} cast<{1}, {0}>(const {0} x) {{
+ return {2};
+}}
+
 kernel void scatter_kernel_5(uint linear_index              [[thread_position_in_grid]],
                              constant void * src_           [[buffer(0)]],
                              device void * dst_             [[buffer(1)]],
@@ -208,7 +334,7 @@ kernel void scatter_kernel_5(uint linear_index              [[thread_position_in
     strided_index.w = local_index.w * stride.w;
     strided_index.u = local_index.u * stride.u;
 
-    dst[strided_index.x + strided_index.y + strided_index.z + strided_index.w + strided_index.u] = src[linear_index];
+    dst[strided_index.x + strided_index.y + strided_index.z + strided_index.w + strided_index.u] = cast<{1}>(src[linear_index]);
 }}
 
 kernel void scatter_kernel_4(uint linear_index              [[thread_position_in_grid]],
@@ -229,7 +355,7 @@ kernel void scatter_kernel_4(uint linear_index              [[thread_position_in
     local_index.w = linear_index % size[3];
 
     const packed_uint4 strided_index = local_index * stride;
-    dst[strided_index.x + strided_index.y + strided_index.z + strided_index.w] = src[linear_index];
+    dst[strided_index.x + strided_index.y + strided_index.z + strided_index.w] = cast<{1}>(src[linear_index]);
 }}
 
 kernel void scatter_kernel_3(uint linear_index              [[thread_position_in_grid]],
@@ -249,7 +375,7 @@ kernel void scatter_kernel_3(uint linear_index              [[thread_position_in
     local_index.z = linear_index % size[2];
 
     const packed_uint3 strided_index = local_index * stride;
-    dst[strided_index.x + strided_index.y + strided_index.z] = src[linear_index];
+    dst[strided_index.x + strided_index.y + strided_index.z] = cast<{1}>(src[linear_index]);
 }}
 
 kernel void scatter_kernel_2(uint linear_index              [[thread_position_in_grid]],
@@ -268,7 +394,7 @@ kernel void scatter_kernel_2(uint linear_index              [[thread_position_in
     local_index.y = linear_index % size[1];
 
     const packed_uint2 strided_index = local_index * stride;
-    dst[strided_index.x + strided_index.y] = src[linear_index];
+    dst[strided_index.x + strided_index.y] = cast<{1}>(src[linear_index]);
 }}
 
 kernel void scatter_kernel_1(uint linear_index              [[thread_position_in_grid]],
@@ -284,7 +410,7 @@ kernel void scatter_kernel_1(uint linear_index              [[thread_position_in
 
     const int local_index = linear_index % size;
     const int strided_index = local_index * stride;
-    dst[strided_index] = src[linear_index];
+    dst[strided_index] = cast<{1}>(src[linear_index]);
 }}
 )METAL_SCATTER";
 
@@ -293,6 +419,14 @@ struct __attribute__ ((packed)) packed_uint5{{
   uint32_t x; uint32_t y; uint32_t z; uint32_t w; uint32_t u;
 }};
 
+template<typename Y, typename X>
+Y cast(const X x);
+
+template<>
+{1} cast<{1}, {0}>(const {0} x) {{
+ return {2};
+}}
+
 kernel void gather_kernel_5(uint linear_index               [[thread_position_in_grid]],
                             constant void * src_            [[buffer(0)]],
                             device void * dst_              [[buffer(1)]],
@@ -319,7 +453,7 @@ kernel void gather_kernel_5(uint linear_index               [[thread_position_in
     strided_index.w = local_index.w * stride.w;
     strided_index.u = local_index.u * stride.u;
 
-    dst[linear_index] = src[strided_index.x + strided_index.y + strided_index.z + strided_index.w + strided_index.u];
+    dst[linear_index] = cast<{1}>(src[strided_index.x + strided_index.y + strided_index.z + strided_index.w + strided_index.u]);
 }}
 
 kernel void gather_kernel_4(uint linear_index               [[thread_position_in_grid]],
@@ -340,7 +474,7 @@ kernel void gather_kernel_4(uint linear_index               [[thread_position_in
     local_index.w = linear_index % size[3];
 
     const packed_uint4 strided_index = local_index * stride;
-    dst[linear_index] = src[strided_index.x + strided_index.y + strided_index.z + strided_index.w];
+    dst[linear_index] = cast<{1}>(src[strided_index.x + strided_index.y + strided_index.z + strided_index.w]);
 }}
 
 kernel void gather_kernel_3(uint linear_index               [[thread_position_in_grid]],
@@ -360,7 +494,7 @@ kernel void gather_kernel_3(uint linear_index               [[thread_position_in
     local_index.z = linear_index % size[2];
 
     const packed_uint3 strided_index = local_index * stride;
-    dst[linear_index] = src[strided_index.x + strided_index.y + strided_index.z];
+    dst[linear_index] = cast<{1}>(src[strided_index.x + strided_index.y + strided_index.z]);
 }}
 
 kernel void gather_kernel_2(uint linear_index               [[thread_position_in_grid]],
@@ -379,7 +513,7 @@ kernel void gather_kernel_2(uint linear_index               [[thread_position_in
     local_index.y = linear_index % size[1];
 
     const packed_uint2 strided_index = local_index * stride;
-    dst[linear_index] = src[strided_index.x + strided_index.y];
+    dst[linear_index] = cast<{1}>(src[strided_index.x + strided_index.y]);
 }}
 
 kernel void gather_kernel_1(uint linear_index               [[thread_position_in_grid]],
@@ -395,8 +529,7 @@ kernel void gather_kernel_1(uint linear_index               [[thread_position_in
 
     const int local_index = linear_index % size;
     const int strided_index = local_index * stride;
-    dst[linear_index] = src[strided_index];
+    dst[linear_index] = cast<{1}>(src[strided_index]);
 }}
 )METAL_GATHER";
-}
-}
+} // namespace at::mps
