@@ -59,7 +59,7 @@ static void mps_error_fallback(const c10::OperatorHandle& op, torch::jit::Stack*
 static Tensor slow_conv2d_forward_mps(const Tensor& self,
                                       const Tensor& weight,
                                       IntArrayRef kernel_size,
-                                      const c10::optional<Tensor>& bias,
+                                      const std::optional<Tensor>& bias,
                                       IntArrayRef stride,
                                       IntArrayRef padding) {
   TORCH_CHECK(self.device() == weight.device(),
@@ -85,8 +85,6 @@ TORCH_LIBRARY_IMPL(aten, MPS, m) {
   // These ops are not supported via MPS backend currently, and we fallback to run on CPU.
   // For the rest of unsupported ops the user needs to pass 'PYTORCH_ENABLE_MPS_FALLBACK=1'
   // to fallback on CPU, otherwise we will error out.
-  m.impl("bitwise_left_shift.Tensor_out", torch::CppFunction::makeFromBoxedFunction<&mps_fallback>());
-  m.impl("bitwise_right_shift.Tensor_out", torch::CppFunction::makeFromBoxedFunction<&mps_fallback>());
   m.impl("embedding_renorm_", torch::CppFunction::makeFromBoxedFunction<&mps_fallback>());
   m.impl("linalg_svd", torch::CppFunction::makeFromBoxedFunction<&mps_fallback>());
   m.impl("linalg_svd.U", torch::CppFunction::makeFromBoxedFunction<&mps_fallback>());
