@@ -100,7 +100,7 @@ static void adaptive_max_pool3d_single_out_frame(
   at::parallel_for(0, sizeD, 0, [&](int64_t start, int64_t end) {
     for (const auto d : c10::irange(start, end)) {
       /* loop over output */
-      int64_t ot, oh, ow;
+      int64_t ot = 0, oh = 0, ow = 0;
       for(ot = 0; ot < osizeT; ot++)
       {
         int64_t istartT = start_index(ot, osizeT, isizeT);
@@ -209,7 +209,7 @@ static void adaptive_max_pool3d_backward_single_out_frame(
       const int64_t *ind_p_d = ind_p + d*osizeT*osizeH*osizeW;
 
       /* calculate max points */
-      int64_t ot, oh, ow;
+      int64_t ot = 0, oh = 0, ow = 0;
       for(ot = 0; ot < osizeT; ot++)
       {
         for(oh = 0; oh < osizeH; oh++)
@@ -357,13 +357,13 @@ TORCH_IMPL_FUNC(adaptive_max_pool3d_backward_out_cpu)
   int dimH = 2;
   int dimW = 3;
   int64_t sizeB = 1;
-  int64_t sizeD;
-  int64_t isizeT;
-  int64_t isizeH;
-  int64_t isizeW;
-  int64_t osizeT;
-  int64_t osizeH;
-  int64_t osizeW;
+  int64_t sizeD = 0;
+  int64_t isizeT = 0;
+  int64_t isizeH = 0;
+  int64_t isizeW = 0;
+  int64_t osizeT = 0;
+  int64_t osizeH = 0;
+  int64_t osizeW = 0;
 
   /* get contiguous gradOutput */
   auto gradOutput_ = gradOutput.contiguous();
