@@ -18,12 +18,11 @@
 
 #if !AT_NNPACK_ENABLED()
 
-namespace at {
-namespace native {
+namespace at::native {
 
 at::Tensor _nnpack_spatial_convolution(
     const Tensor& input,
-    const Tensor& weight, const c10::optional<Tensor>& bias_opt,
+    const Tensor& weight, const std::optional<Tensor>& bias_opt,
     const IntArrayRef padding,
     const IntArrayRef stride) {
   throw std::runtime_error(
@@ -34,8 +33,7 @@ bool _nnpack_available() {
   return false;
 }
 
-} // namespace native
-} // namespace at
+} // namespace at::native
 
 #else
 
@@ -46,8 +44,7 @@ bool _nnpack_available() {
 #include <ATen/Parallel.h>
 #include <c10/util/irange.h>
 
-namespace at {
-namespace native {
+namespace at::native {
 
 static bool init_nnpack() {
   static c10::once_flag once_;
@@ -101,34 +98,46 @@ bool _nnpack_available() {
   return init_nnpack();
 }
 
+namespace {
+struct Workspace {
+  void* buffer = nullptr;
+  size_t size = 0;
+
+  void deallocate() {
+    if (buffer) {
+      // NOLINTNEXTLINE(cppcoreguidelines-no-malloc)
+      std::free(buffer);
+      buffer = nullptr;
+    }
+  }
+
+  void allocate() {
+    deallocate();
+
+    // NNPack has alignment requirements
+    constexpr size_t nnpack_memory_alignment_boundary = 64;
+
+    // Won't work on Windows, but NNPACK doesn't support Windows either
+    auto res = posix_memalign(&buffer, nnpack_memory_alignment_boundary, size);
+    if (res != 0) {
+      TORCH_CHECK(false, "posix_memalign failed:", strerror(errno), " (", errno, ")");
+    }
+    return;
+  }
+
+  ~Workspace() {
+    deallocate();
+  }
+};
+} // namespace
+
 // Make thread_local for safety in cases where we have multiple threads running
 // Convs at once
-static thread_local void* workspace = nullptr;
-static thread_local size_t workspace_size = 0;
-
-static inline void deallocate_workspace() {
-  if (workspace) {
-    // NOLINTNEXTLINE(cppcoreguidelines-no-malloc)
-    std::free(workspace);
-    workspace = nullptr;
-  }
-}
-
-static inline void allocate_workspace() {
-  if (workspace) {
-    deallocate_workspace();
-  }
-
-  // NNPack has alignment requirements
-  constexpr size_t nnpack_memory_alignment_boundary = 64;
-
-  // Won't work on Windows, but NNPACK doesn't support Windows either
-  posix_memalign(&workspace, nnpack_memory_alignment_boundary, workspace_size);
-}
+static thread_local Workspace workspace;
 
 Tensor _nnpack_spatial_convolution(
     const Tensor& input,
-    const Tensor& weight, const c10::optional<Tensor>& bias_opt,
+    const Tensor& weight, const std::optional<Tensor>& bias_opt,
     const IntArrayRef padding,
     const IntArrayRef stride) {
   // See [Note: hacky wrapper removal for optional tensor]
@@ -237,8 +246,8 @@ Tensor _nnpack_spatial_convolution(
             weight_.data_ptr<float>(),
             bias_.data_ptr<float>(),
             output.data_ptr<float>() + batch * output_size_per_batch,
-            workspace,
-            &workspace_size,
+            workspace.buffer,
+            &workspace.size,
             nnp_activation_identity,
             nullptr,
             nnpack_threadpool(),
@@ -264,8 +273,8 @@ Tensor _nnpack_spatial_convolution(
         weight_.data_ptr<float>(),
         bias_.data_ptr<float>(),
         output.data_ptr<float>(),
-        workspace,
-        &workspace_size,
+        workspace.buffer,
+        &workspace.size,
         nnp_activation_identity,
         nullptr,
         nnpack_threadpool(),
@@ -281,11 +290,11 @@ Tensor _nnpack_spatial_convolution(
     if (status != nnp_status_success) {
       throw std::runtime_error("NNPACK SpatialConvolution_updateOutput failed");
     }
-    allocate_workspace();
+    workspace.allocate();
   };
 
   // If no workspace created yet, allocate it
-  if (workspace == nullptr) {
+  if (workspace.buffer == nullptr) {
     size_and_allocate_ws();
   }
 
@@ -294,7 +303,7 @@ Tensor _nnpack_spatial_convolution(
 
   if (status == nnp_status_insufficient_buffer) {
     // Need to reallocate the workspace
-    deallocate_workspace();
+    workspace.deallocate();
     size_and_allocate_ws();
 
     // Try one more time
@@ -308,7 +317,6 @@ Tensor _nnpack_spatial_convolution(
   return output;
 }
 
-} // namespace native
-} // namespace at
+} // namespace at::native
 
 #endif // AT_NNPACK_ENABLED
