@@ -6,8 +6,7 @@
 #include <ATen/native/cpu/StackKernel.h>
 #include <ATen/native/cpu/SerialStackImpl.h>
 
-namespace at {
-namespace native {
+namespace at::native {
 
 namespace {
 
@@ -22,5 +21,4 @@ void stack_serial_kernel(Tensor& result, TensorList tensors, int64_t dim) {
 
 REGISTER_DISPATCH(stack_serial_stub, &stack_serial_kernel);
 
-} // namespace native
-} // namespace at
+} // namespace at::native
