@@ -55,7 +55,7 @@ const Tensor& resize_cuda_(
   }
   auto* self_ = self.unsafeGetTensorImpl();
   int64_t old_storage_nbytes = self_->unsafe_storage() ? self_->unsafe_storage().nbytes() : 0;
-  resize_impl_cuda_(self_, size, /*strides=*/c10::nullopt);
+  resize_impl_cuda_(self_, size, /*strides=*/std::nullopt);
   if (optional_memory_format.has_value()) {
     auto memory_format =
         optional_memory_format.value();
