@@ -1,7 +1,7 @@
-#include <ATen/native/mkldnn/ConvPrepack.h>
 #include <ATen/native/mkldnn/OpContext.h>
 
 #if AT_MKLDNN_ENABLED()
+#include <ATen/native/mkldnn/ConvPrepack.h>
 
 namespace at {
 namespace native {
@@ -9,7 +9,7 @@ namespace mkldnn {
 
 c10::intrusive_ptr<ConvOpContext> MkldnnConvOpContext::create_context(
     at::Tensor&& weight,
-    c10::optional<at::Tensor>&& bias,
+    std::optional<at::Tensor>&& bias,
     std::vector<int64_t>&& padding,
     std::vector<int64_t>&& stride,
     std::vector<int64_t>&& dilation,
@@ -37,7 +37,7 @@ Tensor MkldnnConvOpContext::run(const Tensor& input) {
 }
 
 void MkldnnConvOpContext::run(const Tensor& input, void* output) {
-  return mkldnn::internal::convolution::run(op_context_, input, output);
+  mkldnn::internal::convolution::run(op_context_, input, output);
 }
 
 } // namespace mkldnn
