@@ -1,6 +1,7 @@
 #define TORCH_ASSERT_ONLY_METHOD_OPERATORS
 #include <ATen/ExpandUtils.h>
 #include <ATen/mps/MPSProfiler.h>
+#include <ATen/native/BinaryOps.h>
 #include <ATen/native/Resize.h>
 #include <ATen/native/mps/OperationUtils.h>
 #include <ATen/ops/bitwise_and_native.h>
@@ -12,7 +13,7 @@
 
 namespace at::native {
 namespace mps {
-static const char* BITWISE_OPS_TEMPLATE = R"METAL(
+static MetalShaderLibrary lib(R"METAL(
 
 kernel void bitwise_and_tensor(constant uint& length [[buffer(0)]],
                          device {0}  *out [[buffer(1)]],
@@ -81,6 +82,50 @@ kernel void bitwise_xor_scalar(constant uint& length [[buffer(0)]],
   out[offset] = a[offset] ^ b;
 }}
 
+kernel void bitwise_lshift_tensor(constant uint& length [[buffer(0)]],
+                         device {0}  *out [[buffer(1)]],
+                         device {1}  *a [[buffer(2)]],
+                         device {2}  *b [[buffer(3)]],
+                         uint offset [[thread_position_in_grid]]) {{
+  if (offset >= length) {{
+    return;
+  }}
+  out[offset] = a[offset] << b [offset];
+}}
+
+kernel void bitwise_lshift_scalar(constant uint& length [[buffer(0)]],
+                         device {0}  *out [[buffer(1)]],
+                         device {1}  *a [[buffer(2)]],
+                         constant {2}  &b [[buffer(3)]],
+                         uint offset [[thread_position_in_grid]]) {{
+  if (offset >= length) {{
+    return;
+  }}
+  out[offset] = a[offset] << b;
+}}
+
+kernel void bitwise_rshift_tensor(constant uint& length [[buffer(0)]],
+                         device {0}  *out [[buffer(1)]],
+                         device {1}  *a [[buffer(2)]],
+                         device {2}  *b [[buffer(3)]],
+                         uint offset [[thread_position_in_grid]]) {{
+  if (offset >= length) {{
+    return;
+  }}
+  out[offset] = a[offset] >> b [offset];
+}}
+
+kernel void bitwise_rshift_scalar(constant uint& length [[buffer(0)]],
+                         device {0}  *out [[buffer(1)]],
+                         device {1}  *a [[buffer(2)]],
+                         constant {2}  &b [[buffer(3)]],
+                         uint offset [[thread_position_in_grid]]) {{
+  if (offset >= length) {{
+    return;
+  }}
+  out[offset] = a[offset] >> b;
+}}
+
 kernel void bitwise_not(constant uint& length [[buffer(0)]],
                          device {0}  *out [[buffer(1)]],
                          device {1}  *a [[buffer(2)]],
@@ -90,7 +135,8 @@ kernel void bitwise_not(constant uint& length [[buffer(0)]],
   }}
   out[offset] = ~a[offset];
 }}
-)METAL";
+)METAL",
+                              3);
 
 static const std::string& getMetalType(const c10::ScalarType& t) {
   // Mapping from c10::ScalarType to integral type that can be used for bitwise ops
@@ -117,57 +163,12 @@ static const std::string& getMetalType(const c10::Scalar& s) {
   return getMetalType(s.type());
 }
 
-static id<MTLLibrary> compileBitwiseOpsLibrary(id<MTLDevice> device,
-                                               const std::string& t1,
-                                               const std::string& t2,
-                                               const std::string& t3) {
-  auto key = t1 + t2 + t3;
-  static std::unordered_map<std::string, id<MTLLibrary>> libMap;
-  auto it = libMap.find(key);
-  if (it != libMap.end()) {
-    return it->second;
-  }
-  NSError* error = nil;
-  MTLCompileOptions* options = [[MTLCompileOptions new] autorelease];
-  [options setLanguageVersion:MTLLanguageVersion2_3];
-  auto rc =
-      [device newLibraryWithSource:[NSString stringWithUTF8String:fmt::format(BITWISE_OPS_TEMPLATE, t1, t2, t3).c_str()]
-                           options:options
-                             error:&error];
-  TORCH_CHECK(rc != nil && error == nil, "Failed to compile library: ", [[error localizedDescription] UTF8String]);
-  libMap[key] = rc;
-  return rc;
-}
-
-static id<MTLComputePipelineState> getCPLState(id<MTLDevice> device,
-                                               const std::string& t1,
-                                               const std::string& t2,
-                                               const std::string& t3,
+template <typename ScalarOrTensor>
+static id<MTLComputePipelineState> getCPLState(const Tensor& t1,
+                                               const Tensor& t2,
+                                               const ScalarOrTensor& t3,
                                                const std::string& fname) {
-  auto key = t1 + t2 + t3 + fname;
-  static std::unordered_map<std::string, id<MTLComputePipelineState>> cplMap;
-  auto it = cplMap.find(key);
-  if (it != cplMap.end()) {
-    return it->second;
-  }
-  NSError* error = nil;
-  auto library = compileBitwiseOpsLibrary(device, t1, t2, t3);
-  id<MTLFunction> func = [library newFunctionWithName:[NSString stringWithUTF8String:fname.c_str()]];
-  TORCH_CHECK(func != nil, "Can't get function ", fname);
-  auto rc = [device newComputePipelineStateWithFunction:func error:&error];
-  TORCH_CHECK(
-      rc != nil && error == nil, "Failed to construct pipeline state: ", [[error localizedDescription] UTF8String]);
-  cplMap[key] = rc;
-  return rc;
-}
-
-static void dispatch1DJob(id<MTLComputeCommandEncoder> commandEncoder,
-                          id<MTLComputePipelineState> cplState,
-                          uint32_t length) {
-  uint32_t maxThreadsPerGroup = [cplState maxTotalThreadsPerThreadgroup];
-  auto size = MTLSizeMake(length, 1, 1);
-  auto threadGroupSize = MTLSizeMake(std::min(maxThreadsPerGroup, length), 1, 1);
-  [commandEncoder dispatchThreads:size threadsPerThreadgroup:threadGroupSize];
+  return lib.getPipelineStateForFunc(fname, {getMetalType(t1), getMetalType(t2), getMetalType(t3)});
 }
 
 static void handle_tensor_tensor_binary_op(const Tensor& self,
@@ -176,8 +177,7 @@ static void handle_tensor_tensor_binary_op(const Tensor& self,
                                            const std::string& kernel_name) {
   using namespace at::mps;
   MPSStream* stream = getCurrentMPSStream();
-  id<MTLComputePipelineState> cplState = getCPLState(
-      MPSDevice::getInstance()->device(), getMetalType(output), getMetalType(self), getMetalType(other), kernel_name);
+  auto cplState = getCPLState(output, self, other, kernel_name);
   uint32_t length = output.numel();
   if (length == 0) {
     return;
@@ -189,17 +189,13 @@ static void handle_tensor_tensor_binary_op(const Tensor& self,
 
     id<MTLComputeCommandEncoder> commandEncoder = stream->commandEncoder();
 
-    id<MTLBuffer> outBuf = __builtin_bit_cast(id<MTLBuffer>, output.storage().data());
-    id<MTLBuffer> selfBuf = __builtin_bit_cast(id<MTLBuffer>, self.storage().data());
-    id<MTLBuffer> otherBuf = __builtin_bit_cast(id<MTLBuffer>, other.storage().data());
-
     [commandEncoder pushDebugGroup:[NSString stringWithFormat:@"Dispatch %s kernel", kernel_name.c_str()]];
     [commandEncoder setComputePipelineState:cplState];
-    [commandEncoder setBytes:&length length:sizeof(length) atIndex:0];
-    [commandEncoder setBuffer:outBuf offset:output.storage_offset() * output.itemsize() atIndex:1];
-    [commandEncoder setBuffer:selfBuf offset:self.storage_offset() * self.itemsize() atIndex:2];
-    [commandEncoder setBuffer:otherBuf offset:other.storage_offset() * other.itemsize() atIndex:3];
-    dispatch1DJob(commandEncoder, cplState, length);
+    mtl_setBytes(commandEncoder, length, 0);
+    mtl_setBuffer(commandEncoder, output, 1);
+    mtl_setBuffer(commandEncoder, self, 2);
+    mtl_setBuffer(commandEncoder, other, 3);
+    mtl_dispatch1DJob(commandEncoder, cplState, length);
 
     getMPSProfiler().endProfileKernel(cplState);
   });
@@ -211,8 +207,7 @@ static void handle_tensor_scalar_binary_op(const Tensor& self,
                                            const std::string& kernel_name) {
   using namespace at::mps;
   MPSStream* stream = getCurrentMPSStream();
-  id<MTLComputePipelineState> cplState = getCPLState(
-      MPSDevice::getInstance()->device(), getMetalType(output), getMetalType(self), getMetalType(other), kernel_name);
+  auto cplState = getCPLState(output, self, other, kernel_name);
   uint64_t sval = other.to<int64_t>();
   uint32_t length = output.numel();
   if (length == 0) {
@@ -224,16 +219,13 @@ static void handle_tensor_scalar_binary_op(const Tensor& self,
 
     id<MTLComputeCommandEncoder> commandEncoder = stream->commandEncoder();
 
-    id<MTLBuffer> outBuf = __builtin_bit_cast(id<MTLBuffer>, output.storage().data());
-    id<MTLBuffer> selfBuf = __builtin_bit_cast(id<MTLBuffer>, self.storage().data());
-
     [commandEncoder pushDebugGroup:[NSString stringWithFormat:@"Dispatch %s kernel", kernel_name.c_str()]];
     [commandEncoder setComputePipelineState:cplState];
-    [commandEncoder setBytes:&length length:sizeof(length) atIndex:0];
-    [commandEncoder setBuffer:outBuf offset:output.storage_offset() * output.itemsize() atIndex:1];
-    [commandEncoder setBuffer:selfBuf offset:self.storage_offset() * self.itemsize() atIndex:2];
-    [commandEncoder setBytes:&sval length:sizeof(sval) atIndex:3];
-    dispatch1DJob(commandEncoder, cplState, length);
+    mtl_setBytes(commandEncoder, length, 0);
+    mtl_setBuffer(commandEncoder, output, 1);
+    mtl_setBuffer(commandEncoder, self, 2);
+    mtl_setBytes(commandEncoder, sval, 3);
+    mtl_dispatch1DJob(commandEncoder, cplState, length);
 
     getMPSProfiler().endProfileKernel(cplState);
   });
@@ -252,7 +244,7 @@ static void _bitwise_op_out_mps(const Tensor& self,
 
   auto output_size = at::infer_size_dimvector(self.sizes(), other.sizes());
   resize_output(output, output_size);
-  if (!output.is_contiguous()) {
+  if (needsGather(output)) {
     output = output.contiguous();
     needs_output_copy = true;
   }
@@ -263,6 +255,10 @@ static void _bitwise_op_out_mps(const Tensor& self,
       output.fill_(c10::Scalar(self.item<int64_t>() | other.item<int64_t>()));
     } else if (op_name == "xor") {
       output.fill_(c10::Scalar(self.item<int64_t>() ^ other.item<int64_t>()));
+    } else if (op_name == "lshift") {
+      output.fill_(c10::Scalar(self.item<int64_t>() << other.item<int64_t>()));
+    } else if (op_name == "rshift") {
+      output.fill_(c10::Scalar(self.item<int64_t>() >> other.item<int64_t>()));
     } else {
       TORCH_CHECK(false, "Unknown operation to be performed over scalars ", op_name);
     }
@@ -293,7 +289,7 @@ static void _bitwise_not_out_mps(const Tensor& self, const Tensor& output_) {
   bool needs_output_copy = false;
 
   resize_output(output, self.sizes());
-  if (!output.is_contiguous()) {
+  if (needsGather(output)) {
     output = output.contiguous();
     needs_output_copy = true;
   }
@@ -312,22 +308,18 @@ static void _bitwise_not_out_mps(const Tensor& self, const Tensor& output_) {
   }
   using namespace at::mps;
   MPSStream* stream = getCurrentMPSStream();
-  id<MTLComputePipelineState> cplState = getCPLState(
-      MPSDevice::getInstance()->device(), getMetalType(output), getMetalType(self), getMetalType(self), "bitwise_not");
+  auto cplState = getCPLState(output, self, self, "bitwise_not");
   dispatch_sync(stream->queue(), ^() {
     getMPSProfiler().beginProfileKernel(cplState, "bitwise_not", {self});
 
     id<MTLComputeCommandEncoder> commandEncoder = stream->commandEncoder();
 
-    id<MTLBuffer> outBuf = __builtin_bit_cast(id<MTLBuffer>, output.storage().data());
-    id<MTLBuffer> selfBuf = __builtin_bit_cast(id<MTLBuffer>, self.storage().data());
-
     [commandEncoder pushDebugGroup:@"Dispatch bitwise_not kernel"];
     [commandEncoder setComputePipelineState:cplState];
-    [commandEncoder setBytes:&length length:sizeof(length) atIndex:0];
-    [commandEncoder setBuffer:outBuf offset:output.storage_offset() * output.itemsize() atIndex:1];
-    [commandEncoder setBuffer:selfBuf offset:self.storage_offset() * self.itemsize() atIndex:2];
-    dispatch1DJob(commandEncoder, cplState, length);
+    mtl_setBytes(commandEncoder, length, 0);
+    mtl_setBuffer(commandEncoder, output, 1);
+    mtl_setBuffer(commandEncoder, self, 2);
+    mtl_dispatch1DJob(commandEncoder, cplState, length);
 
     getMPSProfiler().endProfileKernel(cplState);
   });
@@ -337,6 +329,16 @@ static void _bitwise_not_out_mps(const Tensor& self, const Tensor& output_) {
 }
 
 } // namespace mps
+namespace {
+void lshift_kernel_mps(TensorIteratorBase& iter) {
+  mps::_bitwise_op_out_mps(iter.input(0), iter.input(1), iter.output(0), "lshift");
+}
+
+void rshift_kernel_mps(TensorIteratorBase& iter) {
+  mps::_bitwise_op_out_mps(iter.input(0), iter.input(1), iter.output(0), "rshift");
+}
+
+} // namespace
 
 TORCH_IMPL_FUNC(bitwise_and_out_mps)(const Tensor& self, const Tensor& other, const Tensor& output) {
   mps::_bitwise_op_out_mps(self, other, output, "and");
@@ -353,4 +355,8 @@ TORCH_IMPL_FUNC(bitwise_xor_out_mps)(const Tensor& self, const Tensor& other, co
 TORCH_IMPL_FUNC(bitwise_not_out_mps)(const Tensor& self, const Tensor& output) {
   mps::_bitwise_not_out_mps(self, output);
 }
+
+REGISTER_MPS_DISPATCH(lshift_stub, &lshift_kernel_mps);
+REGISTER_MPS_DISPATCH(rshift_stub, &rshift_kernel_mps);
+
 } // namespace at::native
