@@ -10,7 +10,9 @@
 #include <ATen/Functions.h>
 #include <ATen/NativeFunctions.h>
 #else
+#include <ATen/ops/_batch_norm_with_update_native.h>
 #include <ATen/ops/_native_batch_norm_legit_native.h>
+#include <ATen/ops/batch_norm_backward_native.h>
 #include <ATen/ops/native_batch_norm.h>
 #include <ATen/ops/native_batch_norm_backward_native.h>
 #include <ATen/ops/native_batch_norm_native.h>
@@ -65,10 +67,10 @@ static void get_shapes(MPSShape* input_shape_readonly,
 
 // Inverse standard deviation now becomes variance (without epsilon)
 std::tuple<Tensor&, Tensor&, Tensor&> batch_norm_mps_out(const Tensor& self,
-                                                         const c10::optional<Tensor>& weight_opt,
-                                                         const c10::optional<Tensor>& bias_opt,
-                                                         const c10::optional<Tensor>& running_mean_opt,
-                                                         const c10::optional<Tensor>& running_var_opt,
+                                                         const std::optional<Tensor>& weight_opt,
+                                                         const std::optional<Tensor>& bias_opt,
+                                                         const std::optional<Tensor>& running_mean_opt,
+                                                         const std::optional<Tensor>& running_var_opt,
                                                          bool train,
                                                          double momentum,
                                                          double epsilon,
@@ -151,12 +153,6 @@ std::tuple<Tensor&, Tensor&, Tensor&> batch_norm_mps_out(const Tensor& self,
     else
       channelsDim = num_input_dims - 1;
 
-    bool executeGatherOp = true;
-    if (self.is_contiguous(memory_format)) {
-      memory_format = MemoryFormat::Contiguous;
-      executeGatherOp = false;
-    }
-
     auto cachedGraph = LookUpOrCreateCachedGraph<CachedGraph>(key, [&](auto mpsGraph, auto newCachedGraph) {
       MPSGraphTensor* inputTensor = mpsGraphRankedPlaceHolder(mpsGraph, input_mps_dtype, input_shape);
       MPSGraphTensor* weightTensor = nil;
@@ -300,7 +296,9 @@ std::tuple<Tensor&, Tensor&, Tensor&> batch_norm_mps_out(const Tensor& self,
       newCachedGraph->runningVarInplaceUpdate_ = runningVarInplaceUpdate;
     });
 
-    auto inputPlaceholder = Placeholder(cachedGraph->inputTensor_, self, input_shape, executeGatherOp);
+    const auto needs_gather = memory_format != MemoryFormat::ChannelsLast;
+    auto inputPlaceholder =
+        Placeholder(cachedGraph->inputTensor_, self, input_shape, needs_gather, MPSDataTypeInvalid, needs_gather);
     auto weightPlaceholder = Placeholder();
     if (has_weight)
       weightPlaceholder = Placeholder(cachedGraph->weightTensor_, weight_opt.value(), new_mean_shape);
@@ -323,7 +321,8 @@ std::tuple<Tensor&, Tensor&, Tensor&> batch_norm_mps_out(const Tensor& self,
       runningVarInplaceUpdatePlaceholder = Placeholder(cachedGraph->runningVarInplaceUpdate_, running_var_opt.value());
     }
 
-    auto outputPlaceholder = Placeholder(cachedGraph->outputTensor_, output, input_shape, false);
+    auto outputPlaceholder =
+        Placeholder(cachedGraph->outputTensor_, output, input_shape, false, MPSDataTypeInvalid, needs_gather);
     auto saveMeanPlaceholder = Placeholder(cachedGraph->saveMeanTensor_, save_mean);
     auto saveVarPlaceholder = Placeholder(cachedGraph->saveVarTensor_, save_var);
 
@@ -362,16 +361,16 @@ std::tuple<Tensor&, Tensor&, Tensor&> batch_norm_mps_out(const Tensor& self,
 }
 
 std::tuple<Tensor, Tensor, Tensor> batch_norm_mps(const Tensor& self,
-                                                  const c10::optional<Tensor>& weight_opt,
-                                                  const c10::optional<Tensor>& bias_opt,
-                                                  const c10::optional<Tensor>& running_mean_opt,
-                                                  const c10::optional<Tensor>& running_var_opt,
+                                                  const std::optional<Tensor>& weight_opt,
+                                                  const std::optional<Tensor>& bias_opt,
+                                                  const std::optional<Tensor>& running_mean_opt,
+                                                  const std::optional<Tensor>& running_var_opt,
                                                   bool train,
                                                   double momentum,
                                                   double epsilon) {
   const auto memory_format = self.suggest_memory_format();
 
-  auto output = at::empty(self.sizes(), self.scalar_type(), c10::nullopt, kMPS, c10::nullopt, memory_format);
+  auto output = at::empty(self.sizes(), self.scalar_type(), std::nullopt, kMPS, std::nullopt, memory_format);
 
   int64_t n_input = self.size(1);
 
@@ -379,18 +378,18 @@ std::tuple<Tensor, Tensor, Tensor> batch_norm_mps(const Tensor& self,
                              self.scalar_type(),
                              // TODO: Accumulate type?
                              // at::toAccumulateType(self.scalar_type(), /*is_cuda=*/false),
-                             c10::nullopt,
+                             std::nullopt,
                              kMPS,
-                             c10::nullopt,
-                             c10::nullopt);
+                             std::nullopt,
+                             std::nullopt);
   auto save_var = at::empty({n_input},
                             self.scalar_type(),
                             // TODO: Accumulate type?
                             // at::toAccumulateType(self.scalar_type(), /*is_cuda=*/false),
-                            c10::nullopt,
+                            std::nullopt,
                             kMPS,
-                            c10::nullopt,
-                            c10::nullopt);
+                            std::nullopt,
+                            std::nullopt);
 
   at::native::batch_norm_mps_out(self,
                                  weight_opt,
@@ -406,9 +405,38 @@ std::tuple<Tensor, Tensor, Tensor> batch_norm_mps(const Tensor& self,
   return std::make_tuple(output, save_mean, save_var);
 }
 
+std::tuple<Tensor, Tensor, Tensor, Tensor> _batch_norm_with_update_mps(const Tensor& input,
+                                                                       const std::optional<Tensor>& weight_opt,
+                                                                       const std::optional<Tensor>& bias_opt,
+                                                                       Tensor& running_mean,
+                                                                       Tensor& running_var,
+                                                                       double momentum,
+                                                                       double eps) {
+  auto [output, save_mean, save_var] =
+      batch_norm_mps(input, weight_opt, bias_opt, running_mean, running_var, /*train*/ true, momentum, eps);
+  Tensor reserve = at::empty({0}, input.options().dtype(kByte));
+  return std::tuple<Tensor, Tensor, Tensor, Tensor>(output, save_mean, save_var, reserve);
+}
+
+std::tuple<Tensor&, Tensor&, Tensor&, Tensor&> _batch_norm_with_update_mps_out(const Tensor& input,
+                                                                               const std::optional<Tensor>& weight_opt,
+                                                                               const std::optional<Tensor>& bias_opt,
+                                                                               Tensor& running_mean,
+                                                                               Tensor& running_var,
+                                                                               double momentum,
+                                                                               double eps,
+                                                                               Tensor& out,
+                                                                               Tensor& save_mean,
+                                                                               Tensor& save_var,
+                                                                               Tensor& reserve) {
+  std::tie(out, save_mean, save_var) = batch_norm_mps_out(
+      input, weight_opt, bias_opt, running_mean, running_var, /*update*/ true, momentum, eps, out, save_mean, save_var);
+  return std::tuple<Tensor&, Tensor&, Tensor&, Tensor&>(out, save_mean, save_var, reserve);
+}
+
 std::tuple<Tensor, Tensor, Tensor> _batch_norm_legit_mps(const Tensor& self,
-                                                         const c10::optional<Tensor>& weight_opt,
-                                                         const c10::optional<Tensor>& bias_opt,
+                                                         const std::optional<Tensor>& weight_opt,
+                                                         const std::optional<Tensor>& bias_opt,
                                                          Tensor& running_mean,
                                                          Tensor& running_var,
                                                          bool train,
@@ -418,8 +446,8 @@ std::tuple<Tensor, Tensor, Tensor> _batch_norm_legit_mps(const Tensor& self,
 }
 
 std::tuple<Tensor, Tensor, Tensor> _batch_norm_legit_no_stats_mps(const Tensor& self,
-                                                                  const c10::optional<Tensor>& weight_opt,
-                                                                  const c10::optional<Tensor>& bias_opt,
+                                                                  const std::optional<Tensor>& weight_opt,
+                                                                  const std::optional<Tensor>& bias_opt,
                                                                   bool train,
                                                                   double momentum,
                                                                   double epsilon) {
@@ -427,8 +455,8 @@ std::tuple<Tensor, Tensor, Tensor> _batch_norm_legit_no_stats_mps(const Tensor&
 }
 
 std::tuple<Tensor&, Tensor&, Tensor&> _batch_norm_legit_mps_out(const Tensor& self,
-                                                                const c10::optional<Tensor>& weight_opt,
-                                                                const c10::optional<Tensor>& bias_opt,
+                                                                const std::optional<Tensor>& weight_opt,
+                                                                const std::optional<Tensor>& bias_opt,
                                                                 Tensor& running_mean,
                                                                 Tensor& running_var,
                                                                 bool train,
@@ -442,8 +470,8 @@ std::tuple<Tensor&, Tensor&, Tensor&> _batch_norm_legit_mps_out(const Tensor& se
 }
 
 std::tuple<Tensor&, Tensor&, Tensor&> _batch_norm_legit_no_stats_mps_out(const Tensor& self,
-                                                                         const c10::optional<Tensor>& weight_opt,
-                                                                         const c10::optional<Tensor>& bias_opt,
+                                                                         const std::optional<Tensor>& weight_opt,
+                                                                         const std::optional<Tensor>& bias_opt,
                                                                          bool train,
                                                                          double momentum,
                                                                          double epsilon,
@@ -471,13 +499,36 @@ static string get_mem_string(c10::MemoryFormat memory_format) {
 }
 
 // Batch norm backward
+std::tuple<Tensor, Tensor, Tensor> _new_batch_norm_backward_mps(const Tensor& grad_output,
+                                                                const Tensor& input,
+                                                                const Tensor& weight,
+                                                                const std::optional<Tensor>& running_mean_opt,
+                                                                const std::optional<Tensor>& running_var_opt,
+                                                                const std::optional<Tensor>& save_mean_opt,
+                                                                const std::optional<Tensor>& save_var_opt,
+                                                                bool update,
+                                                                double eps,
+                                                                std::array<bool, 3> grad_input_mask,
+                                                                const Tensor& reserve) {
+  return batch_norm_backward_mps(grad_output,
+                                 input,
+                                 weight,
+                                 running_mean_opt,
+                                 running_var_opt,
+                                 save_mean_opt,
+                                 save_var_opt,
+                                 update,
+                                 eps,
+                                 grad_input_mask);
+}
+
 std::tuple<Tensor, Tensor, Tensor> batch_norm_backward_mps(const Tensor& grad_out,
                                                            const Tensor& input,
-                                                           const c10::optional<Tensor>& weight_opt,
-                                                           const c10::optional<Tensor>& running_mean_opt,
-                                                           const c10::optional<Tensor>& running_var_opt,
-                                                           const c10::optional<Tensor>& save_mean_opt,
-                                                           const c10::optional<Tensor>& save_var_opt,
+                                                           const std::optional<Tensor>& weight_opt,
+                                                           const std::optional<Tensor>& running_mean_opt,
+                                                           const std::optional<Tensor>& running_var_opt,
+                                                           const std::optional<Tensor>& save_mean_opt,
+                                                           const std::optional<Tensor>& save_var_opt,
                                                            bool train,
                                                            double epsilon,
                                                            std::array<bool, 3> grad_input_mask) {
@@ -488,23 +539,23 @@ std::tuple<Tensor, Tensor, Tensor> batch_norm_backward_mps(const Tensor& grad_ou
   const auto memory_format = input.suggest_memory_format();
 
   if (grad_input_mask[0]) {
-    grad_input = at::empty(input.sizes(), input.scalar_type(), c10::nullopt, kMPS, c10::nullopt, memory_format);
+    grad_input = at::empty(input.sizes(), input.scalar_type(), std::nullopt, kMPS, std::nullopt, memory_format);
   }
   // Assuming that if grad_input_mask of weight is 1, then the weight is available
   if (grad_input_mask[1]) {
     grad_weight = at::empty(weight_opt.value().sizes(),
                             weight_opt.value().scalar_type(),
-                            c10::nullopt,
+                            std::nullopt,
                             kMPS,
-                            c10::nullopt,
+                            std::nullopt,
                             at::MemoryFormat::Contiguous);
   }
   if (grad_input_mask[2]) {
     grad_bias = at::empty(weight_opt.value().sizes(),
                           weight_opt.value().scalar_type(),
-                          c10::nullopt,
+                          std::nullopt,
                           kMPS,
-                          c10::nullopt,
+                          std::nullopt,
                           at::MemoryFormat::Contiguous);
   }
 
@@ -806,8 +857,8 @@ std::tuple<Tensor, Tensor, Tensor> batch_norm_backward_mps(const Tensor& grad_ou
 // Layer norm forward for MPS
 std::tuple<Tensor, Tensor, Tensor> layer_norm_mps(const Tensor& input,
                                                   IntArrayRef normalized_shape,
-                                                  const c10::optional<Tensor>& weight_opt,
-                                                  const c10::optional<Tensor>& bias_opt,
+                                                  const std::optional<Tensor>& weight_opt,
+                                                  const std::optional<Tensor>& bias_opt,
                                                   double eps) {
   c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight_opt);
   const Tensor& weight = *weight_maybe_owned;
@@ -867,8 +918,8 @@ std::tuple<Tensor, Tensor, Tensor> layer_norm_backward_mps(const Tensor& grad_ou
                                                            IntArrayRef normalized_shape,
                                                            const Tensor& mean,
                                                            const Tensor& rstd,
-                                                           const c10::optional<Tensor>& weight_opt /* optional */,
-                                                           const c10::optional<Tensor>& bias_opt /* optional */,
+                                                           const std::optional<Tensor>& weight_opt /* optional */,
+                                                           const std::optional<Tensor>& bias_opt /* optional */,
                                                            std::array<bool, 3> grad_input_mask) {
   c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight_opt);
   const Tensor& weight = *weight_maybe_owned;
@@ -888,38 +939,38 @@ std::tuple<Tensor, Tensor, Tensor> layer_norm_backward_mps(const Tensor& grad_ou
   Tensor grad_bias;
   if (grad_input_mask[0]) {
     grad_input = at::empty_like(*X,
-                                c10::nullopt /* dtype */,
-                                c10::nullopt /* layout */,
+                                std::nullopt /* dtype */,
+                                std::nullopt /* layout */,
                                 kMPS /* device */,
-                                c10::nullopt /* pin_memory */,
+                                std::nullopt /* pin_memory */,
                                 at::MemoryFormat::Contiguous);
   }
   if (grad_input_mask[1]) {
     grad_weight = M > 0 ? at::empty_like(*gamma,
-                                         c10::nullopt /* dtype */,
-                                         c10::nullopt /* layout */,
+                                         std::nullopt /* dtype */,
+                                         std::nullopt /* layout */,
                                          kMPS /* device */,
-                                         c10::nullopt /* pin_memory */,
+                                         std::nullopt /* pin_memory */,
                                          at::MemoryFormat::Contiguous)
                         : at::zeros_like(*gamma,
-                                         c10::nullopt /* dtype */,
-                                         c10::nullopt /* layout */,
+                                         std::nullopt /* dtype */,
+                                         std::nullopt /* layout */,
                                          kMPS /* device */,
-                                         c10::nullopt /* pin_memory */,
+                                         std::nullopt /* pin_memory */,
                                          at::MemoryFormat::Contiguous);
   }
   if (grad_input_mask[2]) {
     grad_bias = M > 0 ? at::empty_like(*beta,
-                                       c10::nullopt /* dtype */,
-                                       c10::nullopt /* layout */,
+                                       std::nullopt /* dtype */,
+                                       std::nullopt /* layout */,
                                        kMPS /* device */,
-                                       c10::nullopt /* pin_memory */,
+                                       std::nullopt /* pin_memory */,
                                        at::MemoryFormat::Contiguous)
                       : at::zeros_like(*beta,
-                                       c10::nullopt /* dtype */,
-                                       c10::nullopt /* layout */,
+                                       std::nullopt /* dtype */,
+                                       std::nullopt /* layout */,
                                        kMPS /* device */,
-                                       c10::nullopt /* pin_memory */,
+                                       std::nullopt /* pin_memory */,
                                        at::MemoryFormat::Contiguous);
   }
   if (M > 0) {
