@@ -53,10 +53,10 @@ TORCH_IMPL_FUNC(gather_out_mps)
     }
     auto input_type = getMPSDataType(self);
     auto output_type = getMPSDataType(output);
-    if (input_type == MPSDataTypeUInt8 || ((input_type == MPSDataTypeBool && !is_macos_13_or_newer()))) {
+    if (input_type == MPSDataTypeUInt8) {
       input_type = MPSDataTypeInt8;
     }
-    if (output_type == MPSDataTypeUInt8 || ((output_type == MPSDataTypeBool && !is_macos_13_or_newer()))) {
+    if (output_type == MPSDataTypeUInt8) {
       output_type = MPSDataTypeInt8;
     }
     string key = "gather_out_mps" + getTensorsStringKey({self, index, output}) + ":" + std::to_string(dim);
@@ -306,7 +306,7 @@ TORCH_IMPL_FUNC(scatter_src_out_mps)
 TORCH_IMPL_FUNC(scatter_value_out_mps)
 (const Tensor& self, int64_t dim, const Tensor& index, const Scalar& value, const Tensor& output) {
   Tensor src =
-      at::empty(index.sizes(), self.scalar_type(), c10::nullopt, kMPS, c10::nullopt, self.suggest_memory_format());
+      at::empty(index.sizes(), self.scalar_type(), std::nullopt, kMPS, std::nullopt, self.suggest_memory_format());
   src.fill_(value);
   scatter_mps_general(self, dim, index, const_cast<Tensor&>(src), output, "scatter_value_out_mps", "set");
 }
@@ -329,7 +329,7 @@ TORCH_IMPL_FUNC(scatter_value_reduce_out_mps)
  const c10::string_view reduce,
  const Tensor& output) {
   Tensor src =
-      at::empty(index.sizes(), self.scalar_type(), c10::nullopt, kMPS, c10::nullopt, self.suggest_memory_format());
+      at::empty(index.sizes(), self.scalar_type(), std::nullopt, kMPS, std::nullopt, self.suggest_memory_format());
   src.fill_(value);
   scatter_mps_general(self, dim, index, const_cast<Tensor&>(src), output, "scatter_value_reduce_out_mps", reduce);
 }
