@@ -26,7 +26,6 @@
 #include <ATen/ops/cumsum_native.h>
 #include <ATen/ops/erf_native.h>
 #include <ATen/ops/exp2_native.h>
-#include <ATen/ops/exp_native.h>
 #include <ATen/ops/expm1_native.h>
 #include <ATen/ops/floor_native.h>
 #include <ATen/ops/frac_native.h>
@@ -54,7 +53,6 @@
 #include <ATen/ops/sinh_native.h>
 #include <ATen/ops/sqrt_native.h>
 #include <ATen/ops/tan_native.h>
-#include <ATen/ops/tanh_native.h>
 #include <ATen/ops/trunc_native.h>
 #include <ATen/ops/view_as_real.h>
 #endif
@@ -76,13 +74,12 @@ static bool is_empty_tensor(const Tensor& self) {
 }
 
 static void unary_op_noresize(const Tensor& self, const Tensor& output_, std::string op_name, UnaryOpBlock unaryBlock) {
-  TORCH_CHECK(!(!is_macos_13_or_newer() && self.scalar_type() == ScalarType::Byte),
-              "MPS support unary op with uint8 natively starting from macOS 13.0");
+  static const bool is_macOS_15_0_or_newer = is_macos_13_or_newer(MacOSVersion::MACOS_VER_15_0_PLUS);
 
   auto output = output_;
   bool needsCopyToOutput = false;
   if (needsGather(output)) {
-    output = at::empty(output.sizes(), output.scalar_type(), c10::nullopt, kMPS, c10::nullopt, c10::nullopt);
+    output = at::empty(output.sizes(), output.scalar_type(), std::nullopt, kMPS, std::nullopt, std::nullopt);
     needsCopyToOutput = true;
   }
 
@@ -100,7 +97,7 @@ static void unary_op_noresize(const Tensor& self, const Tensor& output_, std::st
 
     // If self is non-densely mapped in storage, create a dense output-like representation
     at::Tensor self_;
-    if (!is_dense_in_storage(self)) {
+    if (!is_dense_in_storage(self) && !is_macOS_15_0_or_newer) {
       self_ = at::empty_like(output, self.scalar_type());
       mps::mps_copy_(self_, self, false);
     } else {
@@ -152,19 +149,8 @@ MPSGraphTensor* trunc_tensor(MPSGraph* mpsGraph, MPSGraphTensor* inputTensor) {
     return inputTensor;
   }
 
-  if (!is_macos_13_or_newer()) {
-    MPSGraphTensor* zeroTensor = [mpsGraph constantWithScalar:0.0 dataType:inputTensor.dataType];
-    MPSGraphTensor* predicateTensor = [mpsGraph lessThanWithPrimaryTensor:inputTensor
-                                                          secondaryTensor:zeroTensor
-                                                                     name:nil];
-    return [mpsGraph selectWithPredicateTensor:predicateTensor
-                           truePredicateTensor:[mpsGraph ceilWithTensor:inputTensor name:nil]
-                          falsePredicateTensor:[mpsGraph floorWithTensor:inputTensor name:nil]
-                                          name:nil];
-  } else {
-    return [mpsGraph truncateWithTensor:inputTensor name:nil];
-  }
-};
+  return [mpsGraph truncateWithTensor:inputTensor name:nil];
+}
 
 MPSGraphTensor* log1p(MPSGraph* mpsGraph, MPSGraphTensor* inputTensor) {
   MPSGraphTensor* oneTensor = [mpsGraph constantWithScalar:1.0 dataType:inputTensor.dataType];
@@ -236,7 +222,6 @@ CREATE_MPS_STRUCTURED_UNARY_ROUNDING_TORCH_IMPL_FUNC(round_out_mps, round)
     });                                                                                                          \
   }
 
-CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(exp_out_mps, exponent)
 CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(exp2_out_mps, exponentBase2)
 CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(reciprocal_out_mps, reciprocal)
 CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(sqrt_out_mps, squareRoot)
@@ -254,7 +239,6 @@ CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(acos_out_mps, acos)
 CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(atan_out_mps, atan)
 CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(sinh_out_mps, sinh)
 CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(cosh_out_mps, cosh)
-CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(tanh_out_mps, tanh)
 CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(asinh_out_mps, asinh)
 CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(acosh_out_mps, acosh)
 CREATE_MPS_STRUCTURED_UNARY_TORCH_IMPL_FUNC(atanh_out_mps, atanh)
@@ -332,7 +316,7 @@ TORCH_IMPL_FUNC(expm1_out_mps)(const Tensor& self, const Tensor& output) {
   });
 }
 
-static void logit_mps_impl(const Tensor& self, c10::optional<double> eps, Tensor& output, const std::string op_name) {
+static void logit_mps_impl(const Tensor& self, std::optional<double> eps, Tensor& output, const std::string op_name) {
   std::string key = op_name + ":[" + (eps.has_value() ? std::to_string(eps.value()) : "NULL") + "]";
 
   mps::unary_op(self, output, key, ^MPSGraphTensor*(MPSGraph* mpsGraph, MPSGraphTensor* inputTensor) {
@@ -360,19 +344,19 @@ static void logit_mps_impl(const Tensor& self, c10::optional<double> eps, Tensor
   });
 }
 
-Tensor& logit_out_mps(const Tensor& self, c10::optional<double> eps, Tensor& result) {
+Tensor& logit_out_mps(const Tensor& self, std::optional<double> eps, Tensor& result) {
   logit_mps_impl(self, eps, result, "logit_out_mps");
   return result;
 }
 
-Tensor logit_mps(const Tensor& self, c10::optional<double> eps) {
-  Tensor result = at::empty(self.sizes(), ScalarType::Float, c10::nullopt, kMPS, c10::nullopt, c10::nullopt);
+Tensor logit_mps(const Tensor& self, std::optional<double> eps) {
+  Tensor result = at::empty(self.sizes(), ScalarType::Float, std::nullopt, kMPS, std::nullopt, std::nullopt);
   logit_mps_impl(self, eps, result, "logit_mps");
   return result;
 }
 
 TORCH_IMPL_FUNC(logit_backward_out_mps)
-(const Tensor& grad_output, const Tensor& input, c10::optional<double> eps, const Tensor& grad_input) {
+(const Tensor& grad_output, const Tensor& input, std::optional<double> eps, const Tensor& grad_input) {
   using namespace mps;
   using CachedGraph = MPSUnaryGradCachedGraph;
 
@@ -433,7 +417,7 @@ TORCH_IMPL_FUNC(logit_backward_out_mps)
 
 static void cumulative_op_impl(const Tensor& self,
                                int64_t dim,
-                               c10::optional<ScalarType> dtype,
+                               std::optional<ScalarType> dtype,
                                const Tensor& result,
                                MPSCumulativeOpType cumulativeOpType,
                                const std::string& op_name) {
@@ -448,17 +432,6 @@ static void cumulative_op_impl(const Tensor& self,
               "(original dim is ",
               dim,
               ")");
-  if (!is_macos_13_or_newer()) {
-    TORCH_WARN_ONCE(op_name, " supported by MPS on MacOS 13+, please upgrade");
-    Tensor cpu_result;
-    if (cumulativeOpType == MPSCumulativeOpType::CUMSUM) {
-      cpu_result = self.to(at::Device(kCPU)).cumsum(dim, dtype);
-    } else if (cumulativeOpType == MPSCumulativeOpType::CUMPROD) {
-      cpu_result = self.to(at::Device(kCPU)).cumprod(dim, dtype);
-    }
-    at::_copy_from_and_resize(cpu_result, result);
-    return;
-  }
   TORCH_CHECK(!self.is_complex(), "cumulative ops are not yet supported for complex");
   auto input = dtype.has_value() ? self.to(dtype.value()) : self;
 
@@ -491,12 +464,12 @@ static void cumulative_op_impl(const Tensor& self,
 }
 
 TORCH_IMPL_FUNC(cumsum_out_mps)
-(const Tensor& self, int64_t dim, c10::optional<ScalarType> dtype, const Tensor& result) {
+(const Tensor& self, int64_t dim, std::optional<ScalarType> dtype, const Tensor& result) {
   return cumulative_op_impl(self, dim, dtype, result, MPSCumulativeOpType::CUMSUM, "cumsum_out_mps");
 }
 
 TORCH_IMPL_FUNC(cumprod_out_mps)
-(const Tensor& self, int64_t dim, c10::optional<ScalarType> dtype, const Tensor& result) {
+(const Tensor& self, int64_t dim, std::optional<ScalarType> dtype, const Tensor& result) {
   return cumulative_op_impl(self, dim, dtype, result, MPSCumulativeOpType::CUMPROD, "cumprod_out_mps");
 }
 
