@@ -81,7 +81,7 @@ AT_FORALL_OPERATORS(DEFINE_COMPARATOR)
 const Tensor& quantized_resize_cpu_(
     const Tensor& self,
     IntArrayRef size,
-    c10::optional<MemoryFormat> optional_memory_format) {
+    std::optional<MemoryFormat> optional_memory_format) {
   // See Note [Writing Nondeterministic Operations]
   // Nondeterministic because if storage is resized, new elements are uninitialized
   globalContext().alertNotDeterministic("quantized_resize_cpu_");
@@ -96,7 +96,7 @@ const Tensor& quantized_resize_cpu_(
       "Can only resize quantized tensors with per-tensor schemes!");
   auto* self_ = self.unsafeGetTensorImpl();
   // NOLINTNEXTLINE(bugprone-argument-comment)
-  resize_impl_cpu_(self_, size, /*strides=*/c10::nullopt);
+  resize_impl_cpu_(self_, size, /*strides=*/std::nullopt);
   return self;
 }
 
