@@ -44,8 +44,8 @@ Tensor qsoftmax_qnnpack(const Tensor& qx, const int64_t dim) {
    */
 
   const int64_t last_dim = qx.dim() - 1;
-  std::optional<std::vector<int64_t>> permuted_dims = c10::nullopt;
-  std::optional<at::Tensor> qx_contig = c10::nullopt;
+  std::optional<std::vector<int64_t>> permuted_dims = std::nullopt;
+  std::optional<at::Tensor> qx_contig = std::nullopt;
   const at::Tensor* qx_contig_ptr = nullptr;
 
   if (qx.stride(dim) == 1) {
@@ -69,7 +69,7 @@ Tensor qsoftmax_qnnpack(const Tensor& qx, const int64_t dim) {
           .memory_format(qx_contig_ptr->suggest_memory_format()),
       qnnpack_softmax_output_scale,
       qnnpack_softmax_output_zero_point,
-      c10::nullopt);
+      std::nullopt);
 
   const size_t channels = qx.size(dim);
   const float input_scale = static_cast<float>(qx.q_scale());
