@@ -159,8 +159,7 @@ void _csr_matmult(
       }
     }
 
-    for (const auto jj : c10::irange(length)) {
-      (void)jj; //Suppress unused variable warning
+    for (C10_UNUSED const auto jj : c10::irange(length)) {
 
       // NOTE: the linked list that encodes col indices
       // is not guaranteed to be sorted.
