@@ -23,7 +23,6 @@ If you want to test float16
 
 """
 
-
 import argparse
 import dataclasses
 import functools
@@ -44,7 +43,6 @@ from os.path import abspath, exists
 from random import randint
 
 import matplotlib.pyplot as plt
-
 import numpy as np
 import pandas as pd
 from matplotlib import rcParams
@@ -52,9 +50,9 @@ from scipy.stats import gmean
 from tabulate import tabulate
 
 import torch
-
 import torch._dynamo
 
+
 rcParams.update({"figure.autolayout": True})
 plt.rc("axes", axisbelow=True)
 
@@ -389,11 +387,6 @@ def get_skip_tests(suite, device, is_training: bool):
             skip_tests.update(module.TorchBenchmarkRunner().skip_models_for_cpu)
         elif device == "cuda":
             skip_tests.update(module.TorchBenchmarkRunner().skip_models_for_cuda)
-    else:
-        if hasattr(module, "SKIP"):
-            skip_tests.update(module.SKIP)
-        if is_training and hasattr(module, "SKIP_TRAIN"):
-            skip_tests.update(module.SKIP_TRAIN)
 
     skip_tests = (f"-x {name}" for name in skip_tests)
     skip_str = " ".join(skip_tests)
@@ -440,7 +433,7 @@ def generate_commands(args, dtypes, suites, devices, compilers, output_dir):
                     if args.enable_cpu_launcher:
                         launcher_cmd = f"python -m torch.backends.xeon.run_cpu {args.cpu_launcher_args}"
                     cmd = f"{launcher_cmd} benchmarks/dynamo/{suite}.py --{testing} --{dtype} -d{device} --output={output_filename}"
-                    cmd = f"{cmd} {base_cmd} {args.extra_args} --no-skip --dashboard"
+                    cmd = f"{cmd} {base_cmd} {args.extra_args} --dashboard"
                     skip_tests_str = get_skip_tests(suite, device, args.training)
                     cmd = f"{cmd} {skip_tests_str}"
 
