@@ -6,6 +6,7 @@ import timeit
 from collections import namedtuple
 
 import benchmark_utils
+
 import numpy as np
 
 import torch
@@ -339,16 +340,12 @@ class BenchmarkRunner:
         return cmd_flag is None or test_flag == cmd_flag
 
     def _check_operator_first_char(self, test_flag, cmd_flag):
-        if cmd_flag is None or test_flag[:1].lower() in cmd_flag:
-            return True
-        return False
+        return cmd_flag is None or test_flag[:1].lower() in cmd_flag
 
     def _check_keep_list(self, test_flag, cmd_flag_list):
-        if cmd_flag_list is None or any(
+        return cmd_flag_list is None or any(
             test_flag == cmd_flag for cmd_flag in cmd_flag_list
-        ):
-            return True
-        return False
+        )
 
     def _keep_test(self, test_case):
         # TODO: consider regex matching for test filtering.
@@ -362,7 +359,7 @@ class BenchmarkRunner:
         )
 
         # Filter framework, operator, test_name, tag, forward_only
-        if (
+        return (
             self._check_keep(op_test_config.test_name, self.args.test_name)
             and self._check_keep_list(test_case.op_bench.module_name(), operators)
             and self._check_operator_first_char(
@@ -381,10 +378,7 @@ class BenchmarkRunner:
                 or "device" not in test_case.test_config.input_config
                 or self.args.device in op_test_config.test_name
             )
-        ):
-            return True
-
-        return False
+        )
 
     def _print_test_case_info(self, test_case):
         # Print out the test name and skip the real execution
