@@ -7,18 +7,13 @@ import warnings
 import torch
 import torch.distributed as dist
 import torch.testing._internal.common_methods_invocations as common_ops
-
-from torch.distributed._tensor import DeviceMesh, DTensor, Replicate
-
+from torch.distributed._tensor import DeviceMesh, DTensor
 from torch.overrides import resolve_name
 from torch.testing._internal.common_device_type import (
     instantiate_device_type_tests,
     ops,
 )
-from torch.testing._internal.common_methods_invocations import (
-    DecorateInfo,
-    op_db,
-)
+from torch.testing._internal.common_methods_invocations import DecorateInfo, op_db
 from torch.testing._internal.common_utils import (
     run_tests,
     suppress_warnings,
@@ -28,7 +23,9 @@ from torch.testing._internal.distributed._tensor.common_dtensor import (
     DTensorConverter,
     DTensorOpTestBase,
 )
-from torch.utils._pytree import tree_flatten, tree_map
+from torch.utils import _pytree as pytree
+from torch.utils._pytree import tree_map
+
 
 # rewrite common size variables to sth can be sharded evenly
 # we can enable uneven shards later, but need to adjust more on
@@ -96,13 +93,15 @@ dtensor_fails = {
     # get full support with varying sharding specs
     xfail("__getitem__"),
     xfail("__rsub__"),
+    xfail("_chunk_cat"),
     xfail("_native_batch_norm_legit"),
-    xfail("_softmax_backward_data"),
+    xfail("_upsample_bilinear2d_aa"),
     xfail("addbmm"),
     xfail("addmv"),
     xfail("addr"),
     xfail("all"),
     xfail("allclose"),
+    xfail("alias_copy"),
     xfail("amax"),
     xfail("amin"),
     xfail("aminmax"),
@@ -113,13 +112,13 @@ dtensor_fails = {
     xfail("argsort"),
     xfail("as_strided"),
     xfail("as_strided", "partial_views"),
+    xfail("as_strided_copy"),
     xfail("as_strided_scatter"),
-    xfail("baddbmm"),
     xfail("bernoulli"),
+    xfail("_batch_norm_with_update"),
     xfail("block_diag"),
     xfail("broadcast_shapes"),
     xfail("cauchy"),
-    xfail("cartesian_prod"),
     xfail("cdist"),
     xfail("cholesky"),
     xfail("cholesky_inverse"),
@@ -131,26 +130,22 @@ dtensor_fails = {
     xfail("combinations"),
     xfail("complex"),
     xfail("constant_pad_nd"),
-    xfail("corrcoef"),
     xfail("count_nonzero"),
-    xfail("cov"),
     xfail("cross"),
     xfail("cummax"),
     xfail("cummin"),
     xfail("cumsum"),
     xfail("cumulative_trapezoid"),
-    xfail("diag"),
-    xfail("diag_embed"),
-    xfail("diagflat"),
-    xfail("diagonal"),
-    xfail("diagonal_copy"),
     xfail("diagonal_scatter"),
     xfail("dist"),
     xfail("dot"),
-    xfail("einsum"),
     xfail("empty"),
+    xfail("empty_strided"),
     xfail("empty_like"),
+    xfail("empty_permuted"),
+    xfail("expand_copy"),
     xfail("exponential"),
+    xfail("equal"),
     xfail("eye"),
     xfail("fft.fft2"),
     xfail("fft.fft"),
@@ -190,23 +185,20 @@ dtensor_fails = {
     xfail("index_copy"),
     xfail("index_fill"),
     xfail("index_put"),
-    xfail("index_reduce"),
+    xfail("index_reduce", "prod"),
+    xfail("index_reduce", "mean"),
+    xfail("index_reduce", "amax"),
+    xfail("index_reduce", "amin"),
     xfail("index_select"),
     xfail("isin"),
-    xfail("isinf"),
-    xfail("isneginf"),
-    xfail("isposinf"),
     xfail("kthvalue"),
     xfail("linalg.cholesky"),
     xfail("linalg.cholesky_ex"),
-    xfail("linalg.cond"),
     xfail("linalg.cross"),
     xfail("linalg.det"),
     xfail("linalg.det", "singular"),
     xfail("linalg.eig"),
-    xfail("linalg.eigh"),
     xfail("linalg.eigvals"),
-    xfail("linalg.eigvalsh"),
     xfail("linalg.householder_product"),
     xfail("linalg.inv"),
     xfail("linalg.inv_ex"),
@@ -228,27 +220,22 @@ dtensor_fails = {
     xfail("linalg.norm", "subgradients_at_zero"),
     xfail("linalg.pinv"),
     xfail("linalg.pinv", "hermitian"),
-    xfail("linalg.qr"),
     xfail("linalg.slogdet"),
     xfail("linalg.solve"),
     xfail("linalg.solve_ex"),
     xfail("linalg.solve_triangular"),
-    xfail("linalg.svd"),
-    xfail("linalg.svdvals"),
     xfail("linalg.tensorinv"),
     xfail("linalg.tensorsolve"),
     xfail("linalg.vander"),
     xfail("linalg.vecdot"),
-    xfail("linalg.vector_norm"),
     xfail("linspace"),
+    xfail("linspace", "tensor_overload"),
     xfail("log_normal"),
-    xfail("log_softmax"),
-    xfail("log_softmax", "with_dtype"),
     xfail("logcumsumexp"),
     xfail("logdet"),
     xfail("logspace"),
+    xfail("logspace", "tensor_overload"),
     xfail("logsumexp"),
-    xfail("lt"),
     xfail("lu"),
     xfail("lu_solve"),
     xfail("lu_unpack"),
@@ -261,23 +248,14 @@ dtensor_fails = {
     xfail("masked.argmin"),
     xfail("masked.cumprod"),
     xfail("masked.cumsum"),
-    xfail("masked.log_softmax"),
-    xfail("masked.logaddexp"),
     xfail("masked.logsumexp"),
     xfail("masked.median"),
-    xfail("masked.norm"),
-    xfail("masked.prod"),
-    xfail("masked.softmin"),
-    xfail("masked.softmax"),
-    xfail("masked.sum"),
     xfail("matrix_exp"),
     xfail("max", "binary"),
-    xfail("max", "reduction_no_dim"),
     xfail("max", "reduction_with_dim"),
     xfail("maximum"),
     xfail("median"),
     xfail("min", "binary"),
-    xfail("min", "reduction_no_dim"),
     xfail("min", "reduction_with_dim"),
     xfail("minimum"),
     xfail("mode"),
@@ -291,7 +269,6 @@ dtensor_fails = {
     xfail("nansum"),
     xfail("native_batch_norm"),
     xfail("native_dropout_backward"),
-    xfail("native_layer_norm"),
     xfail("narrow_copy"),
     xfail("ne"),
     xfail("new_empty"),
@@ -315,11 +292,11 @@ dtensor_fails = {
     xfail("nn.functional.celu"),
     xfail("nn.functional.conv1d"),
     xfail("nn.functional.conv2d"),
+    xfail("nn.functional.conv3d"),
     xfail("nn.functional.conv_transpose1d"),
     xfail("nn.functional.conv_transpose2d"),
     xfail("nn.functional.conv_transpose3d"),
     xfail("nn.functional.cosine_similarity"),
-    xfail("nn.functional.cross_entropy"),
     xfail("nn.functional.ctc_loss"),
     xfail("nn.functional.dropout"),
     xfail("nn.functional.dropout2d"),
@@ -327,7 +304,6 @@ dtensor_fails = {
     xfail("nn.functional.elu"),
     xfail("nn.functional.fractional_max_pool2d"),
     xfail("nn.functional.fractional_max_pool3d"),
-    xfail("nn.functional.gaussian_nll_loss"),
     xfail("nn.functional.glu"),
     xfail("nn.functional.grid_sample"),
     xfail("nn.functional.group_norm"),
@@ -342,8 +318,8 @@ dtensor_fails = {
     xfail("nn.functional.interpolate", "bilinear"),
     xfail("nn.functional.interpolate", "linear"),
     xfail("nn.functional.interpolate", "nearest"),
+    xfail("nn.functional.interpolate", "nearest-exact"),
     xfail("nn.functional.interpolate", "trilinear"),
-    xfail("nn.functional.layer_norm"),
     xfail("nn.functional.leaky_relu"),
     xfail("nn.functional.linear"),
     xfail("nn.functional.local_response_norm"),
@@ -361,24 +337,22 @@ dtensor_fails = {
     xfail("nn.functional.mish"),
     xfail("nn.functional.mse_loss"),
     xfail("nn.functional.multi_margin_loss"),
+    xfail("nn.functional.multi_head_attention_forward"),
     xfail("nn.functional.multilabel_margin_loss"),
     xfail("nn.functional.multilabel_soft_margin_loss"),
-    xfail("nn.functional.nll_loss"),
     xfail("nn.functional.normalize"),
-    xfail("nn.functional.pad", "circular"),
     xfail("nn.functional.pad", "constant"),
     xfail("nn.functional.pad", "reflect"),
     xfail("nn.functional.pad", "replicate"),
+    xfail("nn.functional.pad", "replicate_negative"),
     xfail("nn.functional.pairwise_distance"),
     xfail("nn.functional.pdist"),
     xfail("nn.functional.pixel_shuffle"),
     xfail("nn.functional.pixel_unshuffle"),
-    xfail("nn.functional.poisson_nll_loss"),
     xfail("nn.functional.prelu"),
     xfail("nn.functional.relu6"),
     xfail("nn.functional.rrelu"),
     xfail("nn.functional.selu"),
-    xfail("nn.functional.silu"),
     xfail("nn.functional.smooth_l1_loss"),
     xfail("nn.functional.soft_margin_loss"),
     xfail("nn.functional.softplus"),
@@ -390,19 +364,15 @@ dtensor_fails = {
     xfail("nn.functional.upsample_bilinear"),
     xfail("nn.functional.upsample_nearest"),
     xfail("nonzero"),
-    xfail("norm"),
-    xfail("norm", "fro"),
-    xfail("norm", "inf"),
-    xfail("norm", "nuc"),
     xfail("normal"),
     xfail("normal", "number_mean"),
+    xfail("normal", "in_place"),
     xfail("ormqr"),
     xfail("ones"),
     xfail("pca_lowrank"),
     xfail("pinverse"),
     xfail("polar"),
     xfail("put"),
-    xfail("qr"),
     xfail("quantile"),
     xfail("rand_like"),
     xfail("randint_like"),
@@ -418,7 +388,6 @@ dtensor_fails = {
     xfail("rsub"),
     xfail("scalar_tensor"),
     xfail("scatter_add"),
-    xfail("scatter"),
     xfail("scatter_reduce", "amax"),
     xfail("scatter_reduce", "amin"),
     xfail("scatter_reduce", "mean"),
@@ -474,20 +443,15 @@ dtensor_fails = {
     xfail("std_mean"),
     xfail("std_mean", "unbiased"),
     xfail("stft"),
-    xfail("svd"),
     xfail("svd_lowrank"),
-    xfail("t"),
-    xfail("take_along_dim"),
+    xfail("t_copy"),
     xfail("take"),
     xfail("tensor_split"),
     xfail("to_sparse"),
-    xfail("topk"),
     xfail("trace"),
     xfail("trapezoid"),
     xfail("trapz"),
     xfail("triangular_solve"),
-    xfail("tril"),
-    xfail("triu"),
     xfail("unbind"),
     xfail("unfold"),
     xfail("unfold_copy"),
@@ -496,16 +460,17 @@ dtensor_fails = {
     xfail("unique_consecutive"),
     xfail("unique"),
     xfail("unsafe_split"),
+    xfail("unsafe_chunk"),
+    xfail("_unsafe_masked_index"),
+    xfail("_unsafe_masked_index_put_accumulate"),
     xfail("var_mean"),
     xfail("var_mean", "unbiased"),
     xfail("vdot"),
     xfail("view_copy"),
-    xfail("view_as_complex"),
-    xfail("where"),
     xfail("zeros"),
     # ops inside this might even fail without dtensor
     # tests, as we rescale op db common test size factor (i.e. L, M, S)
-    # which triggered the orignal function run failures with input
+    # which triggered the original function run failures with input
     # generation becomes wrong, we skip them for now but should enable later.
     # TODO: need to clean this list and remove all cases
     skip("argwhere"),
@@ -538,7 +503,6 @@ dtensor_fails = {
     skip("prod"),
     skip("_segment_reduce", "lengths"),
     skip("_segment_reduce", "offsets"),
-
     # TODO: fix the following ops
     skip("squeeze"),
 }
@@ -555,7 +519,6 @@ skip_bw = [
 ]
 
 
-
 OP_DB_WORLD_SIZE = 4
 # DEVICE_TYPE = "cuda" if torch.cuda.is_available() and torch.cuda.device_count() >= OP_DB_WORLD_SIZE else "cpu"
 # TODO: debug cuda illegal memory access issue and re-enable cuda tests
@@ -593,11 +556,10 @@ class TestDTensorOps(DTensorOpTestBase):
         self.check_dtensor_func(test, op)
 
     def assert_ref_dtensor_equal(self, dtensor_rs, rs):
-        flat_dtensor_rs, _ = tree_flatten(dtensor_rs)
-        flat_rs, _ = tree_flatten(rs)
+        flat_dtensor_rs = pytree.tree_leaves(dtensor_rs)
+        flat_rs = pytree.tree_leaves(rs)
         self.assertEqual(len(flat_dtensor_rs), len(flat_rs))
         for dtensor_r, r in zip(flat_dtensor_rs, flat_rs):
-
             if not isinstance(r, torch.Tensor):
                 continue
 
@@ -615,7 +577,7 @@ class TestDTensorOps(DTensorOpTestBase):
                 f"dtensor requires_grad: {dtensor_r.requires_grad}",
             )
 
-            self.assertEqualOnRank(dtensor_r.to_local(), r)
+            self.assertEqualOnRank(dtensor_r, r)
 
     def run_dtensor_crossref(self, func, args, kwargs):
         to_dtensor = DTensorConverter(self.mesh, args, kwargs)
@@ -623,10 +585,7 @@ class TestDTensorOps(DTensorOpTestBase):
         def concat_res_if_necessary(func, res: object) -> object:
             # concat the result on corresponding dim for ops like
             # split, so that we can call backward on a single tensor
-            if (
-                (resolve_name(func) is not None)
-                and ("split" in resolve_name(func))
-            ):
+            if (resolve_name(func) is not None) and ("split" in resolve_name(func)):
                 dim = args[2] if len(args) == 3 else 0
                 return torch.cat(res, dim=dim)
             else:
@@ -637,11 +596,7 @@ class TestDTensorOps(DTensorOpTestBase):
         rs = concat_res_if_necessary(func, rs)
 
         def to_replicate(e: object) -> object:
-            return (
-                e.redistribute(self.mesh, self.mesh.ndim * [Replicate()])
-                if isinstance(e, DTensor)
-                else e
-            )
+            return e.full_tensor() if isinstance(e, DTensor) else e
 
         try:
             # Suppress warnings, this doesn't matter for test_meta.py
@@ -663,10 +618,10 @@ class TestDTensorOps(DTensorOpTestBase):
                         # errors
                         dtensor_rs = func(*dtensor_args, **dtensor_kwargs)
 
-                        # we need to skip tests containing tensors of zero elmeents for now.
+                        # we need to skip tests containing tensors of zero elements for now.
                         # see issue: https://github.com/pytorch/tau/issues/470
                         # TODO remove this once issue above fixed.
-                        flat_args, _ = tree_flatten(dtensor_rs)
+                        flat_args = pytree.tree_leaves(dtensor_rs)
                         if any(
                             isinstance(e, torch.Tensor) and e.numel() == 0
                             for e in flat_args
@@ -702,7 +657,6 @@ class TestDTensorOps(DTensorOpTestBase):
 
         return rs
 
-
     def check_dtensor_func(self, test_func, opinfo, dry_run=False):
         try:
             test_func()
