@@ -15,24 +15,22 @@ import time
 from abc import ABC, abstractmethod
 from base64 import b64encode
 from datetime import datetime, timedelta
-from typing import Callable, Optional, Tuple, cast
+from typing import Callable, cast, Optional, Tuple
 from unittest import TestCase
-from unittest.mock import MagicMock, Mock, call, patch
+from unittest.mock import call, MagicMock, Mock, patch, PropertyMock
 
-from torch.distributed import Store
+import torch.distributed as dist
+from torch.distributed import HashStore, Store
 from torch.distributed.elastic.rendezvous import (
     RendezvousClosedError,
     RendezvousError,
+    RendezvousInfo,
     RendezvousParameters,
     RendezvousStateError,
+    RendezvousStoreInfo,
     RendezvousTimeoutError,
 )
 from torch.distributed.elastic.rendezvous.dynamic_rendezvous import (
-    DynamicRendezvousHandler,
-    RendezvousBackend,
-    RendezvousSettings,
-    RendezvousTimeout,
-    Token,
     _Action,
     _BackendRendezvousStateHolder,
     _DistributedRendezvousOpExecutor,
@@ -46,13 +44,20 @@ from torch.distributed.elastic.rendezvous.dynamic_rendezvous import (
     _RendezvousState,
     _RendezvousStateHolder,
     create_handler,
+    DynamicRendezvousHandler,
+    RendezvousBackend,
+    RendezvousSettings,
+    RendezvousTimeout,
+    Token,
 )
 
 
 class CustomAssertMixin:
     assertDictEqual: Callable
 
-    def assert_state_equal(self, actual: _RendezvousState, expected: _RendezvousState) -> None:
+    def assert_state_equal(
+        self, actual: _RendezvousState, expected: _RendezvousState
+    ) -> None:
         self.assertDictEqual(vars(actual), vars(expected))
 
     def assert_state_empty(self, actual: _RendezvousState) -> None:
@@ -87,7 +92,8 @@ class RendezvousTimeoutTest(TestCase):
         for join_timeout in join_timeouts:
             with self.subTest(join_timeout=join_timeout):
                 with self.assertRaisesRegex(
-                    ValueError, rf"^The join timeout \({join_timeout}\) must be positive.$"
+                    ValueError,
+                    rf"^The join timeout \({join_timeout}\) must be positive.$",
                 ):
                     timeout = RendezvousTimeout(join_timeout)
 
@@ -143,8 +149,12 @@ class RendezvousStateTest(TestCase):
         for num_nodes, max_byte_size in expected_max_sizes:
             with self.subTest(num_nodes=num_nodes, max_byte_size=max_byte_size):
                 for i in range(num_nodes):
-                    node_running = _NodeDesc(f"dummy{i}.dummy1-dummy1-dummy1-dummy1.com", 12345, i)
-                    node_waiting = _NodeDesc(f"dummy{i}.dummy2-dummy2-dummy2-dummy2.com", 67890, i)
+                    node_running = _NodeDesc(
+                        f"dummy{i}.dummy1-dummy1-dummy1-dummy1.com", 12345, i
+                    )
+                    node_waiting = _NodeDesc(
+                        f"dummy{i}.dummy2-dummy2-dummy2-dummy2.com", 67890, i
+                    )
 
                     state.participants[node_running] = i
 
@@ -269,7 +279,9 @@ class BackendRendezvousStateHolderTest(TestCase, CustomAssertMixin):
         return state
 
     def _create_state_holder(self) -> _BackendRendezvousStateHolder:
-        return _BackendRendezvousStateHolder(self._backend, self._settings, self._cache_duration)
+        return _BackendRendezvousStateHolder(
+            self._backend, self._settings, self._cache_duration
+        )
 
     def test_init_initializes_state_holder(self) -> None:
         state_holder = self._create_state_holder()
@@ -361,7 +373,9 @@ class BackendRendezvousStateHolderTest(TestCase, CustomAssertMixin):
 
         self._backend.set_state_internal(state)
 
-        with patch("torch.distributed.elastic.rendezvous.dynamic_rendezvous.time") as mock_time:
+        with patch(
+            "torch.distributed.elastic.rendezvous.dynamic_rendezvous.time"
+        ) as mock_time:
             for cache_duration in [1, 5, 10]:
                 with self.subTest(cache_duration=cache_duration):
                     self._cache_duration = cache_duration
@@ -397,7 +411,9 @@ class BackendRendezvousStateHolderTest(TestCase, CustomAssertMixin):
 
         self._backend.set_state_internal(state)
 
-        with patch("torch.distributed.elastic.rendezvous.dynamic_rendezvous.time") as mock_time:
+        with patch(
+            "torch.distributed.elastic.rendezvous.dynamic_rendezvous.time"
+        ) as mock_time:
             self._cache_duration = 1
 
             state_holder = self._create_state_holder()
@@ -568,7 +584,9 @@ class DistributedRendezvousOpExecutorTest(TestCase, CustomAssertMixin):
         if settings is None:
             settings = self._create_settings()
 
-        return _DistributedRendezvousOpExecutor(self._node, self._state_holder, settings)
+        return _DistributedRendezvousOpExecutor(
+            self._node, self._state_holder, settings
+        )
 
     def _run_action(self, action: _Action) -> None:
         op_executor = self._create_op_executor()
@@ -644,14 +662,18 @@ class DistributedRendezvousOpExecutorTest(TestCase, CustomAssertMixin):
                 node = _NodeDesc(f"dummy{i}", 1, 1)
                 rank = i
             else:
-                node = _NodeDesc(f"dummy{num_participants - i - 1}", 1, 1)  # Add in reverse.
+                node = _NodeDesc(
+                    f"dummy{num_participants - i - 1}", 1, 1
+                )  # Add in reverse.
                 rank = 0
 
             state.participants[node] = rank
 
             state.last_heartbeats[node] = self._now
 
-    def test_run_adds_to_participants_and_starts_last_call_if_min_nodes_is_reached(self) -> None:
+    def test_run_adds_to_participants_and_starts_last_call_if_min_nodes_is_reached(
+        self,
+    ) -> None:
         for num_participants in range(3):
             self._state = _RendezvousState()
 
@@ -817,12 +839,16 @@ class DistributedRendezvousOpExecutorTest(TestCase, CustomAssertMixin):
         self.assertListEqual(self._mock_state_holder.mock_calls, [call.sync()])
 
     def test_run_delays_execution_if_sync_requested(self) -> None:
-        with patch("torch.distributed.elastic.rendezvous.dynamic_rendezvous._delay") as mock_delay:
+        with patch(
+            "torch.distributed.elastic.rendezvous.dynamic_rendezvous._delay"
+        ) as mock_delay:
             self._run_action(_Action.SYNC)
 
             mock_delay.assert_called_once_with(seconds=1)
 
-        self.assertListEqual(self._mock_state_holder.mock_calls, [call.sync(), call.sync()])
+        self.assertListEqual(
+            self._mock_state_holder.mock_calls, [call.sync(), call.sync()]
+        )
 
 
 class AbstractTestRendezvousOp(ABC):
@@ -850,7 +876,9 @@ class AbstractTestRendezvousOp(ABC):
         mock_datetime = self._datetime_patch.start()
         mock_datetime.utcnow.return_value = self._now
 
-        self._time_patch = patch("torch.distributed.elastic.rendezvous.dynamic_rendezvous.time")
+        self._time_patch = patch(
+            "torch.distributed.elastic.rendezvous.dynamic_rendezvous.time"
+        )
 
         mock_time = self._time_patch.start()
         mock_time.monotonic.return_value = self._deadline
@@ -932,14 +960,38 @@ class TestRendezvousJoinOp(AbstractTestRendezvousOp, TestCase):
 
             self._assert_action(expected_action)
 
-    def test_waits_next_round_if_rendezvous_is_complete(self) -> None:
+    def test_treat_as_redundancy_for_next_rendezvous_if_rendezvous_is_complete(
+        self,
+    ) -> None:
+        self._max_nodes = 1
+
+        self._state.complete = True
+
+        self._assert_action(_Action.ADD_TO_REDUNDANCY_LIST)
+
+    def test_waits_next_round_if_rendezvous_is_complete_and_node_is_redundant(
+        self,
+    ) -> None:
+        self._state.redundancy_list.add(self._node)
+
         self._max_nodes = 1
 
         self._state.complete = True
 
         self._assert_waits_rendezvous_completion()
 
-    def test_waits_next_round_if_rendezvous_is_complete_and_node_is_in_wait_list(self) -> None:
+    def test_remove_from_rednundancy_list(self) -> None:
+        self._state.redundancy_list.add(self._node)
+
+        self._max_nodes = 2
+
+        self._state.complete = True
+
+        self._assert_action(_Action.REMOVE_FROM_REDUNDANCY_LIST)
+
+    def test_waits_next_round_if_rendezvous_is_complete_and_node_is_in_wait_list(
+        self,
+    ) -> None:
         self._state.wait_list.add(self._node)
 
         self._state.complete = True
@@ -981,14 +1033,18 @@ class TestRendezvousJoinOp(AbstractTestRendezvousOp, TestCase):
 
         self._assert_action(_Action.ERROR_TIMEOUT)
 
-    def test_raises_timeout_if_rollback_deadline_exceeded_and_node_is_participant(self) -> None:
+    def test_raises_timeout_if_rollback_deadline_exceeded_and_node_is_participant(
+        self,
+    ) -> None:
         self._deadline = 0
 
         self._state.participants[self._node] = 0
 
         self._assert_action(_Action.ERROR_TIMEOUT)
 
-    def test_raises_timeout_if_rollback_deadline_exceeded_and_node_is_in_wait_list(self) -> None:
+    def test_raises_timeout_if_rollback_deadline_exceeded_and_node_is_in_wait_list(
+        self,
+    ) -> None:
         self._deadline = 0
 
         self._state.wait_list.add(self._node)
@@ -1004,13 +1060,35 @@ class TestRendezvousJoinOp(AbstractTestRendezvousOp, TestCase):
 
         self._assert_action(_Action.REMOVE_FROM_PARTICIPANTS)
 
-    def test_removes_from_wait_list_if_timed_out_but_rollback_deadline_is_not_reached(self) -> None:
+    def test_removes_from_wait_list_if_timed_out_but_rollback_deadline_is_not_reached(
+        self,
+    ) -> None:
         self._deadline = 5
 
         self._state.wait_list.add(self._node)
 
         self._assert_action(_Action.REMOVE_FROM_WAIT_LIST)
 
+    def test_no_timeout_for_redundant_node(self) -> None:
+        self._max_nodes = 1
+        self._deadline = 0
+        self._state.complete = True
+
+        self._state.redundancy_list.add(self._node)
+
+        self._assert_action(_Action.SYNC)
+
+    def test_keep_alive_for_redundant_node(self) -> None:
+        self._deadline = 0
+        self._max_nodes = 1
+        self._state.complete = True
+
+        self._state.redundancy_list.add(self._node)
+
+        keep_alive_time = self._now - self._keep_alive_interval
+        self._state.last_heartbeats[self._node] = keep_alive_time
+        self._assert_action(_Action.KEEP_ALIVE)
+
 
 class TestRendezvousCloseOp(AbstractTestRendezvousOp, TestCase):
     def _create_op(self) -> Callable:
@@ -1053,7 +1131,9 @@ class TestRendezvousKeepAliveOp(AbstractTestRendezvousOp, TestCase):
     def test_finishes_if_no_keep_alive_update_is_needed(self) -> None:
         delta = timedelta(seconds=1)
 
-        self._state.last_heartbeats[self._node] = self._now - self._keep_alive_interval + delta
+        self._state.last_heartbeats[self._node] = (
+            self._now - self._keep_alive_interval + delta
+        )
 
         self._assert_action(_Action.FINISH)
 
@@ -1077,9 +1157,11 @@ class DynamicRendezvousHandlerTest(TestCase):
 
         self._store = DummyStore()
 
-        self._mock_store_get = MagicMock(return_value=b"dummy_value")
+        self._mock_store_get = MagicMock(return_value=b"123")
+        self._mock_store_set = MagicMock()
 
         setattr(self._store, "get", self._mock_store_get)  # noqa: B010
+        setattr(self._store, "set", self._mock_store_set)  # noqa: B010
 
         self._state_holder = FakeRendezvousStateHolder()
 
@@ -1089,6 +1171,16 @@ class DynamicRendezvousHandlerTest(TestCase):
 
         self._state = self._state_holder.state
 
+        self._tcp_store_mock = DummyStore()
+
+        patcher = patch.object(
+            DynamicRendezvousHandler,
+            "_create_tcp_store_server",
+            return_value=self._tcp_store_mock,
+        )
+        patcher.start()
+        self.addCleanup(patcher.stop)
+
     def _create_handler(self) -> DynamicRendezvousHandler:
         settings = RendezvousSettings(
             run_id="dummy_run_id",
@@ -1109,6 +1201,36 @@ class DynamicRendezvousHandlerTest(TestCase):
             self._node, settings, "dummy_backend", self._store, self._state_holder
         )
 
+    def test_share_store_creates_tcp_store(self):
+        handler = self._create_handler()
+
+        shared_store_info = RendezvousStoreInfo("host", 54321)
+        with patch.object(RendezvousStoreInfo, "build", return_value=shared_store_info):
+            rdzv_info = handler.next_rendezvous()
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, "host")
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_port, 54321)
+        self.assertEqual(handler._shared_tcp_store_server, self._tcp_store_mock)
+
+        rdzv_info = handler.next_rendezvous()
+        self.assertEqual(handler._shared_tcp_store_server, self._tcp_store_mock)
+
+    def test_share_store_when_tcp_store(self):
+        handler = self._create_handler()
+
+        with patch.object(dist, "PrefixStore", new=Mock):
+            handler._store = Mock(spec=dist.TCPStore)
+            type(handler._store).host = PropertyMock(return_value="host")
+            type(handler._store).port = PropertyMock(return_value=54321)
+            rdzv_info = handler.next_rendezvous()
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, "host")
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_port, 54321)
+            self.assertEqual(handler._shared_tcp_store_server, handler._store)
+
+            rdzv_info = handler.next_rendezvous()
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, "host")
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_port, 54321)
+            self.assertEqual(handler._shared_tcp_store_server, handler._store)
+
     @patch("torch.distributed.elastic.rendezvous.dynamic_rendezvous._delay")
     def test_next_rendezvous_skews_the_first_join_attempt(self, mock_delay) -> None:
         for round, expected_call_count in [(0, True), (1, False)]:
@@ -1131,14 +1253,16 @@ class DynamicRendezvousHandlerTest(TestCase):
 
         handler = self._create_handler()
 
-        store, rank, world_size = handler.next_rendezvous()
+        rdzv_info = handler.next_rendezvous()
 
-        self.assertEqual(rank, 2)
-        self.assertEqual(world_size, 3)
+        self.assertEqual(rdzv_info.rank, 2)
+        self.assertEqual(rdzv_info.world_size, 3)
 
-        _ = store.get("dummy_key")
+        _ = rdzv_info.store.get("dummy_key")
 
-        self._mock_store_get.assert_called_once_with("torch.rendezvous.dummy_run_id.0/dummy_key")
+        self._mock_store_get.assert_called_with(
+            "torch.rendezvous.dummy_run_id.0/dummy_key"
+        )
 
     def test_next_rendezvous_respects_the_requested_timeout(self) -> None:
         self._mock_sync.side_effect = lambda: time.sleep(0.3)
@@ -1451,7 +1575,9 @@ class CreateHandlerTest(TestCase):
         self.assertEqual(handler.settings.min_nodes, self._params.min_nodes)
         self.assertEqual(handler.settings.max_nodes, self._params.max_nodes)
         self.assertEqual(handler.settings.timeout.join, self._expected_timeout.join)
-        self.assertEqual(handler.settings.timeout.last_call, self._expected_timeout.last_call)
+        self.assertEqual(
+            handler.settings.timeout.last_call, self._expected_timeout.last_call
+        )
         self.assertEqual(handler.settings.timeout.close, self._expected_timeout.close)
 
     def test_create_handler_returns_handler_if_timeout_is_not_specified(self) -> None:
@@ -1470,3 +1596,300 @@ class CreateHandlerTest(TestCase):
             with self.assertRaises(RendezvousError):
                 create_handler(self._store, self._backend, self._params)
                 record_mock.assert_called_once()
+
+    def test_create_handler_rdzv_local_addr(self) -> None:
+        params = RendezvousParameters(
+            backend=self._backend.name,
+            endpoint="dummy_endpoint",
+            run_id="dummy_run_id",
+            min_nodes=1,
+            max_nodes=1,
+            join_timeout="50",
+            last_call_timeout="60",
+            close_timeout="70",
+            local_addr="127.0.0.2",
+        )
+        store = HashStore()
+        handler = create_handler(store, self._backend, params)
+        rdzv_info = handler.next_rendezvous()
+        self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, "127.0.0.2")
+
+
+def _ignore_exception(exception_type: Exception, fn: Callable):
+    try:
+        fn()
+    except exception_type as e:
+        pass
+
+
+def _wait_for(condition, timeout=10, interval=1, name=None):
+    def _wait_while():
+        while True:
+            if condition():
+                break
+            else:
+                time.sleep(interval)
+
+    wait_thread = threading.Thread(target=_wait_while, name=name)
+    wait_thread.start()
+    wait_thread.join(timeout=timeout)
+
+
+class _CapturingThread(threading.Thread):
+    def __init__(self, target=None, name=None, args=None, kwargs=None):
+        if args is None:
+            args = ()
+        if kwargs is None:
+            kwargs = {}
+        threading.Thread.__init__(
+            self, target=target, args=args, kwargs=kwargs, name=name
+        )
+        self._result = None
+
+    def run(self):
+        if self._target is not None:
+            self._result = self._target(*self._args, **self._kwargs)
+
+    def join(self, *args):
+        threading.Thread.join(self, *args)
+        return self._result
+
+
+class IntegrationTest(TestCase):
+    def setUp(self) -> None:
+        self._store = HashStore()
+        self._handlers = []
+        self._backend = _InMemoryRendezvousBackend()
+
+    def tearDown(self) -> None:
+        for handler in self._handlers:
+            handler._stop_heartbeats()
+
+    def _create_handler(self, **kwargs) -> DynamicRendezvousHandler:
+        params = {
+            "backend": self._backend.name,
+            "endpoint": "dummy_endpoint",
+            "run_id": "dummy_run_id",
+            "min_nodes": 2,
+            "max_nodes": 2,
+            "join_timeout": "5",
+            "local_addr": f"127.0.0.{len(self._handlers)}",
+        }
+        params.update(**kwargs)
+
+        rzdv_params = RendezvousParameters(**params)
+
+        handler = create_handler(self._store, self._backend, rzdv_params)
+        self._handlers.append(handler)
+        return handler
+
+    def test_all_nodes_join_rendezvous(self) -> None:
+        handler1 = self._create_handler(min_nodes=2, max_nodes=2)
+        handler2 = self._create_handler(min_nodes=2, max_nodes=2)
+
+        handler1_thread = _CapturingThread(target=handler1.next_rendezvous)
+        handler2_thread = _CapturingThread(target=handler2.next_rendezvous)
+
+        handler1_thread.start()
+        handler2_thread.start()
+
+        rdzv_info1: RendezvousInfo = handler1_thread.join()
+        rdzv_info2: RendezvousInfo = handler2_thread.join()
+        self.assertEqual(rdzv_info1.store.underlying_store, self._store)
+        self.assertEqual(rdzv_info2.store.underlying_store, self._store)
+
+        self.assertNotEqual(rdzv_info1.rank, rdzv_info2.rank)
+
+        self.assertEqual(rdzv_info1.world_size, 2)
+        self.assertEqual(rdzv_info2.world_size, 2)
+
+    def test_redundancy_list(self) -> None:
+        handler1 = self._create_handler(min_nodes=2, max_nodes=2)
+        handler2 = self._create_handler(min_nodes=2, max_nodes=2)
+        handler3 = self._create_handler(min_nodes=2, max_nodes=2)
+
+        handler1_thread = _CapturingThread(target=handler1.next_rendezvous)
+        handler2_thread = _CapturingThread(target=handler2.next_rendezvous)
+        handler3_thread = _CapturingThread(
+            target=_ignore_exception,
+            args=(RendezvousTimeoutError, lambda: handler3.next_rendezvous()),
+        )
+
+        handler1_thread.start()
+        handler2_thread.start()
+
+        # establish successful rendezvous
+        handler1_thread.join()
+        handler2_thread.join()
+
+        # expect to register in redundancy list
+        handler3_thread.start()
+
+        # wait until the handler3 is registered in the redundancy list
+        _wait_for(lambda: pickle.loads(self._backend.get_state()[0]).redundancy_list)
+
+        state_and_token = self._backend.get_state()
+        state = pickle.loads(state_and_token[0])
+        addresses = [node.addr for node in state.redundancy_list]
+        self.assertListEqual(addresses, ["127.0.0.2"])
+
+    def test_redundancy_transition_to_wait_list_then_join_rendezvous(self) -> None:
+        handler1 = self._create_handler(
+            min_nodes=1,
+            max_nodes=2,
+        )
+        handler2 = self._create_handler(
+            min_nodes=1,
+            max_nodes=2,
+            keep_alive_interval=timedelta(seconds=1),
+        )
+        handler3 = self._create_handler(
+            min_nodes=1,
+            max_nodes=2,
+        )
+
+        handler1_thread = _CapturingThread(target=handler1.next_rendezvous)
+        handler2_thread = _CapturingThread(target=handler2.next_rendezvous)
+
+        handler3_thread = _CapturingThread(
+            target=_ignore_exception,
+            args=(RendezvousTimeoutError, lambda: handler3.next_rendezvous()),
+        )
+
+        handler1_thread.start()
+        handler2_thread.start()
+
+        # establish successful rendezvous
+        handler1_thread.join()
+        handler2_thread.join()
+
+        handler3_thread.start()
+
+        _wait_for(lambda: pickle.loads(self._backend.get_state()[0]).redundancy_list)
+
+        handler2._stop_heartbeats()
+
+        _wait_for(
+            lambda: len(pickle.loads(self._backend.get_state()[0]).participants) == 1
+        )
+        _wait_for(
+            lambda: len(pickle.loads(self._backend.get_state()[0]).wait_list) == 1
+        )
+
+    def test_use_agent_store_is_true_by_default(self):
+        handler = self._create_handler(
+            min_nodes=1,
+            max_nodes=2,
+        )
+
+        self.assertTrue(handler.use_agent_store)
+
+    @patch.dict(os.environ, {"TORCH_DISABLE_SHARE_RDZV_TCP_STORE": "1"})
+    def test_use_agent_store_is_disabled(self):
+        handler = self._create_handler(
+            min_nodes=1,
+            max_nodes=2,
+        )
+
+        self.assertFalse(handler.use_agent_store)
+
+    @patch.object(dist, "PrefixStore")
+    def test_share_tcp_store_from_backend(self, prefix_store_class_mock):
+        prefix_store = Mock(spec=dist.PrefixStore)
+        prefix_store_class_mock.return_value = prefix_store
+
+        tcp_store = Mock(spec=dist.TCPStore)
+        expected_addr = "expected_address"
+        expected_port = 54321
+        type(tcp_store).host = PropertyMock(return_value=expected_addr)
+        type(tcp_store).port = PropertyMock(return_value=expected_port)
+        # this will be injected
+        self._store = tcp_store
+
+        handler1 = self._create_handler(min_nodes=2, max_nodes=2)
+        handler2 = self._create_handler(min_nodes=2, max_nodes=2)
+
+        handler1_thread = _CapturingThread(target=handler1.next_rendezvous)
+        handler2_thread = _CapturingThread(target=handler2.next_rendezvous)
+
+        handler1_thread.start()
+        handler2_thread.start()
+
+        rdzv_info1: RendezvousInfo = handler1_thread.join()
+        rdzv_info2: RendezvousInfo = handler2_thread.join()
+
+        self.assertEqual(rdzv_info1.store, prefix_store)
+        self.assertEqual(rdzv_info2.store, prefix_store)
+        prefix_store_class_mock.assert_called_with(
+            "torch.rendezvous.dummy_run_id.0", tcp_store
+        )
+
+        self.assertEqual(
+            rdzv_info1.bootstrap_store_info, rdzv_info2.bootstrap_store_info
+        )
+
+        self.assertEqual(rdzv_info1.bootstrap_store_info.master_addr, expected_addr)
+        self.assertEqual(rdzv_info1.bootstrap_store_info.master_port, expected_port)
+
+    @patch.dict(os.environ, {"TORCH_DISABLE_SHARE_RDZV_TCP_STORE": "1"})
+    @patch.object(dist, "PrefixStore")
+    def test_share_tcp_store_is_disabled(self, prefix_store_class_mock):
+        prefix_store = Mock()
+        prefix_store_class_mock.return_value = prefix_store
+
+        prefix_store.set.return_value = None
+        prefix_store.get.return_value = b"123"
+        tcp_store = Mock(spec=dist.TCPStore)
+        # this will be injected
+        self._store = tcp_store
+
+        handler1 = self._create_handler(min_nodes=2, max_nodes=2)
+        handler2 = self._create_handler(min_nodes=2, max_nodes=2)
+
+        handler1_thread = _CapturingThread(target=handler1.next_rendezvous)
+        handler2_thread = _CapturingThread(target=handler2.next_rendezvous)
+
+        handler1_thread.start()
+        handler2_thread.start()
+
+        rdzv_info1: RendezvousInfo = handler1_thread.join()
+        rdzv_info2: RendezvousInfo = handler2_thread.join()
+
+        self.assertEqual(rdzv_info1.store, prefix_store)
+        self.assertEqual(rdzv_info2.store, prefix_store)
+        prefix_store_class_mock.assert_called_with(
+            "torch.rendezvous.dummy_run_id.0", self._store
+        )
+        self.assertEqual(rdzv_info1.bootstrap_store_info.master_port, 123)
+        self.assertEqual(rdzv_info2.bootstrap_store_info.master_port, 123)
+
+
+class _InMemoryRendezvousBackend(RendezvousBackend):
+    def __init__(self) -> None:
+        self._lock = threading.Lock()
+        self._state = None
+        self._token = None
+
+    @property
+    def name(self):
+        return "_in_memory_backend"
+
+    def get_state(self):
+        with self._lock:
+            if self._state is None:
+                return None
+            return (self._state, self._token)
+
+        return self._state
+
+    def set_state(self, state, token):
+        if state is None:
+            raise ValueError("State cannot be None.")
+        with self._lock:
+            if token is None and self._token is not None:
+                return None
+            if self._token != token:
+                return None
+
+            self._state = state
+            self._token = self._token + 1 if self._token is not None else 0
