@@ -16,7 +16,9 @@ from torch.distributed.fsdp import (
     ShardingStrategy,
 )
 from torch.distributed.fsdp._common_utils import clean_tensor_name
-from torch.distributed.fsdp.flat_param import FlatParameter
+from torch.distributed.fsdp._flat_param import FlatParameter
+from torch.distributed.fsdp.fully_sharded_data_parallel import FLAT_PARAM
+from torch.distributed.fsdp.wrap import ModuleWrapPolicy
 from torch.nn.parallel.distributed import DistributedDataParallel as DDP
 from torch.testing._internal.common_distributed import skip_if_lt_x_gpu
 from torch.testing._internal.common_fsdp import (
@@ -28,6 +30,7 @@ from torch.testing._internal.common_fsdp import (
 )
 from torch.testing._internal.common_utils import run_tests, TEST_WITH_DEV_DBG_ASAN
 
+
 if not dist.is_available():
     print("Distributed not available, skipping tests", file=sys.stderr)
     sys.exit(0)
@@ -434,7 +437,7 @@ class TestUnshardParams(TestUnshardParamsBase):
             CUDAInitMode.CUDA_BEFORE,
             deterministic=True,
         )
-        model.register_buffer("buffer", torch.ones(1))
+        model.buffer = nn.Buffer(torch.ones(1))
         # Wrap the top-level with FSDP since `named_parameters()` and
         # `named_buffers` will contain FSDP prefixes if called on a non-FSDP
         # root module
@@ -447,7 +450,7 @@ class TestUnshardParams(TestUnshardParamsBase):
             ),
             self.process_group,
         )
-        fsdp_model.register_buffer("buffer", torch.ones(1))
+        fsdp_model.buffer = nn.Buffer(torch.ones(1))
         with FSDP.summon_full_params(fsdp_model):
             for call in ["named_parameters", "named_buffers"]:
                 for (n1, p1), (n2, p2) in itertools.zip_longest(
@@ -622,6 +625,19 @@ class TestUnshardParams(TestUnshardParamsBase):
             for param in fsdp_model.parameters():
                 self.assertTrue(param.grad is None)
 
+    @skip_if_lt_x_gpu(2)
+    def test_unshard_submodule(self):
+        model = nn.Sequential(
+            nn.Sequential(nn.Linear(16, 16), nn.Linear(16, 16)),
+            nn.Sequential(nn.Linear(16, 16), nn.Linear(16, 16)),
+        ).cuda()
+        model = FSDP(model, auto_wrap_policy=ModuleWrapPolicy((nn.Sequential,)))
+        with FSDP.summon_full_params(model[0]):
+            # Check that the summoned module does not have its flat parameter
+            for param_name, param in model[0].named_parameters():
+                self.assertFalse(FLAT_PARAM in param_name)
+            self.assertGreater(len(list(model[0].parameters())), 1)
+
 
 class TestUnshardParamsNoShard(TestUnshardParamsBase):
     @property
@@ -660,7 +676,7 @@ class TestUnshardParamsErrors(TestUnshardParamsBase):
     @skip_if_lt_x_gpu(2)
     def test_unshard_params_from_forward_raises(self):
         class MyModule(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.a = nn.Parameter(torch.zeros(5))
 
