@@ -15,6 +15,7 @@ from itertools import groupby
 import torch
 import torch.distributed as c10d
 
+
 if not c10d.is_available() or not c10d.is_gloo_available():
     print("c10d GLOO not available, skipping tests", file=sys.stderr)
     sys.exit(0)
@@ -45,7 +46,6 @@ from torch.testing._internal.common_distributed import (
     requires_gloo,
     simple_sparse_reduce_tests,
     skip_if_lt_x_gpu,
-    skip_if_win32,
     verify_ddp_error_logged,
 )
 from torch.testing._internal.common_utils import (
@@ -1480,47 +1480,6 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
         for i, tensor in enumerate(tensors):
             self.assertEqual(torch.full(size, float(i * self.world_size)), tensor)
 
-    @skip_if_win32()
-    @requires_gloo()
-    def test_round_robin(self):
-        num_process_groups = 2
-        store = c10d.FileStore(self.file_name, self.world_size)
-        c10d.init_process_group(
-            backend="gloo", store=store, rank=self.rank, world_size=self.world_size
-        )
-        pg = c10d._round_robin_process_groups(
-            [c10d.new_group(pg_options=self.opts()) for i in range(num_process_groups)]
-        )
-
-        # Run a few collectives so that we have called each process group
-        for _ in range(num_process_groups + 1):
-            tensor = torch.full([100, 100], float(self.rank))
-            pg.broadcast(tensor, root=0).wait()
-            self.assertEqual(torch.full([100, 100], 0.0), tensor)
-
-    @skip_if_win32()
-    @requires_gloo()
-    def test_round_robin_create_destroy(self):
-        store = c10d.FileStore(self.file_name, self.world_size)
-        c10d.init_process_group(
-            backend="gloo", store=store, rank=self.rank, world_size=self.world_size
-        )
-
-        def create(num, prefix):
-            return c10d._round_robin_process_groups(
-                [c10d.new_group(pg_options=self.opts()) for i in range(num)]
-            )
-
-        # Run create/use/destroy twice
-        for i in range(2):
-            num_process_groups = 2
-            pg = create(num=num_process_groups, prefix=i)
-            for _ in range(3):
-                tensor = torch.ones([10, 10])
-                pg.allreduce(tensor).wait()
-                self.assertEqual(torch.full([10, 10], float(self.world_size)), tensor)
-            del pg
-
 
 class DistributedDataParallelTest(
     test_c10d_common.CommonDistributedDataParallelTest, MultiProcessTestCase
@@ -1599,7 +1558,7 @@ class DistributedDataParallelTest(
         """
 
         class GlobalLocalUnusedParamModule(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.t0 = Task()
                 self.t1 = Task()
@@ -1681,7 +1640,7 @@ class DistributedDataParallelTest(
         """
 
         class FindUnusedParamModule(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.t0 = Task()
                 self.t1 = Task()
@@ -1734,7 +1693,7 @@ class DistributedDataParallelTest(
         process_group = self._get_process_group()
 
         class IgnoredOutput(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.fc1 = nn.Linear(2, 10, bias=False)
                 self.fc2 = nn.Linear(10, 4, bias=False)
@@ -1776,7 +1735,7 @@ class DistributedDataParallelTest(
         process_group = self._get_process_group()
 
         class IgnoredOutputWithUnusedParameters(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.fc1 = nn.Linear(2, 10, bias=False)
                 self.fc2 = nn.Linear(10, 4, bias=False)
@@ -1885,7 +1844,7 @@ class DistributedDataParallelTest(
         )
 
         class TestModel(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.fc1 = nn.Linear(2, 10, bias=False)
                 self.fc2 = nn.Linear(10, 4, bias=False)
@@ -2187,7 +2146,7 @@ class DistributedDataParallelTest(
 
 
 class ReducerModule(nn.Module):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.fc1 = nn.Linear(2, 10, bias=False)
         self.fc2 = nn.Linear(10, 4, bias=False)
@@ -2218,7 +2177,6 @@ class ReducerTest(TestCase):
             os.remove(self.file.name)
         except OSError as e:
             print(str(e))
-            pass
 
     @requires_gloo()
     def test_single_dtype_single_bucket(self):
@@ -2505,62 +2463,6 @@ class GlooProcessGroupWithDispatchedCollectivesTests(
         dist.monitored_barrier()
 
 
-class CompilerTest(test_c10d_common.CompilerTest):
-    @property
-    def world_size(self):
-        return 2
-
-    def _get_default_group(self):
-        store = c10d.FileStore(self.file_name, self.world_size)
-        dist.init_process_group(
-            backend="gloo",
-            rank=self.rank,
-            world_size=self.world_size,
-            store=store,
-        )
-        return dist.distributed_c10d._get_default_group()
-
-    def test_allreduce_work_wait_cpu(self):
-        self._test_allreduce_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_allreduce_work_wait_gpu(self):
-        self._test_allreduce_work_wait(torch.ones(2, 2, device=self.rank) * self.rank)
-
-    def test_allgather_work_wait_cpu(self):
-        self._test_allgather_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_allgather_work_wait_gpu(self):
-        self._test_allgather_work_wait(torch.ones(2, 2, device=self.rank) * self.rank)
-
-    def test_broadcast_work_wait_cpu(self):
-        self._test_broadcast_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_broadcast_work_wait_gpu(self):
-        self._test_broadcast_work_wait(torch.ones(2, 2, device=self.rank) * self.rank)
-
-    def test_scatter_work_wait_cpu(self):
-        self._test_scatter_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_scatter_work_wait_gpu(self):
-        self._test_scatter_work_wait(torch.ones(2, 2, device=self.rank) * self.rank)
-
-    def test_nested_comm_tensor_wrapping(self):
-        self._test_nested_comm_tensor_wrapping(torch.ones(2, 2) * self.rank)
-
-    def test_consecutive_comm_work_wait_cpu(self):
-        self._test_consecutive_comm_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_consecutive_comm_work_wait_gpu(self):
-        self._test_consecutive_comm_work_wait(
-            torch.ones(2, 2, device=self.rank) * self.rank
-        )
-
-
 class LargeCommTest(test_c10d_common.AbstractLargeCommTest, MultiProcessTestCase):
     def setUp(self):
         super().setUp()
