@@ -3,6 +3,7 @@
 import datetime
 import os
 import socket
+import struct
 import sys
 import tempfile
 import threading
@@ -16,10 +17,8 @@ import torch.distributed.distributed_c10d as c10d
 import torch.distributed.rpc as rpc
 from torch.distributed import DistError, DistNetworkError, DistStoreError
 from torch.testing._internal.common_distributed import MultiThreadedTestCase
-from torch.testing._internal.common_utils import (
-    instantiate_parametrized_tests,
-    parametrize,
-)
+from torch.testing._internal.common_utils import instantiate_parametrized_tests
+
 
 if not dist.is_available():
     print("torch.distributed not available, skipping tests", file=sys.stderr)
@@ -40,6 +39,7 @@ from torch.testing._internal.common_utils import (
     TestCase,
 )
 
+
 # load_tests from common_utils is used to automatically filter tests for
 # sharding on sandcastle. This line silences flake warnings
 load_tests = load_tests
@@ -278,11 +278,11 @@ class TCPStoreTest(TestCase, StoreTestBase):
         )
 
     def test_address_already_in_use(self):
-        err_msg_reg = "^The server socket has failed to listen on any local "
-        with self.assertRaisesRegex(RuntimeError, err_msg_reg):
-            addr = DEFAULT_HOSTNAME
-            port = common.find_free_port()
+        addr = DEFAULT_HOSTNAME
+        port = common.find_free_port()
 
+        err_msg_reg = f"^The server socket has failed to listen on any local .*{port}"
+        with self.assertRaisesRegex(RuntimeError, err_msg_reg):
             # Use noqa to silence flake8.
             # Need to store in an unused variable here to ensure the first
             # object is not destroyed before the second object is created.
@@ -312,6 +312,30 @@ class TCPStoreTest(TestCase, StoreTestBase):
         self.assertEqual(store1.libuvBackend, self._use_libuv)
         self.assertEqual(store2.libuvBackend, self._use_libuv)
 
+    def test_repr(self) -> None:
+        # server
+        store1 = self._create_store()
+        self.assertRegex(
+            repr(store1),
+            r"TCPStore\("
+            r"client=TCPClient\(SocketImpl\(fd=\d+, addr=\[?localhost\]?:\d+, remote=\[?localhost\]?:\d+\)\), "
+            r"server=TCPServer\(port=\d+\)\)",
+        )
+
+        # client
+        store2 = dist.TCPStore(
+            store1.host,
+            store1.port,
+            world_size=2,
+            is_master=False,
+        )
+        self.assertRegex(
+            repr(store2),
+            r"TCPStore\("
+            r"client=TCPClient\(SocketImpl\(fd=\d+, addr=\[?localhost\]?:\d+, remote=\[?localhost\]?:\d+\)\), "
+            r"server=<nullptr>\)",
+        )
+
     @skip_if_win32()
     @retry_on_connect_failures
     def test_init_pg_and_rpc_with_same_socket(self):
@@ -554,7 +578,7 @@ class PrefixTCPStoreTest(TestCase, StoreTestBase):
 
 
 class MyPythonStore(dist.Store):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.store = {}
 
@@ -710,28 +734,34 @@ class RendezvousTCPTest(TestCase):
     @retry_on_connect_failures(connect_errors=(CONNECT_TIMEOUT, ADDRESS_IN_USE))
     def test_tcp_store_timeout_set(self):
         url = self.create_tcp_url()
-        test_store_timeout = timedelta(seconds=10)
-        gen0 = dist.rendezvous(url + "&rank=0", timeout=test_store_timeout)
+        test_store_timeout = timedelta(seconds=0.1)
+        gen0 = dist.rendezvous(url + "&rank=0", timeout=timedelta(seconds=10))
         store0, rank0, size0 = next(gen0)
-        # this should time out in 10s. If the timeout passed into rendezvous was
+        store0.set_timeout(test_store_timeout)
+        # this should time out in 0.1s. If the timeout passed into rendezvous was
         # not respected, it will take much longer to timeout.
         start = time.time()
-        with self.assertRaisesRegex(RuntimeError, "Timeout"):
+        with self.assertRaisesRegex(
+            DistStoreError, "wait timeout after 100ms, keys: /nonexistant key"
+        ):
             store0.get("nonexistant key")
 
         end = time.time()
         time_diff = end - start
-        self.assertGreater(test_store_timeout.seconds * 10, time_diff)
+        self.assertGreater(10, time_diff)
 
     def test_tcp_store_timeout_doest_break_client(self):
         url = self.create_tcp_url()
-        test_store_timeout = timedelta(seconds=10)
-        gen0 = dist.rendezvous(url + "&rank=0", timeout=test_store_timeout)
+        test_store_timeout = timedelta(seconds=0.1)
+        gen0 = dist.rendezvous(url + "&rank=0", timeout=timedelta(seconds=10))
         store0, rank0, size0 = next(gen0)
+        store0.set_timeout(test_store_timeout)
         # this should time out in 10s. If the timeout passed into rendezvous was
         # not respected, it will take much longer to timeout.
         start = time.time()
-        with self.assertRaisesRegex(RuntimeError, "Timeout"):
+        with self.assertRaisesRegex(
+            DistStoreError, "wait timeout after 100ms, keys: /the_key"
+        ):
             store0.get("the_key")
 
         store0.set("the_key", "x")
@@ -740,7 +770,7 @@ class RendezvousTCPTest(TestCase):
 
         end = time.time()
         time_diff = end - start
-        self.assertGreater(test_store_timeout.seconds * 10, time_diff)
+        self.assertGreater(10, time_diff)
 
     def test_tcp_store_url_with_libuv(self):
         url = self.create_tcp_url()
@@ -750,7 +780,7 @@ class RendezvousTCPTest(TestCase):
 
 
 class DummyStore(dist.Store):
-    def __init__(self):
+    def __init__(self) -> None:
         self.appends = []
         self.multi_sets = []
         self.multi_gets = []
@@ -841,19 +871,11 @@ class TestPythonStore(TestCase):
 
 
 class TestMultiThreadedWait(MultiThreadedTestCase):
-    # TODO (xilunwu): Use less hacky means of instantiating stores.
-    # Note, stores accumulate values per test.
-    stores = [
-        dist.FileStore(tempfile.NamedTemporaryFile(delete=False).name, 1),
-        dist.HashStore(),
-        dist.PrefixStore(
-            "pre", dist.FileStore(tempfile.NamedTemporaryFile(delete=False).name, 1)
-        ),
-        create_tcp_store(use_libuv=False),
-        create_tcp_store(use_libuv=True),
-        dist.PrefixStore("pre", create_tcp_store(use_libuv=False)),
-        dist.PrefixStore("pre", create_tcp_store(use_libuv=True)),
-    ]
+    file_store = dist.FileStore(tempfile.NamedTemporaryFile(delete=False).name, 1)
+    hash_store = dist.HashStore()
+
+    tcp_store = create_tcp_store(use_libuv=False)
+    tcp_store_uv = create_tcp_store(use_libuv=True)
 
     @property
     def world_size(self):
@@ -863,10 +885,7 @@ class TestMultiThreadedWait(MultiThreadedTestCase):
         super().setUp()
         self._spawn_threads()
 
-    # Iterates over self.stores, keep 7 in sync with len(self.stores).
-    @parametrize("i", range(7))
-    def test_wait(self, i):
-        store = self.stores[i]
+    def _test_wait(self, store):
         store.set_timeout(timedelta(seconds=2))
         if dist.get_rank() == 0:
             store.wait(["key1"])
@@ -874,6 +893,39 @@ class TestMultiThreadedWait(MultiThreadedTestCase):
         if dist.get_rank() == 1:
             store.set("key1", "value1")
 
+    def test_wait_hash_store(self):
+        self._test_wait(self.hash_store)
+
+    def test_wait_file_store(self):
+        self._test_wait(self.file_store)
+
+    def test_wait_prefix_file_store(self):
+        store = dist.PrefixStore("pre", self.file_store)
+        self._test_wait(store)
+
+    def _test_wait_tcp_store(self, master_store):
+        store = (
+            master_store
+            if dist.get_rank() == 0
+            else dist.TCPStore(
+                host_name=master_store.host,
+                port=master_store.port,
+                is_master=False,
+                wait_for_workers=False,
+                use_libuv=False,
+            )
+        )
+        self._test_wait(store)
+
+        prefix_store = dist.PrefixStore("pre", store)
+        self._test_wait(prefix_store)
+
+    def test_wait_tcp_store(self):
+        self._test_wait_tcp_store(self.tcp_store)
+
+    def test_wait_tcp_store_uv(self):
+        self._test_wait_tcp_store(self.tcp_store_uv)
+
 
 instantiate_parametrized_tests(TestMultiThreadedWait)
 
@@ -986,6 +1038,46 @@ class InitPgWithNonUvStore(TestCase):
         dist.destroy_process_group()
 
 
+class TestClientProtocol(TestCase):
+    def test_client_connect(self) -> None:
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock.bind(("localhost", 0))
+        port = sock.getsockname()[1]
+
+        def listen() -> None:
+            sock.listen()
+            conn, _ = sock.accept()
+
+            # VALIDATE
+            # 0x3C85F7CE
+            self.assertEqual(conn.recv(5), b"\x00\xce\xf7\x85\x3c")
+
+            # PING
+            data = conn.recv(5)
+            self.assertEqual(data[0], 13)
+            nonce = struct.unpack("i", data[1:])[0]
+            self.assertEqual(nonce, os.getpid())
+
+            # send PING nonce response
+            conn.sendall(data[1:])
+
+            conn.close()
+
+        thread = threading.Thread(target=listen)
+        thread.start()
+
+        store = dist.TCPStore(
+            host_name="localhost",
+            port=port,
+            world_size=2,
+            is_master=False,
+            timeout=timedelta(seconds=2),
+            wait_for_workers=False,
+        )
+
+        thread.join()
+
+
 if __name__ == "__main__":
     assert (
         not torch.cuda._initialized
