@@ -3,12 +3,14 @@
 import collections
 import re
 import sys
+import time
 from io import StringIO
 
 import torch._dynamo.test_case
 import torch._dynamo.testing
 from torch._dynamo.comptime import comptime
 
+
 # Because we don't support free variables in comptime at the moment,
 # we have to communicate via globals.  This also means these tests cannot
 # be run in parallel in a single process (not that you'd... ever want
@@ -95,7 +97,7 @@ s0""",
             """\
 def forward(self, L_x_ : torch.Tensor):
     l_x_ = L_x_
-    y = l_x_ * 2;  l_x_ = None""",
+    y = l_x_ * 2;  l_x_ = y = None""",
         )
 
     def test_print_disas(self):
@@ -158,7 +160,7 @@ def forward(self, L_x_ : torch.Tensor):
         self.assertExpectedInline(
             FILE.getvalue(),
             """\
-- TensorVariable()
+- FakeTensor(..., size=(2,))
 """,
         )
 
@@ -184,8 +186,8 @@ def forward(self, L_x_ : torch.Tensor):
         self.assertExpectedInline(
             FILE.getvalue(),
             """\
-x = TensorVariable()
-y = TensorVariable()
+x = FakeTensor(..., size=(2,))
+y = FakeTensor(..., size=(2,))
 """,
         )
 
@@ -202,6 +204,29 @@ y = TensorVariable()
 
         f(torch.randn(2), torch.randn(2))
 
+    def test_sleep(self):
+        sleep_time = 5
+        cnt = torch._dynamo.testing.CompileCounter()
+
+        @torch._dynamo.optimize(cnt)
+        def f(x, z, should_sleep):
+            if should_sleep:
+                comptime.sleep(sleep_time)
+            y = x * 2
+            return y + 3
+
+        start = time.time()
+        f(torch.randn(2), torch.randn(2), False)
+        total_no_sleep = time.time() - start
+
+        start = time.time()
+        f(torch.randn(2), torch.randn(2), True)
+        total_with_sleep = time.time() - start
+
+        self.assertTrue(total_with_sleep > sleep_time)
+        # Hopefully this won't be flaky
+        self.assertTrue(abs(total_with_sleep - sleep_time - total_no_sleep) < 3)
+
     # Just make sure it doesn't crash
     def test_get_local_closure_variable(self):
         global SELF
@@ -391,7 +416,7 @@ y = TensorVariable()
 def forward(self, L_x_ : torch.Tensor):
     l_x_ = L_x_
     y = l_x_ * 2;  l_x_ = None
-    add = y + 4;  y = None""",
+    add = y + 4;  y = add = None""",
         )
 
 
