@@ -4,19 +4,25 @@ import os
 import sys
 
 import torch
-from torch.fx import symbolic_trace, subgraph_rewriter
+from torch.fx import subgraph_rewriter, symbolic_trace
 from torch.fx.annotate import annotate
+
 # Make the helper files in test/ importable
 from torch.fx.experimental.rewriter import RewritingTracer
 
+
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 sys.path.append(pytorch_test_dir)
 from torch.testing._internal.jit_utils import JitTestCase
 
-if __name__ == '__main__':
-    raise RuntimeError("This test file is not meant to be run directly, use:\n\n"
-                       "\tpython test/test_fx.py TESTNAME\n\n"
-                       "instead.")
+
+if __name__ == "__main__":
+    raise RuntimeError(
+        "This test file is not meant to be run directly, use:\n\n"
+        "\tpython test/test_fx.py TESTNAME\n\n"
+        "instead."
+    )
+
 
 @torch.fx.wrap
 def wrapped_gemm_bias_mul(a, b, bias):
@@ -24,14 +30,15 @@ def wrapped_gemm_bias_mul(a, b, bias):
     mul_res = lin_res * a
     return lin_res, mul_res
 
+
 @torch.fx.wrap
 def wrapped_gemm_bias_mul_with_c(a, b, bias, c):
     lin_res = torch.nn.functional.linear(a, b, bias=bias)
     mul_res = lin_res * c
     return lin_res, mul_res
 
-class TestSubgraphRewriter(JitTestCase):
 
+class TestSubgraphRewriter(JitTestCase):
     def test_subgraph_rewriter_preserves_logic(self):
         class M(torch.nn.Module):
             def forward(self, x):
@@ -110,7 +117,9 @@ class TestSubgraphRewriter(JitTestCase):
 
         x = torch.randn(1, 5)
 
-        matches = subgraph_rewriter.replace_pattern_with_filters(traced, pattern, replacement, [])
+        matches = subgraph_rewriter.replace_pattern_with_filters(
+            traced, pattern, replacement, []
+        )
 
         traced.graph.lint()
 
@@ -297,7 +306,9 @@ class TestSubgraphRewriter(JitTestCase):
         test_outs = traced.forward(x)
         self.assertEqual(ref_outs, test_outs)
 
-    def test_subgraph_rewriter_pattern_output_pattern_node_can_have_users_that_are_not_matched(self):
+    def test_subgraph_rewriter_pattern_output_pattern_node_can_have_users_that_are_not_matched(
+        self,
+    ):
         class M(torch.nn.Module):
             def forward(self, x):
                 y = torch.relu(x)
@@ -326,7 +337,9 @@ class TestSubgraphRewriter(JitTestCase):
         test_outs = traced.forward(x)
         self.assertEqual(ref_outs, test_outs)
 
-    def test_subgraph_rewriter_internal_pattern_nodes_cannot_have_users_that_are_not_matched(self):
+    def test_subgraph_rewriter_internal_pattern_nodes_cannot_have_users_that_are_not_matched(
+        self,
+    ):
         class M(torch.nn.Module):
             def forward(self, x, w1, w2, b1, b2):
                 m0 = torch.cat([w1, w2])
@@ -381,12 +394,13 @@ class TestSubgraphRewriter(JitTestCase):
 
         Here, we want to be able to match the original graph's
         `call_function.add` Node with the pattern graph's
-        `plaeholder.x` Node.
+        `placeholder.x` Node.
 
         Credit to Jerry Zhang (GitHub: jerryzh168) for this test case
         """
+
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.dtype = torch.float16
 
@@ -425,7 +439,7 @@ class TestSubgraphRewriter(JitTestCase):
 
     def test_subgraph_rewriter_replaces_referenced_submodules(self):
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.sigmoid = torch.nn.Sigmoid()
                 self.submod = torch.nn.ReLU()
@@ -435,7 +449,7 @@ class TestSubgraphRewriter(JitTestCase):
                 return self.submod(self.sigmoid(x))
 
         class Pattern(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.sigmoid = torch.nn.Sigmoid()
                 self.submod = torch.nn.ReLU()
@@ -444,7 +458,7 @@ class TestSubgraphRewriter(JitTestCase):
                 return self.submod(self.sigmoid(x))
 
         class Replacement(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.tanh = torch.nn.Tanh()
                 self.submod = torch.nn.ReLU()
@@ -453,7 +467,7 @@ class TestSubgraphRewriter(JitTestCase):
                 return self.submod(self.tanh(x))
 
         class Comparison(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.tanh = torch.nn.Tanh()
                 self.submod = torch.nn.ReLU()
@@ -483,7 +497,6 @@ class TestSubgraphRewriter(JitTestCase):
         self.assertEqual(type(submod), torch.nn.ReLU)
 
     def test_subgraph_rewriter_annotations_int(self):
-
         class M1(torch.nn.Module):
             def forward(self, x):
                 y: int = x
@@ -500,12 +513,11 @@ class TestSubgraphRewriter(JitTestCase):
         module = M2()
         symbolic_traced: torch.fx.GraphModule = symbolic_trace(module)
         for n, m in zip(symbolic_traced.graph.nodes, graph.nodes):
-            if n.op == 'placeholder':
+            if n.op == "placeholder":
                 assert n.type == int
                 assert m.type == int
 
     def test_subgraph_rewriter_replace_consecutive_submodules(self):
-
         def f(x):
             x = torch.sigmoid(x)
             x = torch.sigmoid(x)
@@ -536,7 +548,6 @@ class TestSubgraphRewriter(JitTestCase):
         self.assertEqual(ref_outs, test_outs)
 
     def test_subgraph_rewriter_with_overlapping_matches(self):
-
         def f(x):
             x = torch.sigmoid(x)
             x = torch.sigmoid(x)
@@ -569,7 +580,6 @@ class TestSubgraphRewriter(JitTestCase):
         self.assertEqual(ref_outs, test_outs)
 
     def test_subgraph_rewriter_replace_with_multiple_outputs(self):
-
         def f(x):
             y = torch.sigmoid(x)
             z = torch.relu(x)
@@ -602,7 +612,6 @@ class TestSubgraphRewriter(JitTestCase):
         self.assertEqual(ref_outs, test_outs)
 
     def test_subgraph_rewriter_replace_with_duplicated_outputs(self):
-
         def f(x1, x2):
             x = x1 - x2
             y = torch.sigmoid(x)
@@ -670,7 +679,6 @@ class TestSubgraphRewriter(JitTestCase):
         self.assertEqual(ref_outs, test_outs)
 
     def test_subgraph_rewriter_call_method(self):
-
         class M(torch.nn.Module):
             def forward(self, x):
                 x = x.dequantize()
@@ -701,7 +709,6 @@ class TestSubgraphRewriter(JitTestCase):
         self.assertEqual(ref_outs, test_outs)
 
     def test_subgraph_rewriter_nodes_with_kwargs(self):
-
         class M(torch.nn.Module):
             def __init__(self) -> None:
                 super().__init__()
@@ -737,7 +744,6 @@ class TestSubgraphRewriter(JitTestCase):
         self.assertTrue(found_repalcement_node)
 
     def test_subgraph_rewriter_local_revert(self):
-
         # Following model will have 3 anchors as the matching candidate with the given pattern
         # Anchor 1 and 3 is a real match, but anchor 2 is not.
         # The subgraph rewriter should be able to revert the changes made while matching anchor 2.
@@ -763,9 +769,7 @@ class TestSubgraphRewriter(JitTestCase):
                 # potential match at anchor 1
                 mul_res_1 = in1 * lin_res_2
                 sum_res_1 = mul_res_1 + in1
-                lin_res_3 = torch.nn.functional.linear(
-                    sum_res_1, self.w2, bias=self.b2
-                )
+                lin_res_3 = torch.nn.functional.linear(sum_res_1, self.w2, bias=self.b2)
                 sigmoid_res_1 = torch.sigmoid(lin_res_3)
                 # potential match at anchor 2
                 mul_res_2 = lin_res_3 * sigmoid_res_1
@@ -791,9 +795,8 @@ class TestSubgraphRewriter(JitTestCase):
 
         traced = symbolic_trace(M())
         matches = subgraph_rewriter.replace_pattern(
-            traced,
-            gemm_bias_mul_pattern_with_c,
-            gemm_bias_mul_replacement_with_c)
+            traced, gemm_bias_mul_pattern_with_c, gemm_bias_mul_replacement_with_c
+        )
 
         self.assertEqual(len(matches), 2)
 
@@ -834,7 +837,7 @@ class TestSubgraphRewriter(JitTestCase):
             return x
 
         def second_input_is_scalar(match, original_graph, pattern_graph):
-            """ check the node that's matched to the second input of the pattern graph
+            """check the node that's matched to the second input of the pattern graph
             is a scalar number
             """
             input_idx = 0
@@ -843,24 +846,24 @@ class TestSubgraphRewriter(JitTestCase):
                     if input_idx == 1:
                         num_node = node
                     input_idx += 1
-            if not isinstance(match.nodes_map[num_node], (int, float)):
-                return False
-            return True
+            return isinstance(match.nodes_map[num_node], (int, float))
 
         def check_replacement_nodes(self, traced, matches):
-            replacement_nodes_in_graph = [node for node in traced.graph.nodes if node.target == torch.mul]
+            replacement_nodes_in_graph = [
+                node for node in traced.graph.nodes if node.target == torch.mul
+            ]
             replacement_nodes_in_res = [r for m in matches for r in m.replacements]
-            self.assertEqual(len(replacement_nodes_in_graph), len(replacement_nodes_in_res))
+            self.assertEqual(
+                len(replacement_nodes_in_graph), len(replacement_nodes_in_res)
+            )
             self.assertEqual(replacement_nodes_in_graph, replacement_nodes_in_res)
             return len(replacement_nodes_in_graph)
 
         # match without filter, should find 2 match
         traced = symbolic_trace(M())
         matches = subgraph_rewriter.replace_pattern_with_filters(
-            traced,
-            BinaryOpScalarReLUPattern,
-            BinaryOpScalarReLUReplacement,
-            None)
+            traced, BinaryOpScalarReLUPattern, BinaryOpScalarReLUReplacement, None
+        )
         self.assertEqual(len(matches), 2)
         self.assertEqual(check_replacement_nodes(self, traced, matches), 2)
 
@@ -870,7 +873,8 @@ class TestSubgraphRewriter(JitTestCase):
             traced,
             BinaryOpScalarReLUPattern,
             BinaryOpScalarReLUReplacement,
-            [second_input_is_scalar])
+            [second_input_is_scalar],
+        )
         self.assertEqual(len(matches), 1)
         self.assertEqual(check_replacement_nodes(self, traced, matches), 1)
 
@@ -890,14 +894,17 @@ class TestSubgraphRewriter(JitTestCase):
 
         self.assertEqual(len(matches), 1)
 
-        self.assertExpectedInline(traced.code.strip(), """\
+        self.assertExpectedInline(
+            traced.code.strip(),
+            """\
 def forward(self, x):
     _reshape_alias_copy_default_1 = torch.ops.aten._reshape_alias_copy.default(x, [3, 4], [1, 2]);  x = None
-    return _reshape_alias_copy_default_1""")  # noqa: B950
+    return _reshape_alias_copy_default_1""",
+        )  # noqa: B950
 
     def test_replacement_with_attrs(self):
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.a = torch.tensor([1])
                 self.b = torch.tensor([2])
@@ -906,7 +913,7 @@ def forward(self, x):
                 return x + self.a - self.b
 
         class Pattern(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.a = torch.tensor([1])
 
@@ -914,7 +921,7 @@ def forward(self, x):
                 return x + self.a
 
         class Replacement(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.c = torch.tensor([3])
 
@@ -928,11 +935,15 @@ def forward(self, x):
     def test_matching_variable_arguments(self):
         class M(torch.nn.Module):
             def forward(self, x):
-                return torch.ops.aten.max_pool2d_with_indices.default(x, [2, 2], stride=[2, 2])
+                return torch.ops.aten.max_pool2d_with_indices.default(
+                    x, [2, 2], stride=[2, 2]
+                )
 
         def pattern(x, kernel_size, stride):
             # default padding is [0, 0]
-            return torch.ops.aten.max_pool2d_with_indices.default(x, kernel_size, stride, padding=[0, 0])
+            return torch.ops.aten.max_pool2d_with_indices.default(
+                x, kernel_size, stride, padding=[0, 0]
+            )
 
         traced = symbolic_trace(M())
         matches = subgraph_rewriter.replace_pattern(traced, pattern, pattern)
@@ -951,12 +962,20 @@ def forward(self, x):
             return torch.sub(torch.mul(x, y), y)
 
         traced = symbolic_trace(M())
-        matches = subgraph_rewriter.replace_pattern_with_filters(traced, pattern, replacement)
+        matches = subgraph_rewriter.replace_pattern_with_filters(
+            traced, pattern, replacement
+        )
 
         def check_replacement_nodes(self, traced, matches):
-            replacement_nodes_in_graph = [node for node in traced.graph.nodes if node.target in {torch.sub, torch.mul}]
+            replacement_nodes_in_graph = [
+                node
+                for node in traced.graph.nodes
+                if node.target in {torch.sub, torch.mul}
+            ]
             replacement_nodes_in_res = [r for m in matches for r in m.replacements]
-            self.assertEqual(len(replacement_nodes_in_graph), len(replacement_nodes_in_res))
+            self.assertEqual(
+                len(replacement_nodes_in_graph), len(replacement_nodes_in_res)
+            )
             self.assertEqual(replacement_nodes_in_graph, replacement_nodes_in_res)
             return len(replacement_nodes_in_graph)
 
