@@ -1,7 +1,9 @@
 import sys
+
 import torch
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     script_mod = torch.jit.load(sys.argv[1])
     mod = torch.load(sys.argv[1] + ".orig")
     print(script_mod)
