@@ -19,7 +19,7 @@ class TestUtils(TestCase):
 
     def test_get_fqn_to_example_inputs_simple(self):
         class Sub(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear1 = torch.nn.Linear(5, 5)
                 self.linear2 = torch.nn.Linear(5, 5)
@@ -30,7 +30,7 @@ class TestUtils(TestCase):
                 return x
 
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear1 = torch.nn.Linear(5, 5)
                 self.linear2 = torch.nn.Linear(5, 5)
@@ -57,7 +57,7 @@ class TestUtils(TestCase):
         """ Test that we can get example inputs for functions with default keyword arguments
         """
         class Sub(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear1 = torch.nn.Linear(5, 5)
                 self.linear2 = torch.nn.Linear(5, 5)
@@ -68,7 +68,7 @@ class TestUtils(TestCase):
                 return x
 
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear1 = torch.nn.Linear(5, 5)
                 self.linear2 = torch.nn.Linear(5, 5)
@@ -98,7 +98,7 @@ class TestUtils(TestCase):
         """ Test that we can record complex example inputs such as lists and dicts
         """
         class Sub(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear1 = torch.nn.Linear(5, 5)
                 self.linear2 = torch.nn.Linear(5, 5)
@@ -109,7 +109,7 @@ class TestUtils(TestCase):
                 return x
 
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear1 = torch.nn.Linear(5, 5)
                 self.linear2 = torch.nn.Linear(5, 5)
@@ -191,3 +191,31 @@ class TestUtils(TestCase):
         quantized_tensor = _quantize_weight(float_tensor, observer)
         assert quantized_tensor.int_repr().max().item() == q8_max
         assert quantized_tensor.int_repr().min().item() == q8_min
+
+    def test_uint1_7_dtype(self):
+
+        def up_size(size):
+            return (*size[:-1], size[-1] * 2)
+
+        class UInt4Tensor(torch.Tensor):
+            @staticmethod
+            def __new__(cls, elem, **kwargs):
+                assert elem.dtype is torch.uint8
+                assert not kwargs.get("requires_grad", False)
+                kwargs["requires_grad"] = False
+                return torch.Tensor._make_wrapper_subclass(cls, up_size(elem.shape), dtype=torch.uint4, **kwargs)
+
+            def __init__(self, elem):
+                self.elem = elem
+
+            @classmethod
+            def __torch_dispatch__(cls, func, types, args, kwargs=None):
+                pass
+
+        # make sure it runs
+        x = UInt4Tensor(torch.tensor([
+            [0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF],
+            [0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF],
+            [0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF],
+        ], dtype=torch.uint8))
+        assert x.dtype == torch.uint4
