@@ -57,10 +57,7 @@ class TestArgumentHandler(TestCase):
 
         outputs = {out[0].data_ptr(), out[1].data_ptr(), out[2].data_ptr()}
         self.assertEqual({a.data_ptr()}, argument_handler.dataptrs_read)
-        self.assertEqual(
-            outputs,
-            argument_handler.dataptrs_written,
-        )
+        self.assertEqual(outputs, argument_handler.dataptrs_written)
 
     def test_inplace(self):
         add_inplace_func = torch.ops.aten.add_.Tensor
