@@ -1,54 +1,58 @@
 # Owner(s): ["module: fx"]
 
+import functools
 import math
 import numbers
 import operator
 import pickle
 import sys
+import sympy
 import tempfile
 import unittest
-from typing import Callable, Dict, Union, List, Optional
 from types import BuiltinFunctionType
+from typing import Callable, Dict, List, NamedTuple, Optional, Tuple, Union
 
 import torch
+import torch.fx.experimental.meta_tracer
 import torch.fx.experimental.optimization as optimization
 from torch.fx._symbolic_trace import symbolic_trace
 from torch.fx.experimental import merge_matmul
 from torch.fx.experimental.accelerator_partitioner import Partitioner
-from torch.fx.experimental.normalize import NormalizeOperators, NormalizeArgs
-from torch.fx.passes import graph_manipulation
-from torch.fx.passes.param_fetch import lift_lowering_attrs_to_nodes
+from torch.fx.experimental.normalize import NormalizeArgs, NormalizeOperators
 from torch.fx.experimental.partitioner_utils import (
-    NodeLatency,
-    get_partition_to_latency_mapping,
-    get_latency_of_partitioned_graph,
     Device,
+    get_latency_of_partitioned_graph,
+    get_partition_to_latency_mapping,
+    NodeLatency,
     PartitionerConfig,
     PartitionMode,
 )
 from torch.fx.experimental.rewriter import RewritingTracer
 from torch.fx.experimental.schema_type_annotation import AnnotateTypesWithSchema
-import torch.fx.experimental.meta_tracer
 from torch.fx.graph_module import GraphModule
 from torch.fx.node import Node
 from torch.fx.operator_schemas import (
     _torchscript_type_to_python_type,
+    create_type_hint,
     normalize_function,
     normalize_module,
     type_matches,
-    create_type_hint,
 )
+from torch.fx.passes import graph_manipulation
+from torch.fx.passes.param_fetch import lift_lowering_attrs_to_nodes
 from torch.fx.passes.shape_prop import ShapeProp
 from torch.fx.passes.split_module import split_module
+from torch.fx.passes.annotate_getitem_nodes import annotate_getitem_nodes
 from torch.testing._internal.common_device_type import (
-    ops,
-    onlyCPU,
     instantiate_device_type_tests,
+    onlyCPU,
+    ops,
 )
 from torch.testing._internal.common_methods_invocations import op_db
 from torch.testing._internal.common_nn import module_tests, new_module_tests
-from torch.testing._internal.common_utils import run_tests
+from torch.testing._internal.common_utils import TEST_Z3, run_tests, TestCase
 from torch.testing._internal.jit_utils import JitTestCase
+import torch.utils._pytree as pytree
 
 try:
     import torchvision.models
@@ -117,7 +121,7 @@ class TestFXExperimental(JitTestCase):
 
     def test_large_node_error(self):
         class TestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear = torch.nn.Linear(4, 4)
 
@@ -174,7 +178,7 @@ class TestFXExperimental(JitTestCase):
 
     def test_size_based_partition(self):
         class TestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear = torch.nn.Linear(4, 4)
                 self.c = torch.rand(4)
@@ -206,7 +210,7 @@ class TestFXExperimental(JitTestCase):
 
     def test_partition_device_mapping(self):
         class TestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear = torch.nn.Linear(4, 4)
 
@@ -245,8 +249,8 @@ class TestFXExperimental(JitTestCase):
                     layers.append(torch.nn.ReLU())
                 return layers
 
-            def __init__(self):
-                super(MyRecommendationModule, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 layers = self.create_mlp(4, 4, 4)
                 self.bottom_layers = torch.nn.Sequential(*layers)
                 layers = self.create_mlp(3, 24, 24)
@@ -299,8 +303,8 @@ class TestFXExperimental(JitTestCase):
 
     def test_partition_latency(self):
         class TestModule(torch.nn.Module):
-            def __init__(self):
-                super(TestModule, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.linear = torch.nn.Linear(4, 4)
 
             def forward(self, a):
@@ -356,7 +360,7 @@ class TestFXExperimental(JitTestCase):
 
     def test_cost_aware_partition(self):
         class MyModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear = torch.nn.Linear(4, 4)
 
@@ -418,8 +422,8 @@ class TestFXExperimental(JitTestCase):
 
     def test_aot_based_partition(self):
         class TestModule(torch.nn.Module):
-            def __init__(self):
-                super(TestModule, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.b = torch.rand(4)
                 self.c = torch.rand(4)
 
@@ -477,8 +481,8 @@ class TestFXExperimental(JitTestCase):
 
     def test_saturate_host(self):
         class TestModule(torch.nn.Module):
-            def __init__(self):
-                super(TestModule, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.linear = torch.nn.Linear(4, 4)
 
             def forward(self, a):
@@ -533,8 +537,8 @@ class TestFXExperimental(JitTestCase):
 
     def test_conv_bn_fusion_not_running_state(self):
         class M(torch.nn.Module):
-            def __init__(self):
-                super(M, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.conv = torch.nn.Conv2d(32, 64, 3, stride=2)
                 self.bn = torch.nn.BatchNorm2d(64, eps=1e-05, momentum=0.1, affine=True, track_running_stats=False)
 
@@ -555,6 +559,29 @@ class TestFXExperimental(JitTestCase):
         )
         self.assertEqual(fused(inp), model(inp))
 
+    def test_conv_bn_fusion_mixed_dtype(self):
+        class M(torch.nn.Module):
+            def __init__(self) -> None:
+                super().__init__()
+                self.conv = torch.nn.Conv2d(3, 16, kernel_size=3, stride=1, padding=1, bias=False, dtype=torch.bfloat16)
+                self.bn = torch.nn.BatchNorm2d(16, eps=0.001, momentum=0.1, affine=True, track_running_stats=True)
+
+            def forward(self, x):
+                x = self.conv(x)
+                x = self.bn(x)
+                return x
+
+        model = M().eval()
+
+        traced = symbolic_trace(model)
+        fused = optimization.fuse(traced)
+        inp = torch.randn(1, 3, 64, 64, dtype=torch.bfloat16)
+
+        self.assertTrue(
+            all(not isinstance(m, torch.nn.BatchNorm2d) for m in fused.modules())
+        )
+        self.assertEqual(fused(inp), model(inp))
+
     def test_call_to_assert_no_msg(self):
         class M(torch.nn.Module):
             def forward(self, a, b):
@@ -585,7 +612,7 @@ class TestFXExperimental(JitTestCase):
 
     def test_meta_tracer(self):
         class MetaTracerTestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.emb = torch.nn.Embedding(num_embeddings=42, embedding_dim=16)
                 self.layernorm = torch.nn.LayerNorm(16)
@@ -708,7 +735,7 @@ terrible spacing
 
     def test_subgraph_creation(self):
         class MyModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(3, 4))
                 self.linear = torch.nn.Linear(4, 5)
@@ -761,6 +788,38 @@ terrible spacing
 
         self.assertEqual(orig_out, submodules_out)
 
+    def test_split_module_dead_code(self):
+        class ModWithDeadCode(torch.nn.Module):
+            def forward(self, x):
+                output = x * 2  # we want this
+                dead_line = x + 2  # this is dead
+                return output
+
+        mod = ModWithDeadCode()
+        traced = torch.fx.symbolic_trace(mod)
+
+        # split into before (0), target (1), and after(2)
+        saw_mul = False
+
+        def split_callback(n):
+            nonlocal saw_mul
+            if n.target == operator.mul:
+                saw_mul = True
+                return 1
+
+            if not saw_mul:
+                return 0
+            if saw_mul:
+                return 2
+
+        split = split_module(traced, mod, split_callback)
+
+        x = torch.randn((5,))
+        torch.testing.assert_close(
+            split(x), traced(x)
+        )
+
+
     def test_split_module_kwargs_expansion(self):
         class ModuleWithKwargsExpansion(torch.nn.Module):
             def forward(self, x, **kwargs):
@@ -782,7 +841,7 @@ terrible spacing
 
         x = torch.randn(5, 3)
         foo = torch.randn(5, 3)
-        torch.testing.assert_allclose(split(x, foo=foo), traced(x, foo=foo))
+        torch.testing.assert_close(split(x, foo=foo), traced(x, foo=foo))
 
     @skipIfNoTorchVision
     def test_subgraph_trivial_resnet(self):
@@ -796,7 +855,7 @@ terrible spacing
 
     def test_split_module_default_arg(self):
         class ModelToTrace(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.lin = torch.nn.Linear(512, 512)
 
@@ -814,7 +873,7 @@ terrible spacing
         split = split_module(traced, mtt, lambda node: 0)
 
         x = torch.randn(50, 512)
-        torch.testing.assert_allclose(split(x), traced(x))
+        torch.testing.assert_close(split(x), traced(x))
 
     def test_normalize_binary_operators(self):
         ops_to_test = {
@@ -872,7 +931,7 @@ terrible spacing
             ) -> bool:
                 # `leaves` contains the set of standard `nn.Modules` that are not
                 # currently symbolically traceable. Ideally this set would be empty
-                leaves = set([torch.nn.BatchNorm2d])
+                leaves = {torch.nn.BatchNorm2d}
                 return type(m) in leaves
 
         traced = torch.fx.GraphModule(m, FunctionalTracer().trace(m))
@@ -986,9 +1045,6 @@ class {test_classname}(torch.nn.Module):
 
     def test_normalize_args_preserve_meta(self):
         class MyModule(torch.nn.Module):
-            def __init__(self):
-                super().__init__()
-
             def forward(self, a):
                 return torch.add(a, 3)
 
@@ -1056,7 +1112,7 @@ class {test_classname}(torch.nn.Module):
             ) -> bool:
                 # `leaves` contains the set of standard `nn.Modules` that are not
                 # currently symbolically traceable. Ideally this set would be empty
-                leaves = set([torch.nn.BatchNorm2d])
+                leaves = {torch.nn.BatchNorm2d}
                 return type(m) in leaves
 
         traced_functionals = torch.fx.GraphModule(m, FunctionalTracer().trace(m))
@@ -1080,9 +1136,40 @@ class {test_classname}(torch.nn.Module):
         # Smoke test torchscript compilation since now we're emitting type annotations
         torch.jit.script(traced_functionals_annotated)
 
+    def test_annotate_getitem_node(self):
+        class CustomType:
+            pass
+
+        class CustomNamedTuple(NamedTuple):
+            x: int
+            y: float
+
+        class MyModule(torch.nn.Module):
+            def forward(self, inp: Tuple[CustomType, torch.Tensor], inp2: List[CustomType], inp3: CustomNamedTuple):
+                inp_0 = inp[0]
+                inp_1 = inp[1]
+                inp2_0 = inp2[0]
+                inp3_x = inp3.x
+                inp3_y = inp3.y
+                return inp_0 + inp_1 + inp2_0 + inp3_x + inp3_y
+
+        my_module = MyModule()
+        my_module_traced = torch.fx.symbolic_trace(my_module)
+
+        # by default, fx transform loses type annotation of getitem nodes.
+        for node in my_module_traced.graph.nodes:
+            if node.target == operator.getitem:
+                assert node.type is None
+
+        annotate_getitem_nodes(my_module_traced.graph)
+
+        for node in my_module_traced.graph.nodes:
+            if node.target == operator.getitem:
+                self.assertIsNotNone(node.type, f"Node {node} should be annotated but is not.")
+
     def test_subgraph_uniquename(self):
         class MyModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear = torch.nn.Linear(4, 4)
 
@@ -1112,7 +1199,7 @@ class {test_classname}(torch.nn.Module):
         d_hid = 4
 
         class ExampleCode(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.mm_param = torch.nn.Parameter(torch.randn(d_hid, d_hid))
                 self.mm_param2 = torch.nn.Parameter(torch.randn(d_hid, d_hid))
@@ -1157,14 +1244,14 @@ class {test_classname}(torch.nn.Module):
 
     def test_to_folder(self):
         class Test(torch.nn.Module):
-            def __init__(self):
-                super(Test, self).__init__()
+            def __init__(self) -> None:
+                super().__init__()
                 self.W = torch.nn.Parameter(torch.randn(2))
                 self.seq = torch.nn.Sequential(torch.nn.BatchNorm1d(2, 2))
                 self.linear = torch.nn.Linear(2, 2)
                 self.attr = torch.randn(2)
-                self.register_buffer("attr2", torch.randn(2))
-                self.register_buffer("attr3", torch.ones(2, dtype=torch.int32))
+                self.attr2 = torch.nn.Buffer(torch.randn(2))
+                self.attr3 = torch.nn.Buffer(torch.ones(2, dtype=torch.int32))
 
             def forward(self, x):
                 return self.linear(self.seq(self.W + self.attr + self.attr2 + self.attr3 + x))
@@ -1212,7 +1299,7 @@ class {test_classname}(torch.nn.Module):
         }
 
         class TestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv = torch.nn.Conv2d(3, 3, 2)
                 self.bn = torch.nn.BatchNorm2d(3)
@@ -1361,13 +1448,13 @@ class {test_classname}(torch.nn.Module):
 
     def test_type_matches(self):
         should_be_equal = [
-            (int, type(5)),
-            (numbers.Number, type(5)),
-            (numbers.Number, type(5.0)),
+            (int, int),
+            (numbers.Number, int),
+            (numbers.Number, float),
             (int, type(torch.float)),
-            (Union[int, float], type(5)),
-            (Union[int, float], type(5.0)),
-            (List[int], type(5)),
+            (Union[int, float], int),
+            (Union[int, float], float),
+            (List[int], int),
             (List[int], create_type_hint([int, int])),
             (List[int], create_type_hint((int, int))),
             (List[torch.Tensor], create_type_hint([torch.Tensor, torch.Tensor])),
@@ -1406,7 +1493,7 @@ class {test_classname}(torch.nn.Module):
         import torch.nn as nn
 
         class Foo(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 layers = []
                 layers2 = []
@@ -1479,7 +1566,7 @@ class TestNormalizeOperators(JitTestCase):
     @ops(op_db, allowed_dtypes=(torch.float,))
     def test_normalize_operator_exhaustive(self, device, dtype, op):
         # These ops currently don't trace in FX for various reasons (i.e. they take a list of tensors)
-        fx_fail = {"cat", "stack", "hstack", "vstack", "dstack", "linalg.multi_dot"}
+        fx_fail = {"cat", "stack", "hstack", "vstack", "dstack", "linalg.multi_dot", "_upsample_bilinear2d_aa", "_chunk_cat"}
         sample_inputs_itr = op.sample_inputs(device, dtype, requires_grad=False)
         if isinstance(op.op, torch._ops.OpOverload):
             self.skipTest("normalize operator doesn't work on torch.ops")
@@ -1538,21 +1625,40 @@ class TestNormalizeOperators(JitTestCase):
             param_names = []
             param_values = []
             fx_args = []
-            for idx, v in enumerate(arg_values):
-                if isinstance(v, torch.Tensor):
-                    param_names.append(f"arg_{idx}")
-                    param_values.append(v)
-                    fx_args.append(param_names[-1])
+
+            idx = 0
+
+            def process_arg(arg, name):
+                if isinstance(arg, torch.Tensor):
+                    param_names.append(name)
+                    param_values.append(arg)
+                    return name
                 else:
-                    fx_args.append(f"{repr(v)}")
+                    return f"{repr(arg)}"
+
+            def process_arg_with_idx(arg):
+                nonlocal idx
+                res = process_arg(arg, f"arg_{idx}")
+                idx = idx + 1
+                return res
+
+            def str_arg(arg):
+                if isinstance(arg, tuple):
+                    args = [f"{str_arg(v)}, " for v in arg]
+                    return f"({' '.join(args)})"
+                elif isinstance(arg, list):
+                    args = [f"{str_arg(v)}" for v in arg]
+                    return f"[{', '.join(args)}]"
+                else:
+                    return arg
+
+            for v in arg_values:
+                arg = pytree.tree_map(process_arg_with_idx, v)
+                fx_args.append(str_arg(arg))
 
             for k, v in kwarg_values.items():
-                if isinstance(v, torch.Tensor):
-                    param_names.append(k)
-                    param_values.append(v)
-                    fx_args.append(f"{k} = {k}")
-                else:
-                    fx_args.append(f"{k} = {repr(v)}")
+                arg = pytree.tree_map(functools.partial(process_arg, name=k), v)
+                fx_args.append(f"{k} = {str_arg(arg)}")
 
             code = f"""
 class TestModule(torch.nn.Module):
@@ -1606,7 +1712,7 @@ class TestModule(torch.nn.Module):
                 "include_last_offset",
             },
         )
-        self.assertEqual(norm_args_and_kwargs.args, tuple())
+        self.assertEqual(norm_args_and_kwargs.args, ())
 
     def test_normalize_args_op_overload(self):
         for target in [torch.ops.aten.resize_as_.default, torch.ops.aten.resize_as_]:
@@ -1616,6 +1722,168 @@ class TestModule(torch.nn.Module):
             self.assertIs(kwargs["input"], inp1)
             self.assertIs(kwargs["the_template"], inp2)
 
+
+if TEST_Z3:
+    import z3
+
+    import torch._dynamo.config
+
+    from torch.fx.experimental.validator import SympyToZ3, TranslationValidator, ValidationException, z3str
+    from torch.utils._sympy.functions import FloorDiv, Mod
+
+    class TestTranslationValidation(TestCase):
+        def _prepare_for_translation_validation(self):
+            validator = TranslationValidator()
+
+            # SymPy symbols.
+            s0, s1, s2 = sympy.symbols("s0 s1 s2", integer=True)
+
+            # Z3 symbols.
+            [validator.add_var(s, int) for s in (s0, s1, s2)]
+            z0, z1, z2 = (validator.z3var(s) for s in (s0, s1, s2))
+
+            return (s0, s1, s2), (z0, z1, z2), validator
+
+        def test_sympy_to_z3(self):
+
+            (
+                (s0, s1, s2),
+                (z0, z1, z2),
+                validator,
+            ) = self._prepare_for_translation_validation()
+
+            test_cases = [
+                # Integer constants.
+                (sympy.S.Zero, z3.IntVal(0)),
+                (sympy.S.One, z3.IntVal(1)),
+                (sympy.S.NegativeOne, z3.IntVal(-1)),
+                (sympy.Integer(2), z3.IntVal(2)),
+                (
+                    s0,
+                    z0,
+                ),
+                # Arithmetic operations.
+                *[
+                    (op(s0, s1), op(z0, z1))
+                    for op in (
+                        operator.add,
+                        operator.mul,
+                        operator.pow,
+                    )
+                ],
+                # Logical operations.
+                *[
+                    (sympy_op(s0, s1), z3_op(z0, z1))
+                    for sympy_op, z3_op in (
+                        (sympy.Eq, operator.eq),
+                        (sympy.Ne, operator.ne),
+                        (sympy.Lt, operator.lt),
+                        (sympy.Le, operator.le),
+                        (sympy.Gt, operator.gt),
+                        (sympy.Ge, operator.ge),
+                    )
+                ],
+                # Other operations.
+                (
+                    s0 - s1,
+                    z0 + z3.IntVal(-1) * z1,
+                ),
+                (
+                    s0 / s1,
+                    z3.ToReal(z0) * (z1**-1),
+                ),
+                (FloorDiv(s0, s1), z3.ToInt(z3.ToReal(z0) / z3.ToReal(z1))),
+                (Mod(s0, s1), z0 - z3.ToInt(z3.ToReal(z0) / z3.ToReal(z1)) * z1),
+                (
+                    Mod(s2, (s0 / s1)),
+                    z2
+                    - z3.ToReal(z3.ToInt(z3.ToReal(z2) / (z3.ToReal(z0) * z1**-1)))
+                    * (z3.ToReal(z0) * z1**-1),
+                ),
+                (
+                    Mod(s2, s0**3),
+                    z2 - z3.ToReal(z3.ToInt(z3.ToReal(z2) / z0**3)) * z0**3,
+                ),
+            ]
+
+            toZ3 = SympyToZ3(validator)
+            for sympy_expr, z3_expr in test_cases:
+                result = toZ3.run(sympy_expr)
+                self.assertTrue(
+                    z3_expr.eq(result), msg=f"expected: {z3_expr}. Got: {result}"
+                )
+
+        def test_sat(self):
+            (
+                (s0, s1, s2),
+                (z0, z1, z2),
+                validator,
+            ) = self._prepare_for_translation_validation()
+
+            validator.add_source_expr(z0 > 5)
+            validator.add_source_expr(z1 / 2 > z0)
+
+            # Solutions for target is a subset of the solutions for the source.
+            validator.add_target_expr(s0 > 20)
+            validator.add_target_expr(s1 > s0**2)
+
+            validator.validate()
+
+        def test_unsat(self):
+            (
+                (s0, s1, s2),
+                (z0, z1, z2),
+                validator,
+            ) = self._prepare_for_translation_validation()
+
+            validator.add_source_expr(z0 > 5)
+            validator.add_source_expr(z1 / 2 > z0)
+
+            # Solutions for target is NOT a subset of the solutions for the source.
+            validator.add_target_expr(s0 > 20)
+            # This expression is less restrictive than its counterpart.
+            validator.add_target_expr(s1 > s0 + 2)
+
+            with self.assertRaisesRegex(ValidationException, "translation validation failed."):
+                validator.validate()
+
+        def test_z3str(self):
+            a = z3.Int("a")
+            b = z3.Int("b")
+            special = z3.Real("this.size()[2]")
+
+            test_cases = [
+                (z3.IntVal(42), "42"),
+                # Variable.
+                (a, "a"),
+                # Name with special characters.
+                (special, "this.size()[2]"),
+                # Renamed function fpplications.
+                (a != b, "(!= a b)"),
+                (a ** b, "(pow a b)"),
+                # Chain of associative operations.
+                *[
+                    (op(op(a, 5), b), f"({opstr} 5 a b)")
+                    for op, opstr in [
+                        (operator.add, "+"),
+                        (operator.mul, "*")
+                    ]
+                ],
+                # Revert 'Not' conversions.
+                (a != b, "(!= a b)"),
+                (a < b, "(> b a)"),
+                (a > b, "(> a b)"),
+                # Ignore 'ToInt' and 'ToReal' functions.
+                (z3.ToInt(special) + a, "(+ this.size()[2] a)"),
+                (z3.ToReal(a + b), "(+ a b)"),
+                # Convert to floor division: 'idiv'.
+                (z3.ToInt(z3.ToReal(a) / z3.ToReal(b)), "(idiv a b)"),
+            ]
+
+            for expr, expected in test_cases:
+                self.assertEqual(z3str(expr), expected)
+
+
 instantiate_device_type_tests(TestNormalizeOperators, globals())
 
 if __name__ == "__main__":
