@@ -3,7 +3,6 @@
 # Owner(s): ["module: numpy"]
 
 import sys
-
 from itertools import product
 
 import numpy as np
@@ -17,7 +16,6 @@ from torch.testing._internal.common_device_type import (
     skipMeta,
 )
 from torch.testing._internal.common_dtype import all_types_and_complex_and
-
 from torch.testing._internal.common_utils import run_tests, skipIfTorchDynamo, TestCase
 
 
@@ -329,13 +327,7 @@ class TestNumPyInterop(TestCase):
 
         # list of list or numpy array.
         with self.assertRaisesRegex(ValueError, "expected sequence of length"):
-            torch.tensor(
-                [
-                    [1, 2, 3],
-                    np.random.random(size=(2,)),
-                ],
-                device=device,
-            )
+            torch.tensor([[1, 2, 3], np.random.random(size=(2,))], device=device)
 
     @onlyCPU
     def test_ctor_with_numpy_scalar_ctor(self, device) -> None:
