@@ -1,13 +1,16 @@
 # Owner(s): ["module: unknown"]
 
-import expecttest
 import io
-import numpy as np
 import os
 import shutil
 import sys
 import tempfile
 import unittest
+from pathlib import Path
+
+import expecttest
+import numpy as np
+
 
 TEST_TENSORBOARD = True
 try:
@@ -23,15 +26,6 @@ except ImportError:
     HAS_TORCHVISION = False
 skipIfNoTorchVision = unittest.skipIf(not HAS_TORCHVISION, "no torchvision")
 
-TEST_CAFFE2 = True
-try:
-    import caffe2.python.caffe2_pybind11_state as _caffe2_pybind11_state  # noqa: F401
-    from caffe2.python import brew, cnn, core, workspace
-    from caffe2.python.model_helper import ModelHelper
-except ImportError:
-    TEST_CAFFE2 = False
-skipIfNoCaffe2 = unittest.skipIf(not TEST_CAFFE2, "no caffe2")
-
 TEST_MATPLOTLIB = True
 try:
     import matplotlib
@@ -45,15 +39,16 @@ skipIfNoMatplotlib = unittest.skipIf(not TEST_MATPLOTLIB, "no matplotlib")
 import torch
 from torch.testing._internal.common_utils import (
     instantiate_parametrized_tests,
-    parametrize,
-    TestCase,
-    run_tests,
-    TEST_WITH_ASAN,
-    TEST_WITH_CROSSREF,
-    IS_WINDOWS,
     IS_MACOS,
+    IS_WINDOWS,
+    parametrize,
+    run_tests,
+    TEST_WITH_CROSSREF,
+    TestCase,
+    skipIfTorchDynamo,
 )
 
+
 def tensor_N(shape, dtype=float):
     numel = np.prod(shape)
     x = (np.arange(numel, dtype=dtype)).reshape(shape)
@@ -85,17 +80,16 @@ class BaseTestCase(TestCase):
 
 
 if TEST_TENSORBOARD:
-    from tensorboard.compat.proto.graph_pb2 import GraphDef
-    from torch.utils.tensorboard import summary, SummaryWriter
-    from torch.utils.tensorboard._utils import _prepare_video, convert_to_HWC
-    from tensorboard.compat.proto.types_pb2 import DataType
-    from torch.utils.tensorboard.summary import int_to_half, tensor_proto
-    from torch.utils.tensorboard._convert_np import make_np
-    from torch.utils.tensorboard._pytorch_graph import graph
     from google.protobuf import text_format
     from PIL import Image
-if TEST_TENSORBOARD and TEST_CAFFE2:
-    from torch.utils.tensorboard import _caffe2_graph as c2_graph
+    from tensorboard.compat.proto.graph_pb2 import GraphDef
+    from tensorboard.compat.proto.types_pb2 import DataType
+
+    from torch.utils.tensorboard import summary, SummaryWriter
+    from torch.utils.tensorboard._convert_np import make_np
+    from torch.utils.tensorboard._pytorch_graph import graph
+    from torch.utils.tensorboard._utils import _prepare_video, convert_to_HWC
+    from torch.utils.tensorboard.summary import int_to_half, tensor_proto
 
 class TestTensorBoardPyTorchNumpy(BaseTestCase):
     def test_pytorch_np(self):
@@ -301,9 +295,8 @@ class TestTensorBoardSummaryWriter(BaseTestCase):
         self.assertTrue(passed)
 
     def test_pathlib(self):
-        import pathlib
         with tempfile.TemporaryDirectory(prefix="test_tensorboard_pathlib") as d:
-            p = pathlib.Path(d)
+            p = Path(d)
             with SummaryWriter(p) as writer:
                 writer.add_scalar('test', 1)
 
@@ -527,7 +520,7 @@ class TestTensorBoardPytorchGraph(BaseTestCase):
         dummy_input = (torch.zeros(1, 3),)
 
         class myLinear(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.l = torch.nn.Linear(3, 5)
 
@@ -604,7 +597,7 @@ class TestTensorBoardPytorchGraph(BaseTestCase):
 
     def test_pytorch_graph_dict_input(self):
         class Model(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.l = torch.nn.Linear(3, 5)
 
@@ -612,7 +605,7 @@ class TestTensorBoardPytorchGraph(BaseTestCase):
                 return self.l(x)
 
         class ModelDict(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.l = torch.nn.Linear(3, 5)
 
@@ -647,7 +640,7 @@ class TestTensorBoardPytorchGraph(BaseTestCase):
         # However, it should not raise an error during
         # the add_graph call and still continue.
         class myMLP(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.input_len = 1 * 28 * 28
                 self.fc1 = torch.nn.Linear(self.input_len, 1200)
@@ -754,80 +747,11 @@ class TestTensorBoardNumpy(BaseTestCase):
         res = make_np(np.int64(100000000000))
         self.assertIsInstance(res, np.ndarray) and self.assertEqual(res.shape, (1,))
 
-    @skipIfNoCaffe2
-    def test_caffe2_np(self):
-        workspace.FeedBlob("testBlob", tensor_N(shape=(1, 3, 64, 64)))
-        self.assertIsInstance(make_np('testBlob'), np.ndarray)
-
-    @skipIfNoCaffe2
-    def test_caffe2_np_expect_fail(self):
-        with self.assertRaises(RuntimeError):
-            res = make_np('This_blob_does_not_exist')
-
     def test_pytorch_np_expect_fail(self):
         with self.assertRaises(NotImplementedError):
             res = make_np({'pytorch': 1.0})
 
-    @skipIfNoCaffe2
-    @unittest.skipIf(TEST_WITH_ASAN, "Caffe2 failure with ASAN")
-    def test_caffe2_simple_model(self):
-        model = ModelHelper(name="mnist")
-        # how come those inputs don't break the forward pass =.=a
-        workspace.FeedBlob("data", np.random.randn(1, 3, 64, 64).astype(np.float32))
-        workspace.FeedBlob("label", np.random.randn(1, 1000).astype(int))
 
-        with core.NameScope("conv1"):
-            conv1 = brew.conv(model, "data", 'conv1', dim_in=1, dim_out=20, kernel=5)
-            # Image size: 24 x 24 -> 12 x 12
-            pool1 = brew.max_pool(model, conv1, 'pool1', kernel=2, stride=2)
-            # Image size: 12 x 12 -> 8 x 8
-            conv2 = brew.conv(model, pool1, 'conv2', dim_in=20, dim_out=100, kernel=5)
-            # Image size: 8 x 8 -> 4 x 4
-            pool2 = brew.max_pool(model, conv2, 'pool2', kernel=2, stride=2)
-        with core.NameScope("classifier"):
-            # 50 * 4 * 4 stands for dim_out from previous layer multiplied by the image size
-            fc3 = brew.fc(model, pool2, 'fc3', dim_in=100 * 4 * 4, dim_out=500)
-            relu = brew.relu(model, fc3, fc3)
-            pred = brew.fc(model, relu, 'pred', 500, 10)
-            softmax = brew.softmax(model, pred, 'softmax')
-            xent = model.LabelCrossEntropy([softmax, "label"], 'xent')
-            # compute the expected loss
-            loss = model.AveragedLoss(xent, "loss")
-        model.net.RunAllOnMKL()
-        model.param_init_net.RunAllOnMKL()
-        model.AddGradientOperators([loss], skip=1)
-        blob_name_tracker = {}
-        graph = c2_graph.model_to_graph_def(
-            model,
-            blob_name_tracker=blob_name_tracker,
-            shapes={},
-            show_simplified=False,
-        )
-        compare_proto(graph, self)
-
-    @skipIfNoCaffe2
-    def test_caffe2_simple_cnnmodel(self):
-        model = cnn.CNNModelHelper("NCHW", name="overfeat")
-        workspace.FeedBlob("data", np.random.randn(1, 3, 64, 64).astype(np.float32))
-        workspace.FeedBlob("label", np.random.randn(1, 1000).astype(int))
-        with core.NameScope("conv1"):
-            conv1 = model.Conv("data", "conv1", 3, 96, 11, stride=4)
-            relu1 = model.Relu(conv1, conv1)
-            pool1 = model.MaxPool(relu1, "pool1", kernel=2, stride=2)
-        with core.NameScope("classifier"):
-            fc = model.FC(pool1, "fc", 4096, 1000)
-            pred = model.Softmax(fc, "pred")
-            xent = model.LabelCrossEntropy([pred, "label"], "xent")
-            loss = model.AveragedLoss(xent, "loss")
-
-        blob_name_tracker = {}
-        graph = c2_graph.model_to_graph_def(
-            model,
-            blob_name_tracker=blob_name_tracker,
-            shapes={},
-            show_simplified=False,
-        )
-        compare_proto(graph, self)
 
 class TestTensorProtoSummary(BaseTestCase):
     @parametrize(
@@ -837,6 +761,7 @@ class TestTensorProtoSummary(BaseTestCase):
             (torch.bfloat16, DataType.DT_BFLOAT16),
         ],
     )
+    @skipIfTorchDynamo("Unsuitable test for Dynamo, behavior changes with version")
     def test_half_tensor_proto(self, tensor_type, proto_type):
         float_values = [1.0, 2.0, 3.0]
         actual_proto = tensor_proto(
