@@ -1,10 +1,12 @@
+from __future__ import annotations
+
 import argparse
 import json
 import os
 import xml.etree.ElementTree as ET
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import Any, Dict, Generator, Tuple
+from typing import Any, Generator
 
 from tools.stats.upload_stats_lib import (
     download_s3_artifacts,
@@ -14,13 +16,14 @@ from tools.stats.upload_stats_lib import (
 )
 from tools.stats.upload_test_stats import process_xml_element
 
+
 TESTCASE_TAG = "testcase"
 SEPARATOR = ";"
 
 
 def process_report(
     report: Path,
-) -> Dict[str, Dict[str, int]]:
+) -> dict[str, dict[str, int]]:
     """
     Return a list of disabled tests that should be re-enabled and those that are still
     flaky (failed or skipped)
@@ -36,7 +39,7 @@ def process_report(
     # * Skipped tests from unittest
     #
     # We want to keep track of how many times the test fails (num_red) or passes (num_green)
-    all_tests: Dict[str, Dict[str, int]] = {}
+    all_tests: dict[str, dict[str, int]] = {}
 
     for test_case in root.iter(TESTCASE_TAG):
         parsed_test_case = process_xml_element(test_case)
@@ -116,7 +119,7 @@ def get_test_reports(
         yield from Path(".").glob("**/*.xml")
 
 
-def get_disabled_test_name(test_id: str) -> Tuple[str, str, str, str]:
+def get_disabled_test_name(test_id: str) -> tuple[str, str, str, str]:
     """
     Follow flaky bot convention here, if that changes, this will also need to be updated
     """
@@ -133,7 +136,7 @@ def prepare_record(
     flaky: bool,
     num_red: int = 0,
     num_green: int = 0,
-) -> Tuple[Any, Dict[str, Any]]:
+) -> tuple[Any, dict[str, Any]]:
     """
     Prepare the record to save onto S3
     """
@@ -162,7 +165,7 @@ def prepare_record(
 def save_results(
     workflow_id: int,
     workflow_run_attempt: int,
-    all_tests: Dict[str, Dict[str, int]],
+    all_tests: dict[str, dict[str, int]],
 ) -> None:
     """
     Save the result to S3, so it can go to Rockset
@@ -228,7 +231,7 @@ def main(repo: str, workflow_run_id: int, workflow_run_attempt: int) -> None:
     Find the list of all disabled tests that should be re-enabled
     """
     # Aggregated across all jobs
-    all_tests: Dict[str, Dict[str, int]] = {}
+    all_tests: dict[str, dict[str, int]] = {}
 
     for report in get_test_reports(
         args.repo, args.workflow_run_id, args.workflow_run_attempt
