@@ -1,11 +1,12 @@
 # mypy: allow-untyped-defs
-import torch
 import numpy as np
 
+import torch
 from torch.ao.nn.quantized.modules.utils import WeightedQuantizedModule
 from torch.ao.quantization.experimental.observer import APoTObserver
 from torch.ao.quantization.experimental.quantizer import quantize_APoT
 
+
 class LinearAPoT(WeightedQuantizedModule):
     r"""
     A quantized linear module with quantized tensor as inputs and outputs
@@ -39,9 +40,20 @@ class LinearAPoT(WeightedQuantizedModule):
 
         observer(weight2quantize)
 
-        self.alpha, self.gamma, self.quantization_levels, self.level_indices = observer.calculate_qparams(signed=False)
+        (
+            self.alpha,
+            self.gamma,
+            self.quantization_levels,
+            self.level_indices,
+        ) = observer.calculate_qparams(signed=False)
 
-        quantized_weight = quantize_APoT(weight2quantize, self.alpha, self.gamma, self.quantization_levels, self.level_indices)
+        quantized_weight = quantize_APoT(
+            weight2quantize,
+            self.alpha,
+            self.gamma,
+            self.quantization_levels,
+            self.level_indices,
+        )
         self.weight = quantized_weight.data
         self.weight_transposed = torch.transpose(self.weight, 0, 1)
 
@@ -58,8 +70,8 @@ class LinearAPoT(WeightedQuantizedModule):
         blocks = []
 
         while x:
-            blocks.append(x[0:self.k])
-            x = x[self.k:]
+            blocks.append(x[0 : self.k])
+            x = x[self.k :]
 
         return blocks
 
@@ -94,7 +106,6 @@ class LinearAPoT(WeightedQuantizedModule):
 
         return product
 
-
     def matmul(self, decomposed_weight, activation):
         r"""
         Perform matrix multiplication between decomposed_weight and
@@ -137,20 +148,26 @@ class LinearAPoT(WeightedQuantizedModule):
         weight_rows = self.weight_transposed.size()[0]
         weight_cols = self.weight_transposed.size()[1]
 
-        decomposed_weight: np.ndarray = np.empty(shape=(weight_rows, weight_cols), dtype=object)
+        decomposed_weight: np.ndarray = np.empty(
+            shape=(weight_rows, weight_cols), dtype=object
+        )
         for row in range(weight_rows):
             for col in range(weight_cols):
-                decomposed_weight[row][col] = self.decompose_APoT(bin(self.weight_transposed[row][col]))
+                decomposed_weight[row][col] = self.decompose_APoT(
+                    bin(self.weight_transposed[row][col])
+                )
 
         result = self.matmul(decomposed_weight, activation).type(torch.FloatTensor)
 
         return result
 
     @classmethod
-    def from_reference(cls,  # type: ignore[override]
-                       ref_qlinear,
-                       alpha: torch.Tensor,
-                       gamma: torch.Tensor,
-                       quantization_levels: torch.Tensor,
-                       level_indices: torch.Tensor):
+    def from_reference(  # type: ignore[override]
+        cls,
+        ref_qlinear,
+        alpha: torch.Tensor,
+        gamma: torch.Tensor,
+        quantization_levels: torch.Tensor,
+        level_indices: torch.Tensor,
+    ):
         raise NotImplementedError
