@@ -20,11 +20,13 @@ namespace F = torch::nn::functional;
 static F::PadFuncOptions::mode_t _get_pad_mode_from_conv_padding_mode(
     torch::nn::detail::conv_padding_mode_t conv_padding_mode) {
   F::PadFuncOptions::mode_t pad_mode;
-  if (c10::get_if<torch::enumtype::kReflect>(&conv_padding_mode)) {
+  if (std::holds_alternative<torch::enumtype::kReflect>(conv_padding_mode)) {
     pad_mode = torch::kReflect;
-  } else if (c10::get_if<torch::enumtype::kReplicate>(&conv_padding_mode)) {
+  } else if (std::holds_alternative<torch::enumtype::kReplicate>(
+                 conv_padding_mode)) {
     pad_mode = torch::kReplicate;
-  } else if (c10::get_if<torch::enumtype::kCircular>(&conv_padding_mode)) {
+  } else if (std::holds_alternative<torch::enumtype::kCircular>(
+                 conv_padding_mode)) {
     pad_mode = torch::kCircular;
   } else {
     TORCH_CHECK(
@@ -52,7 +54,7 @@ Conv1dImpl::Conv1dImpl(Conv1dOptions options_)
                      .padding_mode(options_.padding_mode())) {}
 
 Tensor Conv1dImpl::forward(const Tensor& input) {
-  if (!c10::get_if<enumtype::kZeros>(&options.padding_mode())) {
+  if (!std::get_if<enumtype::kZeros>(&options.padding_mode())) {
     return F::detail::conv1d(
         F::pad(
             input,
@@ -91,7 +93,7 @@ Conv2dImpl::Conv2dImpl(Conv2dOptions options_)
                      .padding_mode(options_.padding_mode())) {}
 
 Tensor Conv2dImpl::_conv_forward(const Tensor& input, const Tensor& weight) {
-  if (!c10::get_if<enumtype::kZeros>(&options.padding_mode())) {
+  if (!std::get_if<enumtype::kZeros>(&options.padding_mode())) {
     return F::detail::conv2d(
         F::pad(
             input,
@@ -134,7 +136,7 @@ Conv3dImpl::Conv3dImpl(Conv3dOptions options_)
                      .padding_mode(options_.padding_mode())) {}
 
 Tensor Conv3dImpl::forward(const Tensor& input) {
-  if (!c10::get_if<enumtype::kZeros>(&options.padding_mode())) {
+  if (!std::get_if<enumtype::kZeros>(&options.padding_mode())) {
     return F::detail::conv3d(
         F::pad(
             input,
@@ -167,14 +169,14 @@ template class ConvNdImpl<3, Conv3dImpl>;
 template <size_t D, typename Derived>
 std::vector<int64_t> ConvTransposeNdImpl<D, Derived>::_output_padding(
     const Tensor& input,
-    const c10::optional<at::IntArrayRef>& output_size,
+    const std::optional<at::IntArrayRef>& output_size,
     const ExpandingArray<D>& stride,
     const ExpandingArray<D>& padding,
     const ExpandingArray<D>& kernel_size) {
   std::vector<int64_t> ret;
-  c10::optional<at::IntArrayRef> output_size_ = output_size;
+  std::optional<at::IntArrayRef> output_size_ = output_size;
 
-  if (output_size_ == c10::nullopt) {
+  if (output_size_ == std::nullopt) {
     ret = at::IntArrayRef(this->options.output_padding()).vec();
   } else {
     auto k = input.dim() - 2;
@@ -246,8 +248,8 @@ ConvTranspose1dImpl::ConvTranspose1dImpl(ConvTranspose1dOptions options_)
 
 Tensor ConvTranspose1dImpl::forward(
     const Tensor& input,
-    const c10::optional<at::IntArrayRef>& output_size) {
-  if (!c10::get_if<enumtype::kZeros>(&options.padding_mode())) {
+    const std::optional<at::IntArrayRef>& output_size) {
+  if (!std::get_if<enumtype::kZeros>(&options.padding_mode())) {
     TORCH_CHECK(
         false, "Only `zeros` padding mode is supported for ConvTranspose1d");
   }
@@ -283,8 +285,8 @@ ConvTranspose2dImpl::ConvTranspose2dImpl(ConvTranspose2dOptions options_)
 
 Tensor ConvTranspose2dImpl::forward(
     const Tensor& input,
-    const c10::optional<at::IntArrayRef>& output_size) {
-  if (!c10::get_if<enumtype::kZeros>(&options.padding_mode())) {
+    const std::optional<at::IntArrayRef>& output_size) {
+  if (!std::get_if<enumtype::kZeros>(&options.padding_mode())) {
     TORCH_CHECK(
         false, "Only `zeros` padding mode is supported for ConvTranspose2d");
   }
@@ -320,8 +322,8 @@ ConvTranspose3dImpl::ConvTranspose3dImpl(ConvTranspose3dOptions options_)
 
 Tensor ConvTranspose3dImpl::forward(
     const Tensor& input,
-    const c10::optional<at::IntArrayRef>& output_size) {
-  if (!c10::get_if<enumtype::kZeros>(&options.padding_mode())) {
+    const std::optional<at::IntArrayRef>& output_size) {
+  if (!std::get_if<enumtype::kZeros>(&options.padding_mode())) {
     TORCH_CHECK(
         false, "Only `zeros` padding mode is supported for ConvTranspose3d");
   }
