@@ -12,10 +12,7 @@
 #include <ATen/ATen.h>
 #include <torch/csrc/autograd/generated/Functions.h>
 
-namespace torch {
-namespace autograd {
-namespace generated {
-namespace details {
+namespace torch::autograd::generated::details {
 
 extern const char* kCudnnDoubleBackwardMsg;
 
@@ -34,15 +31,15 @@ struct TORCH_API IndexRangeGenerator {
   size_t i = 0;
 };
 
-TORCH_API Tensor toNonOptFwGrad(const c10::optional<Tensor>& t);
-TORCH_API Tensor toNonOptPrimal(const c10::optional<Tensor>& t);
-TORCH_API Tensor toNonOptTensor(const c10::optional<Tensor>& t);
+TORCH_API Tensor toNonOptFwGrad(const std::optional<Tensor>& t);
+TORCH_API Tensor toNonOptPrimal(const std::optional<Tensor>& t);
+TORCH_API Tensor toNonOptTensor(const std::optional<Tensor>& t);
 
-TORCH_API inline c10::optional<Tensor> wrap_opt_if(
+TORCH_API inline std::optional<Tensor> wrap_opt_if(
     const Tensor& t,
     const bool cond) {
-  using OptTensor = c10::optional<Tensor>;
-  return cond ? OptTensor(t) : static_cast<OptTensor>(c10::nullopt);
+  using OptTensor = std::optional<Tensor>;
+  return cond ? OptTensor(t) : static_cast<OptTensor>(std::nullopt);
 }
 
 TORCH_API Tensor
@@ -78,26 +75,26 @@ Tensor scale_grad_by_count(
 at::Tensor norm_backward(
     const at::Tensor& grad,
     const at::Tensor& self,
-    const optional<at::Scalar>& p_,
+    const std::optional<at::Scalar>& p_,
     const at::Tensor& norm);
 at::Tensor norm_backward(
     at::Tensor grad,
     const at::Tensor& self,
-    const optional<at::Scalar>& p_,
+    const std::optional<at::Scalar>& p_,
     at::Tensor norm,
     at::IntArrayRef dim,
     bool keepdim);
 Tensor norm_jvp(
     const Tensor& self_p,
     const Tensor& self_t,
-    const optional<Scalar>& p_,
+    const std::optional<Scalar>& p_,
     Tensor norm,
     IntArrayRef dim,
     bool keepdim);
 Tensor norm_jvp(
     const Tensor& grad,
     const Tensor& self,
-    const optional<Scalar>& p_,
+    const std::optional<Scalar>& p_,
     Tensor norm);
 Tensor _nested_from_padded_backward(
     const Tensor& grad,
@@ -127,38 +124,44 @@ at::Tensor pow_backward(
     const at::Tensor& self,
     const at::Scalar& exponent_);
 at::Tensor pow_backward_self(
-    at::Tensor grad,
+    const at::Tensor& grad,
     const at::Tensor& self,
     const at::Tensor& exponent);
 at::Tensor pow_backward_exponent(
-    at::Tensor grad,
+    const at::Tensor& grad,
     const at::Tensor& self,
     const at::Tensor& exponent,
-    at::Tensor result);
+    const at::Tensor& result);
 at::Tensor pow_backward_exponent(
-    at::Tensor grad,
+    const at::Tensor& grad,
     const at::Scalar& base,
     const at::Tensor& exponent,
-    at::Tensor result);
-at::Tensor angle_backward(at::Tensor grad, const at::Tensor& self);
+    const at::Tensor& result);
+at::Tensor angle_backward(const at::Tensor& grad, const at::Tensor& self);
 template <typename T>
-at::Tensor mul_tensor_backward(Tensor grad, T other, ScalarType self_st);
-template <typename T>
-at::Tensor div_tensor_self_backward(Tensor grad, T other, ScalarType self_st);
-at::Tensor div_tensor_other_backward(Tensor grad, Tensor self, Tensor other);
+at::Tensor mul_tensor_backward(const Tensor& grad, T other, ScalarType self_st);
 template <typename T>
 at::Tensor div_tensor_self_backward(
-    Tensor grad,
+    const Tensor& grad,
+    T other,
+    ScalarType self_st);
+at::Tensor div_tensor_other_backward(
+    const Tensor& grad,
+    const Tensor& self,
+    const Tensor& other);
+template <typename T>
+at::Tensor div_tensor_self_backward(
+    const Tensor& grad,
     T other,
     ScalarType self_st,
-    const c10::optional<c10::string_view>& rounding_mode);
+    const std::optional<c10::string_view>& rounding_mode);
 at::Tensor div_tensor_other_backward(
-    Tensor grad,
-    Tensor self,
-    Tensor other,
-    const c10::optional<c10::string_view>& rounding_mode);
+    const Tensor& grad,
+    const Tensor& self,
+    const Tensor& other,
+    const std::optional<c10::string_view>& rounding_mode);
 at::Tensor mvlgamma_backward(
-    at::Tensor grad,
+    const at::Tensor& grad,
     const at::Tensor& self,
     int64_t p);
 at::Tensor permute_backwards(const at::Tensor& grad, at::IntArrayRef fwd_dims);
@@ -226,16 +229,30 @@ at::Tensor logsumexp_jvp(
     const at::Tensor& self_t,
     IntArrayRef dim,
     bool keepdim);
+at::Tensor safe_logsumexp_jvp(
+    const at::Tensor& self_p,
+    const at::Tensor& self_t,
+    IntArrayRef dim,
+    bool keepdim);
 at::Tensor logcumsumexp_backward(
     at::Tensor grad,
     const at::Tensor& self,
-    at::Tensor result,
+    const at::Tensor& result,
     int64_t dim);
 at::Tensor logcumsumexp_jvp(
     const at::Tensor& self_p,
     const at::Tensor& self_t,
     int64_t dim);
 at::Tensor unbind_backward(const variable_list& grads, int64_t dim);
+at::Tensor unbind_backward_nested(
+    const variable_list& grads,
+    const Tensor& nt_sizes,
+    int64_t dim,
+    const at::TensorOptions& options);
+at::Tensor unbind_backward_nested_jagged(
+    const variable_list& grads,
+    const Tensor& self,
+    int64_t dim);
 at::Tensor unsqueeze_to(const at::Tensor& self, c10::SymIntArrayRef sym_sizes);
 at::Tensor unsqueeze_to(
     const at::Tensor& self,
@@ -261,8 +278,8 @@ std::vector<at::Tensor> block_diag_backward(
 at::Tensor clamp_backward(
     const at::Tensor& grad,
     const at::Tensor& self,
-    const optional<at::Scalar>& min,
-    const optional<at::Scalar>& max);
+    const std::optional<at::Scalar>& min,
+    const std::optional<at::Scalar>& max);
 at::Tensor clamp_backward(
     const at::Tensor& grad,
     const at::Tensor& self,
@@ -306,8 +323,8 @@ at::Tensor mm_mat1_sparse_backward(
 std::tuple<Tensor, Tensor, Tensor> sparse_sampled_addmm_backward(
     const Tensor& grad,
     const Tensor& self,
-    const c10::optional<Tensor>& mat1,
-    const c10::optional<Tensor>& mat2,
+    const std::optional<Tensor>& mat1,
+    const std::optional<Tensor>& mat2,
     const Scalar& alpha,
     const Scalar& beta,
     const std::array<bool, 3>& grad_input_mask);
@@ -337,8 +354,8 @@ at::Tensor repeat_backward(
     at::SymIntArrayRef repeats,
     at::SymIntArrayRef input_shape);
 at::Tensor _fused_dropout_backward(
-    at::Tensor grad,
-    at::Tensor mask,
+    const at::Tensor& grad,
+    const at::Tensor& mask,
     double p1m);
 at::Tensor infinitely_differentiable_native_dropout_backward(
     const at::Tensor& grad,
@@ -350,7 +367,7 @@ at::Tensor native_dropout_double_backward(
     const at::Tensor& mask,
     double scale);
 at::Tensor evenly_distribute_backward(
-    at::Tensor grad,
+    const at::Tensor& grad,
     const at::Tensor& input,
     const at::Tensor& value);
 Tensor sgn_backward(const Tensor& x, const Tensor& gx, const Tensor& sgn);
@@ -359,21 +376,21 @@ at::Tensor var_backward(
     at::Tensor grad,
     const at::Tensor& self,
     at::OptionalIntArrayRef dim,
-    const c10::optional<c10::Scalar>& correction,
+    const std::optional<c10::Scalar>& correction,
     bool keepdim);
 at::Tensor var_jvp(
     const at::Tensor& self_t,
     const at::Tensor& self_p,
     const at::Tensor& result,
     at::OptionalIntArrayRef dim_opt,
-    const c10::optional<c10::Scalar>& correction,
+    const std::optional<c10::Scalar>& correction,
     bool keepdim);
 at::Tensor std_backward(
     const at::Tensor& result,
     const at::Tensor& grad,
     const at::Tensor& self,
     at::OptionalIntArrayRef dim,
-    const c10::optional<c10::Scalar>& correction,
+    const std::optional<c10::Scalar>& correction,
     bool keepdim);
 Tensor mean_backward(
     const Tensor& grad,
@@ -386,7 +403,7 @@ Tensor var_mean_backward(
     const Tensor& gmean,
     const Tensor& self,
     at::OptionalIntArrayRef dim_opt,
-    const c10::optional<c10::Scalar>& correction,
+    const std::optional<c10::Scalar>& correction,
     bool keepdim);
 Tensor std_mean_backward(
     const Tensor& gstd,
@@ -394,12 +411,8 @@ Tensor std_mean_backward(
     const Tensor& self,
     const Tensor& std,
     at::OptionalIntArrayRef dim_opt,
-    const c10::optional<c10::Scalar>& correction,
+    const std::optional<c10::Scalar>& correction,
     bool keepdim);
-at::Tensor masked_scatter_backward(
-    const at::Tensor& grad,
-    const at::Tensor& mask,
-    c10::SymIntArrayRef sizes);
 at::Tensor cholesky_backward(
     const at::Tensor& grad,
     bool upper,
@@ -409,10 +422,10 @@ at::Tensor cholesky_jvp(
     const at::Tensor& L,
     bool upper);
 at::Tensor cholesky_inverse_backward(
-    at::Tensor grad,
-    at::Tensor L,
+    const at::Tensor& grad,
+    const at::Tensor& L,
     bool upper,
-    at::Tensor inverse);
+    const at::Tensor& inverse);
 at::Tensor cholesky_inverse_jvp(
     const at::Tensor& F,
     const at::Tensor& dF,
@@ -420,6 +433,11 @@ at::Tensor cholesky_inverse_jvp(
     bool upper);
 Tensor pinv_jvp(const Tensor& A, const Tensor& pinvA, const Tensor& dA);
 Tensor pinv_backward(const Tensor& grad, const Tensor& pinvA, const Tensor& A);
+Tensor chunk_backward_nested(
+    const std::vector<torch::autograd::Variable>& grads,
+    const Tensor& self,
+    int64_t chunks,
+    int64_t dim);
 at::Tensor split_with_sizes_backward(
     const std::vector<torch::autograd::Variable>& grads,
     c10::SymIntArrayRef split_sizes,
@@ -430,10 +448,11 @@ at::Tensor _nested_split_with_sizes_backward(
     const std::vector<torch::autograd::Variable>& grads,
     c10::SymIntArrayRef split_sizes,
     int64_t dim,
-    const Tensor& self);
+    const Tensor& nt_sizes,
+    const at::TensorOptions& options);
 at::Tensor split_backward(
     const std::vector<torch::autograd::Variable>& grads,
-    c10::SymInt split_size,
+    const c10::SymInt& split_size,
     int64_t dim,
     c10::SymIntArrayRef sizes,
     const at::TensorOptions& options);
@@ -460,33 +479,33 @@ at::Tensor infinitely_differentiable_mish_backward(
 Tensor infinitely_differentiable_logit_backward(
     const Tensor& grad,
     const Tensor& self,
-    c10::optional<double> eps);
+    std::optional<double> eps);
 Tensor binary_cross_entropy_target_backward(
     const Tensor& grad,
     const Tensor& self,
     const Tensor& target,
-    const c10::optional<Tensor>& weight,
+    const std::optional<Tensor>& weight,
     int64_t reduction);
 Tensor binary_cross_entropy_double_backward_target(
     const Tensor& grad,
     const Tensor& grad_output,
     const Tensor& self,
     const Tensor& target,
-    const c10::optional<Tensor>& weight,
+    const std::optional<Tensor>& weight,
     int64_t reduction);
 Tensor binary_cross_entropy_with_logits_backward(
     const Tensor& grad,
     const Tensor& input,
     const Tensor& target,
-    const c10::optional<Tensor>& weight_opt,
-    const c10::optional<Tensor>& pos_weight_opt,
+    const std::optional<Tensor>& weight_opt,
+    const std::optional<Tensor>& pos_weight_opt,
     int64_t reduction);
 at::Tensor binary_cross_entropy_with_logits_target_backward(
     const at::Tensor& grad_output,
     const at::Tensor& self,
     const at::Tensor& target,
-    const c10::optional<at::Tensor>& weight,
-    const c10::optional<at::Tensor>& pos_weight,
+    const std::optional<at::Tensor>& weight,
+    const std::optional<at::Tensor>& pos_weight,
     int64_t reduction);
 at::Tensor log_sigmoid_double_backward(
     const at::Tensor& grad,
@@ -501,13 +520,13 @@ at::Tensor binary_cross_entropy_double_backward(
     const at::Tensor& grad,
     const at::Tensor& input,
     const at::Tensor& target,
-    const c10::optional<at::Tensor>& weight,
+    const std::optional<at::Tensor>& weight,
     int64_t reduction);
 at::Tensor binary_cross_entropy_double_backward_grad_output(
     const at::Tensor& grad,
     const at::Tensor& input,
     const at::Tensor& target,
-    const c10::optional<at::Tensor>& weight,
+    const std::optional<at::Tensor>& weight,
     int64_t reduction);
 at::Tensor smooth_l1_loss_double_backward(
     const at::Tensor& grad,
@@ -569,10 +588,10 @@ at::Tensor sparse_constructor_values_backward(
 at::Tensor embedding_dense_double_backward_symint(
     const at::Tensor& grad,
     const at::Tensor& indices,
-    c10::SymInt padding_idx);
+    const c10::SymInt& padding_idx);
 at::Tensor index_backward(
     at::Tensor zeros_like_self,
-    const torch::List<c10::optional<Tensor>>& indices,
+    const torch::List<std::optional<Tensor>>& indices,
     const at::Tensor& grad);
 at::Tensor _cudnn_ctc_loss_backward(
     const at::Tensor& grad_out,
@@ -606,8 +625,8 @@ Tensor slice_backward_wrapper(
     const at::Tensor& grad,
     const c10::SymIntArrayRef& input_sizes,
     int64_t dim,
-    c10::optional<c10::SymInt> start,
-    c10::optional<c10::SymInt> end,
+    std::optional<c10::SymInt> start,
+    std::optional<c10::SymInt> end,
     c10::SymInt step);
 std::tuple<Tensor, Tensor> linalg_eig_jvp(
     const Tensor& dA,
@@ -662,9 +681,9 @@ std::tuple<Tensor, Tensor> linalg_solve_triangular_backward(
     std::array<bool, 2> output_mask);
 std::tuple<Tensor, Tensor, Tensor> _trilinear_backward(
     const Tensor& grad_out,
-    const c10::optional<Tensor>& i1,
-    const c10::optional<Tensor>& i2,
-    const c10::optional<Tensor>& i3,
+    const std::optional<Tensor>& i1,
+    const std::optional<Tensor>& i2,
+    const std::optional<Tensor>& i3,
     IntArrayRef expand1,
     IntArrayRef expand2,
     IntArrayRef expand3,
@@ -687,17 +706,17 @@ Tensor linalg_matrix_exp_differential(
     bool adjoint);
 std::tuple<Tensor, Tensor, Tensor> batchnorm_double_backward(
     const Tensor& input,
-    const c10::optional<Tensor>& gamma,
+    const std::optional<Tensor>& gamma,
     const Tensor& ggI,
     const Tensor& ggG,
     const Tensor& ggB,
     const Tensor& gO,
-    const c10::optional<Tensor>& running_mean,
-    const c10::optional<Tensor>& running_var,
+    const std::optional<Tensor>& running_mean,
+    const std::optional<Tensor>& running_var,
     bool training,
     double eps,
-    const c10::optional<Tensor>& save_mean,
-    const c10::optional<Tensor>& save_invstd,
+    const std::optional<Tensor>& save_mean,
+    const std::optional<Tensor>& save_invstd,
     std::array<bool, 3> output_mask);
 std::tuple<Tensor, Tensor> _euclidean_dist_backward(
     const Tensor& grad,
@@ -720,7 +739,7 @@ Tensor fft_r2c_backward(
     at::IntArrayRef dim,
     int64_t normalization,
     bool onesided,
-    c10::SymInt last_dim_size);
+    const c10::SymInt& last_dim_size);
 Tensor fft_c2r_backward(
     const Tensor& grad,
     IntArrayRef dim,
@@ -731,7 +750,8 @@ std::tuple<Tensor, Tensor> cholesky_solve_backward(
     const Tensor& self,
     const Tensor& input2,
     const Tensor& result,
-    const bool upper);
+    const bool upper,
+    std::array<bool, 2> output_mask);
 Tensor cholesky_solve_jvp(
     const Tensor& X,
     const Tensor& U,
@@ -746,9 +766,9 @@ infinitely_differentiable_native_group_norm_backward(
     const Tensor& X,
     const Tensor& mean,
     const Tensor& rstd,
-    const c10::optional<Tensor>& gamma,
+    const std::optional<Tensor>& gamma,
     c10::SymInt N,
-    c10::SymInt C,
+    const c10::SymInt& C,
     c10::SymInt HxW,
     int64_t group,
     double eps,
@@ -763,14 +783,14 @@ Tensor as_strided_backward(
     const TensorGeometry& input_geometry,
     c10::SymIntArrayRef sizes,
     c10::SymIntArrayRef strides,
-    optional<c10::SymInt> storage_offset_);
+    const std::optional<c10::SymInt>& storage_offset_);
 Tensor as_strided_scatter_backward(
-    Tensor grad,
+    const Tensor& grad,
     const TensorGeometry& input_geometry,
-    TensorGeometry src_geometry,
+    const TensorGeometry& src_geometry,
     c10::SymIntArrayRef sizes,
     c10::SymIntArrayRef strides,
-    optional<c10::SymInt> storage_offset);
+    std::optional<c10::SymInt> storage_offset);
 std::tuple<Tensor, Tensor> atan2_backward(
     const Tensor& grad,
     const Tensor& self,
@@ -784,7 +804,7 @@ Tensor amaxamin_jvp(
     bool keepdim);
 std::tuple<Tensor, Tensor, Tensor> layer_norm_double_backward(
     const Tensor& input,
-    const c10::optional<Tensor>& gamma,
+    const std::optional<Tensor>& gamma,
     const Tensor& ggI,
     const Tensor& ggG,
     const Tensor& ggB,
@@ -860,8 +880,8 @@ Tensor linalg_solve_jvp(
 Tensor lu_unpack_backward(
     const Tensor& L_grad,
     const Tensor& U_grad,
-    const c10::SymInt m,
-    const c10::SymInt n);
+    const c10::SymInt& m,
+    const c10::SymInt& n);
 
 Tensor linalg_det_backward(
     const Tensor& grad,
@@ -913,8 +933,8 @@ Tensor batch_norm_jvp(
     const Tensor& weight_t,
     const Tensor& bias_p,
     const Tensor& bias_t,
-    const c10::optional<Tensor>& running_mean,
-    const c10::optional<Tensor>& running_var,
+    const std::optional<Tensor>& running_mean,
+    const std::optional<Tensor>& running_var,
     const Tensor& saved_mean,
     const Tensor& saved_invstd,
     bool train,
@@ -959,12 +979,12 @@ Tensor convolution_jvp(
     const Tensor& weight_t,
     const Tensor& bias_p,
     const Tensor& bias_t,
-    IntArrayRef stride,
+    at::SymIntArrayRef stride,
     at::SymIntArrayRef padding,
-    IntArrayRef dilation,
+    at::SymIntArrayRef dilation,
     bool transposed,
     at::SymIntArrayRef output_padding,
-    int64_t groups);
+    const c10::SymInt& groups);
 
 Tensor _convolution_jvp(
     const Tensor& input_p,
@@ -973,12 +993,12 @@ Tensor _convolution_jvp(
     const Tensor& weight_t,
     const Tensor& bias_p,
     const Tensor& bias_t,
-    IntArrayRef stride,
+    at::SymIntArrayRef stride,
     at::SymIntArrayRef padding,
-    IntArrayRef dilation,
+    at::SymIntArrayRef dilation,
     bool transposed,
     at::SymIntArrayRef output_padding,
-    int64_t groups,
+    const c10::SymInt& groups,
     bool benchmark,
     bool deterministic,
     bool cudnn_enabled,
@@ -988,10 +1008,14 @@ Tensor convolution_backward_jvp_grad_bias(
     const Tensor& grad_out_t,
     const Tensor& grad_bias);
 
-Tensor cat_jvp(at::ITensorListRef tensors, int64_t dim);
+Tensor cat_jvp(const at::ITensorListRef& tensors, int64_t dim);
 Tensor block_diag_jvp(at::TensorList tensors);
 Tensor stack_jvp(at::TensorList tensors, int64_t dim);
-Tensor cumprod_jvp(Tensor self_t, Tensor self_p, Tensor result, int dim);
+Tensor cumprod_jvp(
+    const Tensor& self_t,
+    const Tensor& self_p,
+    const Tensor& result,
+    int dim);
 Tensor gather_with_keepdimed_indices(
     const Tensor& input,
     int64_t dim,
@@ -1007,12 +1031,12 @@ std::tuple<Tensor, Tensor> _cudnn_convolution_backward(
     const at::Tensor& self,
     const at::Tensor& grad_output,
     const at::Tensor& weight,
-    at::IntArrayRef padding,
-    at::IntArrayRef output_padding,
-    at::IntArrayRef stride,
-    at::IntArrayRef dilation,
+    at::SymIntArrayRef padding,
+    at::SymIntArrayRef output_padding,
+    at::SymIntArrayRef stride,
+    at::SymIntArrayRef dilation,
     bool transposed,
-    int64_t groups,
+    c10::SymInt groups,
     ::std::array<bool, 2> output_mask);
 
 Tensor scatter_reduce_jvp(
@@ -1072,9 +1096,9 @@ mkldnn_rnn_layer_differentiable_backward(
     const Tensor& output,
     const Tensor& hy_,
     const Tensor& cy_,
-    const c10::optional<Tensor>& grad_output_r_opt,
-    const c10::optional<Tensor>& grad_hy_r_opt,
-    const c10::optional<Tensor>& grad_cy_r_opt,
+    const std::optional<Tensor>& grad_output_r_opt,
+    const std::optional<Tensor>& grad_hy_r_opt,
+    const std::optional<Tensor>& grad_cy_r_opt,
     bool reverse,
     int64_t mode,
     int64_t hidden_size,
@@ -1086,7 +1110,6 @@ mkldnn_rnn_layer_differentiable_backward(
     bool batch_first,
     const at::Tensor& workspace);
 
-} // namespace details
-} // namespace generated
-} // namespace autograd
-} // namespace torch
+Tensor values_backward(const Tensor& grad, const Tensor& self);
+
+} // namespace torch::autograd::generated::details
