@@ -9,8 +9,7 @@
 #include <string>
 #include <vector>
 
-namespace torch {
-namespace autograd {
+namespace torch::autograd {
 
 struct TORCH_API Error : public Node {
   Error(std::string msg, edge_list&& next_edges)
@@ -44,10 +43,8 @@ struct TORCH_API NotImplemented : public Error {
 // Identity in forward, Error in backward. Used to implement
 // @once_differentiable
 struct TORCH_API DelayedError : public Node {
-  DelayedError(std::string msg, int num_inputs) : msg(std::move(msg)) {
-    // NOLINTNEXTLINE(clang-analyzer-deadcode.DeadStores)
-    for (const auto i : c10::irange(num_inputs)) {
-      (void)i; // Suppress unused variable warning
+  DelayedError(std::string msg, int64_t num_inputs) : msg(std::move(msg)) {
+    for (const auto _ [[maybe_unused]] : c10::irange(num_inputs)) {
       add_input_metadata(Node::undefined_input());
     }
   }
@@ -107,5 +104,4 @@ struct TORCH_API Identity : public Node {
   variable_list apply(variable_list&& inputs) override;
 };
 
-} // namespace autograd
-} // namespace torch
+} // namespace torch::autograd
