@@ -3,12 +3,11 @@
 #include <torch/csrc/autograd/profiler_kineto.h>
 
 #include <c10/macros/Export.h>
-#include <c10/util/C++17.h>
+#include <c10/util/ApproximateClock.h>
 #include <c10/util/Exception.h>
 #include <c10/util/flat_hash_map.h>
 #include <c10/util/irange.h>
 #include <c10/util/overloaded.h>
-#include <c10/util/variant.h>
 
 #include <torch/csrc/profiler/api.h>
 #include <torch/csrc/profiler/collection.h>
@@ -19,17 +18,16 @@
 #include <torch/csrc/profiler/perf.h>
 #include <torch/csrc/profiler/standalone/itt_observer.h>
 #include <torch/csrc/profiler/standalone/nvtx_observer.h>
+#include <torch/csrc/profiler/standalone/privateuse1_observer.h>
 #include <torch/csrc/profiler/util.h>
 
 #include <ATen/Context.h>
 
-#include <deque>
-#include <limits>
-#include <sstream>
 #include <stdexcept>
 #include <utility>
 
 #ifdef USE_KINETO
+#include <ApproximateClock.h>
 #include <libkineto.h>
 #include <time_since_epoch.h>
 
@@ -50,15 +48,14 @@ __attribute__((weak)) int acc_get_device_type() {
 #endif // USE_KINETO
 
 namespace torch {
-namespace autograd {
-namespace profiler {
+namespace autograd::profiler {
 
 namespace {
-inline int64_t getTimeUs() {
+inline int64_t getTimeNs() {
 #ifdef USE_KINETO
   return libkineto::timeSinceEpoch(std::chrono::system_clock::now());
 #else
-  return torch::profiler::impl::getTime() / 1000;
+  return c10::getTime();
 #endif // USE_KINETO
 }
 
@@ -67,77 +64,102 @@ using torch::profiler::impl::EventType;
 using torch::profiler::impl::ExtraFields;
 using torch::profiler::impl::get_record_concrete_inputs_enabled;
 using torch::profiler::impl::ivalueListToStr;
+using torch::profiler::impl::ivalueToStr;
 using torch::profiler::impl::op_input_t;
 using torch::profiler::impl::ProfilerStateBase;
 using torch::profiler::impl::PyExtraFieldsBase;
 using torch::profiler::impl::Result;
+using torch::profiler::impl::shape;
 using torch::profiler::impl::shapesToStr;
 using torch::profiler::impl::stacksToStr;
 using torch::profiler::impl::strListToStr;
 using torch::profiler::impl::TensorMetadata;
+using torch::profiler::impl::variantShapesToStr;
 
 struct OpArgData {
-  bool has_data;
-  std::vector<std::vector<int64_t>> shapes;
+  bool hasData;
+  std::vector<shape> shapes;
   std::vector<std::string> dtypes;
-  std::vector<c10::IValue> concrete_inputs;
+  std::vector<c10::IValue> concreteInputs;
+  std::vector<std::vector<int64_t>> shapesForKinetoEvent;
+  std::vector<shape> strides;
 };
 
 auto parseArgData(
     const std::vector<op_input_t>& input_shapes,
-    const std::vector<op_input_t>& concrete_inputs) {
+    const std::vector<op_input_t>& concreteInputs) {
   if (input_shapes.empty()) {
-    return OpArgData{false, {}, {}, {}};
+    return OpArgData{false, {}, {}, {}, {}, {}};
   }
 
-  std::vector<std::vector<int64_t>> shapes(input_shapes.size());
+  std::vector<shape> shapes(input_shapes.size());
+  std::vector<shape> strides(input_shapes.size());
+  std::vector<std::vector<int64_t>> shapesForKinetoEvent(input_shapes.size());
+
   std::vector<std::string> dtypes(input_shapes.size());
   std::vector<c10::IValue> concrete_inputs_list;
 
   for (const auto& i : c10::irange(input_shapes.size())) {
-    c10::visit(
+    std::visit(
         c10::overloaded(
             [&](const TensorMetadata& t) {
               shapes[i] = t.sizes_;
+              shapesForKinetoEvent[i] = t.sizes_;
               dtypes[i] = std::string(scalarTypeToTypeMeta(t.dtype_).name());
+              strides[i] = t.strides_;
             },
-            [&](const std::vector<TensorMetadata>&) {
+            [&](const std::vector<TensorMetadata>& l) {
+              std::vector<std::vector<int64_t>> shape;
+              shape.reserve(l.size());
+              std::vector<std::vector<int64_t>> stride;
+              stride.reserve(l.size());
+              for (const auto& t : l) {
+                shape.emplace_back(t.sizes_);
+                stride.emplace_back(t.strides_);
+              }
+              shapes[i] = shape;
+              strides[i] = stride;
               dtypes[i] = "TensorList";
             },
-            [&](const c10::IValue& val) { dtypes[i] = "Scalar"; },
+            [&](const c10::IValue&) { dtypes[i] = "Scalar"; },
             [&](const auto&) {}),
         input_shapes[i]);
   }
 
   // If we recorded concrete inputs, then parse them
-  if (input_shapes.size() == concrete_inputs.size() &&
-      !concrete_inputs.empty()) {
+  if (input_shapes.size() == concreteInputs.size() && !concreteInputs.empty()) {
     concrete_inputs_list.resize(input_shapes.size());
 
     for (const auto& i : c10::irange(input_shapes.size())) {
-      c10::visit(
+      std::visit(
           c10::overloaded(
               [&](const c10::IValue& val) { concrete_inputs_list[i] = val; },
               [&](const auto&) {}),
           input_shapes[i]);
-      c10::visit(
+      std::visit(
           c10::overloaded(
               [&](const c10::IValue& val) {
                 concrete_inputs_list[i] = val;
                 dtypes[i] = "ScalarList";
               },
               [&](const auto&) {}),
-          concrete_inputs[i]);
+          concreteInputs[i]);
     }
   }
 
-  return OpArgData{true, shapes, dtypes, concrete_inputs_list};
+  return OpArgData{
+      true,
+      shapes,
+      dtypes,
+      concrete_inputs_list,
+      shapesForKinetoEvent,
+      strides};
 }
 
 struct MetadataBase {
-  MetadataBase(const std::shared_ptr<Result>& result)
-      : kineto_activity_{result->kineto_activity_} {
-    if (c10::holds_alternative<ExtraFields<EventType::Kineto>>(
+  /* implicit */ MetadataBase(const std::shared_ptr<Result>& result)
+      : kinetoActivity_{result->kineto_activity_} {
+    if (std::holds_alternative<ExtraFields<EventType::Kineto>>(
             result->extra_fields_)) {
       // In order to add metadata we have to downcast from
       // `libkineto::ITraceActivity` to `libkineto::GenericTraceActivity`. We
@@ -148,22 +170,27 @@ struct MetadataBase {
       if (!(SOFT_ASSERT(!hasKinetoActivity()))) {
         result->kineto_activity_ = nullptr;
       }
-      kineto_activity_ = result->kineto_activity_;
+      kinetoActivity_ = result->kineto_activity_;
     }
   }
 
   void addMetadata(const std::string& key, const std::string& value) {
-    if (kineto_activity_ && !value.empty() && value != "\"\"") {
-      torch::profiler::impl::kineto::addMetadata(kineto_activity_, key, value);
+    if (kinetoActivity_ && !value.empty() && value != "\"\"") {
+      torch::profiler::impl::kineto::addMetadata(
+          // NOLINTNEXTLINE(cppcoreguidelines-pro-type-const-cast)
+          const_cast<torch::profiler::impl::kineto::activity_t*>(
+              kinetoActivity_),
+          key,
+          value);
     }
   }
 
   bool hasKinetoActivity() const {
-    return kineto_activity_ != nullptr;
+    return kinetoActivity_ != nullptr;
   }
 
  private:
-  const torch::profiler::impl::kineto::activity_t* kineto_activity_{nullptr};
+  const torch::profiler::impl::kineto::activity_t* kinetoActivity_{nullptr};
 };
 
 struct AddTensorboardFields : public MetadataBase {
@@ -179,7 +206,7 @@ struct AddTensorboardFields : public MetadataBase {
     result->visit_if_base<PyExtraFieldsBase>([&, this](const auto& i) -> void {
       this->addMetadata("Python id", std::to_string(i.id_));
 
-      c10::optional<std::string> parent_id;
+      std::optional<std::string> parent_id;
       std::shared_ptr<Result> parent = result->parent_.lock();
       while (parent && !parent_id.has_value()) {
         parent->visit_if_base<PyExtraFieldsBase>(
@@ -218,13 +245,28 @@ struct AddGenericMetadata : public MetadataBase {
     const auto arg_data =
         parseArgData(op_event.inputs_, op_event.concrete_inputs_);
 
-    if (arg_data.has_data) {
-      addMetadata("Input Dims", shapesToStr(arg_data.shapes));
-      addMetadata("Input type", strListToStr(arg_data.dtypes));
-      if (!arg_data.concrete_inputs.empty()) {
-        addMetadata(
-            "Concrete Inputs", ivalueListToStr(arg_data.concrete_inputs));
+    if (arg_data.hasData) {
+      if (get_record_concrete_inputs_enabled()) {
+        addMetadata("Input Dims", variantShapesToStr(arg_data.shapes));
+        addMetadata("Input Strides", variantShapesToStr(arg_data.strides));
+      } else {
+        addMetadata("Input Dims", shapesToStr(arg_data.shapesForKinetoEvent));
       }
+      addMetadata("Input type", strListToStr(arg_data.dtypes));
+      if (!arg_data.concreteInputs.empty()) {
+        addMetadata(
+            "Concrete Inputs", ivalueListToStr(arg_data.concreteInputs));
+      }
+    }
+
+    // Add metadata for kwinputs if exist
+    for (const auto& [key, val] : op_event.kwinputs_) {
+      bool isString = val.isString();
+      addMetadata(key, ivalueToStr(val, isString));
+    }
+    // Add extra metadata if any
+    for (const auto& [key, val] : op_event.extra_meta_) {
+      addMetadata(key, val);
     }
 
     if (config_ && !config_->experimental_config.performance_events.empty()) {
@@ -242,6 +284,8 @@ struct AddGenericMetadata : public MetadataBase {
       addMetadata("Fwd thread id", std::to_string(op_event.forward_tid_));
       addMetadata("Sequence number", std::to_string(op_event.sequence_number_));
     }
+    addMetadata(
+        "Record function id", std::to_string(op_event.record_function_id_));
   }
 
   void operator()(ExtraFields<EventType::Backend>& backend_event) {
@@ -280,8 +324,8 @@ struct KinetoThreadLocalState : public ProfilerStateBase {
       const ProfilerConfig& config,
       std::set<torch::profiler::impl::ActivityType> activities)
       : ProfilerStateBase(config),
-        start_time_(getTimeUs()),
-        record_queue_(config, std::move(activities)) {}
+        startTime(getTimeNs()),
+        recordQueue(config, std::move(activities)) {}
   ~KinetoThreadLocalState() override = default;
 
   static KinetoThreadLocalState* get(bool global) {
@@ -298,8 +342,8 @@ struct KinetoThreadLocalState : public ProfilerStateBase {
 
   void reportVulkanEventToProfiler(torch::profiler::impl::vulkan_id_t id) {
     if (!config_.disabled()) {
-      record_queue_.getSubqueue()->emplace_vulkan_event(
-          torch::profiler::impl::getApproximateTime(), id);
+      recordQueue.getSubqueue()->emplace_vulkan_event(
+          c10::getApproximateTime(), id);
     }
   }
 
@@ -310,8 +354,8 @@ struct KinetoThreadLocalState : public ProfilerStateBase {
       size_t total_reserved,
       c10::Device device) override {
     if (config_.profile_memory && !config_.disabled()) {
-      record_queue_.getSubqueue()->emplace_allocation_event(
-          torch::profiler::impl::getApproximateTime(),
+      recordQueue.getSubqueue()->emplace_allocation_event(
+          c10::getApproximateTime(),
           ptr,
           alloc_size,
           total_allocated,
@@ -327,8 +371,8 @@ struct KinetoThreadLocalState : public ProfilerStateBase {
       size_t total_reserved,
       c10::Device device) override {
     if (config_.profile_memory && !config_.disabled()) {
-      record_queue_.getSubqueue()->emplace_ooms_event(
-          torch::profiler::impl::getApproximateTime(),
+      recordQueue.getSubqueue()->emplace_ooms_event(
+          c10::getApproximateTime(),
           alloc_size,
           total_allocated,
           total_reserved,
@@ -337,49 +381,56 @@ struct KinetoThreadLocalState : public ProfilerStateBase {
     }
   }
 
-  const post_process_t& getEventPostProcessingCallback() const {
-    return event_post_process_cb_;
+  void setEventPostProcessingCallback(post_process_t&& cb) {
+    eventPostProcessCb = std::move(cb);
   }
 
-  void setEventPostProcessingCallback(post_process_t&& cb) {
-    event_post_process_cb_ = std::move(cb);
+  void pausePython() {
+    recordQueue.stop();
+  }
+
+  void resumePython() {
+    recordQueue.restart();
   }
 
   std::unique_ptr<torch::profiler::impl::kineto::ActivityTraceWrapper>
   finalizeTrace() {
-    auto end_time = getTimeUs();
-    record_queue_.stop();
+    auto end_time = getTimeNs();
+    recordQueue.stop();
 
     std::lock_guard<std::mutex> guard(state_mutex_);
-    auto converter = clock_converter_.makeConverter();
+    auto converter = clockConverter.makeConverter();
+#ifdef USE_KINETO
+    libkineto::get_time_converter() = converter;
+#endif
     auto records_and_trace =
-        record_queue_.getRecords(std::move(converter), start_time_, end_time);
+        recordQueue.getRecords(std::move(converter), startTime, end_time);
 
     materializeOpEvents(records_and_trace.first);
 
-    // `kineto_events_` does not include Python events. Instead it exposes them
+    // `kinetoEvents` does not include Python events. Instead it exposes them
     // via the `stacks` property.
-    kineto_events_.erase(
+    kinetoEvents.erase(
         std::remove_if(
-            kineto_events_.begin(),
-            kineto_events_.end(),
+            kinetoEvents.begin(),
+            kinetoEvents.end(),
             [](const auto& i) { return i.isPythonFunction(); }),
-        kineto_events_.end());
+        kinetoEvents.end());
 
     return std::move(records_and_trace.second);
   }
 
   template <typename T>
   void invokeCallback(T& t) {
-    if (event_post_process_cb_) {
-      event_post_process_cb_(t.debug_handle_, t.jit_stack_, t.jit_modules_);
+    if (eventPostProcessCb) {
+      eventPostProcessCb(t.debug_handle_, t.jit_stack_, t.jit_modules_);
     }
   }
 
   void materializeOpEvents(std::vector<std::shared_ptr<Result>>& events) {
     for (auto& e : events) {
       if (e->parent_.expired() && e->deviceType() == c10::DeviceType::CPU) {
-        event_tree_.push_back(e);
+        eventTree.push_back(e);
       }
 
       if (e->finished_) {
@@ -388,8 +439,8 @@ struct KinetoThreadLocalState : public ProfilerStateBase {
             [this](ExtraFields<EventType::Backend>& i) { invokeCallback(i); },
             [](auto&) {}));
 
-        kineto_events_.emplace_back(e, config_.experimental_config.verbose);
-        AddTensorboardFields add_tb(e, kineto_events_.back());
+        kinetoEvents.emplace_back(e, config_.experimental_config.verbose);
+        AddTensorboardFields add_tb(e, kinetoEvents.back());
         AddGenericMetadata add_generic(e, &config_);
 
         // It is not safe to use the activity after post processing.
@@ -398,13 +449,13 @@ struct KinetoThreadLocalState : public ProfilerStateBase {
     }
   }
 
-  uint64_t start_time_;
-  torch::profiler::impl::ApproximateClockToUnixTimeConverter clock_converter_;
-  torch::profiler::impl::RecordQueue record_queue_;
-  std::vector<KinetoEvent> kineto_events_;
-  std::vector<experimental_event_t> event_tree_;
+  uint64_t startTime;
+  c10::ApproximateClockToUnixTimeConverter clockConverter;
+  torch::profiler::impl::RecordQueue recordQueue;
+  std::vector<KinetoEvent> kinetoEvents;
+  std::vector<experimental_event_t> eventTree;
   // Optional, if event post-processing is enabled.
-  post_process_t event_post_process_cb_;
+  post_process_t eventPostProcessCb;
 };
 
 template <bool use_global_state_ptr = false>
@@ -414,7 +465,7 @@ std::unique_ptr<at::ObserverContext> onFunctionEnter(
   if (!state_ptr) {
     return nullptr;
   }
-  return state_ptr->record_queue_.getSubqueue()->begin_op(fn);
+  return state_ptr->recordQueue.getSubqueue()->begin_op(fn);
 }
 
 // @lint-ignore CLANGTIDY clang-diagnostic-unused-parameter
@@ -430,10 +481,9 @@ void onFunctionExit(
   auto* kineto_ctx_ptr =
       static_cast<torch::profiler::impl::KinetoObserverContext*>(ctx_ptr);
   TORCH_INTERNAL_ASSERT(kineto_ctx_ptr != nullptr);
-  kineto_ctx_ptr->event_->end_time_ =
-      torch::profiler::impl::getApproximateTime();
+  kineto_ctx_ptr->event_->end_time_ = c10::getApproximateTime();
   if (!config.experimental_config.performance_events.empty()) {
-    state_ptr->record_queue_.getSubqueue()->disable_perf_profiler(
+    state_ptr->recordQueue.getSubqueue()->disable_perf_profiler(
         *kineto_ctx_ptr->event_->counters_);
   }
   kineto_ctx_ptr->event_->basic_fields_.end_tid_ =
@@ -482,6 +532,12 @@ void pushProfilingCallbacks(const std::unordered_set<at::RecordScope>& scopes) {
   }
 }
 
+struct ProfilerStateInfo {
+  std::shared_ptr<KinetoThreadLocalState> state_ptr;
+  std::unordered_set<at::RecordScope> scopes;
+};
+std::shared_ptr<ProfilerStateInfo> profiler_state_info_ptr{nullptr};
+
 } // namespace
 
 void reportBackendEventToActiveKinetoProfiler(
@@ -500,7 +556,7 @@ void reportBackendEventToActiveKinetoProfiler(
     return;
   }
 
-  state_ptr->record_queue_.getSubqueue()->emplace_backend_event(
+  state_ptr->recordQueue.getSubqueue()->emplace_backend_event(
       start_time_us,
       end_time_us,
       debug_handle,
@@ -529,7 +585,9 @@ void prepareProfiler(
           config.state == ProfilerState::KINETO_PRIVATEUSE1_FALLBACK,
       "Supported only in Kineto profiler");
   torch::profiler::impl::kineto::prepareTrace(
-      /*cpuOnly=*/!(at::hasCUDA() || at::hasXPU() || at::hasMTIA()),
+      /*cpuOnly=*/!(
+          at::hasCUDA() || at::hasXPU() || at::hasMTIA() ||
+          c10::get_privateuse1_backend() != "privateuseone"),
       activities,
       config.experimental_config);
 
@@ -561,6 +619,72 @@ void prepareProfiler(
   }
 }
 
+static void toggleTorchOpCollectionDynamic(bool enable) {
+  auto state_ptr = ProfilerStateBase::get();
+  if (state_ptr) {
+    const auto& config = state_ptr->config();
+    if (enable) {
+      auto scopes = profiler_state_info_ptr->scopes;
+      config.global() ? pushProfilingCallbacks</*global=*/true>(scopes)
+                      : pushProfilingCallbacks</*global=*/false>(scopes);
+    } else {
+      state_ptr->removeCallback();
+    }
+  }
+}
+
+// Set this function to be unused as profiler implementation needs more
+// refactoring to support Python ops collection dynamic toggling
+#ifdef _MSC_VER
+#define UNUSED
+#else
+#define UNUSED __attribute__((unused))
+#endif
+static UNUSED void togglePythonCollectionDynamic(bool enable) {
+  auto state_ptr = ProfilerStateBase::get();
+  if (state_ptr) {
+    auto global = state_ptr->config().global();
+    KinetoThreadLocalState* kineto_thread_local_state_ptr =
+        KinetoThreadLocalState::get(global);
+    if (enable) {
+      kineto_thread_local_state_ptr->resumePython();
+    } else {
+      kineto_thread_local_state_ptr->pausePython();
+    }
+  }
+}
+
+static void toggleCPUCollectionDynamic(bool enable) {
+  toggleTorchOpCollectionDynamic(enable);
+  // For now we only support Torch Op collection dynamic toggling as
+  // implementing Python ops would require not only string parsing to get rid of
+  // the toggling events as well as other unfinished events as well as changes
+  // in stack logic
+  // togglePythonCollectionDynamic(enable);
+}
+
+void toggleCollectionDynamic(
+    const bool enable,
+    const std::set<torch::profiler::impl::ActivityType>& activities) {
+  if (activities.count(torch::autograd::profiler::ActivityType::CPU) > 0 &&
+      activities.count(torch::autograd::profiler::ActivityType::CUDA) == 0) {
+    LOG(WARNING)
+        << "Toggling CPU activity with CUDA activity on may result in traces with CUDA events on artibrary tracks";
+  }
+  for (auto act : activities) {
+    if (act == torch::autograd::profiler::ActivityType::CUDA) {
+      torch::profiler::impl::kineto::toggleCollectionDynamic(enable);
+    } else if (act == torch::autograd::profiler::ActivityType::CPU) {
+      toggleCPUCollectionDynamic(enable);
+    } else {
+      LOG(WARNING)
+          << "Dynamic toggle is only supported for CPU/GPU activity, skipping toggling of "
+          << actToString(act);
+      continue;
+    }
+  }
+}
+
 void enableProfilerWithEventPostProcess(
     const torch::profiler::impl::ProfilerConfig& config,
     const std::set<torch::profiler::impl::ActivityType>& activities,
@@ -597,6 +721,9 @@ void enableProfiler(
   } else if (config.state == ProfilerState::ITT) {
     torch::profiler::impl::pushITTCallbacks(config, scopes);
     return;
+  } else if (config.state == ProfilerState::PRIVATEUSE1) {
+    torch::profiler::impl::pushPRIVATEUSE1CallbacksStub(config, scopes);
+    return;
   }
 
   TORCH_CHECK(
@@ -609,8 +736,8 @@ void enableProfiler(
       has_cpu || !config.global(),
       "Ondemand profiling must enable CPU tracing");
 
-  KinetoThreadLocalState::push(
-      std::make_shared<KinetoThreadLocalState>(config, activities));
+  auto state_ptr = std::make_shared<KinetoThreadLocalState>(config, activities);
+  KinetoThreadLocalState::push(state_ptr);
 
   if (has_cpu) {
     config.global() ? pushProfilingCallbacks</*global=*/true>(scopes)
@@ -620,9 +747,42 @@ void enableProfiler(
   if (!config.global()) {
     torch::profiler::impl::kineto::startTrace();
   }
+
+  if (has_cpu) {
+    auto state_info_ptr = std::make_shared<ProfilerStateInfo>();
+    state_info_ptr->state_ptr = state_ptr;
+    state_info_ptr->scopes = scopes;
+    profiler_state_info_ptr = state_info_ptr;
+  }
+}
+
+bool isProfilerEnabledInMainThread() {
+  return profiler_state_info_ptr != nullptr;
+}
+
+void enableProfilerInChildThread() {
+  auto state_info_ptr = profiler_state_info_ptr;
+  TORCH_CHECK(state_info_ptr, "Profiler is not enabled in main thread.");
+  TORCH_CHECK(
+      KinetoThreadLocalState::get(/*global=*/false) == nullptr,
+      "Profiler is already enabled in this thread.");
+
+  KinetoThreadLocalState::push(state_info_ptr->state_ptr);
+  pushProfilingCallbacks</*global=*/false>(state_info_ptr->scopes);
+}
+
+void disableProfilerInChildThread() {
+  auto state_ptr = ProfilerStateBase::pop();
+  TORCH_CHECK(
+      state_ptr,
+      "Can't disable Kineto profiler when it's not running in this thread");
+  state_ptr->removeCallback();
 }
 
 std::unique_ptr<ProfilerResult> disableProfiler() {
+  // releasing to inform child threads to stop profiling
+  profiler_state_info_ptr = nullptr;
+
   auto state_ptr = ProfilerStateBase::pop();
   const auto& config = state_ptr->config();
   TORCH_CHECK(
@@ -632,7 +792,8 @@ std::unique_ptr<ProfilerResult> disableProfiler() {
            config.state == ProfilerState::KINETO_PRIVATEUSE1_FALLBACK ||
            config.state == ProfilerState::KINETO_ONDEMAND ||
            config.state == ProfilerState::NVTX ||
-           config.state == ProfilerState::ITT),
+           config.state == ProfilerState::ITT ||
+           config.state == ProfilerState::PRIVATEUSE1),
       "Can't disable Kineto profiler when it's not running");
 
   state_ptr->removeCallback();
@@ -644,9 +805,11 @@ std::unique_ptr<ProfilerResult> disableProfiler() {
     return std::make_unique<ProfilerResult>();
   }
 
-  // Shared among NVTX, KINETO, KINETO_GPU_FALLBACK, KINETO_PRIVATEUSE1_FALLBACK
+  // Shared among NVTX, PRIVATEUSE1, KINETO, KINETO_GPU_FALLBACK,
+  // KINETO_PRIVATEUSE1_FALLBACK
   std::unique_ptr<ProfilerResult> result;
-  if (state_ptr->config().state == ProfilerState::NVTX) {
+  if (state_ptr->config().state == ProfilerState::NVTX ||
+      state_ptr->config().state == ProfilerState::PRIVATEUSE1) {
     result = std::make_unique<ProfilerResult>();
   }
 
@@ -657,17 +820,17 @@ std::unique_ptr<ProfilerResult> disableProfiler() {
         std::static_pointer_cast<KinetoThreadLocalState>(state_ptr);
     auto trace = kineto_state_ptr->finalizeTrace();
     result = std::make_unique<ProfilerResult>(
-        kineto_state_ptr->start_time_,
-        std::move(kineto_state_ptr->kineto_events_),
+        kineto_state_ptr->startTime,
+        std::move(kineto_state_ptr->kinetoEvents),
         std::move(trace),
-        std::move(kineto_state_ptr->event_tree_));
+        std::move(kineto_state_ptr->eventTree));
   }
 
   return result;
 }
 
 KinetoEvent::KinetoEvent(
-    std::shared_ptr<const torch::profiler::impl::Result> result,
+    const std::shared_ptr<const torch::profiler::impl::Result>& result,
     const bool verbose)
     : result_{result} {
   TORCH_INTERNAL_ASSERT(result != nullptr);
@@ -677,16 +840,17 @@ KinetoEvent::KinetoEvent(
     auto parent = result_->parent_.lock();
     while (parent != nullptr) {
       parent->visit_if_base<PyExtraFieldsBase>(
-          [&](const auto& i) { python_stack_.push_back(parent->name()); });
+          [&](const auto&) { python_stack_.push_back(parent->name()); });
       parent = parent->parent_.lock();
     }
   }
 
   result->visit_if_base<ExtraFields<EventType::TorchOp>>([&](const auto& op) {
     auto arg_data = parseArgData(op.inputs_, op.concrete_inputs_);
-    shapes_ = std::move(arg_data.shapes);
+    shapes_ = std::move(arg_data.shapesForKinetoEvent);
     dtypes_ = std::move(arg_data.dtypes);
-    concrete_inputs_ = std::move(arg_data.concrete_inputs);
+    concrete_inputs_ = std::move(arg_data.concreteInputs);
+    kwinputs_ = std::move(op.kwinputs_);
   });
 }
 
@@ -720,29 +884,47 @@ const c10::ArrayRef<c10::IValue> KinetoEvent::concreteInputs() const {
   return concrete_inputs_;
 }
 
+bool KinetoEvent::hasKwinputs() const {
+  return !kwinputs_.empty();
+}
+
+const std::unordered_map<std::string, c10::IValue> KinetoEvent::kwinputs()
+    const {
+  return kwinputs_;
+}
+
 const c10::ArrayRef<std::string> KinetoEvent::stack() const {
   auto get = [&](const auto& i) -> auto& {
     return !i.jit_stack_.empty() ? i.jit_stack_ : python_stack_;
   };
 
-  using out_t = const c10::ArrayRef<std::string>;
-  return result_->visit(c10::overloaded(
-      [&](const ExtraFields<EventType::TorchOp>& i) -> out_t { return get(i); },
-      [&](const ExtraFields<EventType::Backend>& i) -> out_t { return get(i); },
-      [&](const auto&) -> out_t { return python_stack_; }));
+  auto const& extra_fields = result_->extra_fields_;
+  if (auto p = std::get_if<ExtraFields<EventType::TorchOp>>(&extra_fields)) {
+    return get(*p);
+  }
+  if (auto p = std::get_if<ExtraFields<EventType::Backend>>(&extra_fields)) {
+    return get(*p);
+  }
+  return python_stack_;
 }
 
 const c10::ArrayRef<std::string> KinetoEvent::moduleHierarchy() const {
-  return result_->visit(c10::overloaded(
-      [](const ExtraFields<EventType::TorchOp>& e)
-          -> const c10::ArrayRef<std::string> { return e.jit_modules_; },
-      [](const ExtraFields<EventType::Backend>& e)
-          -> const c10::ArrayRef<std::string> { return e.jit_modules_; },
-      [](const auto&) -> const c10::ArrayRef<std::string> { return {}; }));
+  auto const& extra_fields = result_->extra_fields_;
+  if (auto p = std::get_if<ExtraFields<EventType::TorchOp>>(&extra_fields)) {
+    return p->jit_modules_;
+  }
+  if (auto p = std::get_if<ExtraFields<EventType::Backend>>(&extra_fields)) {
+    return p->jit_modules_;
+  }
+  return {};
 }
 
-uint64_t KinetoEvent::durationUs() const {
-  return (result_->endTimeNS() - result_->start_time_ns_) / 1000;
+uint64_t KinetoEvent::endNs() const {
+  return result_->endTimeNS();
+}
+
+uint64_t KinetoEvent::durationNs() const {
+  return (result_->endTimeNS() - result_->start_time_ns_);
 }
 
 int64_t KinetoEvent::debugHandle() const {
@@ -752,16 +934,16 @@ int64_t KinetoEvent::debugHandle() const {
       [](const auto&) -> int64_t { return -1; }));
 }
 
-uint8_t KinetoEvent::deviceIndex() const {
+int KinetoEvent::deviceIndex() const {
   return result_->visit(c10::overloaded(
       [](const ExtraFields<EventType::Allocation>& i) {
-        return static_cast<uint8_t>(i.device_index_);
+        return static_cast<int>(i.device_index_);
       },
       [](const ExtraFields<EventType::OutOfMemory>& i) {
-        return static_cast<uint8_t>(i.device_index_);
+        return static_cast<int>(i.device_index_);
       },
       [&](const auto&) {
-        return static_cast<uint8_t>(result_->kineto_info_.device);
+        return static_cast<int>(result_->kineto_info_.device);
       }));
 }
 
@@ -823,7 +1005,7 @@ FORWARD_FROM_RESULT(endThreadId, endTID())
 FORWARD_FROM_RESULT(activityType, kinetoType())
 FORWARD_FROM_RESULT(name, name())
 FORWARD_FROM_RESULT(deviceType, deviceType())
-FORWARD_FROM_RESULT(startUs, start_time_ns_ / 1000)
+FORWARD_FROM_RESULT(startNs, start_time_ns_)
 FORWARD_FROM_RESULT(correlationId, correlationID())
 FORWARD_FROM_RESULT(deviceResourceId, kineto_info_.resource)
 #undef FORWARD_FROM_RESULT
@@ -851,12 +1033,15 @@ TYPED_ATTR(TorchOp, fwdThreadId, e.sequence_number_ >= 0 ? e.forward_tid_ : 0)
 TYPED_ATTR(TorchOp, scope, static_cast<uint8_t>(e.scope_))
 TYPED_ATTR(TorchOp, hasModuleHierarchy, !e.jit_modules_.empty())
 TYPED_ATTR(TorchOp, isAsync, e.is_async_)
+TYPED_ATTR(TorchOp, extraMeta, e.extra_meta_)
 TYPED_ATTR(TorchOp, fallbackStart, e.device_fallback_.device_event_start_)
 TYPED_ATTR(TorchOp, fallbackEnd, e.device_fallback_.device_event_end_)
 TYPED_ATTR(
     TorchOp,
     flops,
-    !e.extra_args_.empty() ? computeFlops(e.name_, e.extra_args_) : 0)
+    !e.extra_args_.empty()
+        ? torch::profiler::impl::computeFlops(e.name_, e.extra_args_)
+        : 0)
 TYPED_ATTR(Backend, backend, e.backend_)
 TYPED_ATTR(Allocation, nBytes, e.alloc_size_)
 TYPED_ATTR(Kineto, linkedCorrelationId, [&]() {
@@ -872,7 +1057,7 @@ ProfilerResult::ProfilerResult(
     std::unique_ptr<torch::profiler::impl::kineto::ActivityTraceWrapper>&&
         trace,
     std::vector<experimental_event_t>&& event_tree)
-    : trace_start_us_(start_time),
+    : trace_start_ns_(start_time),
       events_(std::move(events)),
       trace_(std::move(trace)),
       event_tree_(std::move(event_tree)) {}
@@ -883,11 +1068,9 @@ void ProfilerResult::save(const std::string& path) {
   trace_->save(path);
 }
 
-} // namespace profiler
-} // namespace autograd
+} // namespace autograd::profiler
 
-namespace profiler {
-namespace impl {
+namespace profiler::impl {
 void _reportVulkanEventToProfiler(vulkan_id_t id) {
   auto state_ptr = ::torch::autograd::profiler::KinetoThreadLocalState::get(
       /*global=*/false);
@@ -895,7 +1078,6 @@ void _reportVulkanEventToProfiler(vulkan_id_t id) {
     state_ptr->reportVulkanEventToProfiler(id);
   }
 }
-} // namespace impl
-} // namespace profiler
+} // namespace profiler::impl
 
 } // namespace torch
