@@ -9,7 +9,7 @@
 #include <torch/csrc/autograd/python_variable.h>
 #include <torch/csrc/autograd/utils/wrap_outputs.h>
 #include <torch/csrc/jit/frontend/tracer.h>
-#include <torch/csrc/utils/cuda_lazy_init.h>
+#include <torch/csrc/utils/device_lazy_init.h>
 #include <torch/csrc/utils/out_types.h>
 #include <torch/csrc/utils/pybind.h>
 #include <torch/csrc/utils/pycfunction_helpers.h>
@@ -21,6 +21,7 @@
 
 #include <ATen/ATen.h>
 #include <ATen/FunctionalTensorWrapper.h>
+#include <ATen/native/Resize.h>
 
 #include <Python.h>
 #include <fmt/format.h>
@@ -40,8 +41,7 @@ using at::TensorOptions;
 using torch::utils::check_out_type_matches;
 using namespace torch::autograd::utils;
 
-namespace torch {
-namespace autograd {
+namespace torch::autograd {
 
 // NOLINTNEXTLINE(cppcoreguidelines-avoid-non-const-global-variables)
 PyObject* THPVariableFunctionsModule = nullptr;
@@ -61,7 +61,7 @@ inline Tensor dispatch_range(
     const Scalar& end,
     const Scalar& step,
     const TensorOptions& options) {
-  torch::utils::maybe_initialize_cuda(options);
+  torch::utils::maybe_initialize_device(options);
   pybind11::gil_scoped_release no_gil;
   DeviceGuard device_guard(options.device());
   return torch::range(start, end, step, options);
@@ -221,12 +221,12 @@ static PyObject* THPVariable_sparse_coo_tensor(
     PyObject* kwargs) {
   HANDLE_TH_ERRORS
   static PythonArgParser parser({
-      "sparse_coo_tensor(PyObject* indices, PyObject* values, *, ScalarType dtype=None, Device? device=None, bool requires_grad=False, bool check_invariants=None)",
-      "sparse_coo_tensor(PyObject* indices, PyObject* values, IntArrayRef size, *, ScalarType dtype=None, Device? device=None, bool requires_grad=False, bool check_invariants=None, bool is_coalesced=None)",
+      "sparse_coo_tensor(PyObject* indices, PyObject* values, *, ScalarType dtype=None, Device? device=None, bool pin_memory=False, bool requires_grad=False, bool check_invariants=None)",
+      "sparse_coo_tensor(PyObject* indices, PyObject* values, IntArrayRef size, *, ScalarType dtype=None, Device? device=None, bool pin_memory=False, bool requires_grad=False, bool check_invariants=None, bool is_coalesced=None)",
       "sparse_coo_tensor(IntArrayRef size, *, ScalarType dtype=None, Device? device=None, bool requires_grad=False, bool check_invariants=None)",
   });
 
-  ParsedArgs<8> parsed_args;
+  ParsedArgs<9> parsed_args;
   auto r = parser.parse(args, kwargs, parsed_args);
   if (r.has_torch_function()) {
     return handle_torch_function(
@@ -279,6 +279,10 @@ static PyObject* THPVariable_get_device(
 
   ParsedArgs<1> parsed_args;
   auto r = parser.parse(args, kwargs, parsed_args);
+  if (r.has_torch_function()) {
+    return handle_torch_function(
+        r, nullptr, args, kwargs, THPVariableFunctionsModule, "torch");
+  }
 
   if (r.idx == 0) {
     return wrap(r.tensor(0).get_device());
@@ -356,167 +360,6 @@ static PyObject* THPVariable_numel(
     PyObject* args,
     PyObject* kwargs);
 
-static PyObject* THPVariable__to_functional_tensor(
-    PyObject* self,
-    PyObject* args,
-    PyObject* kwargs) {
-  HANDLE_TH_ERRORS
-  static PythonArgParser parser(
-      {"_to_functional_tensor(Tensor t, *, bool mirror_autograd_meta=False)"},
-      /*traceable=*/true);
-
-  ParsedArgs<2> parsed_args;
-  auto r = parser.parse(args, kwargs, parsed_args);
-  auto self_ = r.tensor(0);
-  auto mirror_autograd_meta = r.toBool(1);
-  auto wrapped = at::functionalization::impl::to_functional_tensor(self_);
-  if (mirror_autograd_meta) {
-    // Here, we unsafely set the grad function on the wrapper to be the same as
-    // the inner. We expect this grad_fn to NEVER be used. It's needed so that
-    // .is_leaf metadata is accurate on the wrapper
-    auto inner_autograd_meta = impl::get_autograd_meta(self_);
-    if (inner_autograd_meta) {
-      wrapped.set_requires_grad(self_.requires_grad());
-      if (wrapped.requires_grad()) {
-        auto new_grad_fn = std::shared_ptr<torch::autograd::Error>(
-            new torch::autograd::Error(
-                "Cannot backprop through mirrored meta, file a bug in PyTorch"),
-            torch::autograd::deleteNode);
-        torch::autograd::set_history(wrapped, new_grad_fn);
-      }
-    }
-  }
-  return wrap(std::move(wrapped));
-  END_HANDLE_TH_ERRORS
-}
-
-static PyObject* THPVariable__from_functional_tensor(
-    PyObject* self,
-    PyObject* args,
-    PyObject* kwargs) {
-  HANDLE_TH_ERRORS
-  static PythonArgParser parser(
-      {"_from_functional_tensor(Tensor t)"}, /*traceable=*/true);
-
-  ParsedArgs<1> parsed_args;
-  auto r = parser.parse(args, kwargs, parsed_args);
-  auto self_ = r.tensor(0);
-  auto unwrapped = at::functionalization::impl::from_functional_tensor(self_);
-  return wrap(std::move(unwrapped));
-  END_HANDLE_TH_ERRORS
-}
-
-static PyObject* THPVariable__freeze_functional_tensor(
-    PyObject* self,
-    PyObject* args,
-    PyObject* kwargs) {
-  HANDLE_TH_ERRORS
-  static PythonArgParser parser(
-      {"_freeze_functional_tensor(Tensor t)"}, /*traceable=*/true);
-
-  ParsedArgs<1> parsed_args;
-  auto r = parser.parse(args, kwargs, parsed_args);
-  auto self_ = r.tensor(0);
-  at::functionalization::impl::freeze_functional_tensor(self_);
-  Py_RETURN_NONE;
-  END_HANDLE_TH_ERRORS
-}
-
-static PyObject* THPVariable__is_functional_tensor(
-    PyObject* self,
-    PyObject* args,
-    PyObject* kwargs) {
-  HANDLE_TH_ERRORS
-  static PythonArgParser parser(
-      {"_is_functional_tensor(Tensor t)"}, /*traceable=*/true);
-
-  ParsedArgs<1> parsed_args;
-  auto r = parser.parse(args, kwargs, parsed_args);
-  auto self_ = r.tensor(0);
-  if (at::functionalization::impl::isFunctionalTensor(self_)) {
-    Py_RETURN_TRUE;
-  } else {
-    Py_RETURN_FALSE;
-  }
-  END_HANDLE_TH_ERRORS
-}
-
-static PyObject* THPVariable__functionalize_has_metadata_mutation(
-    PyObject* self,
-    PyObject* args,
-    PyObject* kwargs) {
-  HANDLE_TH_ERRORS
-  static PythonArgParser parser(
-      {"_functionalize_has_metadata_mutation(Tensor t)"}, /*traceable=*/true);
-
-  ParsedArgs<1> parsed_args;
-  auto r = parser.parse(args, kwargs, parsed_args);
-  auto self_ = r.tensor(0);
-  TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(self_));
-  auto wrapper = at::functionalization::impl::unsafeGetFunctionalWrapper(self_);
-  if (wrapper->has_metadata_mutation()) {
-    Py_RETURN_TRUE;
-  } else {
-    Py_RETURN_FALSE;
-  }
-  END_HANDLE_TH_ERRORS
-}
-
-static PyObject* THPVariable__enable_functionalization(
-    PyObject* self,
-    PyObject* args,
-    PyObject* kwargs) {
-  HANDLE_TH_ERRORS
-  static PythonArgParser parser(
-      {"_enable_functionalization(*, bool reapply_views=False)"},
-      /*traceable=*/true);
-  ParsedArgs<1> parsed_args;
-  auto r = parser.parse(args, kwargs, parsed_args);
-  const auto reapply_views = r.toBool(0);
-
-  if (c10::impl::tls_is_dispatch_key_included(at::DispatchKey::Functionalize)) {
-    TORCH_INTERNAL_ASSERT(
-        false,
-        "multiple layers of mode-style functionalization nesting is not"
-        " currently supported, outside of the functionalize() transform");
-  }
-  c10::impl::tls_set_dispatch_key_included(
-      at::DispatchKey::Functionalize, true);
-  if (reapply_views) {
-    at::functionalization::impl::setFunctionalizationReapplyViewsTLS(true);
-  }
-  Py_RETURN_NONE;
-  END_HANDLE_TH_ERRORS
-}
-
-static PyObject* THPVariable__disable_functionalization(
-    PyObject* self,
-    PyObject* args,
-    PyObject* kwargs) {
-  HANDLE_TH_ERRORS
-  c10::impl::tls_set_dispatch_key_included(
-      at::DispatchKey::Functionalize, false);
-  at::functionalization::impl::setFunctionalizationReapplyViewsTLS(false);
-  Py_RETURN_NONE;
-  END_HANDLE_TH_ERRORS
-}
-
-static PyObject* THPVariable__sync(
-    PyObject* self,
-    PyObject* args,
-    PyObject* kwargs) {
-  HANDLE_TH_ERRORS
-  static PythonArgParser parser({"_sync(Tensor t)"}, /*traceable=*/true);
-
-  ParsedArgs<1> parsed_args;
-  auto r = parser.parse(args, kwargs, parsed_args);
-  auto self_ = r.tensor(0);
-  TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(self_));
-  at::functionalization::impl::sync(self_);
-  Py_RETURN_NONE;
-  END_HANDLE_TH_ERRORS
-}
-
 // XXX: ops that are bound here are not exposed to the C++ api nor the JIT.
 // Any new ops added here should be accompanied with a comment why they are not
 // being registered through native_functions.yaml, and be tagged cpp / JIT
@@ -535,39 +378,6 @@ static PyMethodDef torch_functions_manual[] = {
      castPyCFunctionWithKeywords(THPVariable_frombuffer),
      METH_VARARGS | METH_KEYWORDS | METH_STATIC,
      nullptr},
-    {"_is_functional_tensor",
-     castPyCFunctionWithKeywords(THPVariable__is_functional_tensor),
-     METH_VARARGS | METH_KEYWORDS | METH_STATIC,
-     nullptr},
-    {"_to_functional_tensor",
-     castPyCFunctionWithKeywords(THPVariable__to_functional_tensor),
-     METH_VARARGS | METH_KEYWORDS | METH_STATIC,
-     nullptr},
-    {"_from_functional_tensor",
-     castPyCFunctionWithKeywords(THPVariable__from_functional_tensor),
-     METH_VARARGS | METH_KEYWORDS | METH_STATIC,
-     nullptr},
-    {"_freeze_functional_tensor",
-     castPyCFunctionWithKeywords(THPVariable__freeze_functional_tensor),
-     METH_VARARGS | METH_KEYWORDS | METH_STATIC,
-     nullptr},
-    {"_sync",
-     castPyCFunctionWithKeywords(THPVariable__sync),
-     METH_VARARGS | METH_KEYWORDS | METH_STATIC,
-     nullptr},
-    {"_enable_functionalization",
-     castPyCFunctionWithKeywords(THPVariable__enable_functionalization),
-     METH_VARARGS | METH_KEYWORDS | METH_STATIC,
-     nullptr},
-    {"_disable_functionalization",
-     castPyCFunctionWithKeywords(THPVariable__disable_functionalization),
-     METH_VARARGS | METH_KEYWORDS | METH_STATIC,
-     nullptr},
-    {"_functionalize_has_metadata_mutation",
-     castPyCFunctionWithKeywords(
-         THPVariable__functionalize_has_metadata_mutation),
-     METH_VARARGS | METH_KEYWORDS | METH_STATIC,
-     nullptr},
     {"nonzero",
      castPyCFunctionWithKeywords(THPVariable_nonzero),
      METH_VARARGS | METH_KEYWORDS | METH_STATIC,
@@ -789,7 +599,204 @@ void initTorchFunctions(PyObject* module) {
           module, "_VariableFunctions", THPVariableFunctionsModule) < 0) {
     throw python_error();
   }
+
+  // pybind registrations to torch module
+  // TODO: move these from torch.* to torch._C.*
+  auto py_module = py::module::import("torch");
+
+  py_module.def(
+      "_functionalize_are_all_mutations_under_no_grad_or_inference_mode",
+      [](const at::Tensor& t) {
+        TORCH_INTERNAL_ASSERT(
+            at::functionalization::impl::isFunctionalTensor(t));
+        return at::functionalization::impl::
+            are_all_mutations_under_no_grad_or_inference_mode(t);
+      });
+  py_module.def(
+      "_functionalize_was_inductor_storage_resized", [](const at::Tensor& t) {
+        TORCH_INTERNAL_ASSERT(
+            at::functionalization::impl::isFunctionalTensor(t));
+        auto impl = at::functionalization::impl::unsafeGetFunctionalWrapper(t);
+        return impl->was_inductor_storage_resized();
+      });
+  py_module.def(
+      "_functionalize_are_all_mutations_hidden_from_autograd",
+      [](const at::Tensor& t) {
+        TORCH_INTERNAL_ASSERT(
+            at::functionalization::impl::isFunctionalTensor(t));
+        return at::functionalization::impl::
+            are_all_mutations_hidden_from_autograd(t);
+      });
+  py_module.def(
+      "_functionalize_mark_mutation_hidden_from_autograd",
+      [](const at::Tensor& t) {
+        TORCH_INTERNAL_ASSERT(
+            at::functionalization::impl::isFunctionalTensor(t));
+        at::functionalization::impl::mark_mutation_hidden_from_autograd(t);
+      });
+  py_module.def(
+      "_functionalize_apply_view_metas",
+      [](const at::Tensor& tensor, const at::Tensor& base) {
+        TORCH_INTERNAL_ASSERT(
+            at::functionalization::impl::isFunctionalTensor(tensor));
+        auto impl =
+            at::functionalization::impl::unsafeGetFunctionalWrapper(tensor);
+        return impl->apply_view_metas(base);
+      });
+  py_module.def("_functionalize_is_symbolic", [](const at::Tensor& t) {
+    TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(t));
+    auto impl = at::functionalization::impl::unsafeGetFunctionalWrapper(t);
+    return impl->is_symbolic();
+  });
+  py_module.def("_functionalize_sync", [](const at::Tensor& t) {
+    TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(t));
+    at::functionalization::impl::sync(t);
+  });
+  py_module.def("_functionalize_commit_update", [](const at::Tensor& t) {
+    TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(t));
+    at::functionalization::impl::commit_update(t);
+  });
+  py_module.def(
+      "_functionalize_replace", [](const at::Tensor& t, const at::Tensor& o) {
+        TORCH_INTERNAL_ASSERT(
+            at::functionalization::impl::isFunctionalTensor(t));
+        TORCH_INTERNAL_ASSERT(
+            !at::functionalization::impl::isFunctionalTensor(o));
+        at::functionalization::impl::replace_(t, o);
+      });
+  py_module.def("_is_functional_tensor_base", [](const at::Tensor& t) {
+    TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(t));
+    return at::functionalization::impl::isBaseTensor(t);
+  });
+  py_module.def("_functionalize_is_multi_output_view", [](const at::Tensor& t) {
+    TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(t));
+    auto t_impl = at::functionalization::impl::unsafeGetFunctionalWrapper(t);
+    return t_impl->is_multi_output_view();
+  });
+  py_module.def(
+      "_functionalize_enable_reapply_views",
+      [](bool reapply_views = false) {
+        auto old =
+            at::functionalization::impl::getFunctionalizationReapplyViewsTLS();
+        at::functionalization::impl::setFunctionalizationReapplyViewsTLS(
+            reapply_views);
+        return old;
+      },
+      py::arg("reapply_views") = false);
+  py_module.def(
+      "_functionalize_has_metadata_mutation", [](const at::Tensor& t) {
+        TORCH_INTERNAL_ASSERT(
+            at::functionalization::impl::isFunctionalTensor(t));
+        auto t_impl =
+            at::functionalization::impl::unsafeGetFunctionalWrapper(t);
+        return t_impl->has_metadata_mutation();
+      });
+  py_module.def("_functionalize_has_data_mutation", [](const at::Tensor& t) {
+    TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(t));
+    auto t_impl = at::functionalization::impl::unsafeGetFunctionalWrapper(t);
+    return t_impl->has_data_mutation();
+  });
+  py_module.def(
+      "_functionalize_get_storage_size", [](const at::Tensor& t, bool before) {
+        TORCH_INTERNAL_ASSERT(
+            at::functionalization::impl::isFunctionalTensor(t));
+        auto wrapper =
+            at::functionalization::impl::unsafeGetFunctionalWrapper(t);
+        auto size = wrapper->get_storage_size(/*before=*/before);
+        return size;
+      });
+  py_module.def("_functionalize_set_storage_changed", [](const at::Tensor& t) {
+    TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(t));
+    auto wrapper = at::functionalization::impl::unsafeGetFunctionalWrapper(t);
+    wrapper->set_storage_changed();
+  });
+  py_module.def("_functionalize_was_storage_changed", [](const at::Tensor& t) {
+    TORCH_INTERNAL_ASSERT(at::functionalization::impl::isFunctionalTensor(t));
+    auto wrapper = at::functionalization::impl::unsafeGetFunctionalWrapper(t);
+    return wrapper->was_storage_changed();
+  });
+  py_module.def(
+      "_functionalize_unsafe_set", [](at::Tensor& dst, const at::Tensor& src) {
+        // Forcefully/unsafely dumps src.storage into dst.
+        // This API is technically and not specific to functionalization
+        // (it just runs set_() without the safety checks).
+        // But its main intended purpose today is during functionalization.
+        // In particular: when we generate a new FunctionalTensor from a view
+        // op, we need to ensure it shares a storage with the view input.
+        //
+        // Other subclasses shouldn't really need to care about this,
+        // because we define aliasing on wrapper subclasses such that:
+        // - differentiable aliasing: subclass_x and subclass_y share a ._base.
+        // - non-differentiable aliasing: aliasing of subclass_x and subclass_y
+        //   is defined recursively based on the aliasing of their inner
+        //   tensors.
+        at::native::checkSetStorage(
+            dst,
+            src.storage(),
+            dst.sym_storage_offset(),
+            dst.sym_sizes(),
+            dst.sym_strides());
+      });
+  py_module.def(
+      "_functionalize_mark_mutation_hidden_from_autograd",
+      [](const at::Tensor& t) {
+        TORCH_INTERNAL_ASSERT(
+            at::functionalization::impl::isFunctionalTensor(t));
+        at::functionalization::impl::mark_mutation_hidden_from_autograd(t);
+      });
+  py_module.def("_is_functional_tensor", [](const at::Tensor& t) {
+    return at::functionalization::impl::isFunctionalTensor(t);
+  });
+  py_module.def("_to_functional_tensor", [](const at::Tensor& t) {
+    return at::functionalization::impl::to_functional_tensor(t);
+  });
+  py_module.def("_from_functional_tensor", [](const at::Tensor& t) {
+    return at::functionalization::impl::from_functional_tensor(t);
+  });
+  py_module.def("_freeze_functional_tensor", [](const at::Tensor& t) {
+    at::functionalization::impl::freeze_functional_tensor(t);
+  });
+  py_module.def(
+      "_enable_functionalization",
+      [](bool reapply_views = false) {
+        if (c10::impl::tls_is_dispatch_key_included(
+                at::DispatchKey::Functionalize)) {
+          TORCH_INTERNAL_ASSERT(
+              false,
+              "multiple layers of mode-style functionalization nesting is not"
+              " currently supported, outside of the functionalize() transform");
+        }
+        c10::impl::tls_set_dispatch_key_included(
+            at::DispatchKey::Functionalize, true);
+        if (reapply_views) {
+          at::functionalization::impl::setFunctionalizationReapplyViewsTLS(
+              true);
+        }
+      },
+      py::arg("reapply_views") = false);
+  py_module.def("_disable_functionalization", []() {
+    c10::impl::tls_set_dispatch_key_included(
+        at::DispatchKey::Functionalize, false);
+    at::functionalization::impl::setFunctionalizationReapplyViewsTLS(false);
+  });
+  py_module.def(
+      "_mirror_autograd_meta_to",
+      [](const at::Tensor& src_, const at::Tensor& dst_) {
+        // Here, we unsafely set the grad function on the wrapper to be the same
+        // as the inner. We expect this grad_fn to NEVER be used. It's needed so
+        // that .is_leaf metadata is accurate on the wrapper
+        auto inner_autograd_meta = impl::get_autograd_meta(src_);
+        if (inner_autograd_meta) {
+          dst_.set_requires_grad(src_.requires_grad());
+          if (dst_.requires_grad()) {
+            auto new_grad_fn = std::shared_ptr<torch::autograd::Error>(
+                new torch::autograd::Error(
+                    "Cannot backprop through mirrored meta, file a bug in PyTorch"),
+                torch::autograd::deleteNode);
+            torch::autograd::set_history(dst_, new_grad_fn);
+          }
+        }
+      });
 }
 
-} // namespace autograd
-} // namespace torch
+} // namespace torch::autograd
