@@ -8,16 +8,33 @@
 
 namespace torch::cuda::CUDAPluggableAllocator {
 
+CUDAPluggableAllocatorDeleterContext::CUDAPluggableAllocatorDeleterContext(
+    std::function<FreeFuncType> free_fn,
+    void* data,
+    size_t size,
+    int device,
+    cudaStream_t stream)
+    : free_fn_(free_fn),
+      data_(data),
+      size_(size),
+      device_(device),
+      stream_(stream) {}
+
+void CUDAPluggableAllocatorDeleterContext::free() {
+  free_fn_(data_, size_, device_, stream_);
+  delete this;
+}
+
 int device_count = 0;
 
 void custom_raw_deleter(void* ptr);
 
 _AllocationMetadata::_AllocationMetadata()
-    : size(0), device_idx(-1), stream(0) {}
+    : size(0), device_idx(-1), stream{} {}
 
 _AllocationMetadata::_AllocationMetadata(
     size_t size,
-    int device_idx,
+    c10::DeviceIndex device_idx,
     cudaStream_t stream)
     : size(size), device_idx(device_idx), stream(stream) {}
 
@@ -26,9 +43,9 @@ _AllocationMetadata::_AllocationMetadata(
 // This avoids having to link against libtorch for C++ based custom allocators
 // And also use this from python
 CUDAPluggableAllocator::CUDAPluggableAllocator(
-    std::function<void*(size_t, int, cudaStream_t)> alloc_fn,
-    std::function<void(void*, size_t, int, cudaStream_t)> free_fn)
-    : alloc_fn_(alloc_fn), free_fn_(free_fn) {}
+    std::function<MallocFuncType> alloc_fn,
+    std::function<FreeFuncType> free_fn)
+    : alloc_fn_(std::move(alloc_fn)), free_fn_(std::move(free_fn)) {}
 
 CUDAPluggableAllocator::CUDAPluggableAllocator(CUDAPluggableAllocator& other)
     : alloc_fn_(other.alloc_fn_),
@@ -38,53 +55,53 @@ CUDAPluggableAllocator::CUDAPluggableAllocator(CUDAPluggableAllocator& other)
       memory_fraction_fn_(other.memory_fraction_fn_),
       base_alloc_fn_(other.base_alloc_fn_),
       record_stream_fn_(other.record_stream_fn_),
-      begin_allocate_stream_to_pool_fn_(
-          other.begin_allocate_stream_to_pool_fn_),
-      end_allocate_stream_to_pool_fn_(other.end_allocate_stream_to_pool_fn_),
+      begin_allocate_to_pool_fn_(other.begin_allocate_to_pool_fn_),
+      end_allocate_to_pool_fn_(other.end_allocate_to_pool_fn_),
       relase_pool_fn_(other.relase_pool_fn_) {}
 
 void CUDAPluggableAllocator::set_init_fn(std::function<void(int)> init_fn) {
-  init_fn_ = init_fn;
+  init_fn_ = std::move(init_fn);
 }
 
 void CUDAPluggableAllocator::set_reset_fn(std::function<void()> reset_fn) {
-  reset_fn_ = reset_fn;
+  reset_fn_ = std::move(reset_fn);
 }
 
 void CUDAPluggableAllocator::set_memory_fraction_fn(
     std::function<void(double, int)> memory_fraction_fn) {
-  memory_fraction_fn_ = memory_fraction_fn;
+  memory_fraction_fn_ = std::move(memory_fraction_fn);
 }
 
 void CUDAPluggableAllocator::set_base_alloc_fn(
     std::function<void*(void*, size_t*)> base_alloc_fn) {
-  base_alloc_fn_ = base_alloc_fn;
+  base_alloc_fn_ = std::move(base_alloc_fn);
 }
 
 void CUDAPluggableAllocator::set_record_stream_fn(
     std::function<void(void* ptr, cudaStream_t stream)> record_stream_fn) {
-  record_stream_fn_ = record_stream_fn;
+  record_stream_fn_ = std::move(record_stream_fn);
 }
 
-void CUDAPluggableAllocator::set_begin_allocate_stream_to_pool(
-    std::function<void(int, cudaStream_t, c10::cuda::MempoolId_t)>
+void CUDAPluggableAllocator::set_begin_allocate_to_pool(
+    std::function<
+        void(int, c10::cuda::MempoolId_t, std::function<bool(cudaStream_t)>)>
         capture_begin_fn) {
-  begin_allocate_stream_to_pool_fn_ = capture_begin_fn;
+  begin_allocate_to_pool_fn_ = std::move(capture_begin_fn);
 }
 
-void CUDAPluggableAllocator::set_end_allocate_stream_to_pool_fn(
-    std::function<void(int, cudaStream_t)> capture_about_to_end_fn) {
-  end_allocate_stream_to_pool_fn_ = capture_about_to_end_fn;
+void CUDAPluggableAllocator::set_end_allocate_to_pool_fn(
+    std::function<void(int, c10::cuda::MempoolId_t)> capture_about_to_end_fn) {
+  end_allocate_to_pool_fn_ = std::move(capture_about_to_end_fn);
 }
 
 void CUDAPluggableAllocator::set_release_pool(
     std::function<void(int, c10::cuda::MempoolId_t)> capture_destroy_fn) {
-  relase_pool_fn_ = capture_destroy_fn;
+  relase_pool_fn_ = std::move(capture_destroy_fn);
 }
 
 void* CUDAPluggableAllocator::malloc(
     size_t size,
-    int device,
+    c10::DeviceIndex device,
     cudaStream_t stream) {
   void* r = alloc_fn_(size, device, stream);
   {
@@ -94,14 +111,15 @@ void* CUDAPluggableAllocator::malloc(
   return r;
 }
 
-c10::DataPtr CUDAPluggableAllocator::allocate(size_t size) const {
-  int device;
+c10::DataPtr CUDAPluggableAllocator::allocate(size_t size) {
+  c10::DeviceIndex device = -1;
   C10_CUDA_CHECK(c10::cuda::GetDevice(&device));
   cudaStream_t stream = c10::cuda::getCurrentCUDAStream(device);
-  void* r =
-      const_cast<CUDAPluggableAllocator*>(this)->malloc(size, device, stream);
+  void* r = this->malloc(size, device, stream);
+  auto* ctx = new CUDAPluggableAllocatorDeleterContext(
+      free_fn_, r, size, device, stream);
   c10::DataPtr data_ptr = {
-      r, r, raw_deleter(), c10::Device(c10::DeviceType::CUDA, device)};
+      r, ctx, raw_deleter(), c10::Device(c10::DeviceType::CUDA, device)};
   return data_ptr;
 }
 
@@ -110,7 +128,7 @@ c10::DeleterFnPtr CUDAPluggableAllocator::raw_deleter() const {
 }
 
 void* CUDAPluggableAllocator::raw_alloc(size_t nbytes) {
-  int device;
+  c10::DeviceIndex device = -1;
   C10_CUDA_CHECK(c10::cuda::GetDevice(&device));
   cudaStream_t stream = c10::cuda::getCurrentCUDAStream(device);
   return malloc(nbytes, device, stream);
@@ -119,15 +137,15 @@ void* CUDAPluggableAllocator::raw_alloc(size_t nbytes) {
 void* CUDAPluggableAllocator::raw_alloc_with_stream(
     size_t nbytes,
     cudaStream_t stream) {
-  int device;
+  c10::DeviceIndex device = -1;
   C10_CUDA_CHECK(c10::cuda::GetDevice(&device));
   return malloc(nbytes, device, stream);
 }
 
 void CUDAPluggableAllocator::raw_delete(void* ptr) {
-  cudaStream_t stream;
-  int device_idx;
-  size_t size;
+  cudaStream_t stream{};
+  c10::DeviceIndex device_idx = -1;
+  size_t size = 0;
   {
     const std::lock_guard<std::mutex> lock(allocator_mutex_);
     TORCH_CHECK(
@@ -153,19 +171,23 @@ bool CUDAPluggableAllocator::initialized() {
   return initialized_;
 }
 
-void CUDAPluggableAllocator::setMemoryFraction(double fraction, int device) {
+void CUDAPluggableAllocator::setMemoryFraction(
+    double fraction,
+    c10::DeviceIndex device) {
   if (memory_fraction_fn_) {
     memory_fraction_fn_(fraction, device);
   }
 }
 
-void CUDAPluggableAllocator::emptyCache(void) {
+void CUDAPluggableAllocator::emptyCache() {
   if (reset_fn_) {
     return reset_fn_();
   }
 }
 
-void CUDAPluggableAllocator::cacheInfo(int dev_id, size_t* largestBlock) {
+void CUDAPluggableAllocator::cacheInfo(
+    c10::DeviceIndex device,
+    size_t* largestBlock) {
   TORCH_CHECK(
       false,
       "CUDAPluggableAllocator does not yet support cacheInfo. "
@@ -188,22 +210,22 @@ void CUDAPluggableAllocator::recordStream(
   }
 }
 
-c10::cuda::CUDACachingAllocator::DeviceStats CUDAPluggableAllocator::
-    getDeviceStats(int device) {
+c10::CachingDeviceAllocator::DeviceStats CUDAPluggableAllocator::getDeviceStats(
+    c10::DeviceIndex device) {
   TORCH_CHECK(
       false,
       "CUDAPluggableAllocator does not yet support getDeviceStats. "
       "If you need it, please file an issue describing your use case.");
 }
 
-void CUDAPluggableAllocator::resetAccumulatedStats(int device) {
+void CUDAPluggableAllocator::resetAccumulatedStats(c10::DeviceIndex device) {
   TORCH_CHECK(
       false,
       "CUDAPluggableAllocator does not yet support resetAccumulatedStats. "
       "If you need it, please file an issue describing your use case.");
 }
 
-void CUDAPluggableAllocator::resetPeakStats(int device) {
+void CUDAPluggableAllocator::resetPeakStats(c10::DeviceIndex device) {
   TORCH_CHECK(
       false,
       "CUDAPluggableAllocator does not yet support resetPeakStats. "
@@ -218,6 +240,14 @@ c10::cuda::CUDACachingAllocator::SnapshotInfo CUDAPluggableAllocator::
       "If you need it, please file an issue describing your use case.");
 }
 
+c10::cuda::CUDACachingAllocator::ShareableHandle CUDAPluggableAllocator::
+    shareIpcHandle(void* ptr) {
+  TORCH_CHECK(
+      false,
+      "CUDAPluggableAllocator does not yet support shareIPcHandle. "
+      "If you need it, please file an issue describing your use case.");
+}
+
 std::shared_ptr<void> CUDAPluggableAllocator::getIpcDevPtr(std::string handle) {
   TORCH_CHECK(
       false,
@@ -226,25 +256,25 @@ std::shared_ptr<void> CUDAPluggableAllocator::getIpcDevPtr(std::string handle) {
 }
 
 // CUDAGraph interactions
-void CUDAPluggableAllocator::beginAllocateStreamToPool(
-    int device,
-    cudaStream_t stream,
-    c10::cuda::MempoolId_t mempool_id) {
-  if (begin_allocate_stream_to_pool_fn_) {
-    begin_allocate_stream_to_pool_fn_(device, stream, mempool_id);
+void CUDAPluggableAllocator::beginAllocateToPool(
+    c10::DeviceIndex device,
+    c10::cuda::MempoolId_t mempool_id,
+    std::function<bool(cudaStream_t)> filter) {
+  if (begin_allocate_to_pool_fn_) {
+    begin_allocate_to_pool_fn_(device, mempool_id, std::move(filter));
   }
 }
 
-void CUDAPluggableAllocator::endAllocateStreamToPool(
-    int device,
-    cudaStream_t stream) {
-  if (end_allocate_stream_to_pool_fn_) {
-    end_allocate_stream_to_pool_fn_(device, stream);
+void CUDAPluggableAllocator::endAllocateToPool(
+    c10::DeviceIndex device,
+    c10::cuda::MempoolId_t mempool_id) {
+  if (end_allocate_to_pool_fn_) {
+    end_allocate_to_pool_fn_(device, mempool_id);
   }
 }
 
 void CUDAPluggableAllocator::releasePool(
-    int device,
+    c10::DeviceIndex device,
     c10::cuda::MempoolId_t mempool_id) {
   if (relase_pool_fn_) {
     relase_pool_fn_(device, mempool_id);
@@ -270,9 +300,17 @@ void CUDAPluggableAllocator::attachOutOfMemoryObserver(
       "If you need it, please file an issue describing your use case.");
 }
 
+void CUDAPluggableAllocator::attachAllocatorTraceTracker(
+    c10::cuda::CUDACachingAllocator::AllocatorTraceTracker tracker) {
+  TORCH_CHECK(
+      false,
+      "CUDAPluggableAllocator does not support attachAllocatorTraceTracker. "
+      "attachAllocatorTraceTracker is only used inside Pytorch.");
+}
+
 std::shared_ptr<c10::cuda::CUDACachingAllocator::AllocatorState>
 CUDAPluggableAllocator::getCheckpointState(
-    int device,
+    c10::DeviceIndex device,
     at::cuda::MempoolId_t id) {
   TORCH_CHECK(
       false,
@@ -282,7 +320,7 @@ CUDAPluggableAllocator::getCheckpointState(
 
 c10::cuda::CUDACachingAllocator::CheckpointDelta CUDAPluggableAllocator::
     setCheckpointPoolState(
-        int device,
+        c10::DeviceIndex device,
         std::shared_ptr<c10::cuda::CUDACachingAllocator::AllocatorState> pps) {
   TORCH_CHECK(
       false,
@@ -290,7 +328,9 @@ c10::cuda::CUDACachingAllocator::CheckpointDelta CUDAPluggableAllocator::
       "If you need it, please file an issue describing your use case.");
 }
 
-void CUDAPluggableAllocator::enablePeerAccess(int dev, int dev_to_access) {
+void CUDAPluggableAllocator::enablePeerAccess(
+    c10::DeviceIndex dev,
+    c10::DeviceIndex dev_to_access) {
   c10::cuda::CUDAGuard device_guard(dev);
   cudaError_t err = cudaDeviceEnablePeerAccess(dev_to_access, 0);
   if (err == cudaErrorPeerAccessAlreadyEnabled) {
@@ -316,6 +356,14 @@ std::string CUDAPluggableAllocator::name() {
   return "pluggable";
 }
 
+void CUDAPluggableAllocator::copy_data(
+    void* dest,
+    const void* src,
+    std::size_t count) const {
+  C10_CUDA_CHECK(
+      cudaMemcpy(dest, src, count, cudaMemcpyKind::cudaMemcpyDeviceToDevice));
+}
+
 std::shared_ptr<c10::cuda::CUDACachingAllocator::CUDAAllocator>
     current_custom_allocator;
 
@@ -327,16 +375,17 @@ getCurrentAllocator() {
 // TODO: add more functions in the argument
 std::shared_ptr<c10::cuda::CUDACachingAllocator::CUDAAllocator>
 createCustomAllocator(
-    std::function<void*(size_t, int, cudaStream_t)> alloc_fn,
-    std::function<void(void*, size_t, int, cudaStream_t)> free_fn) {
+    std::function<MallocFuncType> alloc_fn,
+    std::function<FreeFuncType> free_fn) {
   std::shared_ptr<CUDAPluggableAllocator> allocator(
-      new CUDAPluggableAllocator(alloc_fn, free_fn));
+      new CUDAPluggableAllocator(std::move(alloc_fn), std::move(free_fn)));
   allocator->init(device_count);
   return allocator;
 }
 
 void changeCurrentAllocator(
-    std::shared_ptr<c10::cuda::CUDACachingAllocator::CUDAAllocator> allocator) {
+    const std::shared_ptr<c10::cuda::CUDACachingAllocator::CUDAAllocator>&
+        allocator) {
   TORCH_CHECK(
       !c10::cuda::CUDACachingAllocator::allocator.load()->initialized(),
       "Can't swap an already initialized allocator");
@@ -344,8 +393,8 @@ void changeCurrentAllocator(
   current_custom_allocator = allocator;
 }
 
-void custom_raw_deleter(void* ptr) {
-  current_custom_allocator->raw_delete(ptr);
+void custom_raw_deleter(void* ctx) {
+  reinterpret_cast<CUDAPluggableAllocatorDeleterContext*>(ctx)->free();
 }
 
 } // namespace torch::cuda::CUDAPluggableAllocator
