@@ -8,6 +8,7 @@
 #include <utility>
 #include <vector>
 
+#include <c10/util/thread_name.h>
 #include <fmt/format.h>
 #include <torch/csrc/distributed/c10d/TCPStore.hpp>
 #include <torch/csrc/distributed/c10d/TCPStoreBackend.hpp>
@@ -30,7 +31,10 @@ Other callbacks don't provide exception safety so avoid there.
 
 */
 
-#define DEFAULT_BACKLOG 16384
+// This controls how many un-accepted TCP connections can be waiting in the
+// backlog. This should be at least world size to avoid issues on init. We set
+// it to -1 to use the host max value which is controlled by `soconnmax`.
+#define DEFAULT_BACKLOG -1
 #define MAX_KEY_COUNT (128 * 1024)
 #define MAX_STRING_LEN (8 * 1024)
 #define MAX_PAYLOAD_LEN (8 * 1024 * 1024)
@@ -124,7 +128,7 @@ class UvTcpSocket : public UvHandle {
       try {
         uv_socket->processBuf(buf, nread);
       } catch (std::exception& ex) {
-        C10D_INFO("Error processing client message: {}", ex.what());
+        C10D_WARNING("Error processing client message: {}", ex.what());
         uv_socket->close();
       }
     }
@@ -133,12 +137,17 @@ class UvTcpSocket : public UvHandle {
  public:
   explicit UvTcpSocket(uv_loop_t* loop) {
     uv_tcp_init(loop, &client);
+    if (int err = uv_tcp_nodelay(&client, 1)) {
+      C10D_WARNING(
+          "The no-delay option cannot be enabled for the client socket. err={}",
+          err);
+    }
   }
 
   void startRead() {
     int res = uv_read_start((uv_stream_t*)&client, alloc_buffer, read_callback);
     if (res) {
-      C10D_INFO(
+      C10D_WARNING(
           "Failed to setup read callback. client:{} code:{} name:{} desc:{}.",
           (void*)this,
           res,
@@ -227,7 +236,9 @@ class UvTcpServer : public UvTcpSocket {
       TORCH_CHECK(
           uv_res == 0,
           "UV Store addr parsing failure. ",
-          "useIpv6: ",
+          "port: ",
+          port,
+          ", useIpv6: ",
           useIpv6,
           ", code: ",
           uv_res,
@@ -241,7 +252,9 @@ class UvTcpServer : public UvTcpSocket {
       TORCH_CHECK(
           uv_res == 0,
           "The server socket has failed to bind. ",
-          "useIpv6: ",
+          "port: ",
+          port,
+          ", useIpv6: ",
           useIpv6,
           ", code: ",
           uv_res,
@@ -255,7 +268,9 @@ class UvTcpServer : public UvTcpSocket {
       TORCH_CHECK(
           uv_res == 0,
           "The server socket has failed to listen on any local network address. ",
-          "useIpv6: ",
+          "port: ",
+          port,
+          ", useIpv6: ",
           useIpv6,
           ", code: ",
           uv_res,
@@ -356,7 +371,7 @@ class WriterPayload : public c10::intrusive_ptr_target {
     auto handle = wp->handle;
 
     if (status) {
-      C10D_INFO(
+      C10D_WARNING(
           "Write to client failed. code:{} name:{} desc:{}.",
           status,
           uv_err_name(status),
@@ -386,7 +401,7 @@ class WriterPayload : public c10::intrusive_ptr_target {
         &req, (uv_stream_t*)handle->unsafeGetHandle(), &buf, 1, write_done);
 
     if (res) {
-      C10D_INFO(
+      C10D_WARNING(
           "Write setup to client failed. code:{} name:{} desc:{}.",
           res,
           uv_err_name(res),
@@ -669,6 +684,10 @@ class UvClient : public UvTcpSocket {
           return;
       } else {
         switch ((QueryType)command) {
+          case QueryType::PING:
+            if (!parse_ping_command())
+              return;
+            break;
           case QueryType::SET:
             if (!parse_set_command())
               return;
@@ -740,6 +759,18 @@ class UvClient : public UvTcpSocket {
     return true;
   }
 
+  bool parse_ping_command() {
+    uint32_t nonce;
+    if (!stream.read_value(nonce)) {
+      return false;
+    }
+
+    StreamWriter sw(iptr());
+    sw.write_value(nonce);
+    sw.send();
+    return true;
+  }
+
   bool parse_set_command() {
     std::string key;
     if (!stream.read_key(key))
@@ -993,7 +1024,7 @@ void LibUVStoreDaemon::onConnect(int status) {
     tcpServer->accept(client);
     client->startRead();
   } catch (std::exception& e) {
-    C10D_INFO("Failed to accept client due to {}", e.what());
+    C10D_WARNING("Failed to accept client due to {}", e.what());
     client->close();
   }
 }
@@ -1064,6 +1095,8 @@ void LibUVStoreDaemon::print_active_handles(uv_handle_t* handle, void* arg) {
 }
 
 void LibUVStoreDaemon::run() {
+  c10::setThreadName("pt_tcpstore_uv");
+
   C10D_DEBUG("Uv main loop running");
   int res = uv_run(&loop, UV_RUN_DEFAULT);
   if (res) {
@@ -1108,7 +1141,7 @@ void LibUVStoreDaemon::run() {
 void LibUVStoreDaemon::stop() {
   int res = uv_async_send(&exit_handle);
   if (res) {
-    C10D_INFO(
+    C10D_WARNING(
         "uv_async_send failed with:{} errn:{} desc:{}\n",
         res,
         uv_err_name(res),
