@@ -1,10 +1,7 @@
 #include <torch/csrc/distributed/rpc/metrics/RpcMetricsHandler.h> // @manual
-namespace torch {
-namespace distributed {
-namespace rpc {
+
+namespace torch::distributed::rpc {
 C10_DEFINE_REGISTRY(
     RpcMetricsHandlerRegistry,
     torch::distributed::rpc::RpcMetricsHandler);
-} // namespace rpc
-} // namespace distributed
-} // namespace torch
+} // namespace torch::distributed::rpc
