@@ -2,9 +2,7 @@
 
 #include <torch/csrc/distributed/rpc/python_rpc_handler.h>
 
-namespace torch {
-namespace distributed {
-namespace rpc {
+namespace torch::distributed::rpc {
 
 UnpickledPythonCall::UnpickledPythonCall(
     const SerializedPyObj& serializedPyObj,
@@ -33,6 +31,4 @@ const py::object& UnpickledPythonCall::pythonUdf() const {
   return pythonUdf_;
 }
 
-} // namespace rpc
-} // namespace distributed
-} // namespace torch
+} // namespace torch::distributed::rpc
