@@ -1,52 +1,28 @@
 #include <torch/csrc/dynamo/cpython_defs.h>
-
-#ifdef _WIN32
-#define unlikely(x) (x)
-#else
-#define unlikely(x) __builtin_expect((x), 0)
-#endif
-
-#define CHECK(cond)                                                     \
-  if (unlikely(!(cond))) {                                              \
-    fprintf(stderr, "DEBUG CHECK FAILED: %s:%d\n", __FILE__, __LINE__); \
-    abort();                                                            \
-  } else {                                                              \
-  }
-
-#if IS_PYTHON_3_13_PLUS
-// Gave up after fixing a few of these
-// pycore_opcode.h is gone (new is pycore_opcode_metadata.h ?)
-// f_code is gone (new is f_executable?)
-
-// Fake definitions for what we removed
-const uint8_t* THP_PyOpcode_Caches = NULL;
-const int THP_PyOpcode_Caches_size = 0;
-
-#else
-
-// NOTE: all `assert`s below are converted to `CHECK`s
+#include <torch/csrc/dynamo/cpython_includes.h>
+#include <torch/csrc/dynamo/debug_macros.h>
 
 #if IS_PYTHON_3_11_PLUS
 
-// Problem in CPython includes when mixing core and non-core build
-// The fix was not backported to 3.12 so this is needed here
-// https://github.com/python/cpython/issues/105268
-#if IS_PYTHON_3_12_PLUS
-#undef _PyGC_FINALIZED
+#define Py_BUILD_CORE
+#define NEED_OPCODE_TABLES // To get _PyOpcode_Deopt, _PyOpcode_Caches
+
+#if IS_PYTHON_3_13_PLUS
+#include <cpython/code.h> // To get PyUnstable_Code_GetFirstFree
+#define NEED_OPCODE_METADATA
+#include <internal/pycore_opcode_metadata.h>
+#undef NEED_OPCODE_METADATA
+#else
+#include <internal/pycore_opcode.h>
 #endif
 
-#define Py_BUILD_CORE
-#include <internal/pycore_pystate.h>
-#define NEED_OPCODE_TABLES // To get _PyOpcode_Deopt, _PyOpcode_Caches
-#include <internal/pycore_opcode.h>
 #undef NEED_OPCODE_TABLES
-#include <internal/pycore_frame.h>
 #undef Py_BUILD_CORE
 
 // As a simple way to reduce the impact of ABI changes on the CPython side, this check forces
 // us to manually re-check that the function didn't change on the next major version
-#if PY_VERSION_HEX >= 0x030D0000 // 3.13
-#error "Please ensure that the functions below still match the CPython implementation for 3.13"
+#if IS_PYTHON_3_14_PLUS
+#error "Please ensure that the functions below still match the CPython implementation for 3.14"
 #endif
 
 // https://github.com/python/cpython/blob/a7715ccfba5b86ab09f86ec56ac3755c93b46b48/Objects/frameobject.c#L1079
@@ -56,8 +32,8 @@ THP_PyFrame_OpAlreadyRan(_PyInterpreterFrame *frame, int opcode, int oparg)
     // This only works when opcode is a non-quickened form:
     CHECK(_PyOpcode_Deopt[opcode] == opcode);
     int check_oparg = 0;
-    for (_Py_CODEUNIT *instruction = _PyCode_CODE(frame->f_code);
-         instruction < frame->prev_instr; instruction++)
+    for (_Py_CODEUNIT *instruction = _PyCode_CODE(F_CODE(frame));
+         instruction < PREV_INSTR(frame) ; instruction++)
     {
         int check_opcode = _PyOpcode_Deopt[_Py_OPCODE(*instruction)];
         check_oparg |= _Py_OPARG(*instruction);
@@ -77,185 +53,10 @@ THP_PyFrame_OpAlreadyRan(_PyInterpreterFrame *frame, int opcode, int oparg)
 
 #if IS_PYTHON_3_12_PLUS
 
-// https://github.com/python/cpython/blob/0325a8a8cdba6c091bcbbb3c995f3bf1d1217012/Objects/frameobject.c#L1136
-// Initialize frame free variables if needed
-// free_vars_copied argument added in order to let caller know that the COPY_FREE_VARS
-// codepath occurred.
-static void
-frame_init_get_vars(_PyInterpreterFrame *frame, int *free_vars_copied)
-{
-    // COPY_FREE_VARS has no quickened forms, so no need to use _PyOpcode_Deopt
-    // here:
-    PyCodeObject *co = frame->f_code;
-    int lasti = _PyInterpreterFrame_LASTI(frame);
-    if (!(lasti < 0 && _PyCode_CODE(co)->op.code == COPY_FREE_VARS
-          && PyFunction_Check(frame->f_funcobj)))
-    {
-        /* Free vars are initialized */
-        return;
-    }
-
-    /* Free vars have not been initialized -- Do that */
-    PyObject *closure = ((PyFunctionObject *)frame->f_funcobj)->func_closure;
-    int offset = PyCode_GetFirstFree(co);
-    for (int i = 0; i < co->co_nfreevars; ++i) {
-        PyObject *o = PyTuple_GET_ITEM(closure, i);
-        frame->localsplus[offset + i] = Py_NewRef(o);
-    }
-    // COPY_FREE_VARS doesn't have inline CACHEs, either:
-    frame->prev_instr = _PyCode_CODE(frame->f_code);
-
-    *free_vars_copied = 1;
-}
-
-// https://github.com/python/cpython/blob/0325a8a8cdba6c091bcbbb3c995f3bf1d1217012/Objects/frameobject.c#L1162
-static int
-frame_get_var(_PyInterpreterFrame *frame, PyCodeObject *co, int i,
-              PyObject **pvalue)
-{
-    _PyLocals_Kind kind = _PyLocals_GetKind(co->co_localspluskinds, i);
-
-    /* If the namespace is unoptimized, then one of the
-       following cases applies:
-       1. It does not contain free variables, because it
-          uses import * or is a top-level namespace.
-       2. It is a class namespace.
-       We don't want to accidentally copy free variables
-       into the locals dict used by the class.
-    */
-    if (kind & CO_FAST_FREE && !(co->co_flags & CO_OPTIMIZED)) {
-        return 0;
-    }
-
-    PyObject *value = frame->localsplus[i];
-    if (frame->stacktop) {
-        if (kind & CO_FAST_FREE) {
-            // The cell was set by COPY_FREE_VARS.
-            CHECK(value != NULL && PyCell_Check(value));
-            value = PyCell_GET(value);
-        }
-        else if (kind & CO_FAST_CELL) {
-            // Note that no *_DEREF ops can happen before MAKE_CELL
-            // executes.  So there's no need to duplicate the work
-            // that MAKE_CELL would otherwise do later, if it hasn't
-            // run yet.
-            if (value != NULL) {
-                if (PyCell_Check(value) &&
-                        THP_PyFrame_OpAlreadyRan(frame, MAKE_CELL, i)) {
-                    // (likely) MAKE_CELL must have executed already.
-                    value = PyCell_GET(value);
-                }
-                // (likely) Otherwise it it is an arg (kind & CO_FAST_LOCAL),
-                // with the initial value set when the frame was created...
-                // (unlikely) ...or it was set to some initial value by
-                // an earlier call to PyFrame_LocalsToFast().
-            }
-        }
-    }
-    else {
-        CHECK(value == NULL);
-    }
-    *pvalue = value;
-    return 1;
-}
-
-// https://github.com/python/cpython/blob/0325a8a8cdba6c091bcbbb3c995f3bf1d1217012/Objects/frameobject.c#L1213
-static PyObject *
-THP_PyFrame_GetLocals(_PyInterpreterFrame *frame, int include_hidden, int *free_vars_copied)
-{
-    /* Merge fast locals into f->f_locals */
-    PyObject *locals = frame->f_locals;
-    if (locals == NULL) {
-        locals = frame->f_locals = PyDict_New();
-        if (locals == NULL) {
-            return NULL;
-        }
-    }
-    PyObject *hidden = NULL;
-
-    /* If include_hidden, "hidden" fast locals (from inlined comprehensions in
-       module/class scopes) will be included in the returned dict, but not in
-       frame->f_locals; the returned dict will be a modified copy. Non-hidden
-       locals will still be updated in frame->f_locals. */
-    if (include_hidden) {
-        hidden = PyDict_New();
-        if (hidden == NULL) {
-            return NULL;
-        }
-    }
-
-    frame_init_get_vars(frame, free_vars_copied);
-
-    PyCodeObject *co = frame->f_code;
-    for (int i = 0; i < co->co_nlocalsplus; i++) {
-        PyObject *value;  // borrowed reference
-        if (!frame_get_var(frame, co, i, &value)) {
-            continue;
-        }
-
-        PyObject *name = PyTuple_GET_ITEM(co->co_localsplusnames, i);
-        _PyLocals_Kind kind = _PyLocals_GetKind(co->co_localspluskinds, i);
-        if (kind & CO_FAST_HIDDEN) {
-            if (include_hidden && value != NULL) {
-                if (PyObject_SetItem(hidden, name, value) != 0) {
-                    goto error;
-                }
-            }
-            continue;
-        }
-        if (value == NULL) {
-            if (PyObject_DelItem(locals, name) != 0) {
-                if (PyErr_ExceptionMatches(PyExc_KeyError)) {
-                    PyErr_Clear();
-                }
-                else {
-                    goto error;
-                }
-            }
-        }
-        else {
-            if (PyObject_SetItem(locals, name, value) != 0) {
-                goto error;
-            }
-        }
-    }
-
-    if (include_hidden && PyDict_Size(hidden)) {
-        PyObject *innerlocals = PyDict_New();
-        if (innerlocals == NULL) {
-            goto error;
-        }
-        if (PyDict_Merge(innerlocals, locals, 1) != 0) {
-            Py_DECREF(innerlocals);
-            goto error;
-        }
-        if (PyDict_Merge(innerlocals, hidden, 1) != 0) {
-            Py_DECREF(innerlocals);
-            goto error;
-        }
-        locals = innerlocals;
-    }
-    else {
-        Py_INCREF(locals);
-    }
-    Py_CLEAR(hidden);
-
-    return locals;
-
-  error:
-    Py_XDECREF(hidden);
-    return NULL;
-}
-
-// https://github.com/python/cpython/blob/0325a8a8cdba6c091bcbbb3c995f3bf1d1217012/Objects/frameobject.c#L1301
 int
 THP_PyFrame_FastToLocalsWithError(_PyInterpreterFrame *frame, int *free_vars_copied)
 {
-    PyObject *locals = THP_PyFrame_GetLocals(frame, 0, free_vars_copied);
-    if (locals == NULL) {
-        return -1;
-    }
-    Py_DECREF(locals);
+    // functionality moved to framelocals_mapping.cpp
     return 0;
 }
 
@@ -276,14 +77,14 @@ THP_PyFrame_FastToLocalsWithError(_PyInterpreterFrame *frame, int *free_vars_cop
         if (locals == NULL)
             return -1;
     }
-    co = frame->f_code;
+    co = F_CODE(frame);
     fast = _PyFrame_GetLocalsArray(frame);
     // COPY_FREE_VARS has no quickened forms, so no need to use _PyOpcode_Deopt
     // here:
     int lasti = _PyInterpreterFrame_LASTI(frame);
     if (lasti < 0 && _Py_OPCODE(_PyCode_CODE(co)[0]) == COPY_FREE_VARS) {
         /* Free vars have not been initialized -- Do that */
-        PyCodeObject *co = frame->f_code;
+        PyCodeObject *co = F_CODE(frame);
         PyObject *closure = frame->f_func->func_closure;
         int offset = co->co_nlocals + co->co_nplaincellvars;
         for (int i = 0; i < co->co_nfreevars; ++i) {
@@ -292,7 +93,7 @@ THP_PyFrame_FastToLocalsWithError(_PyInterpreterFrame *frame, int *free_vars_cop
             frame->localsplus[offset + i] = o;
         }
         // COPY_FREE_VARS doesn't have inline CACHEs, either:
-        frame->prev_instr = _PyCode_CODE(frame->f_code);
+        PREV_INSTR(frame) = _PyCode_CODE(F_CODE(frame));
 
         *free_vars_copied = 1;
     }
@@ -422,7 +223,11 @@ THP_PyFrame_New_NoTrack(const PyCodeObject *code)
     f->f_trace = NULL;
     f->f_trace_lines = 1;
     f->f_trace_opcodes = 0;
+#if IS_PYTHON_3_13_PLUS
+    f->f_extra_locals = NULL;
+#else
     f->f_fast_as_locals = 0;
+#endif
     f->f_lineno = 0;
     return f;
 }
@@ -435,7 +240,7 @@ THP_PyFrame_MakeAndSetFrameObject(_PyInterpreterFrame *frame)
     PyObject *error_type = NULL, *error_value = NULL, *error_traceback = NULL;
     PyErr_Fetch(&error_type, &error_value, &error_traceback);
 
-    PyFrameObject *f = THP_PyFrame_New_NoTrack(frame->f_code);
+    PyFrameObject *f = THP_PyFrame_New_NoTrack(F_CODE(frame));
     if (f == NULL) {
         Py_XDECREF(error_type);
         Py_XDECREF(error_value);
@@ -495,8 +300,8 @@ THP_take_ownership(PyFrameObject *f, _PyInterpreterFrame *frame)
     if (_PyFrame_IsIncomplete(frame)) {
         // This may be a newly-created generator or coroutine frame. Since it's
         // dead anyways, just pretend that the first RESUME ran:
-        PyCodeObject *code = frame->f_code;
-        frame->prev_instr = _PyCode_CODE(code) + code->_co_firsttraceable;
+        PyCodeObject *code = F_CODE(frame);
+        PREV_INSTR(frame) = _PyCode_CODE(code) + code->_co_firsttraceable;
     }
     CHECK(!_PyFrame_IsIncomplete(frame));
     CHECK(f->f_back == NULL);
@@ -534,7 +339,11 @@ THP_PyFrame_Clear(_PyInterpreterFrame *frame)
         _PyFrame_GetGenerator(frame)->gi_frame_state == FRAME_CLEARED);
     // GH-99729: Clearing this frame can expose the stack (via finalizers). It's
     // crucial that this frame has been unlinked, and is no longer visible:
+#if IS_PYTHON_3_13_PLUS
+    CHECK(_PyThreadState_GET()->current_frame != frame);
+#else
     CHECK(_PyThreadState_GET()->cframe->current_frame != frame);
+#endif
     if (frame->frame_obj) {
         PyFrameObject *f = frame->frame_obj;
         frame->frame_obj = NULL;
@@ -557,7 +366,7 @@ THP_PyFrame_Clear(_PyInterpreterFrame *frame)
     #else
     Py_DECREF(frame->f_func);
     #endif
-    Py_DECREF(frame->f_code);
+    Py_DECREF(F_CODE(frame));
 }
 
 // https://github.com/python/cpython/blob/fad48ea1816be3125ea51edcdfe2f999d6ade796/Objects/obmalloc.c#L635
@@ -688,5 +497,3 @@ const uint8_t* THP_PyOpcode_Caches = NULL;
 const int THP_PyOpcode_Caches_size = 0;
 
 #endif
-
-#endif // CPython 3.13
\ No newline at end of file
