@@ -1,6 +1,5 @@
 #pragma once
 #include <c10/macros/Macros.h>
-#include <c10/util/C++17.h>
 #include <c10/util/Exception.h>
 #include <torch/csrc/Export.h>
 #include <torch/csrc/jit/frontend/parser_constants.h>
@@ -14,13 +13,7 @@
 #include <string>
 #include <vector>
 
-C10_CLANG_DIAGNOSTIC_PUSH()
-#if C10_CLANG_HAS_WARNING("-Wshorten-64-to-32")
-C10_CLANG_DIAGNOSTIC_IGNORE("-Wshorten-64-to-32")
-#endif
-
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
 // single character tokens are just the character itself '+'
 // multi-character tokens need an entry here
@@ -144,7 +137,7 @@ TORCH_API int stringToKind(const std::string& str);
 struct TokenTrie;
 using TokenTrieRef = std::unique_ptr<TokenTrie>;
 struct TokenTrie {
-  TokenTrie() : kind(0) {}
+  TokenTrie() = default;
   void insert(const char* str, int tok) {
     if (*str == '\0') {
       AT_ASSERT(kind == 0);
@@ -163,7 +156,7 @@ struct TokenTrie {
     child_tries.emplace_back(std::make_unique<TokenTrie>());
     child_tries.back()->insert(str + 1, tok);
   }
-  int kind; // 0 == invalid token
+  int kind{0}; // 0 == invalid token
 
   std::vector<char> child_chars;
   std::vector<TokenTrieRef> child_tries;
@@ -173,7 +166,6 @@ struct TokenTrie {
 // once.
 struct TORCH_API SharedParserData {
   SharedParserData() : head(new TokenTrie()) {
-    std::stringstream ss;
     for (const char* c = valid_single_char_tokens; *c; c++) {
       std::string str(1, *c);
       head->insert(str.c_str(), *c);
@@ -241,7 +233,7 @@ struct TORCH_API SharedParserData {
     // invariant: the next token is not whitespace or newline
     *start = pos;
     // check for a valid number
-    size_t len;
+    size_t len = 0;
     if (isNumber(pos.rest_line(), 0, &len)) {
       *end = *start;
       *end += len;
@@ -323,8 +315,7 @@ struct TORCH_API SharedParserData {
     if (first == '-' || first == '+' || isalpha(first))
       return false;
     const char* startptr = str.data() + start;
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-    char* endptr;
+    char* endptr = nullptr;
     torch::jit::strtod_c(startptr, &endptr);
     *len = endptr - startptr;
     // check if the number is complex valued
@@ -388,7 +379,7 @@ struct TORCH_API SharedParserData {
   }
 
   // Make an exception ignoring comments for type annotation comments
-  bool isTypeComment(StringCordView str, size_t pos) {
+  bool isTypeComment(const StringCordView& str, size_t pos) {
     const std::string type_string = "# type:";
     if (str.size() < pos + type_string.length()) {
       return false;
@@ -417,8 +408,7 @@ struct Token {
 struct Lexer {
   explicit Lexer(std::shared_ptr<Source> source)
       : source(std::move(source)),
-        pos(0),
-        nesting(0),
+
         indent_stack(),
         next_tokens(),
         shared(sharedParserData()) {
@@ -428,11 +418,11 @@ struct Lexer {
   }
   // Return the current token, and then move to the next one
   Token next() {
-    if (next_tokens.size() == 0)
+    if (next_tokens.empty())
       reportError("Lexer invariant violated: empty token queue");
     Token r = std::move(next_tokens.front());
     next_tokens.erase(next_tokens.begin());
-    if (next_tokens.size() == 0) {
+    if (next_tokens.empty()) {
       lex();
     }
     return r;
@@ -498,9 +488,8 @@ struct Lexer {
         break;
       case TK_WHITESPACE:
       case TK_WHITESPACE_EOF: {
-        const auto depth = static_cast<int64_t>(
-            r.kind == TK_WHITESPACE_EOF ? indent_stack.front()
-                                        : r.range.size());
+        const auto depth =
+            r.kind == TK_WHITESPACE_EOF ? indent_stack.front() : r.range.size();
         // note: TK_WHITESPACE_EOF is whitespace right before the EOF token
         // just like we allow the code to be indented to a particular initial
         // indent level, we allow the final indent to be anything and set
@@ -517,9 +506,8 @@ struct Lexer {
           while (indent_stack.back() != depth) {
             indent_stack.pop_back();
             next_tokens.emplace_back(TK_DEDENT, r.range);
-            if (indent_stack.size() == 0) {
-              reportError(
-                  "invalid indent level " + c10::guts::to_string(depth), r);
+            if (indent_stack.empty()) {
+              reportError("invalid indent level " + std::to_string(depth), r);
             }
           }
           return; // We've already queued the tokens
@@ -531,8 +519,6 @@ struct Lexer {
     next_tokens.push_back(std::move(r));
   }
   Token lexRaw(bool whitespace_token = false) {
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-    int kind;
     AT_ASSERT(source);
     if (current == nullptr) {
       AT_ASSERT(pos == 0);
@@ -542,6 +528,7 @@ struct Lexer {
 
     StringCordView::Iterator start_iter = *current;
     StringCordView::Iterator end_iter = *current;
+    int kind = 0;
     if (!shared.match(
             *current,
             nesting > 0,
@@ -564,14 +551,12 @@ struct Lexer {
 
   std::shared_ptr<Source> source;
   std::unique_ptr<StringCordView::Iterator> current;
-  size_t pos;
-  size_t nesting; // depth of ( [ { nesting...
-  std::vector<int> indent_stack; // stack of indentation level of blocks
+  size_t pos{0};
+  size_t nesting{0}; // depth of ( [ { nesting...
+  std::vector<size_t> indent_stack; // stack of indentation level of blocks
   // Invariant: this should always contain at least a single element
   std::vector<Token> next_tokens;
+  // NOLINTNEXTLINE(cppcoreguidelines-avoid-const-or-ref-data-members)
   SharedParserData& shared;
 };
-} // namespace jit
-} // namespace torch
-
-C10_CLANG_DIAGNOSTIC_POP()
+} // namespace torch::jit
