@@ -7,8 +7,7 @@
 
 #include <string>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
 using caffe2::serialize::IStreamAdapter;
 using caffe2::serialize::PyTorchStreamWriter;
@@ -87,5 +86,4 @@ bool _backport_for_mobile_impl(
   return backportManager.backport(oss, writer, from_version, to_version);
 }
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
