@@ -7,12 +7,7 @@
 #include <string>
 #include <unordered_map>
 
-namespace torch {
-namespace jit {
-
-namespace onnx {
-using namespace ::c10::onnx;
-}
+namespace torch::jit {
 
 // Meyer’s Singleton for C++ 14
 ConstantValueMap& ConstantValueMap::getInstance() {
@@ -32,13 +27,30 @@ bool ConstantValueMap::HasRank(const std::string& tensorName) {
       ConstantValueMap::getInstance().rankMap.end();
 }
 
-c10::optional<size_t> ConstantValueMap::GetRank(const std::string& tensorName) {
+std::optional<size_t> ConstantValueMap::GetRank(const std::string& tensorName) {
   if (!HasRank(tensorName)) {
-    return c10::nullopt;
+    return std::nullopt;
   }
   return ConstantValueMap::getInstance().rankMap[tensorName];
 }
 
+void ConstantValueMap::SetAllGraphInputsStatic(bool all_static) {
+  ConstantValueMap::getInstance().allGraphInputsStatic =
+      std::make_optional(all_static);
+}
+
+std::optional<bool> ConstantValueMap::GetAllGraphInputsStatic() {
+  return ConstantValueMap::getInstance().allGraphInputsStatic;
+}
+
+void ConstantValueMap::SetAllGraphInputsReliableComputed(bool computed) {
+  ConstantValueMap::getInstance().allGraphInputsReliableComputed = computed;
+}
+
+bool ConstantValueMap::GetAllGraphInputsReliableComputed() {
+  return ConstantValueMap::getInstance().allGraphInputsReliableComputed;
+}
+
 void ConstantValueMap::SetShape(
     const std::string& tensorName,
     const c10::SymbolicShape& shapeValue) {
@@ -51,10 +63,10 @@ bool ConstantValueMap::HasShape(const std::string& tensorName) {
       ConstantValueMap::getInstance().shapeMap.end();
 }
 
-c10::optional<c10::SymbolicShape> ConstantValueMap::GetShape(
+std::optional<c10::SymbolicShape> ConstantValueMap::GetShape(
     const std::string& tensorName) {
   if (!HasShape(tensorName)) {
-    return c10::nullopt;
+    return std::nullopt;
   }
   return ConstantValueMap::getInstance().shapeMap[tensorName];
 }
@@ -70,10 +82,10 @@ bool ConstantValueMap::HasValue(const std::string& tensorName) {
       ConstantValueMap::getInstance().tensorValueMap.end();
 }
 
-c10::optional<at::Tensor> ConstantValueMap::GetValue(
+std::optional<at::Tensor> ConstantValueMap::GetValue(
     const std::string& tensorName) {
   if (!HasValue(tensorName)) {
-    return c10::nullopt;
+    return std::nullopt;
   }
   return ConstantValueMap::getInstance().tensorValueMap[tensorName];
 }
@@ -94,7 +106,7 @@ std::vector<int64_t> ConstantValueMap::GetCompleteShapeInto1DInt64Vector(
   return shape_value;
 }
 
-c10::optional<std::vector<int64_t>> ConstantValueMap::GetShapeInto1DInt64Vector(
+std::optional<std::vector<int64_t>> ConstantValueMap::GetShapeInto1DInt64Vector(
     const std::string& value_name) {
   if (ConstantValueMap::HasShape(value_name)) {
     auto shape_size = ConstantValueMap::GetShape(value_name).value();
@@ -104,10 +116,10 @@ c10::optional<std::vector<int64_t>> ConstantValueMap::GetShapeInto1DInt64Vector(
       return shape_value;
     }
   }
-  return c10::nullopt;
+  return std::nullopt;
 }
 
-c10::optional<std::vector<int64_t>> ConstantValueMap::
+std::optional<std::vector<int64_t>> ConstantValueMap::
     GetShapeInto1DInt64VectorWithOneUnknown(const std::string& value_name) {
   if (ConstantValueMap::HasShape(value_name)) {
     auto shape_size = ConstantValueMap::GetShape(value_name).value();
@@ -135,7 +147,7 @@ c10::optional<std::vector<int64_t>> ConstantValueMap::
       }
     }
   }
-  return c10::nullopt;
+  return std::nullopt;
 }
 
 // accessor<int64_t, 1> for 1DInt64 case.
@@ -163,10 +175,10 @@ bool ConstantValueMap::HasTypeReliable(const std::string& tensorName) {
       ConstantValueMap::getInstance().typeReliableMap.end();
 }
 
-c10::optional<bool> ConstantValueMap::GetTypeReliable(
+std::optional<bool> ConstantValueMap::GetTypeReliable(
     const std::string& tensorName) {
   if (!HasTypeReliable(tensorName)) {
-    return c10::nullopt;
+    return std::nullopt;
   }
   return ConstantValueMap::getInstance().typeReliableMap[tensorName];
 }
@@ -182,10 +194,10 @@ bool ConstantValueMap::HasUseInferredType(const std::string& tensorName) {
       ConstantValueMap::getInstance().useInferredTypeMap.end();
 }
 
-c10::optional<bool> ConstantValueMap::GetUseInferredType(
+std::optional<bool> ConstantValueMap::GetUseInferredType(
     const std::string& tensorName) {
   if (!HasUseInferredType(tensorName)) {
-    return c10::nullopt;
+    return std::nullopt;
   }
   return ConstantValueMap::getInstance().useInferredTypeMap[tensorName];
 }
@@ -201,10 +213,10 @@ bool ConstantValueMap::HasShapeValue(const std::string& tensorName) {
       ConstantValueMap::getInstance().shapeValueMap.end();
 }
 
-c10::optional<c10::SymbolicShape> ConstantValueMap::GetShapeValue(
+std::optional<c10::SymbolicShape> ConstantValueMap::GetShapeValue(
     const std::string& tensorName) {
   if (!HasShapeValue(tensorName)) {
-    return c10::nullopt;
+    return std::nullopt;
   }
   return ConstantValueMap::getInstance().shapeValueMap[tensorName];
 }
@@ -218,6 +230,10 @@ SymbolDimMap& ConstantValueMap::GetSymbolDimMap() {
   return ConstantValueMap::getInstance().symbolDimMap;
 }
 
+DimSymbolMap& ConstantValueMap::GetDimSymbolMap() {
+  return ConstantValueMap::getInstance().dimSymbolMap;
+}
+
 template <typename Map>
 void UpdateStrKey(
     Map& map,
@@ -262,11 +278,14 @@ void ConstantValueMap::ClearMaps() {
   ConstantValueMap::getInstance().shapeValueMap.clear();
   ConstantValueMap::getInstance().inferredShapeData.clear();
   ConstantValueMap::getInstance().symbolDimMap.clear();
+  ConstantValueMap::getInstance().dimSymbolMap.clear();
+  ConstantValueMap::getInstance().allGraphInputsStatic = std::nullopt;
+  ConstantValueMap::getInstance().allGraphInputsReliableComputed = false;
 }
 
 // For debug only.
 void ConstantValueMap::PrintMaps() {
-  std::cout << "Rank/Shape Map:" << std::endl;
+  std::cout << "Rank/Shape Map:" << '\n';
   for (const auto& x : ConstantValueMap::getInstance().rankMap) {
     std::stringstream ss;
     if (ConstantValueMap::getInstance().shapeMap.find(x.first) !=
@@ -284,45 +303,45 @@ void ConstantValueMap::PrintMaps() {
       }
     }
     ss << " (rank = " << x.second << ")";
-    std::cout << "node " << x.first << ": " << ss.str() << std::endl;
+    std::cout << "node " << x.first << ": " << ss.str() << '\n';
   }
-  std::cout << std::endl;
-  std::cout << "Value Map:" << std::endl;
+  std::cout << '\n';
+  std::cout << "Value Map:" << '\n';
   for (const auto& x : ConstantValueMap::getInstance().tensorValueMap) {
-    std::cout << "node " << x.first << ": " << x.second << std::endl;
+    std::cout << "node " << x.first << ": " << x.second << '\n';
   }
-  std::cout << std::endl;
-  std::cout << "TypeReliable Map:" << std::endl;
+  std::cout << '\n';
+  std::cout << "TypeReliable Map:" << '\n';
   size_t count = 0;
   for (const auto& x : ConstantValueMap::getInstance().typeReliableMap) {
     std::cout << "(node " << x.first << ": " << x.second << "), ";
     count++;
     if (count % 10 == 0) {
-      std::cout << std::endl;
+      std::cout << '\n';
     }
   }
-  std::cout << std::endl;
-  std::cout << "UseInferredType Map:" << std::endl;
+  std::cout << '\n';
+  std::cout << "UseInferredType Map:" << '\n';
   count = 0;
   for (const auto& x : ConstantValueMap::getInstance().useInferredTypeMap) {
     std::cout << "(node " << x.first << ": " << x.second << "), ";
     count++;
     if (count % 10 == 0) {
-      std::cout << std::endl;
+      std::cout << '\n';
     }
   }
-  std::cout << std::endl;
-  std::cout << "ShapeValue Map:" << std::endl;
+  std::cout << '\n';
+  std::cout << "ShapeValue Map:" << '\n';
   count = 0;
   for (const auto& x : ConstantValueMap::getInstance().shapeValueMap) {
     std::cout << "(node " << x.first << ": " << x.second << "), ";
     count++;
     if (count % 10 == 0) {
-      std::cout << std::endl;
+      std::cout << '\n';
     }
   }
-  std::cout << std::endl;
-  std::cout << "InferredShape Map:" << std::endl;
+  std::cout << '\n';
+  std::cout << "InferredShape Map:" << '\n';
   count = 0;
   for (const auto& x : ConstantValueMap::getInstance().inferredShapeData) {
     std::cout << "(node " << x.first << ": ";
@@ -336,20 +355,28 @@ void ConstantValueMap::PrintMaps() {
     std::cout << "), ";
     count++;
     if (count % 10 == 0) {
-      std::cout << std::endl;
+      std::cout << '\n';
     }
   }
-  std::cout << std::endl;
-  std::cout << "SymbolDim Map:" << std::endl;
+  std::cout << '\n';
+  std::cout << "SymbolDim Map:" << '\n';
   count = 0;
   for (const auto& x : ConstantValueMap::getInstance().symbolDimMap) {
     std::cout << "(" << x.first << ": " << x.second << "), ";
     count++;
     if (count % 10 == 0) {
-      std::cout << std::endl;
+      std::cout << '\n';
+    }
+  }
+  std::cout << "DimSymbol Map:" << '\n';
+  count = 0;
+  for (const auto& x : ConstantValueMap::getInstance().dimSymbolMap) {
+    std::cout << "(" << x.first << ": " << x.second << "), ";
+    count++;
+    if (count % 10 == 0) {
+      std::cout << '\n';
     }
   }
 }
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
