@@ -57,8 +57,8 @@ inline at::Tensor create_empty_from(const at::Tensor& t) {
       c10::typeMetaToScalarType(t.dtype()),
       t.layout(),
       t.device(),
-      c10::nullopt,
-      c10::nullopt);
+      std::nullopt,
+      std::nullopt);
 }
 
 inline at::Tensor create_empty_from(
@@ -69,20 +69,20 @@ inline at::Tensor create_empty_from(
       c10::typeMetaToScalarType(t.dtype()),
       t.layout(),
       t.device(),
-      c10::nullopt,
-      c10::nullopt);
+      std::nullopt,
+      std::nullopt);
 }
 
 inline at::Tensor create_empty(c10::ScalarType dtype) {
   return at::detail::empty_cpu(
-      {0}, dtype, c10::nullopt, c10::nullopt, c10::nullopt, c10::nullopt);
+      {0}, dtype, std::nullopt, std::nullopt, std::nullopt, std::nullopt);
 }
 
 inline at::Tensor create_empty_from(
     const at::Tensor& t,
     c10::ScalarType dtype) {
   return at::detail::empty_cpu(
-      {0}, dtype, t.layout(), t.device(), c10::nullopt, c10::nullopt);
+      {0}, dtype, t.layout(), t.device(), std::nullopt, std::nullopt);
 }
 
 inline at::Tensor create_empty_from(const at::Tensor& t, c10::Layout layout) {
@@ -91,8 +91,8 @@ inline at::Tensor create_empty_from(const at::Tensor& t, c10::Layout layout) {
       c10::typeMetaToScalarType(t.dtype()),
       layout,
       t.device(),
-      c10::nullopt,
-      c10::nullopt);
+      std::nullopt,
+      std::nullopt);
 }
 
 inline at::Tensor create_empty_from(const at::Tensor& t, c10::Device device) {
@@ -101,8 +101,8 @@ inline at::Tensor create_empty_from(const at::Tensor& t, c10::Device device) {
       c10::typeMetaToScalarType(t.dtype()),
       t.layout(),
       device,
-      c10::nullopt,
-      c10::nullopt);
+      std::nullopt,
+      std::nullopt);
 }
 
 inline at::Tensor create_empty_from(
@@ -113,7 +113,7 @@ inline at::Tensor create_empty_from(
       c10::typeMetaToScalarType(t.dtype()),
       t.layout(),
       t.device(),
-      c10::nullopt,
+      std::nullopt,
       memory_format);
 }
 
@@ -122,7 +122,7 @@ inline at::Tensor create_empty_from(
     c10::ScalarType dtype,
     c10::MemoryFormat memory_format) {
   return at::detail::empty_cpu(
-      {0}, dtype, t.layout(), t.device(), c10::nullopt, memory_format);
+      {0}, dtype, t.layout(), t.device(), std::nullopt, memory_format);
 }
 
 inline bool checkResizedDataPtr(at::Tensor& t) {
