@@ -362,7 +362,7 @@ struct PythonPrintImpl {
     std::string name = candidate;
     while (used.count(name) || reserved_names.count(name)) {
       // NOLINTNEXTLINE(performance-inefficient-string-concatenation)
-      name = candidate + c10::to_string(next_id[name]++);
+      name = candidate + std::to_string(next_id[name]++);
     }
     used.insert(name);
     return name;
@@ -579,10 +579,11 @@ struct PythonPrintImpl {
 
     auto loop_type = stmt.loopType();
     if (loop_type == LoopView::ModifiedLoop) {
-      throw ErrorReport(stmt.node()->sourceRange())
+      throw(
+          ErrorReport(stmt.node()->sourceRange())
           << "loop cannot be printed as python "
           << "because it has gone through an optimization "
-          << "that combined while and for loops. File a bug";
+          << "that combined while and for loops. File a bug");
     }
 
     bool emit_as_for_loop = loop_type == LoopView::For;
@@ -670,20 +671,22 @@ struct PythonPrintImpl {
 
   void scanLongInlines(
       Node* user,
-      int64_t offset,
+      size_t offset,
       std::vector<Value*>& to_split_reversed) {
     auto it = visited_split_inline_uses_.find(user);
     bool present = it != visited_split_inline_uses_.end();
-    for (int64_t i = offset; i >= (present ? it->second + 1 : 0); --i) {
+    for (int64_t i = static_cast<int64_t>(offset);
+         i >= (present ? it->second + 1 : 0);
+         --i) {
       Value* prev_arg = user->input(i);
       if (isNonConstantInline(prev_arg)) {
         to_split_reversed.push_back(prev_arg);
       }
     }
-    visited_split_inline_uses_[user] = offset;
+    visited_split_inline_uses_[user] = static_cast<int64_t>(offset);
     if (!present && output_inline_.count(user)) {
       Use u = user->output()->uses().at(0);
-      scanLongInlines(u.user, int64_t(u.offset) - 1, to_split_reversed);
+      scanLongInlines(u.user, u.offset - 1, to_split_reversed);
       // -1 because the actual use is still being
       // emitted so it cannot be split
     }
@@ -778,9 +781,10 @@ struct PythonPrintImpl {
     switch (node->kind()) {
       case prim::Return:
         if (enforce_importable_ && node->inputs().size() != 1) {
-          throw ErrorReport(node->sourceRange())
+          throw(
+              ErrorReport(node->sourceRange())
               << "Exportable methods must have a single return value. "
-              << "Normal use of ScriptMethods should enforce this";
+              << "Normal use of ScriptMethods should enforce this");
         }
         if (!node->inputs().empty()) {
           indent();
@@ -868,8 +872,9 @@ struct PythonPrintImpl {
       } break;
       case prim::Closure: {
         if (enforce_importable_) {
-          throw ErrorReport(node->sourceRange())
-              << "closures are not exportable";
+          throw(
+              ErrorReport(node->sourceRange())
+              << "closures are not exportable");
         }
         assignValuesToTheirUniqueNames(node->outputs());
         auto name = useOf(node->output())->str();
@@ -994,11 +999,12 @@ struct PythonPrintImpl {
       case prim::PythonOp: {
         auto value = static_cast<const PythonOp*>(node);
         if (enforce_importable_) {
-          throw ErrorReport(node->sourceRange())
+          throw(
+              ErrorReport(node->sourceRange())
               << "Could not export Python function call '" << value->name()
               << "'. Remove calls to Python functions before export. "
               << "Did you forget to add @script or @script_method annotation? "
-              << "If this is a nn.ModuleList, add it to __constants__";
+              << "If this is a nn.ModuleList, add it to __constants__");
         }
         std::stringstream scalars_stream;
         stmt << "^" << value->name();
@@ -1677,9 +1683,7 @@ uint64_t PythonPrint::minVersion() const {
   return pImpl->min_version_;
 }
 
-PythonPrint::~PythonPrint() = default;
-
-static std::vector<IValue> traverseIValueAndGetObjects(IValue ivalue) {
+static std::vector<IValue> traverseIValueAndGetObjects(const IValue& ivalue) {
   std::vector<IValue> result;
   std::vector<IValue> stack;
   stack.emplace_back(ivalue);
@@ -1716,7 +1720,7 @@ static std::vector<IValue> traverseIValueAndGetObjects(IValue ivalue) {
   return result;
 }
 
-static c10::optional<std::string> printType(
+static std::optional<std::string> printType(
     const c10::Type& type,
     torch::jit::TypeNameUniquer& type_name_uniquer) {
   if (auto dyn = type.castRaw<c10::DynamicType>()) {
@@ -1727,7 +1731,7 @@ static c10::optional<std::string> printType(
   if (namedType && namedType->name()) {
     return type_name_uniquer.getUniqueName(namedType).qualifiedName();
   }
-  return c10::nullopt;
+  return std::nullopt;
 }
 
 void jitModuleToPythonCodeAndConstants(
