@@ -1,14 +1,11 @@
 #include <torch/csrc/jit/tensorexpr/unique_name_manager.h>
 
-#include <c10/util/string_utils.h>
 #include <torch/csrc/jit/tensorexpr/ir.h>
 #include <cctype>
 
-namespace torch {
-namespace jit {
-namespace tensorexpr {
+namespace torch::jit::tensorexpr {
 
-const std::string& UniqueNameManager::get_unique_name(VarPtr v) {
+const std::string& UniqueNameManager::get_unique_name(const VarPtr& v) {
   // Find if we have already encountered this variable.
   auto iter = unique_name_mapping_.find(v);
   if (iter != unique_name_mapping_.end()) {
@@ -18,7 +15,7 @@ const std::string& UniqueNameManager::get_unique_name(VarPtr v) {
   // First use the name_hint as a prefix to check if there is another name
   // with the same prefix.
   std::string name_hint = v->name_hint();
-  if (name_hint == "") {
+  if (name_hint.empty()) {
     name_hint = "v";
   } else if (std::isdigit(name_hint[0])) {
     name_hint = "v" + name_hint;
@@ -30,7 +27,7 @@ const std::string& UniqueNameManager::get_unique_name(VarPtr v) {
     int count_v = count++;
     std::string unique_name = name_hint;
     if (count_v > 0) {
-      unique_name += "_" + c10::to_string(count_v);
+      unique_name += "_" + std::to_string(count_v);
     }
     if (all_unique_names_.count(unique_name) == 0) {
       all_unique_names_.insert(unique_name);
@@ -44,6 +41,4 @@ const std::string& UniqueNameManager::get_unique_name(const VarHandle& v) {
   return get_unique_name(v.node());
 }
 
-} // namespace tensorexpr
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit::tensorexpr
