@@ -9,7 +9,7 @@ LoweringContext::LoweringContext(const std::string& name, BackendDevice device)
 LoweringContext::LoweringContext(
     const std::string& name,
     BackendDevice device,
-    c10::ArrayRef<torch::lazy::Node*> post_order,
+    c10::ArrayRef<const torch::lazy::Node*> post_order,
     Util::EmissionMap emit_status)
     : device_(std::move(device)), emit_status_(std::move(emit_status)) {}
 
