@@ -1,7 +1,9 @@
+#include <c10/util/thread_name.h>
 #include <torch/csrc/Exceptions.h>
 #include <torch/csrc/python_headers.h>
 #include <torch/csrc/utils/object_ptr.h>
 #include <torch/csrc/utils/pybind.h>
+#include <torch/csrc/utils/python_strings.h>
 
 #include <stdexcept>
 
@@ -14,8 +16,7 @@
     throw std::system_error(errno, std::system_category(), ##__VA_ARGS__); \
   }
 
-namespace torch {
-namespace multiprocessing {
+namespace torch::multiprocessing {
 
 namespace {
 
@@ -38,6 +39,19 @@ PyObject* multiprocessing_init(PyObject* _unused, PyObject* noargs) {
   Py_RETURN_TRUE;
 }
 
+PyObject* set_thread_name(PyObject* _unused, PyObject* arg) {
+  TORCH_CHECK(THPUtils_checkString(arg), "invalid argument to setDevice");
+
+  auto name = THPUtils_unpackString(arg);
+  c10::setThreadName(name);
+
+  Py_RETURN_TRUE;
+}
+
+PyObject* get_thread_name(PyObject* _unused, PyObject* noargs) {
+  return THPUtils_packString(c10::getThreadName());
+}
+
 } // namespace
 
 // multiprocessing methods on torch._C
@@ -49,6 +63,18 @@ static PyMethodDef methods[] = {
         METH_NOARGS,
         nullptr,
     },
+    {
+        "_set_thread_name",
+        set_thread_name,
+        METH_O,
+        nullptr,
+    },
+    {
+        "_get_thread_name",
+        get_thread_name,
+        METH_NOARGS,
+        nullptr,
+    },
     {nullptr, nullptr, 0, nullptr},
 };
 
@@ -56,5 +82,4 @@ PyMethodDef* python_functions() {
   return methods;
 }
 
-} // namespace multiprocessing
-} // namespace torch
+} // namespace torch::multiprocessing
