@@ -1,7 +1,9 @@
 #ifdef USE_KINETO
+#include <ATen/Context.h>
 #include <libkineto.h>
 #include <torch/csrc/autograd/profiler_kineto.h>
-#include <cstdlib>
+#include <chrono>
+#include <thread>
 
 // Ondemand tracing is not supported on Apple or edge platform
 #if defined(__APPLE__) || defined(EDGE_PROFILER_USE_KINETO)
@@ -11,8 +13,8 @@
 #endif
 
 namespace torch {
-namespace profiler {
-namespace impl {
+
+namespace profiler::impl {
 
 namespace {
 
@@ -67,22 +69,48 @@ class LibKinetoClient : public libkineto::ClientInterface {
 
 } // namespace
 
-} // namespace impl
-} // namespace profiler
+} // namespace profiler::impl
 
 #if ENABLE_GLOBAL_OBSERVER
 namespace {
 
+int get_init_delay() {
+  const char* delay_c = std::getenv("KINETO_DAEMON_INIT_DELAY_S");
+  if (!delay_c) {
+    return -1;
+  }
+  std::string delay_s{delay_c};
+  try {
+    return std::stoi(delay_s);
+  } catch (const std::invalid_argument& _) {
+    return -1;
+  }
+}
+
 struct RegisterLibKinetoClient {
   RegisterLibKinetoClient() {
     static profiler::impl::LibKinetoClient client;
+    libkineto::api().registerClient(&client);
+
+    auto kineto_init = []() {
+      libkineto_init(
+          /*cpuOnly=*/!(at::hasCUDA() || at::hasXPU() || at::hasMTIA()),
+          /*logOnError=*/true);
+      libkineto::api().suppressLogMessages();
+    };
 
     if (std::getenv("KINETO_USE_DAEMON") != nullptr) {
-      libkineto_init(/*cpuOnly=*/false, /*logOnError=*/true);
-      libkineto::api().suppressLogMessages();
+      int init_delay_s = get_init_delay();
+      if (init_delay_s > 0) {
+        std::thread t([init_delay_s, kineto_init]() {
+          std::this_thread::sleep_for(std::chrono::seconds(init_delay_s));
+          kineto_init();
+        });
+        t.detach();
+      } else {
+        kineto_init();
+      }
     }
-
-    libkineto::api().registerClient(&client);
   }
 } register_libkineto_client;
 
