@@ -10,7 +10,7 @@
 #endif // _WIN32
 
 #include <fmt/format.h>
-#include <algorithm>
+#include <fmt/ranges.h>
 #include <chrono>
 #include <cmath>
 #include <fstream>
@@ -19,7 +19,6 @@
 #include <mutex>
 #include <sstream>
 #include <stack>
-#include <stdexcept>
 #include <vector>
 
 #include <ATen/core/TensorBody.h>
@@ -30,11 +29,28 @@
 #include <torch/csrc/profiler/standalone/execution_trace_observer.h>
 #include <torch/csrc/profiler/util.h>
 
+#ifdef USE_DISTRIBUTED
+#include <torch/csrc/distributed/c10d/ParamCommsUtils.hpp>
+#endif // USE_DISTRIBUTED
+
 using namespace at;
 
-namespace torch {
-namespace profiler {
-namespace impl {
+// Collective property attributes
+// https://github.com/pytorch/pytorch/issues/124674
+#ifdef USE_DISTRIBUTED
+constexpr auto kETCommsName = "collective_name";
+constexpr auto kETInMsgNelems = "in_msg_nelems";
+constexpr auto kETOutMsgNelems = "out_msg_nelems";
+constexpr auto kETInSplit = "in_split_size";
+constexpr auto kETOutSplit = "out_split_size";
+constexpr auto kETGlobalRankStart = "global_rank_start";
+constexpr auto kETGlobalRankStride = "global_rank_stride";
+constexpr auto kETGroupSize = "pg_size";
+constexpr auto kETProcessGroupName = "pg_name";
+constexpr auto kETProcessGroupDesc = "pg_desc";
+#endif // USE_DISTRIBUTED
+
+namespace torch::profiler::impl {
 
 //******************************************************************************
 // JSON output utility functions. To be merged with PyTorch profiler.
@@ -46,72 +62,7 @@ inline std::string vectorToString(const std::vector<T>& v) {
 
 std::string json_str_escape(const std::string& str);
 
-constexpr size_t maxNumElements = 4096;
-
-inline std::string getValueType(
-    const c10::IValue& val,
-    const bool baseType = true,
-    const size_t maxArrayLen = maxNumElements) {
-  std::string type = val.tagKind();
-
-  if (val.isTensor()) {
-    // Add tensor element data type.
-    type += fmt::format("({})", std::string(val.toTensor().dtype().name()));
-  } else if (val.isTuple()) {
-    const auto& val_container = val.toTupleRef().elements();
-    std::vector<std::string> str_array;
-    for (const auto& t : val_container) {
-      str_array.emplace_back(getValueType(t, false));
-    }
-    type += vectorToString(str_array);
-  } else if (val.isList()) {
-    const auto& val_list = val.toList();
-    std::vector<std::string> str_array;
-    str_array.reserve(val_list.size());
-    for (const auto j : c10::irange(val_list.size())) {
-      str_array.push_back(getValueType(val_list.get(j), false));
-      if (j >= maxArrayLen) {
-        LOG(WARNING) << "list size=" << val_list.size()
-                     << " exceeded maxArrayLen=" << maxArrayLen;
-        break;
-      }
-    }
-    type += vectorToString(str_array);
-  }
-  return baseType ? fmt::format("\"{}\"", type) : type;
-}
-
-inline std::string getValueShape(
-    const c10::IValue& val,
-    const size_t maxArrayLen = maxNumElements) {
-  if (val.isTensor()) {
-    auto& tensor = val.toTensor();
-    if (tensor.defined()) {
-      return vectorToString(tensor.sizes().vec());
-    }
-  } else if (val.isTuple()) {
-    const auto& val_container = val.toTupleRef().elements();
-    std::vector<std::string> str_array;
-    for (const auto& t : val_container) {
-      str_array.push_back(getValueShape(t));
-    }
-    return vectorToString(str_array);
-  } else if (val.isList()) {
-    const auto& val_list = val.toList();
-    std::vector<std::string> str_array;
-    str_array.reserve(val_list.size());
-    for (const auto j : c10::irange(val_list.size())) {
-      str_array.push_back(getValueShape(val_list.get(j)));
-      if (j >= maxArrayLen) {
-        LOG(WARNING) << "list size=" << val_list.size()
-                     << " exceeded maxArrayLen=" << maxArrayLen;
-        break;
-      }
-    }
-    return vectorToString(str_array);
-  }
-  return "[]";
-}
+constexpr size_t kMaxNumElements = 4096;
 
 inline std::string getScalarValue(const c10::IValue& val) {
   if (val.isDouble()) {
@@ -127,13 +78,6 @@ inline std::string getScalarValue(const c10::IValue& val) {
     return val.toBool() ? "true" : "false";
   } else if (val.isString()) {
     const std::string& str_val = val.toStringRef();
-    if (str_val.size() > maxNumElements) {
-      LOG(WARNING) << "string size=" << str_val.size()
-                   << " exceeded maxNumElements=" << maxNumElements;
-      return fmt::format(
-          "\"{}\"", json_str_escape(str_val.substr(0, maxNumElements)));
-    }
-
     return fmt::format("\"{}\"", json_str_escape(str_val));
   } else if (val.isDevice()) {
     return fmt::format("\"{}\"", val.toDevice().str());
@@ -155,36 +99,36 @@ inline int32_t processId() {
 
 // ExecutionTraceObserver contains all the states of the observer. Some of them
 // are shared between the enter and exit RecordFunction call backs, some data
-// like the `op_stack` may be accessed across different threads. So we should be
-// careful about data races. A global mutex `g_mutex` is used avoid these races
+// like the `opStack` may be accessed across different threads. So we should be
+// careful about data races. A global mutex `gMutex` is used avoid these races
 // at the cost of performance in large number of threads situations. We may
 // optimize this further to thread local, fine-grained locking, or use thread
 // safe containers.
-struct TORCH_API ExecutionTraceObserver {
+struct TORCH_API ExecutionTraceObserver { // NOLINT
   using ID = size_t;
 
   // Mapping of each thread to its own operator stack
-  std::map<size_t, std::stack<ID>> op_stack{};
+  std::map<size_t, std::stack<ID>> opStack{};
   // Uses the underlying TensorImpl object pointer as the key and map to its
   // unique id.
-  std::map<const void*, ID> object_id{};
+  std::map<const void*, ID> objectId{};
   // Observer run state.
   enum class RunState { uninitialized, disabled, enabled };
 
   // Mutex for multithreaded access to the shared containers.
-  std::mutex g_mutex{};
+  std::recursive_mutex gMutex{};
   // Stream to write output JSON.
   std::ofstream out{};
 
   // Full path to the output file.
-  std::string file_name{};
+  std::string fileName{};
 
   // RecordFunction callback handle for this observer.
-  CallbackHandle cb_handle{INVALID_CALLBACK_HANDLE};
+  CallbackHandle cbHandle{INVALID_CALLBACK_HANDLE};
 
   // Process ID.
   int32_t pid{-1};
-  std::string record_time{};
+  std::string recordTime{};
 
   ExecutionTraceObserver() = default;
 
@@ -201,9 +145,9 @@ struct TORCH_API ExecutionTraceObserver {
     if (state_ == RunState::uninitialized ||
         callbackShouldBeEnabled(state_) != callbackShouldBeEnabled(newState)) {
       if (callbackShouldBeEnabled(newState)) {
-        reenableCallback(cb_handle);
+        reenableCallback(cbHandle);
       } else {
-        disableCallback(cb_handle);
+        disableCallback(cbHandle);
       }
     }
     state_ = newState;
@@ -230,18 +174,21 @@ struct TORCH_API ExecutionTraceObserver {
 using ObserverManager = GlobalStateManager<ExecutionTraceObserver>;
 
 // Uninitialized node has id = 0
-const ExecutionTraceObserver::ID uninitialized_id{0};
+const ExecutionTraceObserver::ID kUninitializedId{0};
 // Root node has id = 1
-const ExecutionTraceObserver::ID root_id{1};
+const ExecutionTraceObserver::ID kRootId{1};
 
-struct FunctionCallContext : public ObserverContext {
+struct FunctionCallContext : public ObserverContext { // NOLINT
   std::string name;
-  ExecutionTraceObserver::ID op_id{uninitialized_id};
-  ExecutionTraceObserver::ID parent_id{uninitialized_id};
-  ExecutionTraceObserver::ID fw_parent_id{uninitialized_id};
-  std::vector<std::string> input_types;
-  std::vector<std::string> input_shapes;
-  std::vector<std::string> input_values;
+  std::string kernelBackend;
+  std::string kernelFile;
+  ExecutionTraceObserver::ID opId{kUninitializedId};
+  ExecutionTraceObserver::ID parentId{kUninitializedId};
+  ExecutionTraceObserver::ID fwParentId{kUninitializedId};
+  std::vector<std::string> inputTypes;
+  std::vector<std::string> inputShapes;
+  std::vector<std::string> inputStrides;
+  std::vector<std::string> inputValues;
 };
 
 // Opens the json file to write the execution trace.
@@ -256,6 +203,21 @@ static std::ofstream openOutputFile(const std::string& name) {
   return stream;
 }
 
+#ifdef USE_DISTRIBUTED
+static inline std::string getAttrJson(
+    const std::string& name,
+    const std::string& type,
+    const std::string& value) {
+  // note name and type are not quoted but value should be if it is a string.
+  return fmt::format(
+      R"JSON(
+  {{"name": "{}", "type": "{}", "value": {}}})JSON",
+      name,
+      type,
+      value);
+}
+#endif
+
 static void writeJsonNode(
     std::ofstream& out,
     const std::string& name,
@@ -268,54 +230,65 @@ static void writeJsonNode(
     const uint64_t tid,
     const uint64_t fw_tid,
     const std::string& inputs = "[]",
-    const std::string& input_shapes = "[]",
-    const std::string& input_types = "[]",
+    const std::string& inputShapes = "[]",
+    const std::string& inputStrides = "[]",
+    const std::string& inputTypes = "[]",
     const std::string& outputs = "[]",
     const std::string& output_shapes = "[]",
+    const std::string& output_strides = "[]",
     const std::string& output_types = "[]",
-    const std::string& operator_schema = "") {
+    const std::string& operator_schema = "",
+    const std::string& kernelBackend = "",
+    const std::string& kernelFile = "",
+    const std::string& additiona_attrs = "") {
   out << fmt::format(
       R"JSON(
     {{
-      "name": "{}", "id": {}, "rf_id": {}, "parent": {}, "fw_parent": {}, "seq_id": {}, "scope": {}, "tid": {}, "fw_tid": {}, "op_schema": "{}",
-      "inputs": {}, "input_shapes": {}, "input_types": {},
-      "outputs": {}, "output_shapes": {}, "output_types": {}
+      "id": {}, "name": "{}", "ctrl_deps": {},
+      "inputs": {{"values": {}, "shapes": {}, "types": {}, "strides": {}}},
+      "outputs": {{"values": {}, "shapes": {}, "types": {}, "strides": {}}},
+      "attrs": [{{"name": "rf_id", "type": "uint64", "value": {}}},{{"name": "fw_parent", "type": "uint64", "value": {}}},{{"name": "seq_id", "type": "int64", "value": {}}},{{"name": "scope", "type": "uint64", "value": {}}},{{"name": "tid", "type": "uint64", "value": {}}},{{"name": "fw_tid", "type": "uint64", "value": {}}},{{"name": "op_schema", "type": "string", "value": "{}"}},{{"name": "kernel_backend", "type": "string", "value": "{}"}},{{"name": "kernel_file", "type": "string", "value": "{}"}}{}]
     }})JSON",
-      name,
       id,
-      rf_id,
+      name,
       parent,
+      inputs,
+      inputShapes,
+      inputTypes,
+      inputStrides,
+      outputs,
+      output_shapes,
+      output_types,
+      output_strides,
+      rf_id,
       fw_parent,
       seq_id,
       scope,
       tid,
       fw_tid,
       operator_schema,
-      inputs,
-      input_shapes,
-      input_types,
-      outputs,
-      output_shapes,
-      output_types);
+      kernelBackend,
+      kernelFile,
+      additiona_attrs);
 }
 
 inline std::string timeString(const std::time_t timepoint) {
   std::ostringstream oss;
-  oss << std::put_time(std::localtime(&timepoint), "%Y-%m-%d %X");
+  oss << std::put_time(std::localtime(&timepoint), "%Y-%m-%d %X"); // NOLINT
   return oss.str();
 }
 
 static bool initExecutionTraceStart(ExecutionTraceObserver& ob) {
-  ob.out = openOutputFile(ob.file_name);
+  ob.out = openOutputFile(ob.fileName);
   // If somehow the output stream failed to open, finish observer here.
   if (!ob.out) {
-    LOG(WARNING) << "Failed to open output file: " << ob.file_name;
+    LOG(WARNING) << "Failed to open output file: " << ob.fileName;
     return false;
   }
 
   // Wall clock time for the first op collection time.
   const auto current_time = std::chrono::system_clock::now();
-  ob.record_time =
+  ob.recordTime =
       timeString(std::chrono::system_clock::to_time_t(current_time));
   // Start timestamp using steady_clock for measurement.
   const auto timestamp =
@@ -325,10 +298,10 @@ static bool initExecutionTraceStart(ExecutionTraceObserver& ob) {
 
   ob.out << fmt::format(
       R"JSON({{
-  "schema": "1.0.1", "pid": {}, "time": "{}", "start_ts": {},
+  "schema": "1.1.1-chakra.0.0.4", "pid": {}, "time": "{}", "start_ts": {},
   "nodes": [)JSON",
       ob.pid,
-      ob.record_time,
+      ob.recordTime,
       timestamp);
   return true;
 }
@@ -338,9 +311,9 @@ static void finalizeExecutionTraceOutput(ExecutionTraceObserver& ob) {
   writeJsonNode(
       ob.out,
       "[pytorch|profiler|execution_trace|process]",
-      root_id,
+      kRootId,
       0, // rf_id
-      root_id, // parent is self
+      kRootId, // parent is self
       0, // fw_parent
       -1, // seq_id
       static_cast<std::underlying_type_t<RecordScope>>(RecordScope::USER_SCOPE),
@@ -360,43 +333,64 @@ static void finalizeExecutionTraceOutput(ExecutionTraceObserver& ob) {
       timestamp);
 
   ob.out.close();
-  VLOG(1) << "PyTorch Execution Trace: written to file " << ob.file_name;
+  VLOG(1) << "PyTorch Execution Trace: written to file " << ob.fileName;
 }
 
 inline ExecutionTraceObserver::ID getObjectID(
     ExecutionTraceObserver& ob,
     const void* t) {
-  auto iter = ob.object_id.find(t);
-  if (iter == ob.object_id.end()) {
-    ExecutionTraceObserver::ID object_id = ob.getNewID();
-    ob.object_id[t] = object_id;
-    return object_id;
+  auto iter = ob.objectId.find(t);
+  if (iter == ob.objectId.end()) {
+    ExecutionTraceObserver::ID objectId = ob.getNewID();
+    ob.objectId[t] = objectId;
+    return objectId;
   }
 
   return iter->second;
 }
 
-inline std::string convertIValue(
+inline std::tuple<std::string, std::string, std::string, std::string>
+convertIValue(
     ExecutionTraceObserver& ob,
     const c10::IValue& val,
-    const size_t maxArrayLen = maxNumElements) {
+    const bool baseType = true,
+    const size_t maxArrayLen = kMaxNumElements) {
+  std::string type = val.tagKind();
   if (val.isTensor()) {
-    const auto t = val.toTensor().unsafeGetTensorImpl();
-    ExecutionTraceObserver::ID tensor_id = getObjectID(ob, t);
+    std::string tensor_shape, tensor_stride, tensor_type, tensor_value;
+
+    const auto& tensor = val.toTensor();
+    const auto tensor_impl = tensor.unsafeGetTensorImpl();
+    if (tensor.defined() && !tensor_impl->has_symbolic_sizes_strides()) {
+      // tensor shape
+      tensor_shape = vectorToString(tensor.sizes().vec());
+      // tensor strides
+      tensor_stride = vectorToString(tensor.strides().vec());
+    } else {
+      tensor_shape = "[]";
+      tensor_stride = "[]";
+    }
+    // tensor dtype
+    type = type + fmt::format("({})", std::string(tensor.dtype().name()));
+    tensor_type = baseType ? fmt::format("\"{}\"", type) : type;
+
+    ExecutionTraceObserver::ID tensor_id = getObjectID(ob, tensor_impl);
     ExecutionTraceObserver::ID storage_id = 0;
     size_t offset = 0;
     size_t numel = 0;
     size_t itemsize = 0;
     std::string device_str = "";
-    if (t->has_storage()) {
-      auto& t_storage = t->storage();
+    // symbolic sizes/strides implies t->storage_offset() will fail
+    if (tensor_impl->has_storage() &&
+        !tensor_impl->has_symbolic_sizes_strides()) {
+      auto& t_storage = tensor_impl->storage();
       storage_id = getObjectID(ob, t_storage.data());
-      offset = t->storage_offset();
-      numel = t->numel();
-      itemsize = t->itemsize();
-      device_str = t->device().str();
+      offset = tensor_impl->storage_offset();
+      numel = tensor_impl->numel();
+      itemsize = tensor_impl->itemsize();
+      device_str = tensor_impl->device().str();
     }
-    return fmt::format(
+    tensor_value = fmt::format(
         "[{},{},{},{},{},\"{}\"]",
         tensor_id,
         storage_id,
@@ -404,40 +398,168 @@ inline std::string convertIValue(
         numel,
         itemsize,
         device_str);
+    return std::make_tuple(
+        tensor_shape, tensor_stride, tensor_type, tensor_value);
   } else if (val.isTuple()) {
-    std::vector<std::string> str_array;
     const auto& val_tuple = val.toTupleRef().elements();
-    for (const auto j : c10::irange(val_tuple.size())) {
-      str_array.push_back(convertIValue(ob, val_tuple[j]));
+    size_t tuple_size = val_tuple.size();
+    std::vector<std::string> shape_array;
+    std::vector<std::string> stride_array;
+    std::vector<std::string> type_array;
+    std::vector<std::string> value_array;
+    for (const auto j : c10::irange(tuple_size)) {
+      auto tuple = convertIValue(ob, val_tuple[j], false, maxArrayLen);
+      shape_array.push_back(std::get<0>(tuple));
+      stride_array.push_back(std::get<1>(tuple));
+      type_array.push_back(std::get<2>(tuple));
+      value_array.push_back(std::get<3>(tuple));
     }
-    return vectorToString(str_array);
+    type = type + vectorToString(type_array);
+    std::string tensor_type = baseType ? fmt::format("\"{}\"", type) : type;
+    return std::make_tuple(
+        vectorToString(shape_array),
+        vectorToString(stride_array),
+        tensor_type,
+        vectorToString(value_array));
   } else if (val.isList()) {
     const auto& val_list = val.toList();
-    std::vector<std::string> str_array;
-    str_array.reserve(val_list.size());
-    for (const auto j : c10::irange(val_list.size())) {
-      str_array.push_back(convertIValue(ob, val_list.get(j)));
+    size_t list_size = val_list.size();
+    std::vector<std::string> shape_array;
+    std::vector<std::string> stride_array;
+    std::vector<std::string> type_array;
+    std::vector<std::string> value_array;
+    for (const auto j : c10::irange(list_size)) {
+      auto tuple = convertIValue(ob, val_list.get(j), false, maxArrayLen);
+      shape_array.push_back(std::get<0>(tuple));
+      stride_array.push_back(std::get<1>(tuple));
+      type_array.push_back(std::get<2>(tuple));
+      value_array.push_back(std::get<3>(tuple));
       if (j >= maxArrayLen) {
         LOG(WARNING) << "list size=" << val_list.size()
                      << " exceeded maxArrayLen=" << maxArrayLen;
         break;
       }
     }
-    return vectorToString(str_array);
+    type = type + vectorToString(type_array);
+    std::string tensor_type = baseType ? fmt::format("\"{}\"", type) : type;
+    return std::make_tuple(
+        vectorToString(shape_array),
+        vectorToString(stride_array),
+        tensor_type,
+        vectorToString(value_array));
   } else {
-    return getScalarValue(val);
+    std::string tensor_shape = "[]";
+    std::string tensor_stride = "[]";
+    std::string tensor_type = baseType ? fmt::format("\"{}\"", type) : type;
+    std::string tensor_value = getScalarValue(val);
+
+    return std::make_tuple(
+        tensor_shape, tensor_stride, tensor_type, tensor_value);
   }
 }
 
 inline void appendValueInfo(
     ExecutionTraceObserver& ob,
     const c10::IValue& val,
-    std::vector<std::string>& values,
+    std::vector<std::string>& shapes,
+    std::vector<std::string>& strides,
     std::vector<std::string>& types,
-    std::vector<std::string>& shapes) {
-  values.push_back(convertIValue(ob, val));
-  types.push_back(getValueType(val));
-  shapes.push_back(getValueShape(val));
+    std::vector<std::string>& values) {
+  auto tuple = convertIValue(ob, val, true);
+
+  shapes.push_back(std::get<0>(tuple));
+  strides.push_back(std::get<1>(tuple));
+  types.push_back(std::get<2>(tuple));
+  values.push_back(std::get<3>(tuple));
+}
+
+inline void handleKernelBackendInfo(
+    FunctionCallContext& fc,
+    const RecordFunction& fn) {
+  // triton kernel related information are in kwinputs
+  const auto& kwinputs = fn.kwinputs();
+  if (kwinputs.find("kernel_backend") != kwinputs.end()) {
+    fc.kernelBackend = kwinputs.at("kernel_backend").toStringRef();
+    if (fc.kernelBackend == "triton") {
+      fc.kernelFile = kwinputs.at("kernel_file").toStringRef();
+      TORCH_INTERNAL_ASSERT(
+          kwinputs.find("kernel_file") != kwinputs.end(),
+          "kernel file is missing in triton kernel");
+      // Remove the path of the file name
+      if (fc.kernelFile.find_last_of('/') != std::string::npos) {
+        fc.kernelFile =
+            fc.kernelFile.substr(fc.kernelFile.find_last_of('/') + 1);
+      }
+
+      // get grid information
+      TORCH_INTERNAL_ASSERT(
+          kwinputs.find("grid") != kwinputs.end(),
+          "grid is missing in triton kernel");
+      fc.inputValues.emplace_back(
+          "\"" + kwinputs.at("grid").toStringRef() + "\"");
+      fc.inputTypes.emplace_back("\"String\"");
+      fc.inputShapes.emplace_back("[]");
+
+      // get stream information
+      TORCH_INTERNAL_ASSERT(
+          kwinputs.find("stream") != kwinputs.end(),
+          "stream is missing in triton kernel");
+      fc.inputValues.emplace_back(
+          std::to_string(kwinputs.at("stream").toInt()));
+      fc.inputTypes.emplace_back("\"Int\"");
+      fc.inputShapes.emplace_back("[]");
+    }
+  }
+}
+
+// Additional attributes for commounication collectives
+inline std::string getCommsNodeAttrs(const RecordFunction& fn) { // NOLINT
+  std::vector<std::string> attrs;
+
+#ifdef USE_DISTRIBUTED
+  // We rely on paramcommsdebug object that is available in thread local info
+  auto debugInfo = dynamic_cast<ParamCommsDebugInfo*>(
+      c10::ThreadLocalDebugInfo::get(c10::DebugInfoKind::PARAM_COMMS_INFO));
+  if (debugInfo == nullptr) {
+    LOG(WARNING) << "ParamCommsDebugInfo not available for function: "
+                 << fn.name();
+    return ", " + getAttrJson("debug", "string", "\"missing comms info\"");
+  }
+
+  // get NcclMeta from record function, this used ParamCommsDebugInfo above
+  auto meta = saveNcclMeta(fn, false /*truncate*/);
+
+  auto addAttr =
+      [&](const char* commsMetaName, const char* etMetaName, const char* type) {
+        auto it = meta.find(commsMetaName);
+        if (it != meta.end()) {
+          attrs.push_back(getAttrJson(etMetaName, type, it->second));
+        }
+      };
+
+  addAttr(kCommsName, kETCommsName, "string");
+  addAttr(kDtype, kDtype, "string");
+
+  addAttr(kInMsgNelems, kETInMsgNelems, "uint64");
+  addAttr(kOutMsgNelems, kETOutMsgNelems, "uint64");
+
+  // following two metadata are lists.
+  addAttr(kInSplit, kETInSplit, "string");
+  addAttr(kOutSplit, kETOutSplit, "string");
+
+  addAttr(kGlobalRankStart, kETGlobalRankStart, "uint64");
+  addAttr(kGlobalRankStride, kETGlobalRankStride, "uint64");
+
+  // pg_name is a string.
+  addAttr(kProcessGroupName, kETProcessGroupName, "string");
+  addAttr(kProcessGroupDesc, kETProcessGroupDesc, "string");
+
+  addAttr(kGroupSize, kETGroupSize, "uint64");
+
+#endif // USE_DISTRIBUTED
+
+  // XXX consider using as string stream?
+  return attrs.empty() ? "" : fmt::format(", {}", fmt::join(attrs, ", "));
 }
 
 static void recordOperatorStart(
@@ -447,18 +569,18 @@ static void recordOperatorStart(
   auto tid = fn.threadId();
 
   try {
-    const std::lock_guard<std::mutex> lock(ob.g_mutex);
+    const std::lock_guard<std::recursive_mutex> lock(ob.gMutex);
 
     // if current thread stack is empty, push the root node to the stack first
-    if (ob.op_stack[tid].empty()) {
+    if (ob.opStack[tid].empty()) {
       auto thread_node_id = ob.getNewID();
-      ob.op_stack[tid].push(thread_node_id);
+      ob.opStack[tid].push(thread_node_id);
       writeJsonNode(
           ob.out,
           "[pytorch|profiler|execution_trace|thread]",
           thread_node_id,
           0, // rf_id
-          root_id,
+          kRootId,
           0, // fw_parent
           -1, // seq_id
           static_cast<std::underlying_type_t<RecordScope>>(
@@ -471,7 +593,7 @@ static void recordOperatorStart(
     auto num_inputs = fn.num_inputs();
     const auto inputs = fn.inputs();
 
-    VLOG(2) << "inputs: " << num_inputs << " " << inputs.size() << std::endl;
+    VLOG(2) << "inputs: " << num_inputs << " " << inputs.size() << '\n';
     // We have two cases: for unboxed kernel, we have num_inputs ==
     // inputs.size() for boxed kernel using stack, there could be more elements
     // on the stack from previous ops.
@@ -487,19 +609,27 @@ static void recordOperatorStart(
 
     for (const auto i : c10::irange(input_start, inputs.size())) {
       appendValueInfo(
-          ob, inputs[i], fc.input_values, fc.input_types, fc.input_shapes);
+          ob,
+          inputs[i],
+          fc.inputShapes,
+          fc.inputStrides,
+          fc.inputTypes,
+          fc.inputValues);
     }
-    fc.parent_id = ob.op_stack[tid].top();
+
+    handleKernelBackendInfo(fc, fn);
+
+    fc.parentId = ob.opStack[tid].top();
     // get parent id from the forward stack, this can be different for
     // autograd ops, which may execute on a different thread than the original
     // thread (which should have the parent op on the stack).
     auto fw_tid = fn.forwardThreadId();
     if (fw_tid != 0) {
-      fc.fw_parent_id = ob.op_stack[fw_tid].top();
+      fc.fwParentId = ob.opStack[fw_tid].top();
     }
-    // all input nodes should have id > op_id
-    fc.op_id = ob.getNewID();
-    ob.op_stack[tid].push(fc.op_id);
+    // all input nodes should have id > opId
+    fc.opId = ob.getNewID();
+    ob.opStack[tid].push(fc.opId);
 
   } catch (const std::exception& e) {
     LOG(WARNING) << "Exception in execution trace observer: " << e.what();
@@ -566,7 +696,7 @@ static void onFunctionExit(const RecordFunction& fn, ObserverContext* ctx_ptr) {
     // We have two cases: for unboxed kernel, we have num_outputs ==
     // outputs.size() for boxed kernel using stack, there could be more elements
     // on the stack from previous ops.
-    VLOG(2) << "outputs: " << num_outputs << " " << outputs.size() << std::endl;
+    VLOG(2) << "outputs: " << num_outputs << " " << outputs.size() << '\n';
     // TORCH_INTERNAL_ASSERT(num_outputs <= outputs.size());
     if (num_outputs > outputs.size()) {
       LOG(WARNING) << "RecordFunction " << fc.name
@@ -578,16 +708,22 @@ static void onFunctionExit(const RecordFunction& fn, ObserverContext* ctx_ptr) {
     size_t output_start = outputs.size() - num_outputs;
 
     std::vector<std::string> output_types;
+    std::vector<std::string> output_strides;
     std::vector<std::string> output_shapes;
     std::vector<std::string> output_values;
     try {
-      const std::lock_guard<std::mutex> lock(ob->g_mutex);
+      const std::lock_guard<std::recursive_mutex> lock(ob->gMutex);
       // remove current op id from stack
 
-      ob->op_stack[fn.threadId()].pop();
+      ob->opStack[fn.threadId()].pop();
       for (const auto i : c10::irange(output_start, outputs.size())) {
         appendValueInfo(
-            *ob, outputs[i], output_values, output_types, output_shapes);
+            *ob,
+            outputs[i],
+            output_shapes,
+            output_strides,
+            output_types,
+            output_values);
       }
 
       std::string op_schema_str{};
@@ -596,28 +732,36 @@ static void onFunctionExit(const RecordFunction& fn, ObserverContext* ctx_ptr) {
         op_schema_str = json_str_escape(c10::toString(op_schema.value()));
       }
 
+      const std::string additiona_attrs =
+          fn.isNcclMeta() ? getCommsNodeAttrs(fn) : "";
+
       writeJsonNode(
           ob->out,
           fc.name,
-          fc.op_id,
+          fc.opId,
           fn.handle(),
-          fc.parent_id,
-          fc.fw_parent_id,
+          fc.parentId,
+          fc.fwParentId,
           fn.seqNr(),
           static_cast<std::underlying_type_t<RecordScope>>(fn.scope()),
           fn.threadId(),
           fn.forwardThreadId(),
-          vectorToString(fc.input_values),
-          vectorToString(fc.input_shapes),
-          vectorToString(fc.input_types),
+          vectorToString(fc.inputValues),
+          vectorToString(fc.inputShapes),
+          vectorToString(fc.inputStrides),
+          vectorToString(fc.inputTypes),
           vectorToString(output_values),
           vectorToString(output_shapes),
+          vectorToString(output_strides),
           vectorToString(output_types),
-          op_schema_str);
+          op_schema_str,
+          fc.kernelBackend,
+          fc.kernelFile,
+          additiona_attrs);
       ob->out << ",";
     } catch (const std::exception& e) {
       LOG(WARNING) << "Exception in execution trace observer: [" << fc.name
-                   << " (" << fc.op_id << ")] " << e.what();
+                   << " (" << fc.opId << ")] " << e.what();
     }
   }
 }
@@ -631,12 +775,12 @@ bool addExecutionTraceObserver(const std::string& output_file_path) {
     auto& ob = *ObserverManager::get();
     ob.pid = processId();
     // Set output
-    ob.file_name = output_file_path;
+    ob.fileName = output_file_path;
     if (!initExecutionTraceStart(ob)) {
       return false;
     }
 
-    ob.cb_handle = addGlobalCallback(
+    ob.cbHandle = addGlobalCallback(
         RecordFunctionCallback(&onFunctionEnter, &onFunctionExit)
             .needsInputs(true)
             .needsOutputs(true)
@@ -646,7 +790,7 @@ bool addExecutionTraceObserver(const std::string& output_file_path) {
 
     VLOG(1) << "PyTorch Execution Trace: added observer, output="
             << output_file_path;
-  } else if (ObserverManager::get()->cb_handle != INVALID_CALLBACK_HANDLE) {
+  } else if (ObserverManager::get()->cbHandle != INVALID_CALLBACK_HANDLE) {
     LOG(WARNING) << "Execution trace observer is already registered.";
   }
   return true;
@@ -659,10 +803,10 @@ void removeExecutionTraceObserver() {
       disableExecutionTraceObserver();
     }
 
-    if (ob->cb_handle != INVALID_CALLBACK_HANDLE) {
+    if (ob->cbHandle != INVALID_CALLBACK_HANDLE) {
       finalizeExecutionTraceOutput(*ob);
-      removeCallback(ob->cb_handle);
-      ob->cb_handle = INVALID_CALLBACK_HANDLE;
+      removeCallback(ob->cbHandle);
+      ob->cbHandle = INVALID_CALLBACK_HANDLE;
       // Release the current ET observer object and reset.
       TORCH_INTERNAL_ASSERT(
           ObserverManager::pop() != nullptr,
@@ -677,7 +821,7 @@ void removeExecutionTraceObserver() {
 }
 
 void enableExecutionTraceObserver() {
-  VLOG(1) << "enableExecutionTraceObserver() ";
+  LOG(WARNING) << "Enabling Execution Trace Observer";
   auto& ob = *ObserverManager::get();
   // Make sure we are not already enabled.
   if (ob.getState() == ExecutionTraceObserver::RunState::enabled) {
@@ -689,7 +833,7 @@ void enableExecutionTraceObserver() {
 }
 
 void disableExecutionTraceObserver() {
-  VLOG(1) << "disableExecutionTraceObserver()";
+  LOG(WARNING) << "Disabling Execution Trace Observer";
   auto& ob = *ObserverManager::get();
   if (ob.getState() != ExecutionTraceObserver::RunState::disabled) {
     ob.setState(ExecutionTraceObserver::RunState::disabled);
@@ -698,6 +842,4 @@ void disableExecutionTraceObserver() {
         << "Trying to disable Execution Trace Observer when it's already disabled.";
   }
 }
-} // namespace impl
-} // namespace profiler
-} // namespace torch
+} // namespace torch::profiler::impl
