@@ -7,29 +7,29 @@
     !__has_include("ext/stdio_filebuf.h")
 namespace torch::unwind {
 std::vector<void*> unwind() {
-  TORCH_CHECK(
-      false,
+  TORCH_WARN_ONCE(
       "record_context_cpp is not support on non-linux non-x86_64 platforms");
+  return {};
 }
 
 std::optional<std::pair<std::string, uint64_t>> libraryFor(void* addr) {
-  TORCH_CHECK(
-      false,
+  TORCH_WARN_ONCE(
       "record_context_cpp is not support on non-linux non-x86_64 platforms");
+  return {};
 }
 
 #ifndef FBCODE_CAFFE2
 std::vector<Frame> symbolize(const std::vector<void*>& frames, Mode mode) {
-  TORCH_CHECK(
-      false,
+  TORCH_WARN_ONCE(
       "record_context_cpp is not support on non-linux non-x86_64 platforms");
+  return {};
 }
 #endif
 
 Stats stats() {
-  TORCH_CHECK(
-      false,
+  TORCH_WARN_ONCE(
       "record_context_cpp is not support on non-linux non-x86_64 platforms");
+  return {};
 }
 
 } // namespace torch::unwind
@@ -37,6 +37,7 @@ Stats stats() {
 #else
 
 #include <c10/util/flat_hash_map.h>
+#include <dlfcn.h>
 #include <elf.h>
 #include <link.h>
 #include <linux/limits.h>
@@ -128,7 +129,9 @@ struct UnwindCache {
   Version currentVersion() {
     Version r;
     dl_iterate_phdr(
-        [](struct dl_phdr_info* info, size_t size, void* data) {
+        [](struct dl_phdr_info* info,
+           size_t size [[maybe_unused]],
+           void* data) {
           Version* v = (Version*)data;
           v->adds_ = info->dlpi_adds;
           v->subs_ = info->dlpi_subs;
@@ -142,7 +145,9 @@ struct UnwindCache {
     all_libraries_.clear();
     ip_cache_.clear();
     dl_iterate_phdr(
-        [](struct dl_phdr_info* info, size_t size, void* data) {
+        [](struct dl_phdr_info* info,
+           size_t size [[maybe_unused]],
+           void* data) {
           auto self = (UnwindCache*)data;
           uint64_t last_addr = 0;
           auto segments = (Elf64_Phdr*)info->dlpi_phdr;
@@ -232,7 +237,7 @@ struct UnwindCache {
   const LibraryInfo& libraryFor(uint64_t addr) {
     auto* r = findLibraryFor(addr);
     if (!r) {
-      for (const auto& l : libraries_with_no_unwind_) {
+      for ([[maybe_unused]] const auto& l : libraries_with_no_unwind_) {
         TORCH_WARN("Did not find a PT_GNU_EH_FRAME segment for ", l);
       }
       libraries_with_no_unwind_.clear();
@@ -290,12 +295,12 @@ std::vector<void*> unwind() {
 
 std::optional<std::pair<std::string, uint64_t>> libraryFor(void* addr) {
   if (!addr) {
-    return c10::nullopt;
+    return std::nullopt;
   }
   std::shared_lock lock(cache_mutex_);
   const LibraryInfo* library_info = unwind_cache.findLibraryFor((uint64_t)addr);
   if (!library_info) {
-    return c10::nullopt;
+    return std::nullopt;
   }
   return std::make_pair(
       library_info->name(), (uint64_t)addr - library_info->load_bias());
