@@ -1,14 +1,13 @@
 #include <ATen/core/dispatch/Dispatcher.h>
 #include <torch/csrc/utils/schema_info.h>
 
-namespace torch {
-namespace utils {
+namespace torch::utils {
 void SchemaInfo::addArgumentValue(
     const std::string& name,
     const at::IValue& value) {
   std::optional<int> index = schema_.argumentIndexWithName(name);
   TORCH_INTERNAL_ASSERT(
-      index != c10::nullopt, "Schema has no argument named ", name);
+      index != std::nullopt, "Schema has no argument named ", name);
   value_map_[name] = value;
   alias_maps_current_ = false;
 }
@@ -102,7 +101,7 @@ bool SchemaInfo::is_mutable(const c10::SchemaArgument& argument) {
 }
 
 bool SchemaInfo::has_argument(c10::string_view name) {
-  return schema_.argumentIndexWithName(name) != c10::nullopt;
+  return schema_.argumentIndexWithName(name) != std::nullopt;
 }
 
 bool SchemaInfo::is_mutable(c10::string_view name) {
@@ -433,5 +432,4 @@ void SchemaInfo::generateAliasMaps() {
   }
 }
 
-} // namespace utils
-} // namespace torch
+} // namespace torch::utils
