@@ -1,13 +1,17 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates
 from ._IR import Pipe, pipe_split, pipeline, SplitPoint
 from .schedules import (
+    _ScheduleForwardOnly,
     Schedule1F1B,
+    ScheduleFlexibleInterleaved1F1B,
     ScheduleGPipe,
     ScheduleInterleaved1F1B,
+    ScheduleInterleavedZeroBubble,
     ScheduleLoopedBFS,
 )
 from .stage import build_stage, PipelineStage
 
+
 __all__ = [
     "Pipe",
     "pipe_split",
@@ -16,7 +20,9 @@ __all__ = [
     "PipelineStage",
     "build_stage",
     "Schedule1F1B",
+    "ScheduleFlexibleInterleaved1F1B",
     "ScheduleGPipe",
     "ScheduleInterleaved1F1B",
     "ScheduleLoopedBFS",
+    "ScheduleInterleavedZeroBubble",
 ]
