@@ -1,3 +1,4 @@
+# mypy: allow-untyped-defs
 r"""
 The following constraints are implemented:
 
@@ -14,6 +15,7 @@ The following constraints are implemented:
 - ``constraints.lower_cholesky``
 - ``constraints.lower_triangular``
 - ``constraints.multinomial``
+- ``constraints.nonnegative``
 - ``constraints.nonnegative_integer``
 - ``constraints.one_hot``
 - ``constraints.positive_integer``
@@ -32,40 +34,43 @@ The following constraints are implemented:
 
 import torch
 
+
 __all__ = [
-    'Constraint',
-    'boolean',
-    'cat',
-    'corr_cholesky',
-    'dependent',
-    'dependent_property',
-    'greater_than',
-    'greater_than_eq',
-    'independent',
-    'integer_interval',
-    'interval',
-    'half_open_interval',
-    'is_dependent',
-    'less_than',
-    'lower_cholesky',
-    'lower_triangular',
-    'multinomial',
-    'nonnegative_integer',
-    'positive',
-    'positive_semidefinite',
-    'positive_definite',
-    'positive_integer',
-    'real',
-    'real_vector',
-    'simplex',
-    'square',
-    'stack',
-    'symmetric',
-    'unit_interval',
+    "Constraint",
+    "boolean",
+    "cat",
+    "corr_cholesky",
+    "dependent",
+    "dependent_property",
+    "greater_than",
+    "greater_than_eq",
+    "independent",
+    "integer_interval",
+    "interval",
+    "half_open_interval",
+    "is_dependent",
+    "less_than",
+    "lower_cholesky",
+    "lower_triangular",
+    "multinomial",
+    "nonnegative",
+    "nonnegative_integer",
+    "one_hot",
+    "positive",
+    "positive_semidefinite",
+    "positive_definite",
+    "positive_integer",
+    "real",
+    "real_vector",
+    "simplex",
+    "square",
+    "stack",
+    "symmetric",
+    "unit_interval",
 ]
 
 
-class Constraint(object):
+class Constraint:
     """
     Abstract base class for constraints.
 
@@ -79,6 +84,7 @@ class Constraint(object):
             an event. The :meth:`check` method will remove this many dimensions
             when computing validity.
     """
+
     is_discrete = False  # Default to continuous.
     event_dim = 0  # Default to univariate.
 
@@ -90,7 +96,7 @@ class Constraint(object):
         raise NotImplementedError
 
     def __repr__(self):
-        return self.__class__.__name__[1:] + '()'
+        return self.__class__.__name__[1:] + "()"
 
 
 class _Dependent(Constraint):
@@ -106,6 +112,7 @@ class _Dependent(Constraint):
             can be computed statically. If not provided, access to the
             ``.event_dim`` attribute will raise a NotImplementedError.
     """
+
     def __init__(self, *, is_discrete=NotImplemented, event_dim=NotImplemented):
         self._is_discrete = is_discrete
         self._event_dim = event_dim
@@ -136,10 +143,32 @@ class _Dependent(Constraint):
         return _Dependent(is_discrete=is_discrete, event_dim=event_dim)
 
     def check(self, x):
-        raise ValueError('Cannot determine validity of dependent constraint')
+        raise ValueError("Cannot determine validity of dependent constraint")
 
 
 def is_dependent(constraint):
+    """
+    Checks if ``constraint`` is a ``_Dependent`` object.
+
+    Args:
+        constraint : A ``Constraint`` object.
+
+    Returns:
+        ``bool``: True if ``constraint`` can be refined to the type ``_Dependent``, False otherwise.
+
+    Examples:
+        >>> import torch
+        >>> from torch.distributions import Bernoulli
+        >>> from torch.distributions.constraints import is_dependent
+
+        >>> dist = Bernoulli(probs = torch.tensor([0.6], requires_grad=True))
+        >>> constraint1 = dist.arg_constraints["probs"]
+        >>> constraint2 = dist.arg_constraints["logits"]
+
+        >>> for constraint in [constraint1, constraint2]:
+        >>>     if is_dependent(constraint):
+        >>>         continue
+    """
     return isinstance(constraint, _Dependent)
 
 
@@ -167,7 +196,10 @@ class _DependentProperty(property, _Dependent):
             can be computed statically. If not provided, access to the
             ``.event_dim`` attribute will raise a NotImplementedError.
     """
-    def __init__(self, fn=None, *, is_discrete=NotImplemented, event_dim=NotImplemented):
+
+    def __init__(
+        self, fn=None, *, is_discrete=NotImplemented, event_dim=NotImplemented
+    ):
         super().__init__(fn)
         self._is_discrete = is_discrete
         self._event_dim = event_dim
@@ -180,7 +212,9 @@ class _DependentProperty(property, _Dependent):
             def support(self):
                 ...
         """
-        return _DependentProperty(fn, is_discrete=self._is_discrete, event_dim=self._event_dim)
+        return _DependentProperty(
+            fn, is_discrete=self._is_discrete, event_dim=self._event_dim
+        )
 
 
 class _IndependentConstraint(Constraint):
@@ -189,6 +223,7 @@ class _IndependentConstraint(Constraint):
     dims in :meth:`check`, so that an event is valid only if all its
     independent entries are valid.
     """
+
     def __init__(self, base_constraint, reinterpreted_batch_ndims):
         assert isinstance(base_constraint, Constraint)
         assert isinstance(reinterpreted_batch_ndims, int)
@@ -209,20 +244,24 @@ class _IndependentConstraint(Constraint):
         result = self.base_constraint.check(value)
         if result.dim() < self.reinterpreted_batch_ndims:
             expected = self.base_constraint.event_dim + self.reinterpreted_batch_ndims
-            raise ValueError(f"Expected value.dim() >= {expected} but got {value.dim()}")
-        result = result.reshape(result.shape[:result.dim() - self.reinterpreted_batch_ndims] + (-1,))
+            raise ValueError(
+                f"Expected value.dim() >= {expected} but got {value.dim()}"
+            )
+        result = result.reshape(
+            result.shape[: result.dim() - self.reinterpreted_batch_ndims] + (-1,)
+        )
         result = result.all(-1)
         return result
 
     def __repr__(self):
-        return "{}({}, {})".format(self.__class__.__name__[1:], repr(self.base_constraint),
-                                   self.reinterpreted_batch_ndims)
+        return f"{self.__class__.__name__[1:]}({repr(self.base_constraint)}, {self.reinterpreted_batch_ndims})"
 
 
 class _Boolean(Constraint):
     """
     Constrain to the two values `{0, 1}`.
     """
+
     is_discrete = True
 
     def check(self, value):
@@ -233,6 +272,7 @@ class _OneHot(Constraint):
     """
     Constrain to one-hot vectors.
     """
+
     is_discrete = True
     event_dim = 1
 
@@ -246,6 +286,7 @@ class _IntegerInterval(Constraint):
     """
     Constrain to an integer interval `[lower_bound, upper_bound]`.
     """
+
     is_discrete = True
 
     def __init__(self, lower_bound, upper_bound):
@@ -254,11 +295,15 @@ class _IntegerInterval(Constraint):
         super().__init__()
 
     def check(self, value):
-        return (value % 1 == 0) & (self.lower_bound <= value) & (value <= self.upper_bound)
+        return (
+            (value % 1 == 0) & (self.lower_bound <= value) & (value <= self.upper_bound)
+        )
 
     def __repr__(self):
         fmt_string = self.__class__.__name__[1:]
-        fmt_string += '(lower_bound={}, upper_bound={})'.format(self.lower_bound, self.upper_bound)
+        fmt_string += (
+            f"(lower_bound={self.lower_bound}, upper_bound={self.upper_bound})"
+        )
         return fmt_string
 
 
@@ -266,6 +311,7 @@ class _IntegerLessThan(Constraint):
     """
     Constrain to an integer interval `(-inf, upper_bound]`.
     """
+
     is_discrete = True
 
     def __init__(self, upper_bound):
@@ -277,7 +323,7 @@ class _IntegerLessThan(Constraint):
 
     def __repr__(self):
         fmt_string = self.__class__.__name__[1:]
-        fmt_string += '(upper_bound={})'.format(self.upper_bound)
+        fmt_string += f"(upper_bound={self.upper_bound})"
         return fmt_string
 
 
@@ -285,6 +331,7 @@ class _IntegerGreaterThan(Constraint):
     """
     Constrain to an integer interval `[lower_bound, inf)`.
     """
+
     is_discrete = True
 
     def __init__(self, lower_bound):
@@ -296,7 +343,7 @@ class _IntegerGreaterThan(Constraint):
 
     def __repr__(self):
         fmt_string = self.__class__.__name__[1:]
-        fmt_string += '(lower_bound={})'.format(self.lower_bound)
+        fmt_string += f"(lower_bound={self.lower_bound})"
         return fmt_string
 
 
@@ -304,6 +351,7 @@ class _Real(Constraint):
     """
     Trivially constrain to the extended real line `[-inf, inf]`.
     """
+
     def check(self, value):
         return value == value  # False for NANs.
 
@@ -312,6 +360,7 @@ class _GreaterThan(Constraint):
     """
     Constrain to a real half line `(lower_bound, inf]`.
     """
+
     def __init__(self, lower_bound):
         self.lower_bound = lower_bound
         super().__init__()
@@ -321,7 +370,7 @@ class _GreaterThan(Constraint):
 
     def __repr__(self):
         fmt_string = self.__class__.__name__[1:]
-        fmt_string += '(lower_bound={})'.format(self.lower_bound)
+        fmt_string += f"(lower_bound={self.lower_bound})"
         return fmt_string
 
 
@@ -329,6 +378,7 @@ class _GreaterThanEq(Constraint):
     """
     Constrain to a real half line `[lower_bound, inf)`.
     """
+
     def __init__(self, lower_bound):
         self.lower_bound = lower_bound
         super().__init__()
@@ -338,7 +388,7 @@ class _GreaterThanEq(Constraint):
 
     def __repr__(self):
         fmt_string = self.__class__.__name__[1:]
-        fmt_string += '(lower_bound={})'.format(self.lower_bound)
+        fmt_string += f"(lower_bound={self.lower_bound})"
         return fmt_string
 
 
@@ -346,6 +396,7 @@ class _LessThan(Constraint):
     """
     Constrain to a real half line `[-inf, upper_bound)`.
     """
+
     def __init__(self, upper_bound):
         self.upper_bound = upper_bound
         super().__init__()
@@ -355,7 +406,7 @@ class _LessThan(Constraint):
 
     def __repr__(self):
         fmt_string = self.__class__.__name__[1:]
-        fmt_string += '(upper_bound={})'.format(self.upper_bound)
+        fmt_string += f"(upper_bound={self.upper_bound})"
         return fmt_string
 
 
@@ -363,6 +414,7 @@ class _Interval(Constraint):
     """
     Constrain to a real interval `[lower_bound, upper_bound]`.
     """
+
     def __init__(self, lower_bound, upper_bound):
         self.lower_bound = lower_bound
         self.upper_bound = upper_bound
@@ -373,7 +425,9 @@ class _Interval(Constraint):
 
     def __repr__(self):
         fmt_string = self.__class__.__name__[1:]
-        fmt_string += '(lower_bound={}, upper_bound={})'.format(self.lower_bound, self.upper_bound)
+        fmt_string += (
+            f"(lower_bound={self.lower_bound}, upper_bound={self.upper_bound})"
+        )
         return fmt_string
 
 
@@ -381,6 +435,7 @@ class _HalfOpenInterval(Constraint):
     """
     Constrain to a real interval `[lower_bound, upper_bound)`.
     """
+
     def __init__(self, lower_bound, upper_bound):
         self.lower_bound = lower_bound
         self.upper_bound = upper_bound
@@ -391,7 +446,9 @@ class _HalfOpenInterval(Constraint):
 
     def __repr__(self):
         fmt_string = self.__class__.__name__[1:]
-        fmt_string += '(lower_bound={}, upper_bound={})'.format(self.lower_bound, self.upper_bound)
+        fmt_string += (
+            f"(lower_bound={self.lower_bound}, upper_bound={self.upper_bound})"
+        )
         return fmt_string
 
 
@@ -400,6 +457,7 @@ class _Simplex(Constraint):
     Constrain to the unit simplex in the innermost (rightmost) dimension.
     Specifically: `x >= 0` and `x.sum(-1) == 1`.
     """
+
     event_dim = 1
 
     def check(self, value):
@@ -414,6 +472,7 @@ class _Multinomial(Constraint):
     checks the weaker condition ``value.sum(-1) <= upper_bound``. In the future
     this may be strengthened to ``value.sum(-1) == upper_bound``.
     """
+
     is_discrete = True
     event_dim = 1
 
@@ -428,6 +487,7 @@ class _LowerTriangular(Constraint):
     """
     Constrain to lower-triangular square matrices.
     """
+
     event_dim = 2
 
     def check(self, value):
@@ -439,11 +499,14 @@ class _LowerCholesky(Constraint):
     """
     Constrain to lower-triangular square matrices with positive diagonals.
     """
+
     event_dim = 2
 
     def check(self, value):
         value_tril = value.tril()
-        lower_triangular = (value_tril == value).view(value.shape[:-2] + (-1,)).min(-1)[0]
+        lower_triangular = (
+            (value_tril == value).view(value.shape[:-2] + (-1,)).min(-1)[0]
+        )
 
         positive_diagonal = (value.diagonal(dim1=-2, dim2=-1) > 0).min(-1)[0]
         return lower_triangular & positive_diagonal
@@ -454,12 +517,15 @@ class _CorrCholesky(Constraint):
     Constrain to lower-triangular square matrices with positive diagonals and each
     row vector being of unit length.
     """
+
     event_dim = 2
 
     def check(self, value):
-        tol = torch.finfo(value.dtype).eps * value.size(-1) * 10  # 10 is an adjustable fudge factor
+        tol = (
+            torch.finfo(value.dtype).eps * value.size(-1) * 10
+        )  # 10 is an adjustable fudge factor
         row_norm = torch.linalg.norm(value.detach(), dim=-1)
-        unit_row_norm = (row_norm - 1.).abs().le(tol).all(dim=-1)
+        unit_row_norm = (row_norm - 1.0).abs().le(tol).all(dim=-1)
         return _LowerCholesky().check(value) & unit_row_norm
 
 
@@ -467,6 +533,7 @@ class _Square(Constraint):
     """
     Constrain to square matrices.
     """
+
     event_dim = 2
 
     def check(self, value):
@@ -474,7 +541,7 @@ class _Square(Constraint):
             size=value.shape[:-2],
             fill_value=(value.shape[-2] == value.shape[-1]),
             dtype=torch.bool,
-            device=value.device
+            device=value.device,
         )
 
 
@@ -494,6 +561,7 @@ class _PositiveSemidefinite(_Symmetric):
     """
     Constrain to positive-semidefinite matrices.
     """
+
     def check(self, value):
         sym_check = super().check(value)
         if not sym_check.all():
@@ -505,6 +573,7 @@ class _PositiveDefinite(_Symmetric):
     """
     Constrain to positive-definite matrices.
     """
+
     def check(self, value):
         sym_check = super().check(value)
         if not sym_check.all():
@@ -518,6 +587,7 @@ class _Cat(Constraint):
     `cseq` at the submatrices at dimension `dim`,
     each of size `lengths[dim]`, in a way compatible with :func:`torch.cat`.
     """
+
     def __init__(self, cseq, dim=0, lengths=None):
         assert all(isinstance(c, Constraint) for c in cseq)
         self.cseq = list(cseq)
@@ -553,6 +623,7 @@ class _Stack(Constraint):
     `cseq` at the submatrices at dimension `dim`,
     in a way compatible with :func:`torch.stack`.
     """
+
     def __init__(self, cseq, dim=0):
         assert all(isinstance(c, Constraint) for c in cseq)
         self.cseq = list(cseq)
@@ -573,8 +644,9 @@ class _Stack(Constraint):
     def check(self, value):
         assert -value.dim() <= self.dim < value.dim()
         vs = [value.select(self.dim, i) for i in range(value.size(self.dim))]
-        return torch.stack([constr.check(v)
-                            for v, constr in zip(vs, self.cseq)], self.dim)
+        return torch.stack(
+            [constr.check(v) for v, constr in zip(vs, self.cseq)], self.dim
+        )
 
 
 # Public interface.
@@ -588,13 +660,13 @@ positive_integer = _IntegerGreaterThan(1)
 integer_interval = _IntegerInterval
 real = _Real()
 real_vector = independent(real, 1)
-positive = _GreaterThan(0.)
-nonnegative = _GreaterThanEq(0.)
+positive = _GreaterThan(0.0)
+nonnegative = _GreaterThanEq(0.0)
 greater_than = _GreaterThan
 greater_than_eq = _GreaterThanEq
 less_than = _LessThan
 multinomial = _Multinomial
-unit_interval = _Interval(0., 1.)
+unit_interval = _Interval(0.0, 1.0)
 interval = _Interval
 half_open_interval = _HalfOpenInterval
 simplex = _Simplex()
