@@ -1,26 +1,45 @@
-from collections import namedtuple
 import warnings
+from collections.abc import Iterable
+from typing import (
+    Any,
+    Callable,
+    List,
+    NamedTuple,
+    Optional,
+    overload,
+    Tuple,
+    TypeVar,
+    Union,
+)
+from typing_extensions import Self
 
 import torch
-from torch import Tensor
-from ... import _VF
-from ..._jit_internal import Optional
-
-from typing import List, Tuple, Union, Iterable
+from torch import _VF, Tensor
 
 
-__all__ = ['PackedSequence', 'invert_permutation', 'pack_padded_sequence', 'pad_packed_sequence', 'pad_sequence',
-           'unpad_sequence', 'pack_sequence', 'unpack_sequence']
+__all__ = [
+    "PackedSequence",
+    "invert_permutation",
+    "pack_padded_sequence",
+    "pad_packed_sequence",
+    "pad_sequence",
+    "unpad_sequence",
+    "pack_sequence",
+    "unpack_sequence",
+]
 
-PackedSequence_ = namedtuple('PackedSequence_',
-                             ['data', 'batch_sizes', 'sorted_indices', 'unsorted_indices'])
+_T = TypeVar("_T")
+_R = TypeVar("_R")
 
-# type annotation for PackedSequence_ to make it compatible with TorchScript
-PackedSequence_.__annotations__ = {'data': torch.Tensor, 'batch_sizes': torch.Tensor,
-                                   'sorted_indices': Optional[torch.Tensor],
-                                   'unsorted_indices': Optional[torch.Tensor]}
 
-def bind(optional, fn):
+class PackedSequence_(NamedTuple):
+    data: torch.Tensor
+    batch_sizes: torch.Tensor
+    sorted_indices: Optional[torch.Tensor]
+    unsorted_indices: Optional[torch.Tensor]
+
+
+def bind(optional: Optional[_T], fn: Callable[[_T], _R]) -> Optional[_R]:
     if optional is None:
         return None
     return fn(optional)
@@ -58,67 +77,68 @@ class PackedSequence(PackedSequence_):
         However, :attr:`batch_sizes` should always be a CPU ``torch.int64`` tensor.
 
         This invariant is maintained throughout :class:`PackedSequence` class,
-        and all functions that construct a `:class:PackedSequence` in PyTorch
+        and all functions that construct a :class:`PackedSequence` in PyTorch
         (i.e., they only pass in tensors conforming to this constraint).
-
     """
-    def __new__(cls, data, batch_sizes=None, sorted_indices=None, unsorted_indices=None):
-        return super(PackedSequence, cls).__new__(
+
+    def __new__(
+        cls,
+        data: Tensor,
+        batch_sizes: Optional[Tensor] = None,
+        sorted_indices: Optional[Tensor] = None,
+        unsorted_indices: Optional[Tensor] = None,
+    ) -> Self:
+        return super().__new__(
             cls,
-            *_packed_sequence_init_args(data, batch_sizes, sorted_indices,
-                                        unsorted_indices))
+            *_packed_sequence_init_args(
+                data, batch_sizes, sorted_indices, unsorted_indices
+            ),
+        )
 
     # NOTE [ device and dtype of a PackedSequence ]
     #
     # See the note above in doc string (starting with ":attr:`data` can be on
     # arbitrary device...").
-    def pin_memory(self):
+    def pin_memory(self) -> Self:
         # Why not convert `batch_sizes`?
         # See NOTE [ device and dtype of a PackedSequence ]
-        return type(self)(self.data.pin_memory(), self.batch_sizes,
-                          bind(self.sorted_indices, lambda t: t.pin_memory()),
-                          bind(self.unsorted_indices, lambda t: t.pin_memory()))
+        return type(self)(
+            self.data.pin_memory(),
+            self.batch_sizes,
+            bind(self.sorted_indices, lambda t: t.pin_memory()),
+            bind(self.unsorted_indices, lambda t: t.pin_memory()),
+        )
 
-    def cuda(self, *args, **kwargs):
-        # Tests to see if 'cuda' should be added to kwargs
-        ex = torch.tensor((), dtype=self.data.dtype, device=self.data.device).to(*args, **kwargs)
-        if ex.is_cuda:
-            return self.to(*args, **kwargs)
-        return self.to(*args, device='cuda', **kwargs)
+    @overload
+    def to(
+        self,
+        dtype: torch.dtype,
+        non_blocking: bool = ...,
+        copy: bool = ...,
+    ) -> Self:
+        ...
 
-    def cpu(self, *args, **kwargs):
+    @overload
+    def to(
+        self,
+        device: Optional[Union[str, torch.device, int]] = ...,
+        dtype: Optional[torch.dtype] = ...,
+        non_blocking: bool = ...,
+        copy: bool = ...,
+    ) -> Self:
+        ...
 
-        ex = torch.tensor((), dtype=self.data.dtype, device=self.data.device).to(*args, **kwargs)
-        if ex.device.type == 'cpu':
-            return self.to(*args, **kwargs)
-        return self.to(*args, device='cpu', **kwargs)
+    @overload
+    def to(
+        self,
+        other: Tensor,
+        non_blocking: bool = ...,
+        copy: bool = ...,
+    ) -> Self:
+        ...
 
-    def double(self):
-        return self.to(dtype=torch.double)
-
-    def float(self):
-        return self.to(dtype=torch.float)
-
-    def half(self):
-        return self.to(dtype=torch.half)
-
-    def long(self):
-        return self.to(dtype=torch.long)
-
-    def int(self):
-        return self.to(dtype=torch.int)
-
-    def short(self):
-        return self.to(dtype=torch.short)
-
-    def char(self):
-        return self.to(dtype=torch.int8)
-
-    def byte(self):
-        return self.to(dtype=torch.uint8)
-
-    def to(self, *args, **kwargs):
-        r"""Performs dtype and/or device conversion on `self.data`.
+    def to(self, *args: Any, **kwargs: Any) -> Self:
+        r"""Perform dtype and/or device conversion on `self.data`.
 
         It has similar signature as :meth:`torch.Tensor.to`, except optional
         arguments like `non_blocking` and `copy` should be passed as kwargs,
@@ -130,7 +150,6 @@ class PackedSequence(PackedSequence_):
             and :class:`torch.device`, then ``self`` is returned.
             Otherwise, returns a copy with the desired configuration.
         """
-
         # Why not convert `batch_sizes`?
         # See NOTE [ device and dtype of a PackedSequence ]
         data = self.data.to(*args, **kwargs)
@@ -138,18 +157,67 @@ class PackedSequence(PackedSequence_):
             return self
         else:
             # Does not forward device or dtype arg/kwargs, device is set from data.device
-            kwargs = {k : v for k, v in filter(lambda t: t[0] != 'device' and t[0] != 'dtype', kwargs.items())}
-            sorted_indices = bind(self.sorted_indices, lambda t: t.to(data.device, **kwargs))
-            unsorted_indices = bind(self.unsorted_indices, lambda t: t.to(data.device, **kwargs))
+            kwargs = dict(
+                filter(lambda t: t[0] != "device" and t[0] != "dtype", kwargs.items())
+            )
+            sorted_indices = bind(
+                self.sorted_indices, lambda t: t.to(data.device, **kwargs)
+            )
+            unsorted_indices = bind(
+                self.unsorted_indices, lambda t: t.to(data.device, **kwargs)
+            )
             return type(self)(data, self.batch_sizes, sorted_indices, unsorted_indices)
 
+    def cuda(self, *args: Any, **kwargs: Any) -> Self:
+        # Tests to see if 'cuda' should be added to kwargs
+        ex = torch.tensor((), dtype=self.data.dtype, device=self.data.device).to(
+            *args, **kwargs
+        )
+        if ex.is_cuda:
+            return self.to(*args, **kwargs)
+        kwargs["device"] = "cuda"
+        return self.to(*args, **kwargs)
+
+    def cpu(self, *args: Any, **kwargs: Any) -> Self:
+        ex = torch.tensor((), dtype=self.data.dtype, device=self.data.device).to(
+            *args, **kwargs
+        )
+        if ex.device.type == "cpu":
+            return self.to(*args, **kwargs)
+        kwargs["device"] = "cpu"
+        return self.to(*args, **kwargs)
+
+    def double(self) -> Self:
+        return self.to(dtype=torch.double)
+
+    def float(self) -> Self:
+        return self.to(dtype=torch.float)
+
+    def half(self) -> Self:
+        return self.to(dtype=torch.half)
+
+    def long(self) -> Self:
+        return self.to(dtype=torch.long)
+
+    def int(self) -> Self:
+        return self.to(dtype=torch.int)
+
+    def short(self) -> Self:
+        return self.to(dtype=torch.short)
+
+    def char(self) -> Self:
+        return self.to(dtype=torch.int8)
+
+    def byte(self) -> Self:
+        return self.to(dtype=torch.uint8)
+
     @property
-    def is_cuda(self):
-        r"""Returns true if `self.data` stored on a gpu"""
+    def is_cuda(self) -> bool:
+        r"""Return true if `self.data` stored on a gpu."""
         return self.data.is_cuda
 
-    def is_pinned(self):
-        r"""Returns true if `self.data` stored on in pinned memory"""
+    def is_pinned(self) -> bool:
+        r"""Return true if `self.data` stored on in pinned memory."""
         return self.data.is_pinned()
 
 
@@ -171,13 +239,14 @@ def _packed_sequence_init_args(
     # support being called as `PackedSequence(data, batch_sizes, sorted_indices)`
     if batch_sizes is not None:
         # TODO: Re-enable this check (.type isn't supported in TorchScript)
-        if batch_sizes.device.type != 'cpu':
+        if batch_sizes.device.type != "cpu":
             raise ValueError(
                 "batch_sizes should always be on CPU. "
                 "Instances of PackedSequence should never be created manually. "
                 "They should be instantiated by functions like pack_sequence "
                 "and pack_padded_sequences in nn.utils.rnn. "
-                "https://pytorch.org/docs/stable/nn.html#torch.nn.utils.rnn.pack_sequence")
+                "https://pytorch.org/docs/stable/nn.html#torch.nn.utils.rnn.pack_sequence"
+            )
         return data, batch_sizes, sorted_indices, unsorted_indices
 
     # support being called as `PackedSequence((data, batch_sizes), *, sorted_indices)`
@@ -193,7 +262,8 @@ def _packed_sequence_init(
     unsorted_indices: Optional[Tensor] = None,
 ) -> PackedSequence:
     data, batch_sizes, sorted_indices, unsorted_indices = _packed_sequence_init_args(
-        data, batch_sizes, sorted_indices, unsorted_indices)
+        data, batch_sizes, sorted_indices, unsorted_indices
+    )
     return PackedSequence(data, batch_sizes, sorted_indices, unsorted_indices)
 
 
@@ -201,23 +271,24 @@ def invert_permutation(permutation: Optional[Tensor]) -> Optional[Tensor]:
     if permutation is None:
         return None
     output = torch.empty_like(permutation, memory_format=torch.legacy_contiguous_format)
-    output.scatter_(0, permutation,
-                    torch.arange(0, permutation.numel(), device=permutation.device))
+    output.scatter_(
+        0, permutation, torch.arange(0, permutation.numel(), device=permutation.device)
+    )
     return output
 
 
 def pack_padded_sequence(
     input: Tensor,
-    lengths: Tensor,
+    lengths: Union[Tensor, List[int]],
     batch_first: bool = False,
     enforce_sorted: bool = True,
 ) -> PackedSequence:
     r"""Packs a Tensor containing padded sequences of variable length.
 
-    :attr:`input` can be of size ``T x B x *`` where `T` is the length of the
-    longest sequence (equal to ``lengths[0]``), ``B`` is the batch size, and
-    ``*`` is any number of dimensions (including 0). If ``batch_first`` is
-    ``True``, ``B x T x *`` :attr:`input` is expected.
+    :attr:`input` can be of size ``T x B x *`` (if :attr:`batch_first` is ``False``)
+    or ``B x T x *`` (if :attr:`batch_first` is ``True``) where ``T`` is the length
+    of the longest sequence, ``B`` is the batch size, and ``*`` is any number of dimensions
+    (including 0).
 
     For unsorted sequences, use `enforce_sorted = False`. If :attr:`enforce_sorted` is
     ``True``, the sequences should be sorted by length in a decreasing order, i.e.
@@ -235,7 +306,7 @@ def pack_padded_sequence(
         lengths (Tensor or list(int)): list of sequence lengths of each batch
             element (must be on the CPU if provided as a tensor).
         batch_first (bool, optional): if ``True``, the input is expected in ``B x T x *``
-            format.
+            format, ``T x B x *`` otherwise.
         enforce_sorted (bool, optional): if ``True``, the input is expected to
             contain sequences sorted by length in a decreasing order. If
             ``False``, the input will get sorted unconditionally. Default: ``True``.
@@ -243,13 +314,19 @@ def pack_padded_sequence(
     Returns:
         a :class:`PackedSequence` object
     """
-    if torch._C._get_tracing_state() and not isinstance(lengths, torch.Tensor):
-        warnings.warn('pack_padded_sequence has been called with a Python list of '
-                      'sequence lengths. The tracer cannot track the data flow of Python '
-                      'values, and it will treat them as constants, likely rendering '
-                      'the trace incorrect for any other combination of lengths.',
-                      stacklevel=2)
-    lengths = torch.as_tensor(lengths, dtype=torch.int64)
+    if not isinstance(lengths, torch.Tensor):
+        if torch._C._get_tracing_state():
+            warnings.warn(
+                "pack_padded_sequence has been called with a Python list of "
+                "sequence lengths. The tracer cannot track the data flow of Python "
+                "values, and it will treat them as constants, likely rendering "
+                "the trace incorrect for any other combination of lengths.",
+                stacklevel=2,
+            )
+        lengths = torch.as_tensor(lengths, dtype=torch.int64, device="cpu")
+    else:
+        lengths = lengths.to(dtype=torch.int64)
+
     if enforce_sorted:
         sorted_indices = None
     else:
@@ -258,8 +335,7 @@ def pack_padded_sequence(
         batch_dim = 0 if batch_first else 1
         input = input.index_select(batch_dim, sorted_indices)
 
-    data, batch_sizes = \
-        _VF._pack_padded_sequence(input, lengths, batch_first)
+    data, batch_sizes = _VF._pack_padded_sequence(input, lengths, batch_first)
     return _packed_sequence_init(data, batch_sizes, sorted_indices, None)
 
 
@@ -269,17 +345,17 @@ def pad_packed_sequence(
     padding_value: float = 0.0,
     total_length: Optional[int] = None,
 ) -> Tuple[Tensor, Tensor]:
-    r"""Pads a packed batch of variable length sequences.
+    r"""Pad a packed batch of variable length sequences.
 
     It is an inverse operation to :func:`pack_padded_sequence`.
 
-    The returned Tensor's data will be of size ``T x B x *``, where `T` is the length
-    of the longest sequence and `B` is the batch size. If ``batch_first`` is True,
-    the data will be transposed into ``B x T x *`` format.
+    The returned Tensor's data will be of size ``T x B x *`` (if :attr:`batch_first` is ``False``)
+    or ``B x T x *`` (if :attr:`batch_first` is ``True``) , where ``T`` is the length of the longest
+    sequence and ``B`` is the batch size.
 
     Example:
         >>> from torch.nn.utils.rnn import pack_padded_sequence, pad_packed_sequence
-        >>> seq = torch.tensor([[1,2,0], [3,0,0], [4,5,6]])
+        >>> seq = torch.tensor([[1, 2, 0], [3, 0, 0], [4, 5, 6]])
         >>> lens = [2, 1, 3]
         >>> packed = pack_padded_sequence(seq, lens, batch_first=True, enforce_sorted=False)
         >>> packed
@@ -303,7 +379,7 @@ def pad_packed_sequence(
     Args:
         sequence (PackedSequence): batch to pad
         batch_first (bool, optional): if ``True``, the output will be in ``B x T x *``
-            format.
+            format, ``T x B x *`` otherwise.
         padding_value (float, optional): values for padded elements.
         total_length (int, optional): if not ``None``, the output will be padded to
             have length :attr:`total_length`. This method will throw :class:`ValueError`
@@ -315,44 +391,45 @@ def pad_packed_sequence(
         containing the list of lengths of each sequence in the batch.
         Batch elements will be re-ordered as they were ordered originally when
         the batch was passed to ``pack_padded_sequence`` or ``pack_sequence``.
-
-
-
-
     """
     max_seq_length = sequence.batch_sizes.size(0)
     if total_length is not None:
         if total_length < max_seq_length:
-            raise ValueError("Expected total_length to be at least the length "
-                             "of the longest sequence in input, but got "
-                             "total_length={} and max sequence length being {}"
-                             .format(total_length, max_seq_length))
+            raise ValueError(
+                "Expected total_length to be at least the length "
+                "of the longest sequence in input, but got "
+                f"total_length={total_length} and max sequence length being {max_seq_length}"
+            )
         max_seq_length = total_length
     padded_output, lengths = _VF._pad_packed_sequence(
-        sequence.data, sequence.batch_sizes, batch_first, padding_value, max_seq_length)
+        sequence.data, sequence.batch_sizes, batch_first, padding_value, max_seq_length
+    )
     unsorted_indices = sequence.unsorted_indices
     if unsorted_indices is not None:
         batch_dim = 0 if batch_first else 1
-        return padded_output.index_select(batch_dim, unsorted_indices), lengths[unsorted_indices.cpu()]
+        return (
+            padded_output.index_select(batch_dim, unsorted_indices),
+            lengths[unsorted_indices.cpu()],
+        )
     return padded_output, lengths
 
 
+# NOTE: for JIT-compatibility, we need to be more restrictive here and use specific types instead of Iterable.
 def pad_sequence(
     sequences: Union[Tensor, List[Tensor]],
     batch_first: bool = False,
     padding_value: float = 0.0,
+    padding_side: str = "right",
 ) -> Tensor:
-    r"""Pad a list of variable length Tensors with ``padding_value``
+    r"""Pad a list of variable length Tensors with :attr:`padding_value`.
 
-    ``pad_sequence`` stacks a list of Tensors along a new dimension,
-    and pads them to equal length. For example, if the input is list of
-    sequences with size ``L x *`` and if batch_first is False, and ``T x B x *``
-    otherwise.
-
-    `B` is batch size. It is equal to the number of elements in ``sequences``.
-    `T` is length of the longest sequence.
-    `L` is length of the sequence.
-    `*` is any number of trailing dimensions, including none.
+    ``pad_sequence`` stacks a list of Tensors along a new dimension, and pads them
+    to equal length. :attr:`sequences` can be list of sequences with size ``L x *``,
+    where `L` is length of the sequence and ``*`` is any number of dimensions
+    (including 0). If :attr:`batch_first` is ``False``, the output is of size
+    ``T x B x *``, and ``B x T x *`` otherwise, where ``B`` is the batch size
+    (the number of elements in :attr:`sequences`), ``T`` is the length of the longest
+    sequence.
 
     Example:
         >>> from torch.nn.utils.rnn import pad_sequence
@@ -369,33 +446,38 @@ def pad_sequence(
 
     Args:
         sequences (list[Tensor]): list of variable length sequences.
-        batch_first (bool, optional): output will be in ``B x T x *`` if True, or in
-            ``T x B x *`` otherwise. Default: False.
+        batch_first (bool, optional): if ``True``, the output will be in ``B x T x *``
+            format, ``T x B x *`` otherwise.
         padding_value (float, optional): value for padded elements. Default: 0.
+        padding_side (str, optional): the side to pad the sequences on.
+            Default: "right".
 
     Returns:
         Tensor of size ``T x B x *`` if :attr:`batch_first` is ``False``.
         Tensor of size ``B x T x *`` otherwise
     """
-
     if not (torch.jit.is_tracing() or torch.jit.is_scripting()):
         # JIT doesn't support `Iterable`
         if not isinstance(sequences, Iterable):
-            msg = ('pad_sequence: Expected iterable for input sequences, but got arg of type: '
-                   f'{type(sequences)}')
+            msg = (
+                "pad_sequence: Expected iterable for input sequences, but got arg of type: "
+                f"{type(sequences)}"
+            )
             raise RuntimeError(msg)
 
         # In JIT context this leads to,
         # RuntimeError: cannot statically infer the expected size of a list in this context
-        sequences = tuple(sequences)
+        sequences = tuple(sequences)  # type: ignore[assignment]
     else:
         # For JIT, we only support Union[Tensor, Tuple[Tensor]]
         if isinstance(sequences, torch.Tensor):
-            sequences = sequences.unbind(0)
+            sequences = sequences.unbind(0)  # type: ignore[assignment]
 
     # assuming trailing dimensions and type of all the Tensors
     # in sequences are same and fetching those from sequences[0]
-    return torch._C._nn.pad_sequence(sequences, batch_first, padding_value)
+    return torch._C._nn.pad_sequence(
+        sequences, batch_first, padding_value, padding_side  # type: ignore[arg-type]
+    )
 
 
 def unpad_sequence(
@@ -403,7 +485,7 @@ def unpad_sequence(
     lengths: Tensor,
     batch_first: bool = False,
 ) -> List[Tensor]:
-    r"""Unpad padded Tensor into a list of variable length Tensors
+    r"""Unpad padded Tensor into a list of variable length Tensors.
 
     ``unpad_sequence`` unstacks padded Tensor into a list of variable length Tensors.
 
@@ -431,14 +513,13 @@ def unpad_sequence(
     Returns:
         a list of :class:`Tensor` objects
     """
-
     unpadded_sequences = []
 
     if not batch_first:
         padded_sequences.transpose_(0, 1)
 
     max_length = padded_sequences.shape[1]
-    idx = torch.arange(max_length)
+    idx = torch.arange(max_length, device=lengths.device)
 
     for seq, length in zip(padded_sequences, lengths):
         mask = idx < length
@@ -448,8 +529,11 @@ def unpad_sequence(
     return unpadded_sequences
 
 
-def pack_sequence(sequences: List[Tensor], enforce_sorted: bool = True) -> PackedSequence:
-    r"""Packs a list of variable length Tensors
+def pack_sequence(
+    sequences: List[Tensor],
+    enforce_sorted: bool = True,
+) -> PackedSequence:
+    r"""Packs a list of variable length Tensors.
 
     Consecutive call of the next functions: ``pad_sequence``, ``pack_padded_sequence``.
 
@@ -461,16 +545,14 @@ def pack_sequence(sequences: List[Tensor], enforce_sorted: bool = True) -> Packe
     is ``True``, the sequences should be sorted in the order of decreasing length.
     ``enforce_sorted = True`` is only necessary for ONNX export.
 
-
     Example:
         >>> from torch.nn.utils.rnn import pack_sequence
-        >>> a = torch.tensor([1,2,3])
-        >>> b = torch.tensor([4,5])
+        >>> a = torch.tensor([1, 2, 3])
+        >>> b = torch.tensor([4, 5])
         >>> c = torch.tensor([6])
         >>> pack_sequence([a, b, c])
         PackedSequence(data=tensor([1, 4, 6, 2, 5, 3]), batch_sizes=tensor([3, 2, 1]), sorted_indices=None, unsorted_indices=None)
 
-
     Args:
         sequences (list[Tensor]): A list of sequences of decreasing length.
         enforce_sorted (bool, optional): if ``True``, checks that the input
@@ -481,19 +563,20 @@ def pack_sequence(sequences: List[Tensor], enforce_sorted: bool = True) -> Packe
         a :class:`PackedSequence` object
     """
     lengths = torch.as_tensor([v.size(0) for v in sequences])
-    return pack_padded_sequence(pad_sequence(sequences), lengths, enforce_sorted=enforce_sorted)
+    return pack_padded_sequence(
+        pad_sequence(sequences), lengths, enforce_sorted=enforce_sorted
+    )
 
 
 def unpack_sequence(packed_sequences: PackedSequence) -> List[Tensor]:
-    r"""Unpacks PackedSequence into a list of variable length Tensors
+    r"""Unpack PackedSequence into a list of variable length Tensors.
 
     ``packed_sequences`` should be a PackedSequence object.
 
-
     Example:
         >>> from torch.nn.utils.rnn import pack_sequence, unpack_sequence
-        >>> a = torch.tensor([1,2,3])
-        >>> b = torch.tensor([4,5])
+        >>> a = torch.tensor([1, 2, 3])
+        >>> b = torch.tensor([4, 5])
         >>> c = torch.tensor([6])
         >>> sequences = [a, b, c]
         >>> print(sequences)
@@ -505,14 +588,12 @@ def unpack_sequence(packed_sequences: PackedSequence) -> List[Tensor]:
         >>> print(unpacked_sequences)
         [tensor([1, 2, 3]), tensor([4, 5]), tensor([6])]
 
-
     Args:
         packed_sequences (PackedSequence): A PackedSequence object.
 
     Returns:
         a list of :class:`Tensor` objects
     """
-
     padded_sequences, lengths = pad_packed_sequence(packed_sequences, batch_first=True)
     unpacked_sequences = unpad_sequence(padded_sequences, lengths, batch_first=True)
     return unpacked_sequences
